@@ -34,6 +34,7 @@ from sphinx.roles import XRefRole
 from sphinx.locale import _
 import sphinx
 
+import os
 import subprocess
 import re
 
@@ -47,6 +48,7 @@ except NameError:
     def is_string_type(s):
         return isinstance(s, str)
 
+
 class TSConfVar(std.Target):
     """
     Description of a traffic server configuration variable.
@@ -116,7 +118,7 @@ class TSConfVar(std.Target):
             title.set_class(self.options.get('class'))
         # This has to be a distinct node before the title. if nested then
         # the browser will scroll forward to just past the title.
-        anchor = nodes.target('', '', names=[cv_name])
+        nodes.target('', '', names=[cv_name])
         # Second (optional) arg is 'msgNode' - no idea what I should pass for that
         # or if it even matters, although I now think it should not be used.
         self.state.document.note_explicit_target(title)
@@ -168,7 +170,19 @@ def metrictypes(typename):
 
 
 def metricunits(unitname):
-    return directives.choice(unitname.lower(), ('ratio', 'percent', 'kbits', 'mbits', 'bytes', 'kbytes', 'mbytes', 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'))
+    return directives.choice(
+        unitname.lower(),
+        ('ratio',
+         'percent',
+         'kbits',
+         'mbits',
+         'bytes',
+         'kbytes',
+         'mbytes',
+         'nanoseconds',
+         'microseconds',
+         'milliseconds',
+         'seconds'))
 
 
 class TSStat(std.Target):
@@ -240,7 +254,7 @@ class TSStat(std.Target):
 
         # This has to be a distinct node before the title. if nested then
         # the browser will scroll forward to just past the title.
-        anchor = nodes.target('', '', names=[stat_name])
+        nodes.target('', '', names=[stat_name])
         # Second (optional) arg is 'msgNode' - no idea what I should pass for that
         # or if it even matters, although I now think it should not be used.
         self.state.document.note_explicit_target(title)
@@ -250,8 +264,8 @@ class TSStat(std.Target):
         fl.append(self.make_field('Collection', stat_group))
         if ('type' in self.options):
             fl.append(self.make_field('Type', self.options['type']))
-        if ('unit' in self.options):
-            fl.append(self.make_field('Units', self.options['unit']))
+        if ('units' in self.options):
+            fl.append(self.make_field('Units', self.options['units']))
         fl.append(self.make_field('Datatype', stat_type))
         if ('introduced' in self.options and len(self.options['introduced']) > 0):
             fl.append(self.make_field('Introduced', self.options['introduced']))
@@ -374,16 +388,21 @@ class TrafficServerDomain(Domain):
 
 
 # get the branch this documentation is building for in X.X.x form
-with open('../configure.ac', 'r') as f:
+REPO_ROOT = os.path.join(os.path.dirname(os.path.dirname(
+    os.environ['DOCUTILSCONFIG'])))
+CONFIGURE_AC = os.path.join(REPO_ROOT, 'configure.ac')
+with open(CONFIGURE_AC, 'r') as f:
     contents = f.read()
     match = re.compile(r'm4_define\(\[TS_VERSION_S],\[(.*?)]\)').search(contents)
     autoconf_version = '.'.join(match.group(1).split('.', 2)[:2] + ['x'])
 
 # get the current branch the local repository is on
-git_branch = subprocess.check_output(['git', 'rev-parse', '--abbrev-ref', 'HEAD'])
+REPO_GIT_DIR = os.path.join(REPO_ROOT, ".git")
+git_branch = subprocess.check_output(['git', '--git-dir', REPO_GIT_DIR,
+                                      'rev-parse', '--abbrev-ref', 'HEAD'])
 
 
-def make_github_link(name, rawtext, text, lineno, inliner, options={}, content=[]):
+def make_github_link(name, rawtext, text, lineno, inliner, options=None, content=None):
     """
     This docutils role lets us link to source code via the handy :ts:git: markup.
     Link references are rooted at the top level source directory. All links resolve
@@ -399,6 +418,10 @@ def make_github_link(name, rawtext, text, lineno, inliner, options={}, content=[
 
             If you want to contribute, take a look at :ts:git:`CONTRIBUTING.md`.
     """
+    if options is None:
+        options = {}
+    if content is None:
+        content = []
     url = 'https://github.com/apache/trafficserver/blob/{}/{}'
     ref = autoconf_version if autoconf_version == git_branch else 'master'
     node = nodes.reference(rawtext, text, refuri=url.format(ref, text), **options)
