@@ -45,11 +45,23 @@ IN6_IS_ADDR_UNSPECIFIED(in6_addr const *addr)
 }
 #endif
 
-// IP protocol stack tags.
+/*
+ * IP protocol stack tags.
+ *
+ * When adding support for an additional protocol, the following minimum steps
+ * should be done:
+ *
+ * 1. This set of string_views should be updated with the new tag.
+ * 2. A populate_protocol function overload should be implemented for the
+ *    appropriate VConnection or ProxySession virtual function.
+ * 3. Traffic Dump should be updated to handle the new tag in:
+ *    plugins/experimental/traffic_dump/session_data.cc
+ */
 extern const std::string_view IP_PROTO_TAG_IPV4;
 extern const std::string_view IP_PROTO_TAG_IPV6;
 extern const std::string_view IP_PROTO_TAG_UDP;
 extern const std::string_view IP_PROTO_TAG_TCP;
+extern const std::string_view IP_PROTO_TAG_QUIC;
 extern const std::string_view IP_PROTO_TAG_TLS_1_0;
 extern const std::string_view IP_PROTO_TAG_TLS_1_1;
 extern const std::string_view IP_PROTO_TAG_TLS_1_2;
@@ -58,6 +70,10 @@ extern const std::string_view IP_PROTO_TAG_HTTP_0_9;
 extern const std::string_view IP_PROTO_TAG_HTTP_1_0;
 extern const std::string_view IP_PROTO_TAG_HTTP_1_1;
 extern const std::string_view IP_PROTO_TAG_HTTP_2_0;
+extern const std::string_view IP_PROTO_TAG_HTTP_QUIC;
+extern const std::string_view IP_PROTO_TAG_HTTP_3;
+extern const std::string_view IP_PROTO_TAG_HTTP_QUIC_D27;
+extern const std::string_view IP_PROTO_TAG_HTTP_3_D27;
 
 struct IpAddr; // forward declare.
 
@@ -117,7 +133,7 @@ union IpEndpoint {
   in_port_t &port();
   /// Port in network order.
   in_port_t port() const;
-  /// Port in host horder.
+  /// Port in host order.
   in_port_t host_order_port() const;
 
   operator sockaddr *() { return &sa; }
@@ -165,8 +181,10 @@ inkcoreapi uint32_t ats_inet_addr(const char *s);
 const char *ats_ip_ntop(const struct sockaddr *addr, char *dst, size_t size);
 
 // --
-/// Size in bytes of an IPv6 address.
-static size_t const TS_IP6_SIZE = sizeof(in6_addr);
+/// Size in bytes of an port and IPv4/IPv6 address.
+static constexpr size_t TS_IP4_SIZE  = sizeof(in_addr_t); ///< 4
+static constexpr size_t TS_IP6_SIZE  = sizeof(in6_addr);  ///< 16
+static constexpr size_t TS_PORT_SIZE = sizeof(in_port_t); ///< 2
 
 /// Reset an address to invalid.
 /// @note Useful for marking a member as not yet set.
@@ -1130,7 +1148,7 @@ uint32_t ats_ip_hash(sockaddr const *addr);
 
 uint64_t ats_ip_port_hash(sockaddr const *addr);
 
-/** Convert address to string as a hexidecimal value.
+/** Convert address to string as a hexadecimal value.
     The string is always nul terminated, the output string is clipped
     if @a dst is insufficient.
     @return The length of the resulting string (not including nul).
@@ -1149,21 +1167,20 @@ struct IpAddr {
   typedef IpAddr self; ///< Self reference type.
 
   /// Default construct (invalid address).
-  IpAddr() : _family(AF_UNSPEC) {}
-  /// Construct as IPv4 @a addr.
-  explicit IpAddr(in_addr_t addr ///< Address to assign.
-                  )
-    : _family(AF_INET)
-  {
-    _addr._ip4 = addr;
-  }
-  /// Construct as IPv6 @a addr.
-  explicit IpAddr(in6_addr const &addr ///< Address to assign.
-                  )
-    : _family(AF_INET6)
-  {
-    _addr._ip6 = addr;
-  }
+  IpAddr() {}
+
+  /** Construct from IPv4 address.
+   *
+   * @param addr Source address.
+   */
+  explicit constexpr IpAddr(in_addr_t addr) : _family(AF_INET), _addr(addr) {}
+
+  /** Construct from IPv6 address.
+   *
+   * @param addr Source address.
+   */
+  explicit constexpr IpAddr(in6_addr const &addr) : _family(AF_INET6), _addr(addr) {}
+
   /// Construct from @c sockaddr.
   explicit IpAddr(sockaddr const *addr) { this->assign(addr); }
   /// Construct from @c sockaddr_in6.
@@ -1212,7 +1229,7 @@ struct IpAddr {
   */
   char *toString(char *dest, ///< [out] Destination string buffer.
                  size_t len  ///< [in] Size of buffer.
-                 ) const;
+  ) const;
 
   /// Equality.
   bool
@@ -1240,7 +1257,6 @@ struct IpAddr {
       - Else: 0.
   */
   uint32_t hash() const;
-  uint64_t hash64() const;
 
   /** The hashing function embedded in a functor.
       @see hash
@@ -1283,14 +1299,22 @@ struct IpAddr {
   /// Test for loopback
   bool isLoopback() const;
 
-  uint16_t _family; ///< Protocol family.
+  /// Test for any addr
+  bool isAnyAddr() const;
+
+  uint16_t _family = AF_UNSPEC; ///< Protocol family.
   /// Address data.
-  union {
+  union Addr {
     in_addr_t _ip4;                                                    ///< IPv4 address storage.
     in6_addr _ip6;                                                     ///< IPv6 address storage.
     uint8_t _byte[TS_IP6_SIZE];                                        ///< As raw bytes.
     uint32_t _u32[TS_IP6_SIZE / (sizeof(uint32_t) / sizeof(uint8_t))]; ///< As 32 bit chunks.
     uint64_t _u64[TS_IP6_SIZE / (sizeof(uint64_t) / sizeof(uint8_t))]; ///< As 64 bit chunks.
+
+    // This is required by the @c constexpr constructor.
+    constexpr Addr() : _ip4(0) {}
+    constexpr Addr(in_addr_t addr) : _ip4(addr) {}
+    constexpr Addr(in6_addr const &addr) : _ip6(addr) {}
   } _addr;
 
   ///< Pre-constructed invalid instance.
@@ -1342,6 +1366,12 @@ IpAddr::isLoopback() const
   return (AF_INET == _family && 0x7F == _addr._byte[0]) || (AF_INET6 == _family && IN6_IS_ADDR_LOOPBACK(&_addr._ip6));
 }
 
+inline bool
+IpAddr::isAnyAddr() const
+{
+  return (AF_INET == _family && INADDR_ANY == _addr._ip4) || (AF_INET6 == _family && IN6_IS_ADDR_UNSPECIFIED(&_addr._ip6));
+}
+
 /// Assign sockaddr storage.
 inline IpAddr &
 IpAddr::assign(sockaddr const *addr)
@@ -1435,18 +1465,6 @@ IpAddr::hash() const
   return zret;
 }
 
-inline uint64_t
-IpAddr::hash64() const
-{
-  uint64_t zret = 0;
-  if (this->isIp4()) {
-    zret = ntohl(_addr._ip4);
-  } else if (this->isIp6()) {
-    zret = _addr._u64[0] ^ _addr._u64[1];
-  }
-  return zret;
-}
-
 /// Write IP @a addr to storage @a dst.
 /// @return @s dst.
 sockaddr *ats_ip_set(sockaddr *dst,      ///< Destination storage.
@@ -1571,17 +1589,14 @@ bwformat(BufferWriter &w, BWFSpec const &spec, IpEndpoint const &addr)
 {
   return bwformat(w, spec, &addr.sa);
 }
-} // namespace ts
 
-namespace std
+namespace bwf
 {
-/// Standard hash support for @a IPAddr.
-template <> struct hash<IpAddr> {
-  size_t
-  operator()(IpAddr const &addr) const
+  namespace detail
   {
-    return addr.hash64();
-  }
-};
+    struct MemDump;
+  } // namespace detail
 
-} // namespace std
+  detail::MemDump Hex_Dump(IpEndpoint const &addr);
+} // namespace bwf
+} // namespace ts
