@@ -26,5 +26,7 @@
  *
  */
 
+#if !(defined(darwin) && defined(__aarch64__))
 #define TS_INLINE
 #include "P_Cache.h"
+#endif
