@@ -27,7 +27,9 @@
 
 // Generic Ram Cache interface
 
-struct RamCache {
+class RamCache
+{
+public:
   // returns 1 on found/stored, 0 on not found/stored, if provided auxkey1 and auxkey2 must match
   virtual int get(CryptoHash *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0) = 0;
   virtual int put(CryptoHash *key, IOBufferData *data, uint32_t len, bool copy = false, uint32_t auxkey1 = 0,
