@@ -25,14 +25,14 @@
 
 #include "SRV.h"
 
-const int DOMAIN_SERVICE_PORT        = NAMESERVER_PORT;
-const int DEFAULT_DOMAIN_NAME_SERVER = 0;
+const int DOMAIN_SERVICE_PORT = NAMESERVER_PORT;
 
-const int MAX_DNS_PACKET_LEN = 8192;
-const int DNS_RR_MAX_COUNT   = (MAX_DNS_PACKET_LEN - HFIXEDSZ + RRFIXEDSZ - 1) / RRFIXEDSZ;
-const int DNS_MAX_ALIASES    = DNS_RR_MAX_COUNT;
-const int DNS_MAX_ADDRS      = DNS_RR_MAX_COUNT;
-const int DNS_HOSTBUF_SIZE   = MAX_DNS_PACKET_LEN;
+const int MAX_DNS_REQUEST_LEN  = NS_PACKETSZ;
+const int MAX_DNS_RESPONSE_LEN = 65536;
+const int DNS_RR_MAX_COUNT     = (MAX_DNS_RESPONSE_LEN - HFIXEDSZ + RRFIXEDSZ - 1) / RRFIXEDSZ;
+const int DNS_MAX_ALIASES      = DNS_RR_MAX_COUNT;
+const int DNS_MAX_ADDRS        = DNS_RR_MAX_COUNT;
+const int DNS_HOSTBUF_SIZE     = MAX_DNS_RESPONSE_LEN;
 
 /**
   All buffering required to handle a DNS receipt. For asynchronous DNS,
@@ -41,10 +41,10 @@ const int DNS_HOSTBUF_SIZE   = MAX_DNS_PACKET_LEN;
 
 */
 struct HostEnt : RefCountObj {
-  struct hostent ent           = {.h_name = nullptr, .h_aliases = nullptr, .h_addrtype = 0, .h_length = 0, .h_addr_list = nullptr};
-  uint32_t ttl                 = 0;
-  int packet_size              = 0;
-  char buf[MAX_DNS_PACKET_LEN] = {0};
+  struct hostent ent = {.h_name = nullptr, .h_aliases = nullptr, .h_addrtype = 0, .h_length = 0, .h_addr_list = nullptr};
+  uint32_t ttl       = 0;
+  int packet_size    = 0;
+  char buf[MAX_DNS_RESPONSE_LEN]         = {0};
   u_char *host_aliases[DNS_MAX_ALIASES]  = {nullptr};
   u_char *h_addr_ptrs[DNS_MAX_ADDRS + 1] = {nullptr};
   u_char hostbuf[DNS_HOSTBUF_SIZE]       = {0};
@@ -68,13 +68,13 @@ struct DNSProcessor : public Processor {
 
     /// Query handler to use.
     /// Default: single threaded handler.
-    DNSHandler *handler;
+    DNSHandler *handler = nullptr;
     /// Query timeout value.
     /// Default: @c DEFAULT_DNS_TIMEOUT (or as set in records.config)
-    int timeout; ///< Timeout value for request.
+    int timeout = 0; ///< Timeout value for request.
     /// Host resolution style.
     /// Default: IPv4, IPv6 ( @c HOST_RES_IPV4 )
-    HostResStyle host_res_style;
+    HostResStyle host_res_style = HOST_RES_IPV4;
 
     /// Default constructor.
     Options();
@@ -121,8 +121,8 @@ struct DNSProcessor : public Processor {
 
   // private:
   //
-  EThread *thread;
-  DNSHandler *handler;
+  EThread *thread     = nullptr;
+  DNSHandler *handler = nullptr;
   ts_imp_res_state l_res;
   IpEndpoint local_ipv6;
   IpEndpoint local_ipv4;
@@ -172,7 +172,7 @@ DNSProcessor::gethostbyaddr(Continuation *cont, IpAddr const *addr, Options cons
   return getby(reinterpret_cast<const char *>(addr), 0, T_PTR, cont, opt);
 }
 
-inline DNSProcessor::Options::Options() : handler(nullptr), timeout(0), host_res_style(HOST_RES_IPV4) {}
+inline DNSProcessor::Options::Options() {}
 
 inline DNSProcessor::Options &
 DNSProcessor::Options::setHandler(DNSHandler *h)
@@ -202,4 +202,4 @@ DNSProcessor::Options::reset()
   return *this;
 }
 
-void ink_dns_init(ModuleVersion version);
+void ink_dns_init(ts::ModuleVersion version);
