@@ -25,17 +25,12 @@
 #pragma once
 #define I_VIO_h
 
-#include "tscore/ink_platform.h"
-#include "I_EventSystem.h"
 #if !defined(I_IOBuffer_h)
 #error "include I_IOBuffer.h"
----include I_IOBuffer.h
 #endif
-#include "tscore/ink_apidefs.h"
-   class Continuation;
+
+class Continuation;
 class VConnection;
-class IOVConnection;
-class MIOBuffer;
 class ProxyMutex;
 
 /**
@@ -73,9 +68,12 @@ class ProxyMutex;
 class VIO
 {
 public:
+  explicit VIO(int aop);
+  VIO();
   ~VIO() {}
+
   /** Interface for the VConnection that owns this handle. */
-  Continuation *get_continuation();
+  Continuation *get_continuation() const;
   void set_continuation(Continuation *cont);
 
   /**
@@ -102,8 +100,8 @@ public:
   /////////////////////
   void set_writer(MIOBuffer *writer);
   void set_reader(IOBufferReader *reader);
-  MIOBuffer *get_writer();
-  IOBufferReader *get_reader();
+  MIOBuffer *get_writer() const;
+  IOBufferReader *get_reader() const;
 
   /**
     Reenable the IO operation.
@@ -140,10 +138,7 @@ public:
   inkcoreapi void reenable_re();
 
   void disable();
-  bool is_disabled();
-
-  VIO(int aop);
-  VIO();
+  bool is_disabled() const;
 
   enum {
     NONE = 0,
@@ -160,7 +155,6 @@ public:
     STAT,
   };
 
-public:
   /**
     Continuation to callback.
 
@@ -168,7 +162,7 @@ public:
     call with events for this operation.
 
   */
-  Continuation *cont;
+  Continuation *cont = nullptr;
 
   /**
     Number of bytes to be done for this operation.
@@ -176,7 +170,7 @@ public:
     The total number of bytes this operation must complete.
 
   */
-  int64_t nbytes;
+  int64_t nbytes = 0;
 
   /**
     Number of bytes already completed.
@@ -186,7 +180,7 @@ public:
     the lock.
 
   */
-  int64_t ndone;
+  int64_t ndone = 0;
 
   /**
     Type of operation.
@@ -194,7 +188,7 @@ public:
     The type of operation that this VIO represents.
 
   */
-  int op;
+  int op = VIO::NONE;
 
   /**
     Provides access to the reader or writer for this operation.
@@ -210,7 +204,7 @@ public:
     functions.
 
   */
-  VConnection *vc_server;
+  VConnection *vc_server = nullptr;
 
   /**
     Reference to the state machine's mutex.
@@ -225,5 +219,3 @@ public:
 private:
   bool _disabled = false;
 };
-
-#include "I_VConnection.h"
