@@ -45,5 +45,5 @@ Thread::set_specific()
 TS_INLINE Thread *
 this_thread()
 {
-  return (Thread *)ink_thread_getspecific(Thread::thread_data_key);
+  return static_cast<Thread *>(ink_thread_getspecific(Thread::thread_data_key));
 }
