@@ -21,7 +21,7 @@
   limitations under the License.
  */
 
-#include "P_EventSystem.h" /* MAGIC_EDITING_TAG */
+#include "P_EventSystem.h"
 #include <sched.h>
 #if TS_USE_HWLOC
 #if HAVE_ALLOCA_H
@@ -37,7 +37,7 @@ class EventProcessor eventProcessor;
 
 class ThreadAffinityInitializer : public Continuation
 {
-  typedef ThreadAffinityInitializer self;
+  using self = ThreadAffinityInitializer;
 
 public:
   /// Default construct.
@@ -193,7 +193,7 @@ ThreadAffinityInitializer::set_affinity(int, Event *)
     hwloc_obj_t obj = hwloc_get_obj_by_type(ink_get_topology(), obj_type, t->id % obj_count);
 #if HWLOC_API_VERSION >= 0x00010100
     int cpu_mask_len = hwloc_bitmap_snprintf(nullptr, 0, obj->cpuset) + 1;
-    char *cpu_mask   = (char *)alloca(cpu_mask_len);
+    char *cpu_mask   = static_cast<char *>(alloca(cpu_mask_len));
     hwloc_bitmap_snprintf(cpu_mask, cpu_mask_len, obj->cpuset);
     Debug("iocore_thread", "EThread: %p %s: %d CPU Mask: %s\n", t, obj_name, obj->logical_index, cpu_mask);
 #else
@@ -380,6 +380,7 @@ EventProcessor::spawn_event_threads(EventType ev_type, int n_threads, size_t sta
   }
   tg->_count = n_threads;
   n_ethreads += n_threads;
+  schedule_spawn(&thread_started, ev_type);
 
   // Separate loop to avoid race conditions between spawn events and updating the thread table for
   // the group. Some thread set up depends on knowing the total number of threads but that can't be
@@ -403,8 +404,7 @@ EventProcessor::initThreadState(EThread *t)
 {
   // Run all thread type initialization continuations that match the event types for this thread.
   for (int i = 0; i < MAX_EVENT_TYPES; ++i) {
-    if (t->is_event_type(i)) { // that event type done here, roll thread start events of that type.
-      ++thread_group[i]._started;
+    if (t->is_event_type(i)) {
       // To avoid race conditions on the event in the spawn queue, create a local one to actually send.
       // Use the spawn queue event as a read only model.
       Event *nev = eventAllocator.alloc();
@@ -498,3 +498,24 @@ EventProcessor::spawn_thread(Continuation *cont, const char *thr_name, size_t st
 
   return e;
 }
+
+bool
+EventProcessor::has_tg_started(int etype)
+{
+  return thread_group[etype]._started == thread_group[etype]._count;
+}
+
+void
+thread_started(EThread *t)
+{
+  // Find what type of thread this is, and increment the "_started" counter of that thread type.
+  for (int i = 0; i < MAX_EVENT_TYPES; ++i) {
+    if (t->is_event_type(i)) {
+      if (++eventProcessor.thread_group[i]._started == eventProcessor.thread_group[i]._count &&
+          eventProcessor.thread_group[i]._afterStartCallback != nullptr) {
+        eventProcessor.thread_group[i]._afterStartCallback();
+      }
+      break;
+    }
+  }
+}
