@@ -64,10 +64,7 @@ public:
         @c NET_EVENT_ACCEPT_SUCCEED
         or @c NET_EVENT_ACCEPT_FAILED on success and failure resp.
     */
-    bool f_callback_on_open;
-    /** Accept only on the loopback address.
-        Default: @c false.
-     */
+
     bool localhost_only;
     /// Are frequent accepts expected?
     /// Default: @c false.
@@ -80,6 +77,9 @@ public:
     /// Socket transmit buffer size.
     /// 0 => OS default.
     int send_bufsize;
+    /// defer accept for @c sockopt.
+    /// 0 => OS default.
+    int defer_accept;
     /// Socket options for @c sockopt.
     /// 0 => do not set options.
     uint32_t sockopt_flags;
@@ -176,8 +176,6 @@ public:
       call back with success. If this behaviour is desired use
       synchronous connect connet_s method.
 
-    @see connect_s()
-
     @param cont Continuation to be called back with events.
     @param addr target address and port to connect to.
     @param options @see NetVCOptions.
@@ -186,33 +184,14 @@ public:
 
   inkcoreapi Action *connect_re(Continuation *cont, sockaddr const *addr, NetVCOptions *options = nullptr);
 
-  /**
-    Open a NetVConnection for connection oriented I/O. This call
-    is simliar to connect method except that the cont is called
-    back only after the connections has been established. In the
-    case of connect the cont could be called back with NET_EVENT_OPEN
-    event and OS could still be in the process of establishing the
-    connection. Re-entrant Callbacks: same as connect. If unix
-    asynchronous type connect is desired use connect_re().
-
-    @param cont Continuation to be called back with events.
-    @param addr Address to which to connect (includes port).
-    @param timeout for connect, the cont will get NET_EVENT_OPEN_FAILED
-      if connection could not be established for timeout msecs. The
-      default is 30 secs.
-    @param options @see NetVCOptions.
-
-    @see connect_re()
-
-  */
-  Action *connect_s(Continuation *cont, sockaddr const *addr, int timeout = NET_CONNECT_TIMEOUT, NetVCOptions *opts = nullptr);
-
   /**
     Initializes the net processor. This must be called before the event threads are started.
 
   */
   virtual void init() = 0;
 
+  virtual void init_socks() = 0;
+
   inkcoreapi virtual NetVConnection *allocate_vc(EThread *) = 0;
 
   /** Private constructor. */
@@ -263,7 +242,7 @@ private:
   object.
 
   @code
-    netProcesors.accept(my_cont, ...);
+    netProcessor.accept(my_cont, ...);
     netProcessor.connect_re(my_cont, ...);
   @endcode
 
@@ -278,3 +257,4 @@ extern inkcoreapi NetProcessor &netProcessor;
 
 */
 extern inkcoreapi NetProcessor &sslNetProcessor;
+extern inkcoreapi NetProcessor &quicNetProcessor;
