@@ -46,7 +46,7 @@ class UDPNetProcessor : public Processor
 public:
   int start(int n_upd_threads, size_t stacksize) override = 0;
 
-  // this function was interanal intially.. this is required for public and
+  // this function was internal initially.. this is required for public and
   // interface probably should change.
   bool CreateUDPSocket(int *resfd, sockaddr const *remote_addr, Action **status, NetVCOptions &opt);
 
@@ -63,15 +63,16 @@ public:
      @param c Continuation that is called back with newly created
      socket.
      @param addr Address to bind (includes port)
+     @param fd File descriptor to use (if exists)
      @param send_bufsize (optional) Socket buffer size for sending.
      Limits how much outstanding data to OS before it is able to send
      to the NIC.
      @param recv_bufsize (optional) Socket buffer size for sending.
      Limits how much can be queued by OS before we read it.
      @return Action* Always returns ACTION_RESULT_DONE if socket was
-     created successfuly, or ACTION_IO_ERROR if not.
+     created successfully, or ACTION_IO_ERROR if not.
   */
-  inkcoreapi Action *UDPBind(Continuation *c, sockaddr const *addr, int send_bufsize = 0, int recv_bufsize = 0);
+  inkcoreapi Action *UDPBind(Continuation *c, sockaddr const *addr, int fd = -1, int send_bufsize = 0, int recv_bufsize = 0);
 
   // Regarding sendto_re, sendmsg_re, recvfrom_re:
   // * You may be called back on 'c' with completion or error status.
