@@ -20,15 +20,24 @@
  */
 
 #include "P_OCSPStapling.h"
+
 #if TS_USE_TLS_OCSP
 
+#include <openssl/bio.h>
 #include <openssl/ssl.h>
+
+#if TS_HAS_BORINGOCSP
+#include <boringocsp/ocsp.h>
+#else
 #include <openssl/ocsp.h>
+#endif
+
 #include "P_Net.h"
 #include "P_SSLConfig.h"
 #include "P_SSLUtils.h"
+#include "SSLStats.h"
 
-// Maxiumum OCSP stapling response size.
+// Maximum OCSP stapling response size.
 // This should be the response for a single certificate and will typically include the responder certificate chain,
 // so 10K should be more than enough.
 #define MAX_STAPLING_DER 10240
@@ -42,6 +51,7 @@ struct certinfo {
   ink_mutex stapling_mutex;
   unsigned char resp_der[MAX_STAPLING_DER];
   unsigned int resp_derlen;
+  bool is_prefetched;
   bool is_expire;
   time_t expire_time;
 };
@@ -55,7 +65,7 @@ using certinfo_map = std::map<X509 *, certinfo *>;
 void
 certinfo_map_free(void * /*parent*/, void *ptr, CRYPTO_EX_DATA * /*ad*/, int /*idx*/, long /*argl*/, void * /*argp*/)
 {
-  certinfo_map *map = (certinfo_map *)ptr;
+  certinfo_map *map = static_cast<certinfo_map *>(ptr);
 
   if (!map) {
     return;
@@ -99,7 +109,7 @@ stapling_get_issuer(SSL_CTX *ssl_ctx, X509 *x)
 
 #ifdef SSL_CTX_select_current_cert
   if (!SSL_CTX_select_current_cert(ssl_ctx, x)) {
-    Warning("OCSP: could not select current certifcate chain %p", x);
+    Warning("OCSP: could not select current certificate chain %p", x);
   }
 #endif
 
@@ -140,104 +150,6 @@ end:
   return issuer;
 }
 
-bool
-ssl_stapling_init_cert(SSL_CTX *ctx, X509 *cert, const char *certname)
-{
-  scoped_X509 issuer;
-  STACK_OF(OPENSSL_STRING) *aia = nullptr;
-
-  if (!cert) {
-    Error("null cert passed in for %s", certname);
-    return false;
-  }
-
-  certinfo_map *map = static_cast<certinfo_map *>(SSL_CTX_get_ex_data(ctx, ssl_stapling_index));
-  if (map && map->find(cert) != map->end()) {
-    Note("certificate already initialized for %s", certname);
-    return false;
-  }
-
-  if (!map) {
-    map = new certinfo_map;
-  }
-  certinfo *cinf = static_cast<certinfo *>(OPENSSL_malloc(sizeof(certinfo)));
-  if (!cinf) {
-    Error("error allocating memory for %s", certname);
-    delete map;
-    return false;
-  }
-
-  // Initialize certinfo
-  cinf->cid         = nullptr;
-  cinf->uri         = nullptr;
-  cinf->certname    = ats_strdup(certname);
-  cinf->resp_derlen = 0;
-  ink_mutex_init(&cinf->stapling_mutex);
-  cinf->is_expire   = true;
-  cinf->expire_time = 0;
-
-  issuer = stapling_get_issuer(ctx, cert);
-  if (issuer == nullptr) {
-    Note("cannot get issuer certificate from %s", certname);
-    goto err;
-  }
-
-  cinf->cid = OCSP_cert_to_id(nullptr, cert, issuer);
-  if (!cinf->cid) {
-    goto err;
-  }
-
-  X509_digest(cert, EVP_sha1(), cinf->idx, nullptr);
-
-  aia = X509_get1_ocsp(cert);
-  if (aia) {
-    cinf->uri = sk_OPENSSL_STRING_pop(aia);
-    X509_email_free(aia);
-  }
-
-  if (!cinf->uri) {
-    Note("no OCSP responder URI for %s", certname);
-    goto err;
-  }
-
-  map->insert(std::make_pair(cert, cinf));
-  SSL_CTX_set_ex_data(ctx, ssl_stapling_index, map);
-
-  Note("successfully initialized stapling for %s into SSL_CTX: %p", certname, ctx);
-  return true;
-
-err:
-  if (cinf->cid) {
-    OCSP_CERTID_free(cinf->cid);
-  }
-
-  if (cinf->certname) {
-    ats_free(cinf->certname);
-  }
-
-  if (cinf) {
-    OPENSSL_free(cinf);
-  }
-  if (map) {
-    delete map;
-  }
-  return false;
-}
-
-static certinfo_map *
-stapling_get_cert_info(SSL_CTX *ctx)
-{
-  certinfo_map *map;
-
-  // Only return the map if it contains at least one element with a valid entry
-  map = static_cast<certinfo_map *>(SSL_CTX_get_ex_data(ctx, ssl_stapling_index));
-  if (map && !map->empty() && map->begin()->second && map->begin()->second->cid) {
-    return map;
-  }
-
-  return nullptr;
-}
-
 static bool
 stapling_cache_response(OCSP_RESPONSE *rsp, certinfo *cinf)
 {
@@ -269,6 +181,147 @@ stapling_cache_response(OCSP_RESPONSE *rsp, certinfo *cinf)
   return true;
 }
 
+bool
+ssl_stapling_init_cert(SSL_CTX *ctx, X509 *cert, const char *certname, const char *rsp_file)
+{
+  scoped_X509 issuer;
+  STACK_OF(OPENSSL_STRING) *aia = nullptr;
+  BIO *rsp_bio                  = nullptr;
+  OCSP_RESPONSE *rsp            = nullptr;
+
+  if (!cert) {
+    Error("null cert passed in for %s", certname);
+    return false;
+  }
+
+  certinfo_map *map = static_cast<certinfo_map *>(SSL_CTX_get_ex_data(ctx, ssl_stapling_index));
+  if (map && map->find(cert) != map->end()) {
+    Note("certificate already initialized for %s", certname);
+    return false;
+  }
+
+  if (!map) {
+    map = new certinfo_map;
+  }
+  certinfo *cinf = static_cast<certinfo *>(OPENSSL_malloc(sizeof(certinfo)));
+  if (!cinf) {
+    Error("error allocating memory for %s", certname);
+    delete map;
+    return false;
+  }
+
+  // Initialize certinfo
+  cinf->cid         = nullptr;
+  cinf->uri         = nullptr;
+  cinf->certname    = ats_strdup(certname);
+  cinf->resp_derlen = 0;
+  ink_mutex_init(&cinf->stapling_mutex);
+  cinf->is_prefetched = rsp_file ? true : false;
+  cinf->is_expire     = true;
+  cinf->expire_time   = 0;
+
+  if (cinf->is_prefetched) {
+#ifndef OPENSSL_IS_BORINGSSL
+    Debug("ssl_ocsp", "using OCSP prefetched response file %s", rsp_file);
+    rsp_bio = BIO_new_file(rsp_file, "r");
+    if (rsp_bio) {
+      rsp = d2i_OCSP_RESPONSE_bio(rsp_bio, nullptr);
+    }
+
+    if (!rsp_bio || !rsp) {
+      Note("cannot get prefetched response for %s from %s", certname, rsp_file);
+      goto err;
+    }
+
+    if (!stapling_cache_response(rsp, cinf)) {
+      Error("stapling_refresh_response: can not cache response");
+      goto err;
+    } else {
+      Debug("ssl_ocsp", "stapling_refresh_response: successful refresh OCSP response");
+      OCSP_RESPONSE_free(rsp);
+      rsp = nullptr;
+      BIO_free(rsp_bio);
+      rsp_bio = nullptr;
+    }
+#else
+    Warning("failed to set prefetched OCSP response; this functionality not supported by BoringSSL");
+#endif
+  }
+
+  issuer = stapling_get_issuer(ctx, cert);
+  if (issuer == nullptr) {
+    Note("cannot get issuer certificate from %s", certname);
+    goto err;
+  }
+
+  cinf->cid = OCSP_cert_to_id(nullptr, cert, issuer);
+  if (!cinf->cid) {
+    goto err;
+  }
+
+  X509_digest(cert, EVP_sha1(), cinf->idx, nullptr);
+
+  aia = X509_get1_ocsp(cert);
+  if (aia) {
+    cinf->uri = sk_OPENSSL_STRING_pop(aia);
+    X509_email_free(aia);
+  }
+
+  if (!cinf->uri) {
+    Note("no OCSP responder URI for %s", certname);
+    goto err;
+  }
+
+#ifdef OPENSSL_IS_BORINGSSL
+  X509_up_ref(cert);
+#endif
+
+  map->insert(std::make_pair(cert, cinf));
+  SSL_CTX_set_ex_data(ctx, ssl_stapling_index, map);
+
+  Note("successfully initialized stapling for %s into SSL_CTX: %p uri=%s", certname, ctx, cinf->uri);
+  return true;
+
+err:
+  if (cinf->cid) {
+    OCSP_CERTID_free(cinf->cid);
+  }
+
+  if (cinf->certname) {
+    ats_free(cinf->certname);
+  }
+
+  if (cinf) {
+    OPENSSL_free(cinf);
+  }
+  if (map) {
+    delete map;
+  }
+
+  if (rsp) {
+    OCSP_RESPONSE_free(rsp);
+  }
+  if (rsp_bio) {
+    BIO_free(rsp_bio);
+  }
+
+  return false;
+}
+
+static certinfo_map *
+stapling_get_cert_info(SSL_CTX *ctx)
+{
+  certinfo_map *map;
+
+  // Only return the map if it contains at least one element with a valid entry
+  map = static_cast<certinfo_map *>(SSL_CTX_get_ex_data(ctx, ssl_stapling_index));
+  if (map && !map->empty() && map->begin()->second && map->begin()->second->cid) {
+    return map;
+  }
+
+  return nullptr;
+}
+
 static int
 stapling_check_response(certinfo *cinf, OCSP_RESPONSE *rsp)
 {
@@ -336,10 +389,12 @@ query_responder(BIO *b, char *host, char *path, OCSP_REQUEST *req, int req_timeo
 
   OCSP_REQ_CTX_free(ctx);
 
-  if (rv) {
+  if (rv == 1) {
     return resp;
   }
 
+  Error("failed to connect to OCSP server; host=%s path=%s", host, path);
+
   return nullptr;
 }
 
@@ -358,7 +413,7 @@ process_responder(OCSP_REQUEST *req, char *host, char *path, char *port, int req
 
   BIO_set_nbio(cbio, 1);
   if (BIO_do_connect(cbio) <= 0 && !BIO_should_retry(cbio)) {
-    Debug("ssl_ocsp", "process_responder: failed to connect to OCSP response server. host=%s port=%s path=%s", host, port, path);
+    Debug("ssl_ocsp", "process_responder: failed to connect to OCSP server; host=%s port=%s path=%s", host, port, path);
     goto end;
   }
   resp = query_responder(cbio, host, path, req, req_timeout);
@@ -377,16 +432,18 @@ stapling_refresh_response(certinfo *cinf, OCSP_RESPONSE **prsp)
   OCSP_REQUEST *req = nullptr;
   OCSP_CERTID *id   = nullptr;
   char *host = nullptr, *port = nullptr, *path = nullptr;
-  int ssl_flag    = 0;
-  int req_timeout = -1;
+  int ssl_flag        = 0;
+  int response_status = 0;
 
-  Debug("ssl_ocsp", "stapling_refresh_response: querying responder");
   *prsp = nullptr;
 
   if (!OCSP_parse_url(cinf->uri, &host, &port, &path, &ssl_flag)) {
+    Debug("ssl_ocsp", "stapling_refresh_response: OCSP_parse_url failed; uri=%s", cinf->uri);
     goto err;
   }
 
+  Debug("ssl_ocsp", "stapling_refresh_response: querying responder; host=%s port=%s path=%s", host, port, path);
+
   req = OCSP_REQUEST_new();
   if (!req) {
     goto err;
@@ -399,25 +456,24 @@ stapling_refresh_response(certinfo *cinf, OCSP_RESPONSE **prsp)
     goto err;
   }
 
-  req_timeout = SSLConfigParams::ssl_ocsp_request_timeout;
-  *prsp       = process_responder(req, host, path, port, req_timeout);
-
+  *prsp = process_responder(req, host, path, port, SSLConfigParams::ssl_ocsp_request_timeout);
   if (*prsp == nullptr) {
     goto done;
   }
 
-  if (OCSP_response_status(*prsp) == OCSP_RESPONSE_STATUS_SUCCESSFUL) {
+  response_status = OCSP_response_status(*prsp);
+  if (response_status == OCSP_RESPONSE_STATUS_SUCCESSFUL) {
     Debug("ssl_ocsp", "stapling_refresh_response: query response received");
     stapling_check_response(cinf, *prsp);
   } else {
-    // TODO: We should log the actual openssl error
-    Error("stapling_refresh_response: responder error");
+    Error("stapling_refresh_response: responder response error; host=%s port=%s path=%s response_status=%d", host, port, path,
+          response_status);
   }
 
   if (!stapling_cache_response(*prsp, cinf)) {
     Error("stapling_refresh_response: can not cache response");
   } else {
-    Debug("ssl_ocsp", "stapling_refresh_response: successful refresh OCSP response");
+    Debug("ssl_ocsp", "stapling_refresh_response: successfully refreshed OCSP response");
   }
   goto done;
 
@@ -441,36 +497,39 @@ done:
 void
 ocsp_update()
 {
-  SSL_CTX *ctx;
+  shared_SSL_CTX ctx;
   OCSP_RESPONSE *resp = nullptr;
   time_t current_time;
 
   SSLCertificateConfig::scoped_config certLookup;
   const unsigned ctxCount = certLookup->count();
 
+  Debug("ssl_ocsp", "updating OCSP data");
   for (unsigned i = 0; i < ctxCount; i++) {
     SSLCertContext *cc = certLookup->get(i);
-    if (cc && cc->ctx) {
-      ctx               = cc->ctx;
-      certinfo *cinf    = nullptr;
-      certinfo_map *map = stapling_get_cert_info(ctx);
-      if (map) {
-        // Walk over all certs associated with this CTX
-        for (certinfo_map::iterator iter = map->begin(); iter != map->end(); ++iter) {
-          cinf = iter->second;
-          ink_mutex_acquire(&cinf->stapling_mutex);
-          current_time = time(nullptr);
-          if (cinf->resp_derlen == 0 || cinf->is_expire || cinf->expire_time < current_time) {
-            ink_mutex_release(&cinf->stapling_mutex);
-            if (stapling_refresh_response(cinf, &resp)) {
-              Debug("Successfully refreshed OCSP for %s certificate. url=%s", cinf->certname, cinf->uri);
-              SSL_INCREMENT_DYN_STAT(ssl_ocsp_refreshed_cert_stat);
+    if (cc) {
+      ctx = cc->getCtx();
+      if (ctx) {
+        certinfo *cinf    = nullptr;
+        certinfo_map *map = stapling_get_cert_info(ctx.get());
+        if (map) {
+          // Walk over all certs associated with this CTX
+          for (auto &iter : *map) {
+            cinf = iter.second;
+            ink_mutex_acquire(&cinf->stapling_mutex);
+            current_time = time(nullptr);
+            if (cinf->resp_derlen == 0 || cinf->is_expire || cinf->expire_time < current_time) {
+              ink_mutex_release(&cinf->stapling_mutex);
+              if (stapling_refresh_response(cinf, &resp)) {
+                Debug("ssl_ocsp", "Successfully refreshed OCSP for %s certificate. url=%s", cinf->certname, cinf->uri);
+                SSL_INCREMENT_DYN_STAT(ssl_ocsp_refreshed_cert_stat);
+              } else {
+                Error("Failed to refresh OCSP for %s certificate. url=%s", cinf->certname, cinf->uri);
+                SSL_INCREMENT_DYN_STAT(ssl_ocsp_refresh_cert_failure_stat);
+              }
             } else {
-              Error("Failed to refresh OCSP for %s certificate. url=%s", cinf->certname, cinf->uri);
-              SSL_INCREMENT_DYN_STAT(ssl_ocsp_refresh_cert_failure_stat);
+              ink_mutex_release(&cinf->stapling_mutex);
             }
-          } else {
-            ink_mutex_release(&cinf->stapling_mutex);
           }
         }
       }
@@ -480,7 +539,11 @@ ocsp_update()
 
 // RFC 6066 Section-8: Certificate Status Request
 int
+#ifndef OPENSSL_IS_BORINGSSL
 ssl_callback_ocsp_stapling(SSL *ssl)
+#else
+ssl_callback_ocsp_stapling(SSL *ssl, void *)
+#endif
 {
   // Assume SSL_get_SSL_CTX() is the same as reaching into the ssl structure
   // Using the official call, to avoid leaking internal openssl knowledge
@@ -490,31 +553,55 @@ ssl_callback_ocsp_stapling(SSL *ssl)
     Debug("ssl_ocsp", "ssl_callback_ocsp_stapling: failed to get certificate map");
     return SSL_TLSEXT_ERR_NOACK;
   }
+
+  if (map->empty()) {
+    Debug("ssl_ocsp", "ssl_callback_ocsp_stapling: certificate map empty");
+    return SSL_TLSEXT_ERR_NOACK;
+  }
+
   // Fetch the specific certificate used in this negotiation
   X509 *cert = SSL_get_certificate(ssl);
   if (!cert) {
     Error("ssl_callback_ocsp_stapling: failed to get certificate");
     return SSL_TLSEXT_ERR_NOACK;
   }
+
+  certinfo *cinf = nullptr;
+#ifndef OPENSSL_IS_BORINGSSL
   certinfo_map::iterator iter = map->find(cert);
-  if (iter == map->end()) {
-    Error("ssl_callback_ocsp_stapling: failed to get certificate information");
+  if (iter != map->end()) {
+    cinf = iter->second;
+  }
+#else
+  for (certinfo_map::iterator iter = map->begin(); iter != map->end(); ++iter) {
+    X509 *key = iter->first;
+    if (key == nullptr) {
+      continue;
+    }
+
+    if (X509_cmp(key, cert) == 0) {
+      cinf = iter->second;
+      break;
+    }
+  }
+#endif
+
+  if (cinf == nullptr) {
+    Error("ssl_callback_ocsp_stapling: failed to get certificate information for ssl=%p", ssl);
     return SSL_TLSEXT_ERR_NOACK;
   }
-  certinfo *cinf = iter->second;
 
   ink_mutex_acquire(&cinf->stapling_mutex);
   time_t current_time = time(nullptr);
-  if (cinf->resp_derlen == 0 || cinf->is_expire || cinf->expire_time < current_time) {
+  if ((cinf->resp_derlen == 0 || cinf->is_expire) || (cinf->expire_time < current_time && !cinf->is_prefetched)) {
     ink_mutex_release(&cinf->stapling_mutex);
     Debug("ssl_ocsp", "ssl_callback_ocsp_stapling: failed to get certificate status for %s", cinf->certname);
     return SSL_TLSEXT_ERR_NOACK;
   } else {
-    unsigned char *p = (unsigned char *)OPENSSL_malloc(cinf->resp_derlen);
-    unsigned int len = cinf->resp_derlen;
+    unsigned char *p = static_cast<unsigned char *>(OPENSSL_malloc(cinf->resp_derlen));
     memcpy(p, cinf->resp_der, cinf->resp_derlen);
     ink_mutex_release(&cinf->stapling_mutex);
-    SSL_set_tlsext_status_ocsp_resp(ssl, p, len);
+    SSL_set_tlsext_status_ocsp_resp(ssl, p, cinf->resp_derlen);
     Debug("ssl_ocsp", "ssl_callback_ocsp_stapling: successfully got certificate status for %s", cinf->certname);
     return SSL_TLSEXT_ERR_OK;
   }
