@@ -21,10 +21,11 @@
   limitations under the License.
  */
 
-#ifndef __P_UNIXNET_H__
-#define __P_UNIXNET_H__
+#pragma once
 
-#include "ts/ink_platform.h"
+#include <bitset>
+
+#include "tscore/ink_platform.h"
 
 #define USE_EDGE_TRIGGER_EPOLL 1
 #define USE_EDGE_TRIGGER_KQUEUE 1
@@ -37,6 +38,9 @@
 #define EVENTIO_ASYNC_SIGNAL 5
 
 #if TS_USE_EPOLL
+#ifndef EPOLLEXCLUSIVE
+#define EPOLLEXCLUSIVE 0
+#endif
 #ifdef USE_EDGE_TRIGGER_EPOLL
 #define USE_EDGE_TRIGGER 1
 #define EVENTIO_READ (EPOLLIN | EPOLLET)
@@ -71,41 +75,63 @@
 struct PollDescriptor;
 typedef PollDescriptor *EventLoop;
 
-class UnixNetVConnection;
+class NetEvent;
 class UnixUDPConnection;
 struct DNSConnection;
 struct NetAccept;
+
+/// Unified API for setting and clearing kernel and epoll events.
 struct EventIO {
-  int fd;
+  int fd = -1; ///< file descriptor, often a system port
 #if TS_USE_KQUEUE || TS_USE_EPOLL && !defined(USE_EDGE_TRIGGER) || TS_USE_PORT
-  int events;
+  int events = 0; ///< a bit mask of enabled events
 #endif
-  EventLoop event_loop;
-  int type;
+  EventLoop event_loop = nullptr; ///< the assigned event loop
+  bool syscall         = true;    ///< if false, disable all functionality (for QUIC)
+  int type             = 0;       ///< class identifier of union data.
   union {
     Continuation *c;
-    UnixNetVConnection *vc;
+    NetEvent *ne;
     DNSConnection *dnscon;
     NetAccept *na;
     UnixUDPConnection *uc;
-  } data;
+  } data; ///< a kind of continuation
+
+  /** The start methods all logically Setup a class to be called
+     when a file descriptor is available for read or write.
+     The type of the classes vary.  Generally the file descriptor
+     is pulled from the class, but there is one option that lets
+     the file descriptor be expressed directly.
+     @param l the event loop
+     @param events a mask of flags (for details `man epoll_ctl`)
+     @return int the number of events created, -1 is error
+   */
   int start(EventLoop l, DNSConnection *vc, int events);
   int start(EventLoop l, NetAccept *vc, int events);
-  int start(EventLoop l, UnixNetVConnection *vc, int events);
+  int start(EventLoop l, NetEvent *ne, int events);
   int start(EventLoop l, UnixUDPConnection *vc, int events);
-  int start(EventLoop l, int fd, Continuation *c, int events);
-  // Change the existing events by adding modify(EVENTIO_READ)
-  // or removing modify(-EVENTIO_READ), for level triggered I/O
+  int start(EventLoop l, int fd, NetEvent *ne, int events);
+  int start_common(EventLoop l, int fd, int events);
+
+  /** Alter the events that will trigger the continuation, for level triggered I/O.
+     @param events add with positive mask(+EVENTIO_READ), or remove with negative mask (-EVENTIO_READ)
+     @return int the number of events created, -1 is error
+   */
   int modify(int events);
-  // Refresh the existing events (i.e. KQUEUE EV_CLEAR), for edge triggered I/O
+
+  /** Refresh the existing events (i.e. KQUEUE EV_CLEAR), for edge triggered I/O
+     @param events mask of events
+     @return int the number of events created, -1 is error
+   */
   int refresh(int events);
+
+  /// Remove the kernel or epoll event. Returns 0 on success.
   int stop();
+
+  /// Remove the epoll event and close the connection. Returns 0 on success.
   int close();
-  EventIO()
-  {
-    type   = 0;
-    data.c = 0;
-  }
+
+  EventIO() { data.c = nullptr; }
 };
 
 #include "P_Net.h"
@@ -115,8 +141,9 @@ struct EventIO {
 #include "P_DNSConnection.h"
 #include "P_UnixUDPConnection.h"
 #include "P_UnixPollDescriptor.h"
+#include <limits>
 
-class UnixNetVConnection;
+class NetEvent;
 class NetHandler;
 typedef int (NetHandler::*NetContHandler)(int, void *);
 typedef unsigned int uint32;
@@ -131,7 +158,6 @@ extern int fds_limit;
 extern ink_hrtime last_transient_accept_error;
 extern int http_accept_port_number;
 
-//#define INACTIVITY_TIMEOUT
 //
 // Configuration Parameter had to move here to share
 // between UnixNet and UnixUDPNet or SSLNet modules.
@@ -164,10 +190,64 @@ struct PollCont : public Continuation {
 
   PollCont(Ptr<ProxyMutex> &m, int pt = net_config_poll_timeout);
   PollCont(Ptr<ProxyMutex> &m, NetHandler *nh, int pt = net_config_poll_timeout);
-  ~PollCont();
-  int pollEvent(int event, Event *e);
+  ~PollCont() override;
+  int pollEvent(int, Event *);
+  void do_poll(ink_hrtime timeout);
 };
 
+/**
+  NetHandler is the processor of NetEvent for the Net sub-system. The NetHandler
+  is the core component of the Net sub-system. Once started, it is responsible
+  for polling socket fds and perform the I/O tasks in NetEvent.
+
+  The NetHandler is executed periodically to perform read/write tasks for
+  NetVConnection. The NetHandler::mainNetEvent() should be viewed as a part of
+  EThread::execute() loop. This is the reason that Net System is a sub-system.
+
+  By get_NetHandler(this_ethread()), you can get the NetHandler object that
+  runs inside the current EThread and then @c startIO / @c stopIO which
+  assign/release a NetEvent to/from NetHandler. Before you call these functions,
+  holding the mutex of this NetHandler is required.
+
+  The NetVConnection provides a set of do_io functions through which you can
+  specify continuations to be called back by its NetHandler. These function
+  calls do not block. Instead they return an VIO object and schedule the
+  callback to the continuation passed in when there are I/O events occurred.
+
+  Multi-thread scheduler:
+
+  The NetHandler should be viewed as multi-threaded schedulers which process
+  NetEvents from their queues. If vc wants to be managed by NetHandler, the vc
+  should be derived from NetEvent. The vc can be made of NetProcessor (allocate_vc)
+  either by directly adding a NetEvent to the queue (NetHandler::startIO), or more
+  conveniently, calling a method service call (NetProcessor::connect_re) which
+  synthesizes the NetEvent and places it in the queue.
+
+  Callback event codes:
+
+  These event codes for do_io_read and reenable(read VIO) task:
+    VC_EVENT_READ_READY, VC_EVENT_READ_COMPLETE,
+    VC_EVENT_EOS, VC_EVENT_ERROR
+
+  These event codes for do_io_write and reenable(write VIO) task:
+    VC_EVENT_WRITE_READY, VC_EVENT_WRITE_COMPLETE
+    VC_EVENT_ERROR
+
+  There is no event and callback for do_io_shutdown / do_io_close task.
+
+  NetVConnection allocation policy:
+
+  VCs are allocated by the NetProcessor and deallocated by NetHandler.
+  A state machine may access the returned, non-recurring NetEvent / VIO until
+  it is closed by do_io_close. For recurring NetEvent, the NetEvent may be
+  accessed until it is closed. Once the NetEvent is closed, it's the
+  NetHandler's responsibility to deallocate it.
+
+  Before assign to NetHandler or after release from NetHandler, it's the
+  NetEvent's responsibility to deallocate itself.
+
+ */
+
 //
 // NetHandler
 //
@@ -176,52 +256,138 @@ struct PollCont : public Continuation {
 //
 class NetHandler : public Continuation, public EThread::LoopTailHandler
 {
+  using self_type = NetHandler; ///< Self reference type.
 public:
   // @a thread and @a trigger_event are redundant - you can get the former from the latter.
   // If we don't get rid of @a trigger_event we should remove @a thread.
-  EThread *thread;
-  Event *trigger_event;
-  QueM(UnixNetVConnection, NetState, read, ready_link) read_ready_list;
-  QueM(UnixNetVConnection, NetState, write, ready_link) write_ready_list;
-  Que(UnixNetVConnection, link) open_list;
-  DList(UnixNetVConnection, cop_link) cop_list;
-  ASLLM(UnixNetVConnection, NetState, read, enable_link) read_enable_list;
-  ASLLM(UnixNetVConnection, NetState, write, enable_link) write_enable_list;
-  Que(UnixNetVConnection, keep_alive_queue_link) keep_alive_queue;
-  uint32_t keep_alive_queue_size;
-  Que(UnixNetVConnection, active_queue_link) active_queue;
-  uint32_t active_queue_size;
-  uint32_t max_connections_per_thread_in;
-  uint32_t max_connections_active_per_thread_in;
+  EThread *thread      = nullptr;
+  Event *trigger_event = nullptr;
+  QueM(NetEvent, NetState, read, ready_link) read_ready_list;
+  QueM(NetEvent, NetState, write, ready_link) write_ready_list;
+  Que(NetEvent, open_link) open_list;
+  DList(NetEvent, cop_link) cop_list;
+  ASLLM(NetEvent, NetState, read, enable_link) read_enable_list;
+  ASLLM(NetEvent, NetState, write, enable_link) write_enable_list;
+  Que(NetEvent, keep_alive_queue_link) keep_alive_queue;
+  uint32_t keep_alive_queue_size = 0;
+  Que(NetEvent, active_queue_link) active_queue;
+  uint32_t active_queue_size = 0;
 
-  // configuration settings for managing the active and keep-alive queues
-  uint32_t max_connections_in;
-  uint32_t max_connections_active_in;
-  uint32_t inactive_threashold_in;
-  uint32_t transaction_no_activity_timeout_in;
-  uint32_t keep_alive_no_activity_timeout_in;
-  uint32_t default_inactivity_timeout;
+  /// configuration settings for managing the active and keep-alive queues
+  struct Config {
+    uint32_t max_connections_in                 = 0;
+    uint32_t max_requests_in                    = 0;
+    uint32_t inactive_threshold_in              = 0;
+    uint32_t transaction_no_activity_timeout_in = 0;
+    uint32_t keep_alive_no_activity_timeout_in  = 0;
+    uint32_t default_inactivity_timeout         = 0;
+
+    /** Return the address of the first value in this struct.
+
+        Doing updates is much easier if we treat this config struct as an array.
+        Making it a method means the knowledge of which member is the first one
+        is localized to this struct, not scattered about.
+     */
+    uint32_t &
+    operator[](int n)
+    {
+      return *(&max_connections_in + n);
+    }
+  };
+  /** Static global config, set and updated per process.
+
+      This is updated asynchronously and then events are sent to the NetHandler instances per thread
+      to copy to the per thread config at a convenient time. Because these are updated independently
+      from the command line, the update events just copy a single value from the global to the
+      local. This mechanism relies on members being identical types.
+  */
+  static Config global_config;
+  Config config; ///< Per thread copy of the @c global_config
+  // Active and keep alive queue values that depend on other configuration values.
+  // These are never updated directly, they are computed from other config values.
+  uint32_t max_connections_per_thread_in = 0;
+  uint32_t max_requests_per_thread_in    = 0;
+  /// Number of configuration items in @c Config.
+  static constexpr int CONFIG_ITEM_COUNT = sizeof(Config) / sizeof(uint32_t);
+  /// Which members of @c Config the per thread values depend on.
+  /// If one of these is updated, the per thread values must also be updated.
+  static const std::bitset<CONFIG_ITEM_COUNT> config_value_affects_per_thread_value;
+  /// Set of thread types in which nethandlers are active.
+  /// This enables signaling the correct instances when the configuration is updated.
+  /// Event type threads that use @c NetHandler must set the corresponding bit.
+  static std::bitset<std::numeric_limits<unsigned int>::digits> active_thread_types;
 
-  int startNetEvent(int event, Event *data);
   int mainNetEvent(int event, Event *data);
-  int waitForActivity(ink_hrtime timeout);
-  void process_enabled_list(NetHandler *);
+  int waitForActivity(ink_hrtime timeout) override;
+  void process_enabled_list();
+  void process_ready_list();
   void manage_keep_alive_queue();
-  bool manage_active_queue(bool ignore_queue_size);
-  void add_to_keep_alive_queue(UnixNetVConnection *vc);
-  void remove_from_keep_alive_queue(UnixNetVConnection *vc);
-  bool add_to_active_queue(UnixNetVConnection *vc);
-  void remove_from_active_queue(UnixNetVConnection *vc);
-  void configure_per_thread();
+  bool manage_active_queue(NetEvent *ne, bool ignore_queue_size);
+  void add_to_keep_alive_queue(NetEvent *ne);
+  void remove_from_keep_alive_queue(NetEvent *ne);
+  bool add_to_active_queue(NetEvent *ne);
+  void remove_from_active_queue(NetEvent *ne);
+
+  /// Per process initialization logic.
+  static void init_for_process();
+  /// Update configuration values that are per thread and depend on other configuration values.
+  void configure_per_thread_values();
+
+  /**
+    Start to handle read & write event on a NetEvent.
+    Initial the socket fd of ne for polling system.
+    Only be called when holding the mutex of this NetHandler.
+
+    @param ne NetEvent to be managed by this NetHandler.
+    @return 0 on success, ne->nh set to this NetHandler.
+            -ERRNO on failure.
+   */
+  int startIO(NetEvent *ne);
+  /**
+    Stop to handle read & write event on a NetEvent.
+    Remove the socket fd of ne from polling system.
+    Only be called when holding the mutex of this NetHandler and must call stopCop(ne) first.
+
+    @param ne NetEvent to be released.
+    @return ne->nh set to nullptr.
+   */
+  void stopIO(NetEvent *ne);
+
+  /**
+    Start to handle active timeout and inactivity timeout on a NetEvent.
+    Put the ne into open_list. All NetEvents in the open_list is checked for timeout by InactivityCop.
+    Only be called when holding the mutex of this NetHandler and must call startIO(ne) first.
+
+    @param ne NetEvent to be managed by InactivityCop
+   */
+  void startCop(NetEvent *ne);
+  /**
+    Stop to handle active timeout and inactivity on a NetEvent.
+    Remove the ne from open_list and cop_list.
+    Also remove the ne from keep_alive_queue and active_queue if its context is IN.
+    Only be called when holding the mutex of this NetHandler.
+
+    @param ne NetEvent to be released.
+   */
+  void stopCop(NetEvent *ne);
 
   // Signal the epoll_wait to terminate.
-  void signalActivity();
+  void signalActivity() override;
+
+  /**
+    Release a ne and free it.
+
+    @param ne NetEvent to be detached.
+   */
+  void free_netevent(NetEvent *ne);
 
   NetHandler();
 
 private:
-  void _close_vc(UnixNetVConnection *vc, ink_hrtime now, int &handle_event, int &closed, int &total_idle_time,
-                 int &total_idle_count);
+  void _close_ne(NetEvent *ne, ink_hrtime now, int &handle_event, int &closed, int &total_idle_time, int &total_idle_count);
+
+  /// Static method used as the callback for runtime configuration updates.
+  static int update_nethandler_config(const char *name, RecDataT, RecData data, void *);
 };
 
 static inline NetHandler *
@@ -275,7 +441,7 @@ check_net_throttle(ThrottleType t)
 {
   int connections = net_connections_to_throttle(t);
 
-  if (connections >= net_connections_throttle)
+  if (net_connections_throttle != 0 && connections >= net_connections_throttle)
     return true;
 
   return false;
@@ -302,9 +468,11 @@ change_net_connections_throttle(const char *token, RecDataT data_type, RecData v
   (void)value;
   (void)data;
   int throttle = fds_limit - THROTTLE_FD_HEADROOM;
-  if (fds_throttle < 0)
+  if (fds_throttle == 0) {
+    net_connections_throttle = fds_throttle;
+  } else if (fds_throttle < 0) {
     net_connections_throttle = throttle;
-  else {
+  } else {
     net_connections_throttle = fds_throttle;
     if (net_connections_throttle > throttle)
       net_connections_throttle = throttle;
@@ -367,78 +535,89 @@ check_transient_accept_error(int res)
   }
 }
 
-//
-// Disable a UnixNetVConnection
-//
+/** Disable reading on the NetEvent @a ne.
+     @param nh Nethandler that owns @a ne.
+     @param ne The @c NetEvent to modify.
+
+     - If write is already disable, also disable the inactivity timeout.
+     - clear read enabled flag.
+     - Remove the @c epoll READ flag.
+     - Take @a ne out of the read ready list.
+*/
 static inline void
-read_disable(NetHandler *nh, UnixNetVConnection *vc)
+read_disable(NetHandler *nh, NetEvent *ne)
 {
-#ifdef INACTIVITY_TIMEOUT
-  if (vc->inactivity_timeout) {
-    if (!vc->write.enabled) {
-      vc->inactivity_timeout->cancel_action();
-      vc->inactivity_timeout = nullptr;
-    }
+  if (!ne->write.enabled) {
+    // Clear the next scheduled inactivity time, but don't clear inactivity_timeout_in,
+    // so the current timeout is used when the NetEvent is reenabled and not the default inactivity timeout
+    ne->next_inactivity_timeout_at = 0;
+    Debug("socket", "read_disable updating inactivity_at %" PRId64 ", NetEvent=%p", ne->next_inactivity_timeout_at, ne);
   }
-#else
-  if (!vc->write.enabled) {
-    vc->set_inactivity_timeout(0);
-    Debug("socket", "read_disable updating inactivity_at %" PRId64 ", NetVC=%p", vc->next_inactivity_timeout_at, vc);
-  }
-#endif
-  vc->read.enabled = 0;
-  nh->read_ready_list.remove(vc);
-  vc->ep.modify(-EVENTIO_READ);
+  ne->read.enabled = 0;
+  nh->read_ready_list.remove(ne);
+  ne->ep.modify(-EVENTIO_READ);
 }
 
+/** Disable writing on the NetEvent @a ne.
+     @param nh Nethandler that owns @a ne.
+     @param ne The @c NetEvent to modify.
+
+     - If read is already disable, also disable the inactivity timeout.
+     - clear write enabled flag.
+     - Remove the @c epoll WRITE flag.
+     - Take @a ne out of the write ready list.
+*/
 static inline void
-write_disable(NetHandler *nh, UnixNetVConnection *vc)
+write_disable(NetHandler *nh, NetEvent *ne)
 {
-#ifdef INACTIVITY_TIMEOUT
-  if (vc->inactivity_timeout) {
-    if (!vc->read.enabled) {
-      vc->inactivity_timeout->cancel_action();
-      vc->inactivity_timeout = nullptr;
-    }
+  if (!ne->read.enabled) {
+    // Clear the next scheduled inactivity time, but don't clear inactivity_timeout_in,
+    // so the current timeout is used when the NetEvent is reenabled and not the default inactivity timeout
+    ne->next_inactivity_timeout_at = 0;
+    Debug("socket", "write_disable updating inactivity_at %" PRId64 ", NetEvent=%p", ne->next_inactivity_timeout_at, ne);
   }
-#else
-  if (!vc->read.enabled) {
-    vc->set_inactivity_timeout(0);
-    Debug("socket", "write_disable updating inactivity_at %" PRId64 ", NetVC=%p", vc->next_inactivity_timeout_at, vc);
-  }
-#endif
-  vc->write.enabled = 0;
-  nh->write_ready_list.remove(vc);
-  vc->ep.modify(-EVENTIO_WRITE);
+  ne->write.enabled = 0;
+  nh->write_ready_list.remove(ne);
+  ne->ep.modify(-EVENTIO_WRITE);
 }
 
 TS_INLINE int
 EventIO::start(EventLoop l, DNSConnection *vc, int events)
 {
-  type = EVENTIO_DNS_CONNECTION;
-  return start(l, vc->fd, (Continuation *)vc, events);
+  type        = EVENTIO_DNS_CONNECTION;
+  data.dnscon = vc;
+  return start_common(l, vc->fd, events);
 }
 TS_INLINE int
 EventIO::start(EventLoop l, NetAccept *vc, int events)
 {
-  type = EVENTIO_NETACCEPT;
-  return start(l, vc->server.fd, (Continuation *)vc, events);
+  type    = EVENTIO_NETACCEPT;
+  data.na = vc;
+  return start_common(l, vc->server.fd, events);
 }
 TS_INLINE int
-EventIO::start(EventLoop l, UnixNetVConnection *vc, int events)
+EventIO::start(EventLoop l, NetEvent *ne, int events)
 {
-  type = EVENTIO_READWRITE_VC;
-  return start(l, vc->con.fd, (Continuation *)vc, events);
+  type    = EVENTIO_READWRITE_VC;
+  data.ne = ne;
+  return start_common(l, ne->get_fd(), events);
 }
+
 TS_INLINE int
 EventIO::start(EventLoop l, UnixUDPConnection *vc, int events)
 {
-  type = EVENTIO_UDP_CONNECTION;
-  return start(l, vc->fd, (Continuation *)vc, events);
+  type    = EVENTIO_UDP_CONNECTION;
+  data.uc = vc;
+  return start_common(l, vc->fd, events);
 }
+
 TS_INLINE int
 EventIO::close()
 {
+  if (!this->syscall) {
+    return 0;
+  }
+
   stop();
   switch (type) {
   default:
@@ -451,22 +630,32 @@ EventIO::close()
     return data.na->server.close();
     break;
   case EVENTIO_READWRITE_VC:
-    return data.vc->con.close();
+    return data.ne->close();
     break;
   }
   return -1;
 }
 
 TS_INLINE int
-EventIO::start(EventLoop l, int afd, Continuation *c, int e)
+EventIO::start(EventLoop l, int afd, NetEvent *ne, int e)
 {
-  data.c     = c;
+  data.ne = ne;
+  return start_common(l, afd, e);
+}
+
+TS_INLINE int
+EventIO::start_common(EventLoop l, int afd, int e)
+{
+  if (!this->syscall) {
+    return 0;
+  }
+
   fd         = afd;
   event_loop = l;
 #if TS_USE_EPOLL
   struct epoll_event ev;
   memset(&ev, 0, sizeof(ev));
-  ev.events   = e;
+  ev.events   = e | EPOLLEXCLUSIVE;
   ev.data.ptr = this;
 #ifndef USE_EDGE_TRIGGER
   events = e;
@@ -495,6 +684,10 @@ EventIO::start(EventLoop l, int afd, Continuation *c, int e)
 TS_INLINE int
 EventIO::modify(int e)
 {
+  if (!this->syscall) {
+    return 0;
+  }
+
   ink_assert(event_loop);
 #if TS_USE_EPOLL && !defined(USE_EDGE_TRIGGER)
   struct epoll_event ev;
@@ -574,6 +767,10 @@ EventIO::modify(int e)
 TS_INLINE int
 EventIO::refresh(int e)
 {
+  if (!this->syscall) {
+    return 0;
+  }
+
   ink_assert(event_loop);
 #if TS_USE_KQUEUE && defined(USE_EDGE_TRIGGER)
   e = e & events;
@@ -615,6 +812,9 @@ EventIO::refresh(int e)
 TS_INLINE int
 EventIO::stop()
 {
+  if (!this->syscall) {
+    return 0;
+  }
   if (event_loop) {
     int retval = 0;
 #if TS_USE_EPOLL
@@ -634,4 +834,68 @@ EventIO::stop()
   return 0;
 }
 
-#endif
+TS_INLINE int
+NetHandler::startIO(NetEvent *ne)
+{
+  ink_assert(this->mutex->thread_holding == this_ethread());
+  ink_assert(ne->get_thread() == this_ethread());
+  int res = 0;
+
+  PollDescriptor *pd = get_PollDescriptor(this->thread);
+  if (ne->ep.start(pd, ne, EVENTIO_READ | EVENTIO_WRITE) < 0) {
+    res = errno;
+    // EEXIST should be ok, though it should have been cleared before we got back here
+    if (errno != EEXIST) {
+      Debug("iocore_net", "NetHandler::startIO : failed on EventIO::start, errno = [%d](%s)", errno, strerror(errno));
+      return -res;
+    }
+  }
+
+  if (ne->read.triggered == 1) {
+    read_ready_list.enqueue(ne);
+  }
+  ne->nh = this;
+  return res;
+}
+
+TS_INLINE void
+NetHandler::stopIO(NetEvent *ne)
+{
+  ink_release_assert(ne->nh == this);
+
+  ne->ep.stop();
+
+  read_ready_list.remove(ne);
+  write_ready_list.remove(ne);
+  if (ne->read.in_enabled_list) {
+    read_enable_list.remove(ne);
+    ne->read.in_enabled_list = 0;
+  }
+  if (ne->write.in_enabled_list) {
+    write_enable_list.remove(ne);
+    ne->write.in_enabled_list = 0;
+  }
+
+  ne->nh = nullptr;
+}
+
+TS_INLINE void
+NetHandler::startCop(NetEvent *ne)
+{
+  ink_assert(this->mutex->thread_holding == this_ethread());
+  ink_release_assert(ne->nh == this);
+  ink_assert(!open_list.in(ne));
+
+  open_list.enqueue(ne);
+}
+
+TS_INLINE void
+NetHandler::stopCop(NetEvent *ne)
+{
+  ink_release_assert(ne->nh == this);
+
+  open_list.remove(ne);
+  cop_list.remove(ne);
+  remove_from_keep_alive_queue(ne);
+  remove_from_active_queue(ne);
+}
