@@ -25,11 +25,12 @@
 #include "ts/apidefs.h"
 #include "tscore/ink_platform.h"
 #include "P_SSLNextProtocolSet.h"
+#include "tscpp/util/TextView.h"
 
 // For currently defined protocol strings, see
 // http://technotes.googlecode.com/git/nextprotoneg.html. The OpenSSL
 // documentation tells us to return a string in "wire format". The
-// draft NPN RFC helpfuly refuses to document the wire format. The
+// draft NPN RFC helpfully refuses to document the wire format. The
 // above link says we need to send length-prefixed strings, but does
 // not say how many bytes the length is. For the record, it's 1.
 
@@ -37,13 +38,13 @@ unsigned char *
 append_protocol(const char *proto, unsigned char *buf)
 {
   size_t sz = strlen(proto);
-  *buf++    = (unsigned char)sz;
+  *buf++    = static_cast<unsigned char>(sz);
   memcpy(buf, proto, sz);
   return buf + sz;
 }
 
-static bool
-create_npn_advertisement(const SSLNextProtocolSet::NextProtocolEndpoint::list_type &endpoints, unsigned char **npn, size_t *len)
+bool
+SSLNextProtocolSet::create_npn_advertisement(const SessionProtocolSet &enabled, unsigned char **npn, size_t *len) const
 {
   const SSLNextProtocolSet::NextProtocolEndpoint *ep;
   unsigned char *advertised;
@@ -59,14 +60,16 @@ create_npn_advertisement(const SSLNextProtocolSet::NextProtocolEndpoint::list_ty
     *len += (strlen(ep->protocol) + 1);
   }
 
-  *npn = advertised = (unsigned char *)ats_malloc(*len);
+  *npn = advertised = static_cast<unsigned char *>(ats_malloc(*len));
   if (!(*npn)) {
     goto fail;
   }
 
   for (ep = endpoints.head; ep != nullptr; ep = endpoints.next(ep)) {
-    Debug("ssl", "advertising protocol %s, %p", ep->protocol, ep->endpoint);
-    advertised = append_protocol(ep->protocol, advertised);
+    if (enabled.contains(globalSessionProtocolNameRegistry.toIndex(ts::TextView{ep->protocol, strlen(ep->protocol)}))) {
+      Debug("ssl", "advertising protocol %s, %p", ep->protocol, ep->endpoint);
+      advertised = append_protocol(ep->protocol, advertised);
+    }
   }
 
   return true;
@@ -78,31 +81,6 @@ fail:
   return false;
 }
 
-// copies th eprotocols but not the endpoints
-
-SSLNextProtocolSet *
-SSLNextProtocolSet::clone() const
-{
-  const SSLNextProtocolSet::NextProtocolEndpoint *ep;
-  SSLNextProtocolSet *newProtoSet = new SSLNextProtocolSet();
-  for (ep = this->endpoints.head; ep != nullptr; ep = this->endpoints.next(ep)) {
-    newProtoSet->registerEndpoint(ep->protocol, ep->endpoint);
-  }
-  return newProtoSet;
-}
-
-bool
-SSLNextProtocolSet::advertiseProtocols(const unsigned char **out, unsigned *len) const
-{
-  if (npn && npnsz) {
-    *out = npn;
-    *len = npnsz;
-    return true;
-  }
-
-  return false;
-}
-
 bool
 SSLNextProtocolSet::registerEndpoint(const char *proto, Continuation *ep)
 {
@@ -113,38 +91,14 @@ SSLNextProtocolSet::registerEndpoint(const char *proto, Continuation *ep)
     return false;
   }
 
-  if (!findEndpoint((const unsigned char *)proto, len)) {
+  if (!findEndpoint(reinterpret_cast<const unsigned char *>(proto), len)) {
     this->endpoints.push(new NextProtocolEndpoint(proto, ep));
-
-    if (npn) {
-      ats_free(npn);
-      npn   = nullptr;
-      npnsz = 0;
-    }
-    create_npn_advertisement(this->endpoints, &npn, &npnsz);
-
     return true;
   }
 
   return false;
 }
 
-bool
-SSLNextProtocolSet::unregisterEndpoint(const char *proto, Continuation *ep)
-{
-  for (NextProtocolEndpoint *e = this->endpoints.head; e; e = this->endpoints.next(e)) {
-    if (strcmp(proto, e->protocol) == 0 && (ep == nullptr || e->endpoint == ep)) {
-      // Protocol must be registered only once; no need to remove
-      // any more entries.
-      this->endpoints.remove(e);
-      create_npn_advertisement(this->endpoints, &npn, &npnsz);
-      return true;
-    }
-  }
-
-  return false;
-}
-
 Continuation *
 SSLNextProtocolSet::findEndpoint(const unsigned char *proto, unsigned len) const
 {
@@ -157,12 +111,10 @@ SSLNextProtocolSet::findEndpoint(const unsigned char *proto, unsigned len) const
   return nullptr;
 }
 
-SSLNextProtocolSet::SSLNextProtocolSet() : npn(nullptr), npnsz(0) {}
+SSLNextProtocolSet::SSLNextProtocolSet() {}
 
 SSLNextProtocolSet::~SSLNextProtocolSet()
 {
-  ats_free(this->npn);
-
   for (NextProtocolEndpoint *ep; (ep = this->endpoints.pop());) {
     delete ep;
   }
