@@ -21,7 +21,6 @@
 
 #pragma once
 
-#include "tscore/Map.h"
 #include "tscore/List.h"
 #include "tscore/ink_mutex.h"
 #include "P_EventSystem.h"
@@ -30,15 +29,30 @@
 #include "P_SSLUtils.h"
 #include "ts/apidefs.h"
 #include <openssl/ssl.h>
+#include <mutex>
+#include <shared_mutex>
 
 #define SSL_MAX_SESSION_SIZE 256
 
+struct ssl_session_cache_exdata {
+  ssl_curve_id curve = 0;
+};
+
+inline void
+hash_combine(uint64_t &seed, uint64_t hash)
+{
+  // using boost's version of hash combine, substituting magic number with a 64bit version
+  // https://www.boost.org/doc/libs/1_43_0/doc/html/hash/reference.html#boost.hash_combine
+  seed ^= hash + 0x9E3779B97F4A7C15 + (seed << 6) + (seed >> 2);
+}
+
 struct SSLSessionID : public TSSslSessionID {
   SSLSessionID(const unsigned char *s, size_t l)
   {
     len = l;
     ink_release_assert(l <= sizeof(bytes));
     memcpy(bytes, s, l);
+    hash();
   }
 
   SSLSessionID(const SSLSessionID &other)
@@ -47,6 +61,7 @@ struct SSLSessionID : public TSSslSessionID {
       memcpy(bytes, other.bytes, other.len);
 
     len = other.len;
+    hash();
   }
 
   bool
@@ -98,15 +113,33 @@ struct SSLSessionID : public TSSslSessionID {
   uint64_t
   hash() const
   {
-    // because the session ids should be uniformly random let's just use the last 64 bits as the hash.
-    // The first bytes could be interpreted as a name, and so not random.
-    if (len >= sizeof(uint64_t)) {
-      return *reinterpret_cast<uint64_t *>(const_cast<char *>(bytes + len - sizeof(uint64_t)));
-    } else if (len) {
-      return static_cast<uint64_t>(bytes[0]);
-    } else {
-      return 0;
+    if (hash_value == 0) {
+      // because the session ids should be uniformly random, we can treat the bits as a hash value
+      // however we need to combine them if the length is longer than 64bits
+      if (len >= sizeof(uint64_t)) {
+        uint64_t seed = 0;
+        for (uint64_t i = 0; i < len; i += sizeof(uint64_t)) {
+          hash_combine(seed, static_cast<uint64_t>(bytes[i]));
+        }
+        hash_value = seed;
+      } else if (len) {
+        hash_value = static_cast<uint64_t>(bytes[0]);
+      } else {
+        hash_value = 0;
+      }
     }
+    return hash_value;
+  }
+
+private:
+  mutable uint64_t hash_value = 0;
+};
+
+struct SSLSessionIDHash {
+  uint64_t
+  operator()(const SSLSessionID &id) const
+  {
+    return id.hash();
   }
 };
 
@@ -116,9 +149,12 @@ public:
   SSLSessionID session_id;
   Ptr<IOBufferData> asn1_data; /* this is the ASN1 representation of the SSL_CTX */
   size_t len_asn1_data;
+  Ptr<IOBufferData> extra_data;
+  ink_hrtime time_stamp;
 
-  SSLSession(const SSLSessionID &id, Ptr<IOBufferData> ssl_asn1_data, size_t len_asn1)
-    : session_id(id), asn1_data(ssl_asn1_data), len_asn1_data(len_asn1)
+  SSLSession(const SSLSessionID &id, const Ptr<IOBufferData> &ssl_asn1_data, size_t len_asn1, Ptr<IOBufferData> &exdata,
+             ink_hrtime ts)
+    : session_id(id), asn1_data(ssl_asn1_data), len_asn1_data(len_asn1), extra_data(exdata), time_stamp(ts)
   {
   }
 
@@ -130,31 +166,35 @@ class SSLSessionBucket
 public:
   SSLSessionBucket();
   ~SSLSessionBucket();
-  void insertSession(const SSLSessionID &, SSL_SESSION *ctx);
-  bool getSession(const SSLSessionID &, SSL_SESSION **ctx);
-  int getSessionBuffer(const SSLSessionID &, char *buffer, int &len);
-  void removeSession(const SSLSessionID &);
+  void insertSession(const SSLSessionID &sid, SSL_SESSION *sess, SSL *ssl);
+  bool getSession(const SSLSessionID &sid, SSL_SESSION **sess, ssl_session_cache_exdata **data);
+  int getSessionBuffer(const SSLSessionID &sid, char *buffer, int &len);
+  void removeSession(const SSLSessionID &sid);
 
 private:
   /* these method must be used while hold the lock */
   void print(const char *) const;
-  void removeOldestSession();
+  void removeOldestSession(const std::unique_lock<std::shared_mutex> &lock);
 
-  Ptr<ProxyMutex> mutex;
-  CountQueue<SSLSession> queue;
+  mutable std::shared_mutex mutex;
+  std::unordered_map<SSLSessionID, SSLSession *, SSLSessionIDHash> bucket_data;
+  std::map<ink_hrtime, SSLSession *> bucket_data_ts;
 };
 
 class SSLSessionCache
 {
 public:
-  bool getSession(const SSLSessionID &sid, SSL_SESSION **sess) const;
+  bool getSession(const SSLSessionID &sid, SSL_SESSION **sess, ssl_session_cache_exdata **data) const;
   int getSessionBuffer(const SSLSessionID &sid, char *buffer, int &len) const;
-  void insertSession(const SSLSessionID &sid, SSL_SESSION *sess);
+  void insertSession(const SSLSessionID &sid, SSL_SESSION *sess, SSL *ssl);
   void removeSession(const SSLSessionID &sid);
   SSLSessionCache();
   ~SSLSessionCache();
 
+  SSLSessionCache(const SSLSessionCache &) = delete;
+  SSLSessionCache &operator=(const SSLSessionCache &) = delete;
+
 private:
-  SSLSessionBucket *session_bucket;
+  SSLSessionBucket *session_bucket = nullptr;
   size_t nbuckets;
 };
