@@ -24,22 +24,14 @@
 #pragma once
 
 // copy from mgmt/Alarms.h
+#define REC_ALARM_UNDEFINED 0
+
 #define REC_ALARM_PROXY_PROCESS_DIED 1
 #define REC_ALARM_PROXY_PROCESS_BORN 2
-#define REC_ALARM_PROXY_PEER_BORN 3
-#define REC_ALARM_PROXY_PEER_DIED 4
-#define REC_ALARM_PROXY_CONFIG_ERROR 5
-#define REC_ALARM_PROXY_SYSTEM_ERROR 6
-#define REC_ALARM_PROXY_LOG_SPACE_CRISIS 7
-#define REC_ALARM_PROXY_CACHE_ERROR 8
-#define REC_ALARM_PROXY_CACHE_WARNING 9
-#define REC_ALARM_PROXY_LOGGING_ERROR 10
-#define REC_ALARM_PROXY_LOGGING_WARNING 11
-// Currently unused: 12
-#define REC_ALARM_REC_TEST 13
-#define REC_ALARM_CONFIG_UPDATE_FAILED 14
-#define REC_ALARM_WEB_ERROR 15
-#define REC_ALARM_PING_FAILURE 16
-#define REC_ALARM_REC_CONFIG_ERROR 17
-#define REC_ALARM_ADD_ALARM 18
-#define REC_ALARM_PROXY_LOG_SPACE_ROLLED 19
+#define REC_ALARM_PROXY_CONFIG_ERROR 3
+#define REC_ALARM_PROXY_SYSTEM_ERROR 4
+#define REC_ALARM_PROXY_CACHE_ERROR 5
+#define REC_ALARM_PROXY_CACHE_WARNING 6
+#define REC_ALARM_PROXY_LOGGING_ERROR 7
+#define REC_ALARM_PROXY_LOGGING_WARNING 8
+#define REC_ALARM_CONFIG_UPDATE_FAILED 9
