@@ -23,19 +23,21 @@
 
 #pragma once
 
-#include "ts/ink_thread.h"
-#include "ts/ink_hash_table.h"
-#include "ts/ink_llqueue.h"
-#include "ts/ink_rwlock.h"
-#include "ts/TextBuffer.h"
+#include "tscore/ink_thread.h"
+#include "tscore/ink_llqueue.h"
+#include "tscore/ink_rwlock.h"
+#include "tscore/TextBuffer.h"
 
 #include "I_RecCore.h"
 #include "P_RecDefs.h"
 #include "P_RecUtils.h"
 
+#include <unordered_set>
+#include <unordered_map>
+
 // records, record hash-table, and hash-table rwlock
 extern RecRecord *g_records;
-extern InkHashTable *g_records_ht;
+extern std::unordered_map<std::string, RecRecord *> g_records_ht;
 extern ink_rwlock g_records_rwlock;
 extern int g_num_records;
 extern RecModeT g_mode_type;
@@ -43,7 +45,7 @@ extern RecModeT g_mode_type;
 // records.config items
 extern const char *g_rec_config_fpath;
 extern LLQ *g_rec_config_contents_llq;
-extern InkHashTable *g_rec_config_contents_ht;
+extern std::unordered_set<std::string> g_rec_config_contents_ht;
 extern ink_mutex g_rec_config_lock;
 
 //-------------------------------------------------------------------------
@@ -68,19 +70,16 @@ RecRecord *RecForceInsert(RecRecord *record);
 //-------------------------------------------------------------------------
 
 RecErrT RecSetRecord(RecT rec_type, const char *name, RecDataT data_type, RecData *data, RecRawStat *raw_stat, RecSourceT source,
-                     bool lock = true, bool inc_version = true);
+                     bool lock = true);
 
 RecErrT RecGetRecord_Xmalloc(const char *name, RecDataT data_type, RecData *data, bool lock = true);
 
 //-------------------------------------------------------------------------
 // Read/Sync to Disk
 //-------------------------------------------------------------------------
-
 RecErrT RecReadStatsFile();
 RecErrT RecSyncStatsFile();
-RecErrT RecReadConfigFile(bool inc_version);
-RecErrT RecWriteConfigFile(textBuffer *tb);
-RecErrT RecSyncConfigToTB(textBuffer *tb, bool *inc_version = nullptr);
+RecErrT RecReadConfigFile();
 
 //-------------------------------------------------------------------------
 // Misc
