@@ -21,20 +21,25 @@
   limitations under the License.
  */
 
-#include "tscore/runroot.h"
 #include "tscore/ink_platform.h"
 #include "tscore/ink_memory.h"
 #include "tscore/ink_string.h"
+#include "tscore/Filenames.h"
 
+#include "RecordsConfig.h"
 #include "P_RecFile.h"
 #include "P_RecCore.h"
 #include "P_RecUtils.h"
 #include "tscore/I_Layout.h"
 
+// This is needed to manage the size of the librecords record. It can't be static, because it needs to be modified
+// and used (read) from several binaries / modules.
+int max_records_entries = REC_INTERNAL_RECORDS + REC_DEFAULT_API_RECORDS;
+
 static bool g_initialized = false;
 
-RecRecord *g_records       = nullptr;
-InkHashTable *g_records_ht = nullptr;
+RecRecord *g_records = nullptr;
+std::unordered_map<std::string, RecRecord *> g_records_ht;
 ink_rwlock g_records_rwlock;
 int g_num_records = 0;
 
@@ -48,9 +53,10 @@ register_record(RecT rec_type, const char *name, RecDataT data_type, RecData dat
   RecRecord *r = nullptr;
 
   // Metrics are restored from persistence before they are registered. In this case, when the registration arrives, we
-  // might find that they yave changed. For example, a metric might change it's type due to a software upgrade. Records
+  // might find that they have changed. For example, a metric might change it's type due to a software upgrade. Records
   // must not flip between config and metrics, but changing within those classes is OK.
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    r = it->second;
     if (REC_TYPE_IS_STAT(rec_type)) {
       ink_release_assert(REC_TYPE_IS_STAT(r->rec_type));
     }
@@ -86,7 +92,7 @@ register_record(RecT rec_type, const char *name, RecDataT data_type, RecData dat
     // Set the r->data to its default value as this is a new record
     RecDataSet(r->data_type, &(r->data), &(data_default));
     RecDataSet(r->data_type, &(r->data_default), &(data_default));
-    ink_hash_table_insert(g_records_ht, name, (void *)r);
+    g_records_ht.emplace(name, r);
 
     if (REC_TYPE_IS_STAT(r->rec_type)) {
       r->stat_meta.persist_type = persist_type;
@@ -110,7 +116,7 @@ register_record(RecT rec_type, const char *name, RecDataT data_type, RecData dat
 static int
 link_int(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  RecInt *rec_int = (RecInt *)cookie;
+  RecInt *rec_int = static_cast<RecInt *>(cookie);
   ink_atomic_swap(rec_int, data.rec_int);
   return REC_ERR_OKAY;
 }
@@ -118,28 +124,28 @@ link_int(const char * /* name */, RecDataT /* data_type */, RecData data, void *
 static int
 link_int32(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  *((int32_t *)cookie) = (int32_t)data.rec_int;
+  *(static_cast<int32_t *>(cookie)) = static_cast<int32_t>(data.rec_int);
   return REC_ERR_OKAY;
 }
 
 static int
 link_uint32(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  *((uint32_t *)cookie) = (uint32_t)data.rec_int;
+  *(static_cast<uint32_t *>(cookie)) = static_cast<uint32_t>(data.rec_int);
   return REC_ERR_OKAY;
 }
 
 static int
 link_float(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  *((RecFloat *)cookie) = data.rec_float;
+  *(static_cast<RecFloat *>(cookie)) = data.rec_float;
   return REC_ERR_OKAY;
 }
 
 static int
 link_counter(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  RecCounter *rec_counter = (RecCounter *)cookie;
+  RecCounter *rec_counter = static_cast<RecCounter *>(cookie);
   ink_atomic_swap(rec_counter, data.rec_counter);
   return REC_ERR_OKAY;
 }
@@ -149,7 +155,7 @@ link_counter(const char * /* name */, RecDataT /* data_type */, RecData data, vo
 static int
 link_byte(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
 {
-  RecByte *rec_byte = (RecByte *)cookie;
+  RecByte *rec_byte = static_cast<RecByte *>(cookie);
   RecByte byte      = static_cast<RecByte>(data.rec_int);
 
   ink_atomic_swap(rec_byte, byte);
@@ -157,7 +163,7 @@ link_byte(const char * /* name */, RecDataT /* data_type */, RecData data, void
 }
 
 // mimic Config.cc::config_string_alloc_cb
-// cookie e.g. is the DEFAULT_xxx_str value which this functiion keeps up to date with
+// cookie e.g. is the DEFAULT_xxx_str value which this function keeps up to date with
 // the latest default applied during a config update from records.config
 static int
 link_string_alloc(const char * /* name */, RecDataT /* data_type */, RecData data, void *cookie)
@@ -170,8 +176,8 @@ link_string_alloc(const char * /* name */, RecDataT /* data_type */, RecData dat
   }
 
   // set new string for DEFAULT_xxx_str tp point to
-  RecString _temp2       = *((RecString *)cookie);
-  *((RecString *)cookie) = _new_value;
+  RecString _temp2                    = *(static_cast<RecString *>(cookie));
+  *(static_cast<RecString *>(cookie)) = _new_value;
   // free previous string DEFAULT_xxx_str points to
   ats_free(_temp2);
 
@@ -197,14 +203,10 @@ RecCoreInit(RecModeT mode_type, Diags *_diags)
   g_num_records = 0;
 
   // initialize record array for our internal stats (this can be reallocated later)
-  g_records = (RecRecord *)ats_malloc(REC_MAX_RECORDS * sizeof(RecRecord));
+  g_records = static_cast<RecRecord *>(ats_malloc(max_records_entries * sizeof(RecRecord)));
 
-  // initialize record hash index
-  g_records_ht = ink_hash_table_create(InkHashTableKeyType_String);
+  // initialize record rwlock
   ink_rwlock_init(&g_records_rwlock);
-  if (!g_records_ht) {
-    return REC_ERR_FAIL;
-  }
 
   // read stats
   if ((mode_type == RECM_SERVER) || (mode_type == RECM_STAND_ALONE)) {
@@ -217,18 +219,14 @@ RecCoreInit(RecModeT mode_type, Diags *_diags)
 
     ink_mutex_init(&g_rec_config_lock);
 
-    g_rec_config_fpath = ats_stringdup(RecConfigReadConfigPath(nullptr, REC_CONFIG_FILE REC_SHADOW_EXT));
+    g_rec_config_fpath = ats_stringdup(RecConfigReadConfigPath(nullptr, ts::filename::RECORDS));
     if (RecFileExists(g_rec_config_fpath) == REC_ERR_FAIL) {
-      ats_free((char *)g_rec_config_fpath);
-      g_rec_config_fpath = ats_stringdup(RecConfigReadConfigPath(nullptr, REC_CONFIG_FILE));
-      if (RecFileExists(g_rec_config_fpath) == REC_ERR_FAIL) {
-        RecLog(DL_Warning, "Could not find '%s', system will run with defaults\n", REC_CONFIG_FILE);
-        file_exists = false;
-      }
+      RecLog(DL_Warning, "Could not find '%s', system will run with defaults\n", ts::filename::RECORDS);
+      file_exists = false;
     }
 
     if (file_exists) {
-      RecReadConfigFile(true);
+      RecReadConfigFile();
     }
   }
 
@@ -238,7 +236,7 @@ RecCoreInit(RecModeT mode_type, Diags *_diags)
 }
 
 //-------------------------------------------------------------------------
-// RecLinkCnfigXXX
+// RecLinkConfigXXX
 //-------------------------------------------------------------------------
 RecErrT
 RecLinkConfigInt(const char *name, RecInt *rec_int)
@@ -313,11 +311,12 @@ RecErrT
 RecRegisterConfigUpdateCb(const char *name, RecConfigUpdateCb update_cb, void *cookie)
 {
   RecErrT err = REC_ERR_FAIL;
-  RecRecord *r;
 
   ink_rwlock_rdlock(&g_records_rwlock);
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_CONFIG(r->rec_type)) {
       /* -- upgrade to support a list of callback functions
@@ -328,7 +327,7 @@ RecRegisterConfigUpdateCb(const char *name, RecConfigUpdateCb update_cb, void *c
          }
        */
 
-      RecConfigUpdateCbList *new_callback = (RecConfigUpdateCbList *)ats_malloc(sizeof(RecConfigUpdateCbList));
+      RecConfigUpdateCbList *new_callback = static_cast<RecConfigUpdateCbList *>(ats_malloc(sizeof(RecConfigUpdateCbList)));
       memset(new_callback, 0, sizeof(RecConfigUpdateCbList));
       new_callback->update_cb     = update_cb;
       new_callback->update_cookie = cookie;
@@ -362,37 +361,41 @@ RecRegisterConfigUpdateCb(const char *name, RecConfigUpdateCb update_cb, void *c
 //-------------------------------------------------------------------------
 // RecGetRecordXXX
 //-------------------------------------------------------------------------
-int
+RecErrT
 RecGetRecordInt(const char *name, RecInt *rec_int, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_INT, &data, lock)) == REC_ERR_OKAY) {
     *rec_int = data.rec_int;
   }
   return err;
 }
 
-int
+RecErrT
 RecGetRecordFloat(const char *name, RecFloat *rec_float, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_FLOAT, &data, lock)) == REC_ERR_OKAY) {
     *rec_float = data.rec_float;
   }
   return err;
 }
 
-int
+RecErrT
 RecGetRecordString(const char *name, char *buf, int buf_len, bool lock)
 {
-  int err = REC_ERR_OKAY;
-  RecRecord *r;
+  RecErrT err = REC_ERR_OKAY;
+
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (!r->registered || (r->data_type != RECD_STRING)) {
       err = REC_ERR_FAIL;
@@ -413,44 +416,48 @@ RecGetRecordString(const char *name, char *buf, int buf_len, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordString_Xmalloc(const char *name, RecString *rec_string, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_STRING, &data, lock)) == REC_ERR_OKAY) {
     *rec_string = data.rec_string;
   }
   return err;
 }
 
-int
+RecErrT
 RecGetRecordCounter(const char *name, RecCounter *rec_counter, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_COUNTER, &data, lock)) == REC_ERR_OKAY) {
     *rec_counter = data.rec_counter;
   }
   return err;
 }
 
-int
+RecErrT
 RecGetRecordByte(const char *name, RecByte *rec_byte, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_INT, &data, lock)) == REC_ERR_OKAY) {
     *rec_byte = data.rec_int;
   }
   return err;
 }
 
-int
+RecErrT
 RecGetRecordBool(const char *name, RecBool *rec_bool, bool lock)
 {
-  int err;
+  RecErrT err;
   RecData data;
+
   if ((err = RecGetRecord_Xmalloc(name, RECD_INT, &data, lock)) == REC_ERR_OKAY) {
     *rec_bool = 0 != data.rec_int;
   }
@@ -461,17 +468,18 @@ RecGetRecordBool(const char *name, RecBool *rec_bool, bool lock)
 // RecGetRec Attributes
 //-------------------------------------------------------------------------
 
-int
+RecErrT
 RecLookupRecord(const char *name, void (*callback)(const RecRecord *, void *), void *data, bool lock)
 {
-  int err = REC_ERR_FAIL;
-  RecRecord *r;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     callback(r, data);
     err = REC_ERR_OKAY;
@@ -485,13 +493,13 @@ RecLookupRecord(const char *name, void (*callback)(const RecRecord *, void *), v
   return err;
 }
 
-int
+RecErrT
 RecLookupMatchingRecords(unsigned rec_type, const char *match, void (*callback)(const RecRecord *, void *), void *data, bool lock)
 {
   int num_records;
   DFA regex;
 
-  if (regex.compile(match, RE_CASE_INSENSITIVE | RE_UNANCHORED) != 0) {
+  if (!regex.compile(match, RE_CASE_INSENSITIVE | RE_UNANCHORED)) {
     return REC_ERR_FAIL;
   }
 
@@ -515,17 +523,18 @@ RecLookupMatchingRecords(unsigned rec_type, const char *match, void (*callback)(
   return REC_ERR_OKAY;
 }
 
-int
+RecErrT
 RecGetRecordType(const char *name, RecT *rec_type, bool lock)
 {
-  int err = REC_ERR_FAIL;
-  RecRecord *r;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     *rec_type = r->rec_type;
     err       = REC_ERR_OKAY;
@@ -539,17 +548,18 @@ RecGetRecordType(const char *name, RecT *rec_type, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordDataType(const char *name, RecDataT *data_type, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (!r->registered) {
       err = REC_ERR_FAIL;
@@ -567,11 +577,10 @@ RecGetRecordDataType(const char *name, RecDataT *data_type, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordPersistenceType(const char *name, RecPersistT *persist_type, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
@@ -579,7 +588,9 @@ RecGetRecordPersistenceType(const char *name, RecPersistT *persist_type, bool lo
 
   *persist_type = RECP_NULL;
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_STAT(r->rec_type)) {
       *persist_type = r->stat_meta.persist_type;
@@ -595,27 +606,30 @@ RecGetRecordPersistenceType(const char *name, RecPersistT *persist_type, bool lo
   return err;
 }
 
-int
-RecGetRecordOrderAndId(const char *name, int *order, int *id, bool lock)
+RecErrT
+RecGetRecordOrderAndId(const char *name, int *order, int *id, bool lock, bool check_sync_cb)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     if (r->registered) {
-      rec_mutex_acquire(&(r->lock));
-      if (order) {
-        *order = r->order;
+      if (!check_sync_cb || r->stat_meta.sync_cb) {
+        rec_mutex_acquire(&(r->lock));
+        if (order) {
+          *order = r->order;
+        }
+        if (id) {
+          *id = r->rsb_id;
+        }
+        err = REC_ERR_OKAY;
+        rec_mutex_release(&(r->lock));
       }
-      if (id) {
-        *id = r->rsb_id;
-      }
-      err = REC_ERR_OKAY;
-      rec_mutex_release(&(r->lock));
     }
   }
 
@@ -626,17 +640,18 @@ RecGetRecordOrderAndId(const char *name, int *order, int *id, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordUpdateType(const char *name, RecUpdateT *update_type, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_CONFIG(r->rec_type)) {
       *update_type = r->config_meta.update_type;
@@ -654,17 +669,18 @@ RecGetRecordUpdateType(const char *name, RecUpdateT *update_type, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordCheckType(const char *name, RecCheckT *check_type, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_CONFIG(r->rec_type)) {
       *check_type = r->config_meta.check_type;
@@ -682,17 +698,18 @@ RecGetRecordCheckType(const char *name, RecCheckT *check_type, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordCheckExpr(const char *name, char **check_expr, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_CONFIG(r->rec_type)) {
       *check_expr = r->config_meta.check_expr;
@@ -710,18 +727,19 @@ RecGetRecordCheckExpr(const char *name, char **check_expr, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordDefaultDataString_Xmalloc(char *name, char **buf, bool lock)
 {
-  int err;
-  RecRecord *r = nullptr;
+  RecErrT err;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
-    *buf = (char *)ats_malloc(sizeof(char) * 1024);
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
+    *buf = static_cast<char *>(ats_malloc(sizeof(char) * 1024));
     memset(*buf, 0, 1024);
     err = REC_ERR_OKAY;
 
@@ -760,17 +778,18 @@ RecGetRecordDefaultDataString_Xmalloc(char *name, char **buf, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordAccessType(const char *name, RecAccessT *access, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     *access = r->config_meta.access_type;
     err     = REC_ERR_OKAY;
@@ -784,17 +803,18 @@ RecGetRecordAccessType(const char *name, RecAccessT *access, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecSetRecordAccessType(const char *name, RecAccessT access, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     r->config_meta.access_type = access;
     err                        = REC_ERR_OKAY;
@@ -808,17 +828,18 @@ RecSetRecordAccessType(const char *name, RecAccessT access, bool lock)
   return err;
 }
 
-int
+RecErrT
 RecGetRecordSource(const char *name, RecSourceT *source, bool lock)
 {
-  int err      = REC_ERR_FAIL;
-  RecRecord *r = nullptr;
+  RecErrT err = REC_ERR_FAIL;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     *source = r->config_meta.source;
     err     = REC_ERR_OKAY;
@@ -899,13 +920,14 @@ RecErrT
 RecGetRecord_Xmalloc(const char *name, RecDataT data_type, RecData *data, bool lock)
 {
   RecErrT err = REC_ERR_OKAY;
-  RecRecord *r;
 
   if (lock) {
     ink_rwlock_rdlock(&g_records_rwlock);
   }
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
+
     rec_mutex_acquire(&(r->lock));
     if (!r->registered || (r->data_type != data_type)) {
       err = REC_ERR_FAIL;
@@ -938,7 +960,8 @@ RecForceInsert(RecRecord *record)
 
   ink_rwlock_wrlock(&g_records_rwlock);
 
-  if (ink_hash_table_lookup(g_records_ht, record->name, (void **)&r)) {
+  if (auto it = g_records_ht.find(record->name); it != g_records_ht.end()) {
+    r                 = it->second;
     r_is_a_new_record = false;
     rec_mutex_acquire(&(r->lock));
     r->rec_type  = record->rec_type;
@@ -972,7 +995,7 @@ RecForceInsert(RecRecord *record)
   }
 
   if (r_is_a_new_record) {
-    ink_hash_table_insert(g_records_ht, r->name, (void *)r);
+    g_records_ht.emplace(r->name, r);
   } else {
     rec_mutex_release(&(r->lock));
   }
@@ -1046,7 +1069,7 @@ char *
 REC_ConfigReadString(const char *name)
 {
   char *t = nullptr;
-  RecGetRecordString_Xmalloc(name, (RecString *)&t);
+  RecGetRecordString_Xmalloc(name, static_cast<RecString *>(&t));
   return t;
 }
 
@@ -1054,7 +1077,7 @@ RecFloat
 REC_ConfigReadFloat(const char *name)
 {
   RecFloat t = 0;
-  RecGetRecordFloat(name, (RecFloat *)&t);
+  RecGetRecordFloat(name, &t);
   return t;
 }
 
@@ -1062,7 +1085,7 @@ RecCounter
 REC_ConfigReadCounter(const char *name)
 {
   RecCounter t = 0;
-  RecGetRecordCounter(name, (RecCounter *)&t);
+  RecGetRecordCounter(name, &t);
   return t;
 }
 
@@ -1074,8 +1097,8 @@ REC_readInteger(const char *name, bool *found, bool lock)
 {
   ink_assert(name);
   RecInt _tmp = 0;
-  bool _found;
-  _found = (RecGetRecordInt(name, &_tmp, lock) == REC_ERR_OKAY);
+  bool _found = (RecGetRecordInt(name, &_tmp, lock) == REC_ERR_OKAY);
+
   if (found) {
     *found = _found;
   }
@@ -1087,8 +1110,8 @@ REC_readFloat(char *name, bool *found, bool lock)
 {
   ink_assert(name);
   RecFloat _tmp = 0.0;
-  bool _found;
-  _found = (RecGetRecordFloat(name, &_tmp, lock) == REC_ERR_OKAY);
+  bool _found   = (RecGetRecordFloat(name, &_tmp, lock) == REC_ERR_OKAY);
+
   if (found) {
     *found = _found;
   }
@@ -1100,8 +1123,8 @@ REC_readCounter(char *name, bool *found, bool lock)
 {
   ink_assert(name);
   RecCounter _tmp = 0;
-  bool _found;
-  _found = (RecGetRecordCounter(name, &_tmp, lock) == REC_ERR_OKAY);
+  bool _found     = (RecGetRecordCounter(name, &_tmp, lock) == REC_ERR_OKAY);
+
   if (found) {
     *found = _found;
   }
@@ -1113,31 +1136,22 @@ REC_readString(const char *name, bool *found, bool lock)
 {
   ink_assert(name);
   RecString _tmp = nullptr;
-  bool _found;
-  _found = (RecGetRecordString_Xmalloc(name, &_tmp, lock) == REC_ERR_OKAY);
+  bool _found    = (RecGetRecordString_Xmalloc(name, &_tmp, lock) == REC_ERR_OKAY);
+
   if (found) {
     *found = _found;
   }
   return _tmp;
 }
 
-// RecConfigReadConfigDir. Note that we handle environmental configuration
-// overrides specially here. Normally we would override the configuration
-// variable when we read records.config but to avoid the bootstrapping
-// problem, we make an explicit check here.
+//-------------------------------------------------------------------------
+// RecConfigReadConfigDir
+//-------------------------------------------------------------------------
 std::string
 RecConfigReadConfigDir()
 {
-  char buf[PATH_NAME_MAX] = {0};
-
   if (const char *env = getenv("PROXY_CONFIG_CONFIG_DIR")) {
-    ink_strlcpy(buf, env, sizeof(buf));
-  } else if (get_runroot().empty()) {
-    RecGetRecordString("proxy.config.config_dir", buf, sizeof(buf));
-  }
-
-  if (strlen(buf) > 0) {
-    return Layout::get()->relative(buf);
+    return Layout::get()->relative(env);
   } else {
     return Layout::get()->sysconfdir;
   }
@@ -1245,7 +1259,7 @@ std::string
 RecConfigReadPersistentStatsPath()
 {
   std::string rundir(RecConfigReadRuntimeDir());
-  return Layout::relative_to(rundir, REC_RAW_STATS_FILE);
+  return Layout::relative_to(rundir, ts::filename::RECORDS_STATS);
 }
 
 void
@@ -1271,11 +1285,12 @@ RecSignalWarning(int sig, const char *fmt, ...)
 void
 RecConfigWarnIfUnregistered()
 {
-  RecDumpRecords(RECT_CONFIG,
-                 [](RecT, void *, int registered_p, const char *name, int, RecData *) -> void {
-                   if (!registered_p) {
-                     Warning("Unrecognized configuration value '%s'", name);
-                   }
-                 },
-                 nullptr);
+  RecDumpRecords(
+    RECT_CONFIG,
+    [](RecT, void *, int registered_p, const char *name, int, RecData *) -> void {
+      if (!registered_p) {
+        Warning("Unrecognized configuration value '%s'", name);
+      }
+    },
+    nullptr);
 }
