@@ -25,33 +25,24 @@
 #include "tscore/ink_string.h"
 #include "tscore/ink_file.h"
 #include "tscore/ink_time.h"
-#include "LocalManager.h"
 #include "MgmtUtils.h"
 #include "Alarms.h"
-#include "tscore/Diags.h"
 
 #include "records/P_RecCore.h"
 
 const char *alarmText[] = {
-  "Unknown Alarm",
-  "[TrafficManager] Traffic Server process was reset.",
-  "[TrafficManager] Traffic Server process established.",
-  "New Peer",
-  "Peer Died",
-  "Invalid Configuration",
-  "System Error",
-  "Log Space Crisis",
-  "Cache Error",
-  "Cache Warning",
-  "Logging Error",
-  "Logging Warning",
-  "Mgmt Debugging Alarm",
-  "Configuration File Update Failed",
-  "Unable to Establish Manager User-Interface Services",
-  "Ping Failure",
-  "",
-  "Add OEM Alarm",
-  "",
+  "Unknown Alarm",                                        // 0
+  "[TrafficManager] Traffic Server process was reset.",   // 1
+  "[TrafficManager] Traffic Server process established.", // 2
+  "Invalid Configuration",                                // 3
+  "System Error",                                         // 4
+  "Cache Error",                                          // 5
+  "Cache Warning",                                        // 6
+  "Logging Error",                                        // 7
+  "Logging Warning",                                      // 8
+  "Alarms configuration update failed",                   // 9
+  "Librecords",                                           // 10 (unclear if needed / used)
+  "Plugin set configuration",                             // 11 (unclear if needed / used)
 };
 
 const int alarmTextNum = sizeof(alarmText) / sizeof(char *);
@@ -73,19 +64,18 @@ alarm_script_dir()
 
 Alarms::Alarms()
 {
-  cur_cb        = 0;
-  cblist        = ink_hash_table_create(InkHashTableKeyType_String);
-  local_alarms  = ink_hash_table_create(InkHashTableKeyType_String);
-  remote_alarms = ink_hash_table_create(InkHashTableKeyType_String);
+  cur_cb = 0;
   ink_mutex_init(&mutex);
-  alarmOEMcount = minOEMkey;
 } /* End Alarms::Alarms */
 
 Alarms::~Alarms()
 {
-  ink_hash_table_destroy(cblist);
-  ink_hash_table_destroy_and_free_values(local_alarms);
-  ink_hash_table_destroy_and_free_values(remote_alarms);
+  for (auto &&it : local_alarms) {
+    ats_free(it.second);
+  }
+  for (auto &&it : remote_alarms) {
+    ats_free(it.second);
+  }
   ink_mutex_destroy(&mutex);
 } /* End Alarms::Alarms */
 
@@ -97,7 +87,7 @@ Alarms::registerCallback(AlarmCallbackFunc func)
   ink_mutex_acquire(&mutex);
   snprintf(cb_buf, sizeof(cb_buf), "%d", cur_cb++);
   Debug("alarm", "[Alarms::registerCallback] Registering Alarms callback");
-  ink_hash_table_insert(cblist, cb_buf, (void *)func);
+  cblist.emplace(cb_buf, func);
   ink_mutex_release(&mutex);
 } /* End Alarms::registerCallback */
 
@@ -106,7 +96,6 @@ Alarms::isCurrentAlarm(alarm_t a, char *ip)
 {
   bool ret = false;
   char buf[80];
-  InkHashTableValue hash_value;
 
   ink_mutex_acquire(&mutex);
   if (!ip) {
@@ -115,9 +104,9 @@ Alarms::isCurrentAlarm(alarm_t a, char *ip)
     snprintf(buf, sizeof(buf), "%d-%s", a, ip);
   }
 
-  if (!ip && ink_hash_table_lookup(local_alarms, buf, &hash_value) != 0) {
+  if (!ip && local_alarms.find(buf) != local_alarms.end()) {
     ret = true;
-  } else if (ip && ink_hash_table_lookup(remote_alarms, buf, &hash_value) != 0) {
+  } else if (ip && remote_alarms.find(buf) != remote_alarms.end()) {
     ret = true;
   }
   ink_mutex_release(&mutex);
@@ -128,7 +117,6 @@ void
 Alarms::resolveAlarm(alarm_t a, char *ip)
 {
   char buf[80];
-  InkHashTableValue hash_value;
 
   ink_mutex_acquire(&mutex);
   if (!ip) {
@@ -137,15 +125,15 @@ Alarms::resolveAlarm(alarm_t a, char *ip)
     snprintf(buf, sizeof(buf), "%d-%s", a, ip);
   }
 
-  if (!ip && ink_hash_table_lookup(local_alarms, buf, &hash_value) != 0) {
-    ink_hash_table_delete(local_alarms, buf);
-    ats_free(((Alarm *)hash_value)->description);
+  if (!ip && local_alarms.find(buf) != local_alarms.end()) {
+    Alarm *hash_value = local_alarms[buf];
+    local_alarms.erase(buf);
+    ats_free(hash_value->description);
     ats_free(hash_value);
-  } else if (ip && ink_hash_table_lookup(remote_alarms, buf, &hash_value) != 0) {
-    char buf2[1024];
-
-    snprintf(buf2, sizeof(buf2), "aresolv: %d\n", a);
-    ink_hash_table_delete(remote_alarms, buf);
+  } else if (ip && remote_alarms.find(buf) != remote_alarms.end()) {
+    Alarm *hash_value = remote_alarms[buf];
+    remote_alarms.erase(buf);
+    ats_free(hash_value->description);
     ats_free(hash_value);
   }
   ink_mutex_release(&mutex);
@@ -162,9 +150,6 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
   int priority;
   char buf[80];
   Alarm *atmp;
-  InkHashTableValue hash_value;
-  InkHashTableEntry *entry;
-  InkHashTableIteratorState iterator_state;
 
   /* Assign correct priorities */
   switch (a) {
@@ -173,24 +158,12 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
     break;
   case MGMT_ALARM_PROXY_CACHE_WARNING:
     return;
-  case MGMT_ALARM_PROXY_PEER_BORN:
-    priority = 3;
-    break;
-  case MGMT_ALARM_PROXY_PEER_DIED:
-    priority = 3;
-    break;
-  case MGMT_ALARM_PING_FAILURE:
-    priority = 1;
-    break;
   case MGMT_ALARM_PROXY_PROCESS_DIED:
     priority = 1;
     break;
   case MGMT_ALARM_PROXY_PROCESS_BORN:
     mgmt_log("[Alarms::signalAlarm] Server Process born\n");
     return;
-  case MGMT_ALARM_ADD_ALARM:
-    priority = 2;
-    break;
   default:
     priority = 2;
     break;
@@ -215,11 +188,11 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
   Debug("alarm", "[Alarms::signalAlarm] Sending Alarm: '%s'", desc);
 
   if (!desc) {
-    desc = (char *)getAlarmText(a);
+    desc = const_cast<char *>(getAlarmText(a));
   }
 
   /*
-   * Exec alarm bin for priority alarms everytime, regardless if they are
+   * Exec alarm bin for priority alarms every time, regardless if they are
    * potentially duplicates. However, only exec this for you own alarms,
    * don't want every node in the cluster reporting the same alarm.
    */
@@ -229,30 +202,24 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
 
   ink_mutex_acquire(&mutex);
   if (!ip) {
-    // if an OEM alarm, then must create the unique key alarm type;
-    // this key is used to hash the new OEM alarm descritption in the hash table
-    if (a == MGMT_ALARM_ADD_ALARM) {
-      a = (alarmOEMcount - minOEMkey) % (maxOEMkey - minOEMkey) + minOEMkey;
-      alarmOEMcount++;
-    }
     snprintf(buf, sizeof(buf), "%d", a);
-    if (ink_hash_table_lookup(local_alarms, buf, &hash_value) != 0) {
+    if (local_alarms.find(buf) != local_alarms.end()) {
       ink_mutex_release(&mutex);
       return;
     }
   } else {
     snprintf(buf, sizeof(buf), "%d-%s", a, ip);
-    if (ink_hash_table_lookup(remote_alarms, buf, &hash_value) != 0) {
+    if (auto it = remote_alarms.find(buf); it != remote_alarms.end()) {
       // Reset the seen flag so that we know the remote alarm is
       //   still active
-      atmp       = (Alarm *)hash_value;
+      atmp       = it->second;
       atmp->seen = true;
       ink_mutex_release(&mutex);
       return;
     }
   }
 
-  atmp              = (Alarm *)ats_malloc(sizeof(Alarm));
+  atmp              = static_cast<Alarm *>(ats_malloc(sizeof(Alarm)));
   atmp->type        = a;
   atmp->linger      = true;
   atmp->seen        = true;
@@ -262,11 +229,11 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
   if (!ip) {
     atmp->local        = true;
     atmp->inet_address = 0;
-    ink_hash_table_insert(local_alarms, (InkHashTableKey)(buf), (atmp));
+    local_alarms.emplace(buf, atmp);
   } else {
     atmp->local        = false;
     atmp->inet_address = inet_addr(ip);
-    ink_hash_table_insert(remote_alarms, (InkHashTableKey)(buf), (atmp));
+    local_alarms.emplace(buf, atmp);
   }
 
   // Swap desc with time-stamped description.  Kinda hackish
@@ -285,19 +252,18 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
   }
   const size_t sz = sizeof(char) * (strlen(desc) + strlen(my_ctime_str) + 4);
   ats_free(atmp->description);
-  atmp->description = (char *)ats_malloc(sz);
+  atmp->description = static_cast<char *>(ats_malloc(sz));
   snprintf(atmp->description, sz, "[%s] %s", my_ctime_str, desc);
 
   ink_mutex_release(&mutex);
 
-  for (entry = ink_hash_table_iterator_first(cblist, &iterator_state); entry != nullptr;
-       entry = ink_hash_table_iterator_next(cblist, &iterator_state)) {
-    AlarmCallbackFunc func = (AlarmCallbackFunc)ink_hash_table_entry_value(remote_alarms, entry);
+  for (auto &&it : cblist) {
+    AlarmCallbackFunc func = it.second;
     Debug("alarm", "[Alarms::signalAlarm] invoke callback for %d", a);
     (*(func))(a, ip, desc);
   }
 
-  /* Priority 2 alarms get signalled if they are the first unsolved occurence. */
+  /* Priority 2 alarms get signaled if they are the first unsolved occurrence. */
   if (priority == 2 && !ip) {
     execAlarmBin(desc);
   }
@@ -312,16 +278,11 @@ Alarms::signalAlarm(alarm_t a, const char *desc, const char *ip)
 void
 Alarms::resetSeenFlag(char *ip)
 {
-  InkHashTableEntry *entry;
-  InkHashTableIteratorState iterator_state;
-
   ink_mutex_acquire(&mutex);
-  for (entry = ink_hash_table_iterator_first(remote_alarms, &iterator_state); entry != nullptr;
-       entry = ink_hash_table_iterator_next(remote_alarms, &iterator_state)) {
-    char *key  = (char *)ink_hash_table_entry_key(remote_alarms, entry);
-    Alarm *tmp = (Alarm *)ink_hash_table_entry_value(remote_alarms, entry);
-
-    if (strstr(key, ip)) {
+  for (auto &&it : remote_alarms) {
+    std::string const &key = it.first;
+    Alarm *tmp             = it.second;
+    if (key.find(ip) != std::string::npos) {
       tmp->seen = false;
     }
   }
@@ -331,24 +292,20 @@ Alarms::resetSeenFlag(char *ip)
 
 /*
  * clearUnSeen(...)
- *   This function is a sweeper functionto clean up those alarms that have
- * been taken care of through otehr local managers or at the peer itself.
+ *   This function is a sweeper function to clean up those alarms that have
+ * been taken care of through other local managers or at the peer itself.
  */
 void
 Alarms::clearUnSeen(char *ip)
 {
-  InkHashTableEntry *entry;
-  InkHashTableIteratorState iterator_state;
-
   ink_mutex_acquire(&mutex);
-  for (entry = ink_hash_table_iterator_first(remote_alarms, &iterator_state); entry != nullptr;
-       entry = ink_hash_table_iterator_next(remote_alarms, &iterator_state)) {
-    char *key  = (char *)ink_hash_table_entry_key(remote_alarms, entry);
-    Alarm *tmp = (Alarm *)ink_hash_table_entry_value(remote_alarms, entry);
+  for (auto &&it : remote_alarms) {
+    std::string const &key = it.first;
+    Alarm *tmp             = it.second;
 
-    if (strstr(key, ip)) {                         /* Make sure alarm is for correct ip */
-      if (!tmp->seen) {                            /* Make sure we did not see it in peer's report */
-        ink_hash_table_delete(remote_alarms, key); /* Safe in iterator? */
+    if (key.find(ip) != std::string::npos) { /* Make sure alarm is for correct ip */
+      if (!tmp->seen) {                      /* Make sure we did not see it in peer's report */
+        remote_alarms.erase(key);
         ats_free(tmp->description);
         ats_free(tmp);
       }
@@ -358,17 +315,6 @@ Alarms::clearUnSeen(char *ip)
   return;
 } /* End Alarms::clearUnSeen */
 
-/*
- * checkSystemNAlert(...)
- *   This function should test the system and signal local alarms. Sending
- * out remote notification commands if necessary.
- */
-void
-Alarms::checkSystemNAlert()
-{
-  return;
-} /* End Alarms::checkSystenNAlert */
-
 void
 Alarms::execAlarmBin(const char *desc)
 {
@@ -387,11 +333,6 @@ Alarms::execAlarmBin(const char *desc)
     return;
   }
 
-  // get email info
-  alarm_email_from_name = REC_readString("proxy.config.product_name", nullptr);
-  alarm_email_from_addr = REC_readString("proxy.config.admin.admin_user", nullptr);
-  alarm_email_to_addr   = REC_readString("proxy.config.alarm_email", nullptr);
-
   ink_filepath_make(cmd_line, sizeof(cmd_line), bindir, alarm_bin);
 
 #ifdef POSIX_THREAD
@@ -404,7 +345,7 @@ Alarms::execAlarmBin(const char *desc)
   } else if (pid > 0) { /* Parent */
     int status;
     bool script_done = false;
-    time_t timeout   = (time_t)REC_readInteger("proxy.config.alarm.script_runtime", nullptr);
+    time_t timeout   = static_cast<time_t>(REC_readInteger("proxy.config.alarm.script_runtime", nullptr));
     if (!timeout) {
       timeout = 5; // default time = 5 secs
     }
@@ -428,13 +369,8 @@ Alarms::execAlarmBin(const char *desc)
       waitpid(pid, &status, 0); // to reap the thread
     }
   } else {
-    int res;
-    if (alarm_email_from_name && alarm_email_from_addr && alarm_email_to_addr) {
-      res = execl(cmd_line, (const char *)alarm_bin, desc, (const char *)alarm_email_from_name, (const char *)alarm_email_from_addr,
-                  (const char *)alarm_email_to_addr, (char *)nullptr);
-    } else {
-      res = execl(cmd_line, (const char *)alarm_bin, desc, (char *)nullptr);
-    }
+    int res = execl(cmd_line, (const char *)alarm_bin, desc, (char *)nullptr);
+
     _exit(res);
   }
 }
