@@ -25,6 +25,7 @@
 #include "tscore/ink_sock.h"
 #include "tscore/ink_file.h"
 #include "tscore/ink_error.h"
+#include "Alarms.h"
 #include "MgmtUtils.h"
 #include "tscore/I_Layout.h"
 #include "tscore/runroot.h"
@@ -37,6 +38,7 @@
 #include "tscpp/util/TextView.h"
 #include "tscore/BufferWriter.h"
 #include "tscore/bwf_std_format.h"
+#include "tscore/Filenames.h"
 
 #if TS_USE_POSIX_CAP
 #include <sys/capability.h>
@@ -150,8 +152,8 @@ LocalManager::clearStats(const char *name)
   //  before the proxy clears them, but this should be rare.
   //
   //  Doing things in the opposite order prevents that race
-  //   but excerbates the race between the node and cluster
-  //   stats getting cleared by progation of clearing the
+  //   but exacerbates the race between the node and cluster
+  //   stats getting cleared by propagation of clearing the
   //   cluster stats
   //
   if (name && *name) {
@@ -184,10 +186,9 @@ LocalManager::processRunning()
   }
 }
 
-LocalManager::LocalManager(bool proxy_on) : BaseManager(), run_proxy(proxy_on)
+LocalManager::LocalManager(bool proxy_on, bool listen) : BaseManager(), run_proxy(proxy_on), listen_for_proxy(listen)
 {
   bool found;
-  std::string rundir(RecConfigReadRuntimeDir());
   std::string bindir(RecConfigReadBinDir());
   std::string sysconfdir(RecConfigReadConfigDir());
 
@@ -311,7 +312,6 @@ LocalManager::initMgmtProcessServer()
 void
 LocalManager::pollMgmtProcessServer()
 {
-  int num;
   struct timeval timeout;
   fd_set fdlist;
 
@@ -354,7 +354,7 @@ LocalManager::pollMgmtProcessServer()
     }
 #endif
 
-    num = mgmt_select(FD_SETSIZE, &fdlist, nullptr, nullptr, &timeout);
+    int num = mgmt_select(FD_SETSIZE, &fdlist, nullptr, nullptr, &timeout);
 
     switch (num) {
     case 0:
@@ -383,7 +383,7 @@ LocalManager::pollMgmtProcessServer()
       if (process_server_sockfd != ts::NO_FD && FD_ISSET(process_server_sockfd, &fdlist)) { /* New connection */
         struct sockaddr_in clientAddr;
         socklen_t clientLen = sizeof(clientAddr);
-        int new_sockfd      = mgmt_accept(process_server_sockfd, (struct sockaddr *)&clientAddr, &clientLen);
+        int new_sockfd      = mgmt_accept(process_server_sockfd, reinterpret_cast<struct sockaddr *>(&clientAddr), &clientLen);
 
         mgmt_log("[LocalManager::pollMgmtProcessServer] New process connecting fd '%d'\n", new_sockfd);
 
@@ -401,21 +401,20 @@ LocalManager::pollMgmtProcessServer()
       if (ts::NO_FD != watched_process_fd && FD_ISSET(watched_process_fd, &fdlist)) {
         int res;
         MgmtMessageHdr mh_hdr;
-        MgmtMessageHdr *mh_full;
-        char *data_raw;
 
         keep_polling = true;
 
         // read the message
-        if ((res = mgmt_read_pipe(watched_process_fd, (char *)&mh_hdr, sizeof(MgmtMessageHdr))) > 0) {
-          mh_full = (MgmtMessageHdr *)alloca(sizeof(MgmtMessageHdr) + mh_hdr.data_len);
+        if ((res = mgmt_read_pipe(watched_process_fd, reinterpret_cast<char *>(&mh_hdr), sizeof(MgmtMessageHdr))) > 0) {
+          MgmtMessageHdr *mh_full = static_cast<MgmtMessageHdr *>(malloc(sizeof(MgmtMessageHdr) + mh_hdr.data_len));
           memcpy(mh_full, &mh_hdr, sizeof(MgmtMessageHdr));
-          data_raw = (char *)mh_full + sizeof(MgmtMessageHdr);
+          char *data_raw = reinterpret_cast<char *>(mh_full) + sizeof(MgmtMessageHdr);
           if ((res = mgmt_read_pipe(watched_process_fd, data_raw, mh_hdr.data_len)) > 0) {
             handleMgmtMsgFromProcesses(mh_full);
           } else if (res < 0) {
             mgmt_fatal(0, "[LocalManager::pollMgmtProcessServer] Error in read (errno: %d)\n", -res);
           }
+          free(mh_full);
         } else if (res < 0) {
           mgmt_fatal(0, "[LocalManager::pollMgmtProcessServer] Error in read (errno: %d)\n", -res);
         }
@@ -484,10 +483,10 @@ LocalManager::pollMgmtProcessServer()
 void
 LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
 {
-  char *data_raw = (char *)mh + sizeof(MgmtMessageHdr);
+  char *data_raw = reinterpret_cast<char *>(mh) + sizeof(MgmtMessageHdr);
   switch (mh->msg_id) {
   case MGMT_SIGNAL_PID:
-    watched_process_pid = *((pid_t *)data_raw);
+    watched_process_pid = *(reinterpret_cast<pid_t *>(data_raw));
     lmgmt->alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_PROCESS_BORN, nullptr);
     proxy_running++;
     proxy_launch_pid         = -1;
@@ -495,25 +494,6 @@ LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
     RecSetRecordInt("proxy.node.proxy_running", 1, REC_SOURCE_DEFAULT);
     break;
 
-  case MGMT_SIGNAL_MACHINE_UP:
-    /*
-       {
-       struct in_addr addr;
-       addr.s_addr = *((unsigned int*)data_raw);
-       alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_PEER_BORN, inet_ntoa(addr));
-       }
-     */
-    break;
-  case MGMT_SIGNAL_MACHINE_DOWN:
-    /*
-       {
-       struct in_addr addr;
-       addr.s_addr = *((unsigned int*)data_raw);
-       alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_PEER_DIED, inet_ntoa(addr));
-       }
-     */
-    break;
-
   // FIX: This is very messy need to correlate mgmt signals and
   // alarms better
   case MGMT_SIGNAL_CONFIG_ERROR:
@@ -522,9 +502,6 @@ LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
   case MGMT_SIGNAL_SYSTEM_ERROR:
     alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_SYSTEM_ERROR, data_raw);
     break;
-  case MGMT_SIGNAL_LOG_SPACE_CRISIS:
-    alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_LOG_SPACE_CRISIS, data_raw);
-    break;
   case MGMT_SIGNAL_CACHE_ERROR:
     alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_CACHE_ERROR, data_raw);
     break;
@@ -537,15 +514,12 @@ LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
   case MGMT_SIGNAL_LOGGING_WARNING:
     alarm_keeper->signalAlarm(MGMT_ALARM_PROXY_LOGGING_WARNING, data_raw);
     break;
-  case MGMT_SIGNAL_CONFIG_FILE_READ:
-    mgmt_log("[LocalManager::handleMgmtMsgFromProcesses] File done '%d'\n", data_raw);
-    break;
   case MGMT_SIGNAL_PLUGIN_SET_CONFIG: {
     char var_name[256];
     char var_value[256];
     MgmtType stype;
     // stype is an enum type, so cast to an int* to avoid warnings. /leif
-    int tokens = sscanf(data_raw, "%255s %d %255s", var_name, (int *)&stype, var_value);
+    int tokens = sscanf(data_raw, "%255s %d %255s", var_name, reinterpret_cast<int *>(&stype), var_value);
     if (tokens != 3) {
       stype = MGMT_INVALID;
     }
@@ -564,24 +538,19 @@ LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
       break;
     }
   } break;
-  case MGMT_SIGNAL_LOG_FILES_ROLLED: {
-    Debug("lm", "Rolling logs %s", (char *)data_raw);
-    break;
-  }
   case MGMT_SIGNAL_LIBRECORDS:
     if (mh->data_len > 0) {
-      executeMgmtCallback(MGMT_SIGNAL_LIBRECORDS, data_raw, mh->data_len);
+      executeMgmtCallback(MGMT_SIGNAL_LIBRECORDS, {data_raw, static_cast<size_t>(mh->data_len)});
     } else {
-      executeMgmtCallback(MGMT_SIGNAL_LIBRECORDS, nullptr, 0);
+      executeMgmtCallback(MGMT_SIGNAL_LIBRECORDS, {});
     }
     break;
   case MGMT_SIGNAL_CONFIG_FILE_CHILD: {
-    static const MgmtMarshallType fields[] = {MGMT_MARSHALL_STRING, MGMT_MARSHALL_STRING, MGMT_MARSHALL_INT};
+    static const MgmtMarshallType fields[] = {MGMT_MARSHALL_STRING, MGMT_MARSHALL_STRING};
     char *parent                           = nullptr;
     char *child                            = nullptr;
-    MgmtMarshallInt options                = 0;
-    if (mgmt_message_parse(data_raw, mh->data_len, fields, countof(fields), &parent, &child, &options) != -1) {
-      configFiles->configFileChild(parent, child, (unsigned int)options);
+    if (mgmt_message_parse(data_raw, mh->data_len, fields, countof(fields), &parent, &child) != -1) {
+      configFiles->configFileChild(parent, child);
     } else {
       mgmt_log("[LocalManager::handleMgmtMsgFromProcesses] "
                "MGMT_SIGNAL_CONFIG_FILE_CHILD mgmt_message_parse error\n");
@@ -590,9 +559,6 @@ LocalManager::handleMgmtMsgFromProcesses(MgmtMessageHdr *mh)
     ats_free_null(parent);
     ats_free_null(child);
   } break;
-  case MGMT_SIGNAL_SAC_SERVER_DOWN:
-    alarm_keeper->signalAlarm(MGMT_ALARM_SAC_SERVER_DOWN, data_raw);
-    break;
 
   default:
     break;
@@ -611,10 +577,10 @@ LocalManager::sendMgmtMsgToProcesses(int msg_id, const char *data_raw, int data_
 {
   MgmtMessageHdr *mh;
 
-  mh           = (MgmtMessageHdr *)alloca(sizeof(MgmtMessageHdr) + data_len);
+  mh           = static_cast<MgmtMessageHdr *>(alloca(sizeof(MgmtMessageHdr) + data_len));
   mh->msg_id   = msg_id;
   mh->data_len = data_len;
-  memcpy((char *)mh + sizeof(MgmtMessageHdr), data_raw, data_len);
+  memcpy(reinterpret_cast<char *>(mh) + sizeof(MgmtMessageHdr), data_raw, data_len);
   sendMgmtMsgToProcesses(mh);
   return;
 }
@@ -639,13 +605,12 @@ LocalManager::sendMgmtMsgToProcesses(MgmtMessageHdr *mh)
     mgmt_log("[LocalManager::SendMgmtMsgsToProcesses]Event is being constructed .\n");
     break;
   case MGMT_EVENT_CONFIG_FILE_UPDATE:
-  case MGMT_EVENT_CONFIG_FILE_UPDATE_NO_INC_VERSION:
     bool found;
     char *fname = nullptr;
-    Rollback *rb;
+    ConfigManager *rb;
     char *data_raw;
 
-    data_raw = (char *)mh + sizeof(MgmtMessageHdr);
+    data_raw = reinterpret_cast<char *>(mh) + sizeof(MgmtMessageHdr);
     fname    = REC_readString(data_raw, &found);
 
     RecT rec_type;
@@ -655,7 +620,7 @@ LocalManager::sendMgmtMsgToProcesses(MgmtMessageHdr *mh)
       mgmt_log("[LocalManager:sendMgmtMsgToProcesses] Unknown file change: '%s'\n", data_raw);
     }
     ink_assert(found);
-    if (!(fname && configFiles && configFiles->getRollbackObj(fname, &rb)) &&
+    if (!(fname && configFiles && configFiles->getConfigObj(fname, &rb)) &&
         (strcmp(data_raw, "proxy.config.body_factory.template_sets_dir") != 0) &&
         (strcmp(data_raw, "proxy.config.ssl.server.ticket_key.filename") != 0)) {
       mgmt_fatal(0, "[LocalManager::sendMgmtMsgToProcesses] "
@@ -666,7 +631,7 @@ LocalManager::sendMgmtMsgToProcesses(MgmtMessageHdr *mh)
   }
 
   if (watched_process_fd != -1) {
-    if (mgmt_write_pipe(watched_process_fd, (char *)mh, sizeof(MgmtMessageHdr) + mh->data_len) <= 0) {
+    if (mgmt_write_pipe(watched_process_fd, reinterpret_cast<char *>(mh), sizeof(MgmtMessageHdr) + mh->data_len) <= 0) {
       // In case of Linux, sometimes when the TS dies, the connection between TS and TM
       // is not closed properly. the socket does not receive an EOF. So, the TM does
       // not detect that the connection and hence TS has gone down. Hence it still
@@ -725,13 +690,10 @@ LocalManager::sendMgmtMsgToProcesses(MgmtMessageHdr *mh)
 }
 
 void
-LocalManager::signalFileChange(const char *var_name, bool incVersion)
+LocalManager::signalFileChange(const char *var_name)
 {
-  if (incVersion) {
-    signalEvent(MGMT_EVENT_CONFIG_FILE_UPDATE, var_name);
-  } else {
-    signalEvent(MGMT_EVENT_CONFIG_FILE_UPDATE_NO_INC_VERSION, var_name);
-  }
+  signalEvent(MGMT_EVENT_CONFIG_FILE_UPDATE, var_name);
+
   return;
 }
 
@@ -746,19 +708,22 @@ void
 LocalManager::signalEvent(int msg_id, const char *data_raw, int data_len)
 {
   MgmtMessageHdr *mh;
+  size_t n = sizeof(MgmtMessageHdr) + data_len;
 
-  mh           = (MgmtMessageHdr *)ats_malloc(sizeof(MgmtMessageHdr) + data_len);
+  mh           = static_cast<MgmtMessageHdr *>(ats_malloc(n));
   mh->msg_id   = msg_id;
   mh->data_len = data_len;
-  memcpy((char *)mh + sizeof(MgmtMessageHdr), data_raw, data_len);
-  ink_assert(enqueue(mgmt_event_queue, mh));
+  auto payload = mh->payload();
+  memcpy(payload.data(), data_raw, data_len);
+  this->enqueue(mh);
+  //  ink_assert(enqueue(mgmt_event_queue, mh));
 
 #if HAVE_EVENTFD
   // we don't care about the actual value of wakeup_fd, so just keep adding 1. just need to
-  // wakeup the fd. also, note that wakeup_fd was initalized to non-blocking so we can
+  // wakeup the fd. also, note that wakeup_fd was initialized to non-blocking so we can
   // directly write to it without any timeout checking.
   //
-  // don't tigger if MGMT_EVENT_LIBRECORD because they happen all the time
+  // don't trigger if MGMT_EVENT_LIBRECORD because they happen all the time
   // and don't require a quick response. for MGMT_EVENT_LIBRECORD, rely on timeouts so
   // traffic_server can spend more time doing other things
   uint64_t one = 1;
@@ -777,21 +742,18 @@ LocalManager::signalEvent(int msg_id, const char *data_raw, int data_len)
 void
 LocalManager::processEventQueue()
 {
-  bool handled_by_mgmt;
+  while (!this->queue_empty()) {
+    bool handled_by_mgmt = false;
 
-  while (!queue_is_empty(mgmt_event_queue)) {
-    handled_by_mgmt = false;
-
-    MgmtMessageHdr *mh = (MgmtMessageHdr *)dequeue(mgmt_event_queue);
-    char *data_raw     = (char *)mh + sizeof(MgmtMessageHdr);
+    MgmtMessageHdr *mh = this->dequeue();
+    auto payload       = mh->payload().rebind<char>();
 
     // check if we have a local file update
-    if (mh->msg_id == MGMT_EVENT_CONFIG_FILE_UPDATE || mh->msg_id == MGMT_EVENT_CONFIG_FILE_UPDATE_NO_INC_VERSION) {
+    if (mh->msg_id == MGMT_EVENT_CONFIG_FILE_UPDATE) {
       // records.config
-      if (!(strcmp(data_raw, REC_CONFIG_FILE))) {
-        bool incVersion = mh->msg_id == MGMT_EVENT_CONFIG_FILE_UPDATE;
-        if (RecReadConfigFile(incVersion) != REC_ERR_OKAY) {
-          mgmt_elog(errno, "[fileUpdated] Config update failed for records.config\n");
+      if (!(strcmp(payload.begin(), ts::filename::RECORDS))) {
+        if (RecReadConfigFile() != REC_ERR_OKAY) {
+          mgmt_elog(errno, "[fileUpdated] Config update failed for %s\n", ts::filename::RECORDS);
         } else {
           RecConfigWarnIfUnregistered();
         }
@@ -804,10 +766,10 @@ LocalManager::processEventQueue()
         // Fix INKqa04984
         // If traffic server hasn't completely come up yet,
         // we will hold off until next round.
-        ink_assert(enqueue(mgmt_event_queue, mh));
+        this->enqueue(mh);
         return;
       }
-      Debug("lm", "[TrafficManager] ==> Sending signal event '%d' %s payload=%d", mh->msg_id, data_raw, mh->data_len);
+      Debug("lm", "[TrafficManager] ==> Sending signal event '%d' %s payload=%d", mh->msg_id, payload.begin(), int(payload.size()));
       lmgmt->sendMgmtMsgToProcesses(mh);
     }
     ats_free(mh);
@@ -835,7 +797,7 @@ LocalManager::startProxy(const char *onetime_options)
   pid_t pid;
 
   // Before we do anything lets check for the existence of
-  // the traffic server binary along with it's execute permmissions
+  // the traffic server binary along with it's execute permissions
   if (access(absolute_proxy_binary, F_OK) < 0) {
     // Error can't find traffic_server
     mgmt_elog(errno, "[LocalManager::startProxy] Unable to find traffic server at %s\n", absolute_proxy_binary);
@@ -844,7 +806,7 @@ LocalManager::startProxy(const char *onetime_options)
   // traffic server binary exists, check permissions
   else if (access(absolute_proxy_binary, R_OK | X_OK) < 0) {
     // Error don't have proper permissions
-    mgmt_elog(errno, "[LocalManager::startProxy] Unable to access %s due to bad permisssions \n", absolute_proxy_binary);
+    mgmt_elog(errno, "[LocalManager::startProxy] Unable to access %s due to bad permissions \n", absolute_proxy_binary);
     return false;
   }
 
@@ -972,36 +934,98 @@ LocalManager::closeProxyPorts()
 void
 LocalManager::listenForProxy()
 {
-  if (!run_proxy) {
+  if (!run_proxy || !listen_for_proxy) {
     return;
   }
 
   // We are not already bound, bind the port
   for (auto &p : lmgmt->m_proxy_ports) {
     if (ts::NO_FD == p.m_fd) {
-      this->bindProxyPort(p);
+      // Check the protocol (TCP or UDP) and create an appropriate socket
+      if (p.isQUIC()) {
+        this->bindUdpProxyPort(p);
+      } else {
+        this->bindTcpProxyPort(p);
+      }
     }
 
-    // read backlong configuration value and overwrite the default value if found
-    bool found;
     std::string_view fam{ats_ip_family_name(p.m_family)};
-    RecInt backlog = REC_readInteger("proxy.config.net.listen_backlog", &found);
-    backlog        = (found && backlog >= 0) ? backlog : ats_tcp_somaxconn();
+    if (p.isQUIC()) {
+      // Can we do something like listen backlog for QUIC(UDP) ??
+      // Do nothing for now
+    } else {
+      // read backlog configuration value and overwrite the default value if found
+      bool found;
+      RecInt backlog = REC_readInteger("proxy.config.net.listen_backlog", &found);
+      backlog        = (found && backlog >= 0) ? backlog : ats_tcp_somaxconn();
 
-    if ((listen(p.m_fd, backlog)) < 0) {
-      mgmt_fatal(errno, "[LocalManager::listenForProxy] Unable to listen on port: %d (%.*s)\n", p.m_port, fam.size(), fam.data());
+      if ((listen(p.m_fd, backlog)) < 0) {
+        mgmt_fatal(errno, "[LocalManager::listenForProxy] Unable to listen on port: %d (%.*s)\n", p.m_port, fam.size(), fam.data());
+      }
     }
+
     mgmt_log("[LocalManager::listenForProxy] Listening on port: %d (%.*s)\n", p.m_port, fam.size(), fam.data());
   }
   return;
 }
 
 /*
- * bindProxyPort()
+ * bindUdpProxyPort()
  *  Function binds the accept port of the proxy
  */
 void
-LocalManager::bindProxyPort(HttpProxyPort &port)
+LocalManager::bindUdpProxyPort(HttpProxyPort &port)
+{
+  int one  = 1;
+  int priv = (port.m_port < 1024 && 0 != geteuid()) ? ElevateAccess::LOW_PORT_PRIVILEGE : 0;
+
+  ElevateAccess access(priv);
+
+  if ((port.m_fd = socket(port.m_family, SOCK_DGRAM, 0)) < 0) {
+    mgmt_fatal(0, "[bindProxyPort] Unable to create socket : %s\n", strerror(errno));
+  }
+
+  if (port.m_family == AF_INET6) {
+    if (setsockopt(port.m_fd, IPPROTO_IPV6, IPV6_V6ONLY, SOCKOPT_ON, sizeof(int)) < 0) {
+      mgmt_log("[bindProxyPort] Unable to set socket options: %d : %s\n", port.m_port, strerror(errno));
+    }
+  }
+  if (setsockopt(port.m_fd, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<char *>(&one), sizeof(int)) < 0) {
+    mgmt_fatal(0, "[bindProxyPort] Unable to set socket options: %d : %s\n", port.m_port, strerror(errno));
+  }
+
+  IpEndpoint ip;
+  if (port.m_inbound_ip.isValid()) {
+    ip.assign(port.m_inbound_ip);
+  } else if (AF_INET6 == port.m_family) {
+    if (m_inbound_ip6.isValid()) {
+      ip.assign(m_inbound_ip6);
+    } else {
+      ip.setToAnyAddr(AF_INET6);
+    }
+  } else if (AF_INET == port.m_family) {
+    if (m_inbound_ip4.isValid()) {
+      ip.assign(m_inbound_ip4);
+    } else {
+      ip.setToAnyAddr(AF_INET);
+    }
+  } else {
+    mgmt_fatal(0, "[bindProxyPort] Proxy port with invalid address type %d\n", port.m_family);
+  }
+  ip.port() = htons(port.m_port);
+  if (bind(port.m_fd, &ip.sa, ats_ip_size(&ip)) < 0) {
+    mgmt_fatal(0, "[bindProxyPort] Unable to bind socket: %d : %s\n", port.m_port, strerror(errno));
+  }
+
+  Debug("lm", "[bindProxyPort] Successfully bound proxy port %d", port.m_port);
+}
+
+/*
+ * bindTcpProxyPort()
+ *  Function binds the accept port of the proxy
+ */
+void
+LocalManager::bindTcpProxyPort(HttpProxyPort &port)
 {
   int one  = 1;
   int priv = (port.m_port < 1024 && 0 != geteuid()) ? ElevateAccess::LOW_PORT_PRIVILEGE : 0;
@@ -1046,7 +1070,7 @@ LocalManager::bindProxyPort(HttpProxyPort &port)
       mgmt_log("[bindProxyPort] Unable to set socket options: %d : %s\n", port.m_port, strerror(errno));
     }
   }
-  if (setsockopt(port.m_fd, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(int)) < 0) {
+  if (setsockopt(port.m_fd, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<char *>(&one), sizeof(int)) < 0) {
     mgmt_fatal(0, "[bindProxyPort] Unable to set socket options: %d : %s\n", port.m_port, strerror(errno));
   }
 
@@ -1095,6 +1119,6 @@ void
 LocalManager::signalAlarm(int alarm_id, const char *desc, const char *ip)
 {
   if (alarm_keeper) {
-    alarm_keeper->signalAlarm((alarm_t)alarm_id, desc, ip);
+    alarm_keeper->signalAlarm(static_cast<alarm_t>(alarm_id), desc, ip);
   }
 }
