@@ -31,8 +31,8 @@
  ***************************************************************************/
 
 #include "mgmtapi.h"
-#include "ts/ink_platform.h"
-#include "ts/ink_sock.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_sock.h"
 #include "LocalManager.h"
 #include "MgmtUtils.h"
 #include "MgmtSocket.h"
@@ -42,14 +42,14 @@
 #include "CoreAPIShared.h"
 #include "NetworkUtilsLocal.h"
 
+#include <unordered_map>
+
 #define TIMEOUT_SECS 1 // the num secs for select timeout
 
-static InkHashTable *accepted_con; // a list of all accepted client connections
+static std::unordered_map<int, ClientT *> accepted_con; // a list of all accepted client connections
 
 static TSMgmtError handle_control_message(int fd, void *msg, size_t msglen);
 
-static RecBool disable_modification = false;
-
 /*********************************************************************
  * create_client
  *
@@ -61,9 +61,9 @@ static RecBool disable_modification = false;
 static ClientT *
 create_client()
 {
-  ClientT *ele = (ClientT *)ats_malloc(sizeof(ClientT));
+  ClientT *ele = static_cast<ClientT *>(ats_malloc(sizeof(ClientT)));
 
-  ele->adr = (struct sockaddr *)ats_malloc(sizeof(struct sockaddr));
+  ele->adr = static_cast<struct sockaddr *>(ats_malloc(sizeof(struct sockaddr)));
   return ele;
 }
 
@@ -93,13 +93,13 @@ delete_client(ClientT *client)
  * output:
  *********************************************************************/
 static void
-remove_client(ClientT *client, InkHashTable *table)
+remove_client(ClientT *client, std::unordered_map<int, ClientT *> &table)
 {
   // close client socket
-  close_socket(client->fd); // close client socket
+  close_socket(client->fd);
 
   // remove client binding from hash table
-  ink_hash_table_delete(table, (char *)&client->fd);
+  table.erase(client->fd);
 
   // free ClientT
   delete_client(client);
@@ -123,23 +123,14 @@ ts_ctrl_main(void *arg)
   int *socket_fd;
   int con_socket_fd; // main socket for listening to new connections
 
-  socket_fd     = (int *)arg;
+  socket_fd     = static_cast<int *>(arg);
   con_socket_fd = *socket_fd;
 
-  // initialize queue for accepted con
-  accepted_con = ink_hash_table_create(InkHashTableKeyType_Word);
-  if (!accepted_con) {
-    return nullptr;
-  }
-
   // now we can start listening, accepting connections and servicing requests
   int new_con_fd; // new socket fd when accept connection
 
-  fd_set selectFDs;                    // for select call
-  InkHashTableEntry *con_entry;        // used to obtain client connection info
-  ClientT *client_entry;               // an entry of fd to alarms mapping
-  InkHashTableIteratorState con_state; // used to iterate through hash table
-  int fds_ready;                       // stores return value for select
+  fd_set selectFDs;      // for select call
+  ClientT *client_entry; // an entry of fd to alarms mapping
   struct timeval timeout;
 
   // loops until TM dies; waits for and processes requests from clients
@@ -154,26 +145,20 @@ ts_ctrl_main(void *arg)
       FD_SET(con_socket_fd, &selectFDs);
       // Debug("ts_main", "[ts_ctrl_main] add fd %d to select set", con_socket_fd);
     }
-    // see if there are more fd to set
-    con_entry = ink_hash_table_iterator_first(accepted_con, &con_state);
 
-    // iterate through all entries in hash table
-    while (con_entry) {
-      client_entry = (ClientT *)ink_hash_table_entry_value(accepted_con, con_entry);
+    for (auto &&it : accepted_con) {
+      client_entry = it.second;
       if (client_entry->fd >= 0) { // add fd to select set
         FD_SET(client_entry->fd, &selectFDs);
         Debug("ts_main", "[ts_ctrl_main] add fd %d to select set", client_entry->fd);
       }
-      con_entry = ink_hash_table_iterator_next(accepted_con, &con_state);
     }
 
     // select call - timeout is set so we can check events at regular intervals
-    fds_ready = mgmt_select(FD_SETSIZE, &selectFDs, (fd_set *)nullptr, (fd_set *)nullptr, &timeout);
+    int fds_ready = mgmt_select(FD_SETSIZE, &selectFDs, (fd_set *)nullptr, (fd_set *)nullptr, &timeout);
 
     // check if have any connections or requests
     if (fds_ready > 0) {
-      RecGetRecordBool("proxy.config.disable_configuration_modification", &disable_modification);
-
       // first check for connections!
       if (con_socket_fd >= 0 && FD_ISSET(con_socket_fd, &selectFDs)) {
         fds_ready--;
@@ -187,7 +172,7 @@ ts_ctrl_main(void *arg)
           socklen_t addr_len = (sizeof(struct sockaddr));
           new_con_fd         = mgmt_accept(con_socket_fd, new_client_con->adr, &addr_len);
           new_client_con->fd = new_con_fd;
-          ink_hash_table_insert(accepted_con, (char *)&new_client_con->fd, new_client_con);
+          accepted_con.emplace(new_client_con->fd, new_client_con);
           Debug("ts_main", "[ts_ctrl_main] Add new client connection");
         }
       } // end if(new_con_fd >= 0 && FD_ISSET(new_con_fd, &selectFDs))
@@ -195,10 +180,11 @@ ts_ctrl_main(void *arg)
       // some other file descriptor; for each one, service request
       if (fds_ready > 0) { // RECEIVED A REQUEST from remote API client
         // see if there are more fd to set - iterate through all entries in hash table
-        con_entry = ink_hash_table_iterator_first(accepted_con, &con_state);
-        while (con_entry) {
+
+        for (auto it = accepted_con.begin(); it != accepted_con.end();) {
           Debug("ts_main", "[ts_ctrl_main] We have a remote client request!");
-          client_entry = (ClientT *)ink_hash_table_entry_value(accepted_con, con_entry);
+          client_entry = it->second;
+          ++it; // prevent the breaking of remove_client
           // got information; check
           if (client_entry->fd && FD_ISSET(client_entry->fd, &selectFDs)) {
             void *req = nullptr;
@@ -209,8 +195,6 @@ ts_ctrl_main(void *arg)
               // occurs when remote API client terminates connection
               Debug("ts_main", "[ts_ctrl_main] ERROR: preprocess_msg - remove client %d ", client_entry->fd);
               remove_client(client_entry, accepted_con);
-              // get next client connection (if any)
-              con_entry = ink_hash_table_iterator_next(accepted_con, &con_state);
               continue;
             }
 
@@ -219,19 +203,14 @@ ts_ctrl_main(void *arg)
 
             if (ret != TS_ERR_OKAY) {
               Debug("ts_main", "[ts_ctrl_main] ERROR: sending response for message (%d)", ret);
-
               // XXX this doesn't actually send a error response ...
-
               remove_client(client_entry, accepted_con);
-              con_entry = ink_hash_table_iterator_next(accepted_con, &con_state);
               continue;
             }
 
           } // end if(client_entry->fd && FD_ISSET(client_entry->fd, &selectFDs))
-
-          con_entry = ink_hash_table_iterator_next(accepted_con, &con_state);
-        } // end while (con_entry)
-      }   // end if (fds_ready > 0)
+        }   // end for (auto it = accepted_con.begin(); it != accepted_con.end();)
+      }     // end if (fds_ready > 0)
 
     } // end if (fds_ready > 0)
 
@@ -241,19 +220,16 @@ ts_ctrl_main(void *arg)
   Debug("ts_main", "[ts_ctrl_main] CLOSING AND SHUTTING DOWN OPERATIONS");
   close_socket(con_socket_fd);
 
-  // iterate through hash table; close client socket connections and remove entry
-  con_entry = ink_hash_table_iterator_first(accepted_con, &con_state);
-  while (con_entry) {
-    client_entry = (ClientT *)ink_hash_table_entry_value(accepted_con, con_entry);
+  for (auto &&it : accepted_con) {
+    client_entry = it.second;
     if (client_entry->fd >= 0) {
       close_socket(client_entry->fd); // close socket
     }
-    ink_hash_table_delete(accepted_con, (char *)&client_entry->fd); // remove binding
-    delete_client(client_entry);                                    // free ClientT
-    con_entry = ink_hash_table_iterator_next(accepted_con, &con_state);
+    accepted_con.erase(client_entry->fd);
+    delete_client(client_entry); // free ClientT
   }
   // all entries should be removed and freed already
-  ink_hash_table_destroy(accepted_con);
+  accepted_con.clear();
 
   ink_thread_exit(nullptr);
   return nullptr;
@@ -368,7 +344,7 @@ send_record_get_response(int fd, const RecRecord *rec)
 static void
 send_record_get(const RecRecord *rec, void *edata)
 {
-  int *fd = (int *)edata;
+  int *fd = static_cast<int *>(edata);
   *fd     = send_record_get_response(*fd, rec);
 }
 
@@ -399,7 +375,7 @@ handle_record_get(int fd, void *req, size_t reqlen)
 
   // If the lookup succeeded, the final error is in "fderr".
   if (ret == TS_ERR_OKAY) {
-    ret = (TSMgmtError)fderr;
+    ret = static_cast<TSMgmtError>(fderr);
   }
 
 done:
@@ -415,7 +391,7 @@ struct record_match_state {
 static void
 send_record_match(const RecRecord *rec, void *edata)
 {
-  record_match_state *match = (record_match_state *)edata;
+  record_match_state *match = static_cast<record_match_state *>(edata);
 
   if (match->err != TS_ERR_OKAY) {
     return;
@@ -499,80 +475,6 @@ fail:
   return send_mgmt_response(fd, OpType::RECORD_SET, &err, &act);
 }
 
-/**************************************************************************
- * handle_file_read
- *
- * purpose: handles request to read a file
- * output: SUCC or ERR
- * note: None
- *************************************************************************/
-static TSMgmtError
-handle_file_read(int fd, void *req, size_t reqlen)
-{
-  int size, version;
-  char *text;
-
-  MgmtMarshallInt optype;
-  MgmtMarshallInt fid;
-
-  MgmtMarshallInt err;
-  MgmtMarshallInt vers  = 0;
-  MgmtMarshallData data = {nullptr, 0};
-
-  err = recv_mgmt_request(req, reqlen, OpType::FILE_READ, &optype, &fid);
-  if (err != TS_ERR_OKAY) {
-    return (TSMgmtError)err;
-  }
-
-  // make CoreAPI call on Traffic Manager side
-  err = ReadFile((TSFileNameT)fid, &text, &size, &version);
-  if (err == TS_ERR_OKAY) {
-    vers     = version;
-    data.ptr = text;
-    data.len = size;
-  }
-
-  err = send_mgmt_response(fd, OpType::FILE_READ, &err, &vers, &data);
-
-  ats_free(text); // free memory allocated by ReadFile
-  return (TSMgmtError)err;
-}
-
-/**************************************************************************
- * handle_file_write
- *
- * purpose: handles request to write a file
- * output: SUCC or ERR
- * note: None
- *************************************************************************/
-static TSMgmtError
-handle_file_write(int fd, void *req, size_t reqlen)
-{
-  MgmtMarshallInt optype;
-  MgmtMarshallInt fid;
-  MgmtMarshallInt vers;
-  MgmtMarshallData data = {nullptr, 0};
-
-  MgmtMarshallInt err;
-
-  err = recv_mgmt_request(req, reqlen, OpType::FILE_WRITE, &optype, &fid, &vers, &data);
-  if (err != TS_ERR_OKAY) {
-    goto done;
-  }
-
-  if (data.ptr == nullptr) {
-    err = TS_ERR_PARAMS;
-    goto done;
-  }
-
-  // make CoreAPI call on Traffic Manager side
-  err = WriteFile((TSFileNameT)fid, (const char *)data.ptr, data.len, vers);
-
-done:
-  ats_free(data.ptr);
-  return send_mgmt_response(fd, OpType::FILE_WRITE, &err);
-}
-
 /**************************************************************************
  * handle_proxy_state_get
  *
@@ -616,7 +518,7 @@ handle_proxy_state_set(int fd, void *req, size_t reqlen)
     return send_mgmt_response(fd, OpType::PROXY_STATE_SET, &err);
   }
 
-  err = ProxyStateSet((TSProxyStateT)state, (TSCacheClearT)clear);
+  err = ProxyStateSet(static_cast<TSProxyStateT>(state), static_cast<TSCacheClearT>(clear));
   return send_mgmt_response(fd, OpType::PROXY_STATE_SET, &err);
 }
 
@@ -673,6 +575,50 @@ handle_restart(int fd, void *req, size_t reqlen)
   return send_mgmt_response(fd, OpType::RESTART, &err);
 }
 
+/**************************************************************************
+ * handle_stop
+ *
+ * purpose: handles request to stop TS
+ * output: TS_ERR_xx
+ * note: None
+ *************************************************************************/
+static TSMgmtError
+handle_stop(int fd, void *req, size_t reqlen)
+{
+  OpType optype;
+  MgmtMarshallInt options;
+  MgmtMarshallInt err;
+
+  err = recv_mgmt_request(req, reqlen, OpType::STOP, &optype, &options);
+  if (err == TS_ERR_OKAY) {
+    err = Stop(options);
+  }
+
+  return send_mgmt_response(fd, OpType::STOP, &err);
+}
+
+/**************************************************************************
+ * handle_drain
+ *
+ * purpose: handles request to drain TS
+ * output: TS_ERR_xx
+ * note: None
+ *************************************************************************/
+static TSMgmtError
+handle_drain(int fd, void *req, size_t reqlen)
+{
+  OpType optype;
+  MgmtMarshallInt options;
+  MgmtMarshallInt err;
+
+  err = recv_mgmt_request(req, reqlen, OpType::DRAIN, &optype, &options);
+  if (err == TS_ERR_OKAY) {
+    err = Drain(options);
+  }
+
+  return send_mgmt_response(fd, OpType::DRAIN, &err);
+}
+
 /**************************************************************************
  * handle_storage_device_cmd_offline
  *
@@ -752,7 +698,7 @@ handle_event_get_mlt(int fd, void *req, size_t reqlen)
   // iterate through list and put into a delimited string list
   memset(buf, 0, MAX_BUF_SIZE);
   while (!queue_is_empty(event_list)) {
-    event_name = (char *)dequeue(event_list);
+    event_name = static_cast<char *>(dequeue(event_list));
     if (event_name) {
       snprintf(buf + buf_pos, (MAX_BUF_SIZE - buf_pos), "%s%c", event_name, REMOTE_DELIM);
       buf_pos += (strlen(event_name) + 1);
@@ -806,101 +752,6 @@ done:
   return send_mgmt_response(fd, OpType::EVENT_ACTIVE, &err, &bval);
 }
 
-/**************************************************************************
- * handle_snapshot
- *
- * purpose: handles request to take/remove/restore a snapshot
- * output: TS_ERR_xx
- *************************************************************************/
-static TSMgmtError
-handle_snapshot(int fd, void *req, size_t reqlen)
-{
-  OpType optype;
-  MgmtMarshallString name = nullptr;
-
-  MgmtMarshallInt err;
-
-  err = recv_mgmt_request(req, reqlen, OpType::SNAPSHOT_TAKE, &optype, &name);
-  if (err != TS_ERR_OKAY) {
-    goto done;
-  }
-
-  if (strlen(name) == 0) {
-    err = TS_ERR_PARAMS;
-    goto done;
-  }
-
-  // call CoreAPI call on Traffic Manager side
-  switch (optype) {
-  case OpType::SNAPSHOT_TAKE:
-    err = SnapshotTake(name);
-    break;
-  case OpType::SNAPSHOT_RESTORE:
-    err = SnapshotRestore(name);
-    break;
-  case OpType::SNAPSHOT_REMOVE:
-    err = SnapshotRemove(name);
-    break;
-  default:
-    err = TS_ERR_FAIL;
-    break;
-  }
-
-done:
-  ats_free(name);
-  return send_mgmt_response(fd, (OpType)optype, &err);
-}
-
-/**************************************************************************
- * handle_snapshot_get_mlt
- *
- * purpose: handles request to get list of snapshots
- * output: TS_ERR_xx
- * note: the req should be the event name
- *************************************************************************/
-static TSMgmtError
-handle_snapshot_get_mlt(int fd, void *req, size_t reqlen)
-{
-  LLQ *snap_list = create_queue();
-  char buf[MAX_BUF_SIZE];
-  char *snap_name;
-  int buf_pos = 0;
-
-  MgmtMarshallInt optype;
-  MgmtMarshallInt err;
-  MgmtMarshallString list = nullptr;
-
-  err = recv_mgmt_request(req, reqlen, OpType::SNAPSHOT_GET_MLT, &optype);
-  if (err != TS_ERR_OKAY) {
-    goto done;
-  }
-
-  // call CoreAPI call on Traffic Manager side; req == event_name
-  err = SnapshotGetMlt(snap_list);
-  if (err != TS_ERR_OKAY) {
-    goto done;
-  }
-
-  // iterate through list and put into a delimited string list
-  memset(buf, 0, MAX_BUF_SIZE);
-  while (!queue_is_empty(snap_list)) {
-    snap_name = (char *)dequeue(snap_list);
-    if (snap_name) {
-      snprintf(buf + buf_pos, (MAX_BUF_SIZE - buf_pos), "%s%c", snap_name, REMOTE_DELIM);
-      buf_pos += (strlen(snap_name) + 1);
-      ats_free(snap_name); // free the llq entry
-    }
-  }
-  buf[buf_pos] = '\0'; // end the string
-
-  // Point the send list to the filled buffer.
-  list = buf;
-
-done:
-  delete_queue(snap_list);
-  return send_mgmt_response(fd, OpType::SNAPSHOT_GET_MLT, &err, &list);
-}
-
 /**************************************************************************
  * handle_stats_reset
  *
@@ -916,17 +767,64 @@ handle_stats_reset(int fd, void *req, size_t reqlen)
 
   err = recv_mgmt_request(req, reqlen, OpType::STATS_RESET_NODE, &optype, &name);
   if (err == TS_ERR_OKAY) {
-    err = StatsReset(optype == OpType::STATS_RESET_CLUSTER, name);
+    err = StatsReset(name);
   }
 
   ats_free(name);
-  return send_mgmt_response(fd, (OpType)optype, &err);
+  return send_mgmt_response(fd, optype, &err);
 }
 
+/**************************************************************************
+ * handle_host_status_up
+ *
+ * purpose: handles request to reset statistics to default values
+ * output: TS_ERR_xx
+ *************************************************************************/
+static TSMgmtError
+handle_host_status_up(int fd, void *req, size_t reqlen)
+{
+  OpType optype;
+  MgmtMarshallString name   = nullptr;
+  MgmtMarshallString reason = nullptr;
+  MgmtMarshallInt err;
+  MgmtMarshallInt down_time;
+
+  err = recv_mgmt_request(req, reqlen, OpType::HOST_STATUS_UP, &optype, &name, &reason, &down_time);
+  if (err == TS_ERR_OKAY) {
+    lmgmt->signalEvent(MGMT_EVENT_HOST_STATUS_UP, static_cast<char *>(req), reqlen);
+  }
+
+  ats_free(name);
+  return send_mgmt_response(fd, optype, &err);
+}
+
+/**************************************************************************
+ * handle_host_status_down
+ *
+ * purpose: handles request to reset statistics to default values
+ * output: TS_ERR_xx
+ *************************************************************************/
+static TSMgmtError
+handle_host_status_down(int fd, void *req, size_t reqlen)
+{
+  OpType optype;
+  MgmtMarshallString name   = nullptr;
+  MgmtMarshallString reason = nullptr;
+  MgmtMarshallInt err;
+  MgmtMarshallInt down_time;
+
+  err = recv_mgmt_request(req, reqlen, OpType::HOST_STATUS_DOWN, &optype, &name, &reason, &down_time);
+  if (err == TS_ERR_OKAY) {
+    lmgmt->signalEvent(MGMT_EVENT_HOST_STATUS_DOWN, static_cast<char *>(req), reqlen);
+  }
+
+  ats_free(name);
+  return send_mgmt_response(fd, optype, &err);
+}
 /**************************************************************************
  * handle_api_ping
  *
- * purpose: handles the API_PING messaghat is sent by API clients to keep
+ * purpose: handles the API_PING message that is sent by API clients to keep
  *    the management socket alive
  * output: TS_ERR_xx. There is no response message.
  *************************************************************************/
@@ -955,7 +853,7 @@ handle_server_backtrace(int fd, void *req, size_t reqlen)
   err = send_mgmt_response(fd, OpType::SERVER_BACKTRACE, &err, &trace);
   ats_free(trace);
 
-  return (TSMgmtError)err;
+  return static_cast<TSMgmtError>(err);
 }
 
 static void
@@ -978,7 +876,7 @@ send_record_describe(const RecRecord *rec, void *edata)
 
   TSMgmtError err = TS_ERR_OKAY;
 
-  record_match_state *match = (record_match_state *)edata;
+  record_match_state *match = static_cast<record_match_state *>(edata);
 
   if (match->err != TS_ERR_OKAY) {
     return;
@@ -1091,7 +989,7 @@ done:
 /**************************************************************************
  * handle_lifecycle_message
  *
- * purpose: handle lifecyle message to plugins
+ * purpose: handle lifecycle message to plugins
  * output: TS_ERR_xx
  * note: None
  *************************************************************************/
@@ -1118,8 +1016,6 @@ struct control_message_handler {
 };
 
 static const control_message_handler handlers[] = {
-  /* FILE_READ                  */ {MGMT_API_PRIVILEGED, handle_file_read},
-  /* FILE_WRITE                 */ {MGMT_API_PRIVILEGED, handle_file_write},
   /* RECORD_SET                 */ {MGMT_API_PRIVILEGED, handle_record_set},
   /* RECORD_GET                 */ {0, handle_record_get},
   /* PROXY_STATE_GET            */ {0, handle_proxy_state_get},
@@ -1127,24 +1023,23 @@ static const control_message_handler handlers[] = {
   /* RECONFIGURE                */ {MGMT_API_PRIVILEGED, handle_reconfigure},
   /* RESTART                    */ {MGMT_API_PRIVILEGED, handle_restart},
   /* BOUNCE                     */ {MGMT_API_PRIVILEGED, handle_restart},
+  /* STOP                       */ {MGMT_API_PRIVILEGED, handle_stop},
+  /* DRAIN                      */ {MGMT_API_PRIVILEGED, handle_drain},
   /* EVENT_RESOLVE              */ {MGMT_API_PRIVILEGED, handle_event_resolve},
   /* EVENT_GET_MLT              */ {0, handle_event_get_mlt},
   /* EVENT_ACTIVE               */ {0, handle_event_active},
   /* EVENT_REG_CALLBACK         */ {0, nullptr},
   /* EVENT_UNREG_CALLBACK       */ {0, nullptr},
   /* EVENT_NOTIFY               */ {0, nullptr},
-  /* SNAPSHOT_TAKE              */ {MGMT_API_PRIVILEGED, handle_snapshot},
-  /* SNAPSHOT_RESTORE           */ {MGMT_API_PRIVILEGED, handle_snapshot},
-  /* SNAPSHOT_REMOVE            */ {MGMT_API_PRIVILEGED, handle_snapshot},
-  /* SNAPSHOT_GET_MLT           */ {0, handle_snapshot_get_mlt},
   /* STATS_RESET_NODE           */ {MGMT_API_PRIVILEGED, handle_stats_reset},
-  /* STATS_RESET_CLUSTER        */ {MGMT_API_PRIVILEGED, handle_stats_reset},
   /* STORAGE_DEVICE_CMD_OFFLINE */ {MGMT_API_PRIVILEGED, handle_storage_device_cmd_offline},
   /* RECORD_MATCH_GET           */ {0, handle_record_match},
   /* API_PING                   */ {0, handle_api_ping},
   /* SERVER_BACKTRACE           */ {MGMT_API_PRIVILEGED, handle_server_backtrace},
   /* RECORD_DESCRIBE_CONFIG     */ {0, handle_record_describe},
   /* LIFECYCLE_MESSAGE          */ {MGMT_API_PRIVILEGED, handle_lifecycle_message},
+  /* HOST_STATUS_UP             */ {MGMT_API_PRIVILEGED, handle_host_status_up},
+  /* HOST_STATUS_DOWN           */ {MGMT_API_PRIVILEGED, handle_host_status_down},
 };
 
 // This should use countof(), but we need a constexpr :-/
@@ -1191,6 +1086,6 @@ handle_control_message(int fd, void *req, size_t reqlen)
   return TS_ERR_OKAY;
 
 fail:
-  mgmt_elog(0, "%s: missing handler for type %d control message\n", __func__, (int)optype);
+  mgmt_elog(0, "%s: missing handler for type %d control message\n", __func__, static_cast<int>(optype));
   return TS_ERR_PARAMS;
 }
