@@ -38,8 +38,6 @@
 #include <sys/param.h>
 #include <ts/remap.h>
 
-#include "ts/ink_config.h"
-
 using std::strlen;
 
 struct AuthRequestContext;
@@ -57,12 +55,12 @@ static TSCont AuthOsDnsContinuation;
 
 struct AuthOptions {
   std::string hostname;
-  int hostport;
-  AuthRequestTransform transform;
-  bool force;
+  int hostport                   = -1;
+  AuthRequestTransform transform = nullptr;
+  bool force                     = false;
 
-  AuthOptions() : hostport(-1), transform(nullptr), force(false) {}
-  ~AuthOptions() {}
+  AuthOptions()  = default;
+  ~AuthOptions() = default;
 };
 
 // Global options; used when we are in global authorization mode.
@@ -139,29 +137,21 @@ static const StateTransition StateTableInit[] = {{TS_EVENT_HTTP_POST_REMAP, Stat
                                                  {TS_EVENT_NONE, nullptr, nullptr}};
 
 struct AuthRequestContext {
-  TSHttpTxn txn;        // Original client transaction we are authorizing.
-  TSCont cont;          // Continuation for this state machine.
-  TSVConn vconn;        // Virtual connection to the auth proxy.
-  TSHttpParser hparser; // HTTP response header parser.
-  HttpHeader rheader;   // HTTP response header.
+  TSHttpTxn txn = nullptr; // Original client transaction we are authorizing.
+  TSCont cont   = nullptr; // Continuation for this state machine.
+  TSVConn vconn = nullptr; // Virtual connection to the auth proxy.
+  TSHttpParser hparser;    // HTTP response header parser.
+  HttpHeader rheader;      // HTTP response header.
   HttpIoBuffer iobuf;
-  const char *method; // Client request method (e.g. GET)
-  bool read_body;
+  const char *method = nullptr; // Client request method (e.g. GET)
+  bool read_body     = true;
 
-  const StateTransition *state;
+  const StateTransition *state = nullptr;
 
   AuthRequestContext()
-    : txn(nullptr),
-      cont(nullptr),
-      vconn(nullptr),
-      hparser(TSHttpParserCreate()),
-      rheader(),
-      iobuf(TS_IOBUFFER_SIZE_INDEX_4K),
-      method(nullptr),
-      read_body(true),
-      state(nullptr)
+    : cont(TSContCreate(dispatch, TSMutexCreate())), hparser(TSHttpParserCreate()), rheader(), iobuf(TS_IOBUFFER_SIZE_INDEX_4K)
+
   {
-    this->cont = TSContCreate(dispatch, TSMutexCreate());
     TSContDataSet(this->cont, this);
   }
 
@@ -180,7 +170,7 @@ struct AuthRequestContext {
   {
     AuthOptions *opt;
 
-    opt = (AuthOptions *)TSHttpTxnArgGet(this->txn, AuthTaggedRequestArg);
+    opt = static_cast<AuthOptions *>(TSUserArgGet(this->txn, AuthTaggedRequestArg));
     return opt ? opt : AuthGlobalOptions;
   }
 
@@ -208,7 +198,7 @@ AuthRequestContext::destroy(AuthRequestContext *auth)
 int
 AuthRequestContext::dispatch(TSCont cont, TSEvent event, void *edata)
 {
-  AuthRequestContext *auth = (AuthRequestContext *)TSContDataGet(cont);
+  AuthRequestContext *auth = static_cast<AuthRequestContext *>(TSContDataGet(cont));
   const StateTransition *s;
 
 pump:
@@ -425,7 +415,6 @@ static TSEvent
 StateAuthProxyCompleteHeaders(AuthRequestContext *auth, void * /* edata ATS_UNUSED */)
 {
   TSHttpStatus status;
-  unsigned nbytes;
 
   HttpDebugHeader(auth->rheader.buffer, auth->rheader.header);
 
@@ -446,7 +435,7 @@ StateAuthProxyCompleteHeaders(AuthRequestContext *auth, void * /* edata ATS_UNUS
     } else {
       // OK, we have a non-chunked response. If there's any content, let's go and
       // buffer it so that we can send it on to the client.
-      nbytes = HttpGetContentLength(auth->rheader.buffer, auth->rheader.header);
+      unsigned nbytes = HttpGetContentLength(auth->rheader.buffer, auth->rheader.header);
       if (nbytes > 0) {
         AuthLogDebug("content length is %u", nbytes);
         return TS_EVENT_HTTP_CONTINUE;
@@ -483,7 +472,7 @@ StateAuthProxySendResponse(AuthRequestContext *auth, void * /* edata ATS_UNUSED
 
   // We must not whack the content length for HEAD responses, since the
   // client already knows that there is no body. Forcing content length to
-  // zero breaks hdiutil(1) on Mac OS X.
+  // zero breaks hdiutil(1) on macOS
   if (TS_HTTP_METHOD_HEAD != auth->method) {
     HttpSetMimeHeader(mbuf, mhdr, TS_MIME_FIELD_CONTENT_LENGTH, 0u);
   }
@@ -602,7 +591,7 @@ StateUnauthorized(AuthRequestContext *auth, void *)
 {
   static const char msg[] = "authorization denied\n";
 
-  TSHttpTxnSetHttpRetStatus(auth->txn, TS_HTTP_STATUS_FORBIDDEN);
+  TSHttpTxnStatusSet(auth->txn, TS_HTTP_STATUS_FORBIDDEN);
   TSHttpTxnErrorBodySet(auth->txn, TSstrdup(msg), sizeof(msg) - 1, TSstrdup("text/plain"));
 
   TSHttpTxnReenable(auth->txn, TS_EVENT_HTTP_ERROR);
@@ -632,14 +621,13 @@ StateAuthorized(AuthRequestContext *auth, void *)
 static bool
 AuthRequestIsTagged(TSHttpTxn txn)
 {
-  return AuthTaggedRequestArg != -1 && TSHttpTxnArgGet(txn, AuthTaggedRequestArg) != nullptr;
+  return AuthTaggedRequestArg != -1 && TSUserArgGet(txn, AuthTaggedRequestArg) != nullptr;
 }
 
 static int
 AuthProxyGlobalHook(TSCont /* cont ATS_UNUSED */, TSEvent event, void *edata)
 {
-  AuthRequestContext *auth;
-  TSHttpTxn txn = (TSHttpTxn)edata;
+  TSHttpTxn txn = static_cast<TSHttpTxn>(edata);
 
   AuthLogDebug("handling event=%d edata=%p", (int)event, edata);
 
@@ -663,12 +651,12 @@ AuthProxyGlobalHook(TSCont /* cont ATS_UNUSED */, TSEvent event, void *edata)
     // Hook this request if we are in global authorization mode or if a
     // remap rule tagged it.
     if (AuthGlobalOptions != nullptr || AuthRequestIsTagged(txn)) {
-      auth        = AuthRequestContext::allocate();
-      auth->state = StateTableInit;
-      auth->txn   = txn;
+      AuthRequestContext *auth = AuthRequestContext::allocate();
+      auth->state              = StateTableInit;
+      auth->txn                = txn;
       return AuthRequestContext::dispatch(auth->cont, event, edata);
     }
-  // fallthru
+    // fallthrough
 
   default:
     return TS_EVENT_NONE;
@@ -696,7 +684,7 @@ AuthParseOptions(int argc, const char **argv)
   for (;;) {
     int opt;
 
-    opt = getopt_long(argc, (char *const *)argv, "", longopt, nullptr);
+    opt = getopt_long(argc, const_cast<char *const *>(argv), "", longopt, nullptr);
     switch (opt) {
     case 'h':
       options->hostname = optarg;
@@ -748,7 +736,8 @@ TSPluginInit(int argc, const char *argv[])
     AuthLogError("plugin registration failed");
   }
 
-  TSReleaseAssert(TSHttpArgIndexReserve("AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) == TS_SUCCESS);
+  TSReleaseAssert(TSUserArgIndexReserve(TS_USER_ARGS_TXN, "AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) ==
+                  TS_SUCCESS);
 
   AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, nullptr);
   AuthGlobalOptions     = AuthParseOptions(argc, argv);
@@ -761,7 +750,8 @@ TSPluginInit(int argc, const char *argv[])
 TSReturnCode
 TSRemapInit(TSRemapInterface * /* api ATS_UNUSED */, char * /* err ATS_UNUSED */, int /* errsz ATS_UNUSED */)
 {
-  TSReleaseAssert(TSHttpArgIndexReserve("AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) == TS_SUCCESS);
+  TSReleaseAssert(TSUserArgIndexReserve(TS_USER_ARGS_TXN, "AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) ==
+                  TS_SUCCESS);
 
   AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, nullptr);
   return TS_SUCCESS;
@@ -788,16 +778,16 @@ TSRemapNewInstance(int argc, char *argv[], void **instance, char * /* err ATS_UN
 void
 TSRemapDeleteInstance(void *instance)
 {
-  AuthOptions *options = (AuthOptions *)instance;
+  AuthOptions *options = static_cast<AuthOptions *>(instance);
   AuthDelete(options);
 }
 
 TSRemapStatus
 TSRemapDoRemap(void *instance, TSHttpTxn txn, TSRemapRequestInfo * /* rri ATS_UNUSED */)
 {
-  AuthOptions *options = (AuthOptions *)instance;
+  AuthOptions *options = static_cast<AuthOptions *>(instance);
 
-  TSHttpTxnArgSet(txn, AuthTaggedRequestArg, options);
+  TSUserArgSet(txn, AuthTaggedRequestArg, options);
   TSHttpTxnHookAdd(txn, TS_HTTP_POST_REMAP_HOOK, AuthOsDnsContinuation);
 
   return TSREMAP_NO_REMAP;
