@@ -23,17 +23,21 @@
 
 #include <list>
 #include <string>
+#include <string_view>
 #include <sstream>
 #include <vector>
 #include <algorithm>
 #include <ctime>
-#include <pthread.h>
+#include <fstream>
 #include <arpa/inet.h>
+#include <limits>
+#include <getopt.h>
 
 #include "ts/ts.h"
 #include "ts/experimental.h"
 #include "ts/remap.h"
-#include "ts/ink_defs.h"
+#include "tscore/ink_defs.h"
+#include "tscpp/util/TextView.h"
 
 #include "HttpDataFetcherImpl.h"
 #include "gzip.h"
@@ -44,16 +48,27 @@ using namespace EsiLib;
 
 #define DEBUG_TAG "combo_handler"
 
-#define MAX_FILE_COUNT 30
-#define MAX_QUERY_LENGTH 3000
+// Because STL vs. C library leads to ugly casting, fix it once.
+inline int
+length(std::string const &str)
+{
+  return static_cast<int>(str.size());
+}
+
+constexpr unsigned DEFAULT_MAX_FILE_COUNT = 100;
+constexpr int MAX_QUERY_LENGTH            = 4096;
+
+unsigned MaxFileCount = DEFAULT_MAX_FILE_COUNT;
+
+// We hardcode "immutable" here because it's not yet defined in the ATS API
+#define HTTP_IMMUTABLE "immutable"
 
 int arg_idx;
 static string SIG_KEY_NAME;
-static vector<string> HEADER_WHITELIST;
+static vector<string> HEADER_ALLOWLIST;
 
 #define DEFAULT_COMBO_HANDLER_PATH "admin/v1/combo"
-static string COMBO_HANDLER_PATH;
-static int COMBO_HANDLER_PATH_SIZE;
+static string COMBO_HANDLER_PATH{DEFAULT_COMBO_HANDLER_PATH};
 
 #define LOG_ERROR(fmt, args...)                                                               \
   do {                                                                                        \
@@ -69,12 +84,12 @@ static int COMBO_HANDLER_PATH_SIZE;
 using StringList = list<string>;
 
 struct ClientRequest {
-  TSHttpStatus status;
-  const sockaddr *client_addr;
+  TSHttpStatus status         = TS_HTTP_STATUS_OK;
+  const sockaddr *client_addr = nullptr;
   StringList file_urls;
-  bool gzip_accepted;
+  bool gzip_accepted = false;
   string defaultBucket; // default Bucket will be set to HOST header
-  ClientRequest() : status(TS_HTTP_STATUS_OK), client_addr(nullptr), gzip_accepted(false), defaultBucket("l"){};
+  ClientRequest() : defaultBucket("l"){};
 };
 
 struct InterceptData {
@@ -82,11 +97,11 @@ struct InterceptData {
   TSCont contp;
 
   struct IoHandle {
-    TSVIO vio;
-    TSIOBuffer buffer;
-    TSIOBufferReader reader;
+    TSVIO vio               = nullptr;
+    TSIOBuffer buffer       = nullptr;
+    TSIOBufferReader reader = nullptr;
 
-    IoHandle() : vio(nullptr), buffer(nullptr), reader(nullptr){};
+    IoHandle() = default;
 
     ~IoHandle()
     {
@@ -136,6 +151,51 @@ struct InterceptData {
   ~InterceptData();
 };
 
+/*
+ * This class is responsible for keeping track of and processing the various
+ * Cache-Control values between all the requested documents
+ */
+struct CacheControlHeader {
+  enum Publicity { PRIVATE, PUBLIC, DEFAULT };
+
+  // Update the object with a document's Cache-Control header
+  void update(TSMBuffer bufp, TSMLoc hdr_loc);
+
+  // Return the Cache-Control for the combined document
+  string generate() const;
+
+  // Cache-Control values we're keeping track of
+  unsigned int _max_age = numeric_limits<unsigned int>::max();
+  Publicity _publicity  = Publicity::DEFAULT;
+  bool _immutable       = true;
+};
+
+class ContentTypeHandler
+{
+public:
+  ContentTypeHandler(std::string &resp_header_fields) : _resp_header_fields(resp_header_fields) {}
+
+  // Returns false if _content_type_allowlist is not empty, and content-type field is either not present or not in the
+  // allowlist.  Adds first Content-type field it encounters in the headers passed to this function.
+  //
+  bool nextObjectHeader(TSMBuffer bufp, TSMLoc hdr_loc);
+
+  // Load allowlist from config file.
+  //
+  static void loadAllowList(std::string const &file_spec);
+
+private:
+  // Add Content-Type field to these.
+  //
+  std::string &_resp_header_fields;
+
+  bool _added_content_type{false};
+
+  static vector<std::string> _content_type_allowlist;
+};
+
+vector<std::string> ContentTypeHandler::_content_type_allowlist;
+
 bool
 InterceptData::init(TSVConn vconn)
 {
@@ -187,6 +247,83 @@ InterceptData::~InterceptData()
   }
 }
 
+void
+CacheControlHeader::update(TSMBuffer bufp, TSMLoc hdr_loc)
+{
+  bool found_immutable = false;
+  bool found_private   = false;
+
+  // Load each value from the Cache-Control header into the vector values
+  TSMLoc field_loc = TSMimeHdrFieldFind(bufp, hdr_loc, TS_MIME_FIELD_CACHE_CONTROL, TS_MIME_LEN_CACHE_CONTROL);
+  if (field_loc != TS_NULL_MLOC) {
+    int n_values = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
+    if ((n_values != TS_ERROR) && (n_values > 0)) {
+      for (int i = 0; i < n_values; i++) {
+        // Grab this current header value
+        int _val_len    = 0;
+        const char *val = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, i, &_val_len);
+
+        // Update max-age if necessary
+        if (strncasecmp(val, TS_HTTP_VALUE_MAX_AGE, TS_HTTP_LEN_MAX_AGE) == 0) {
+          unsigned int max_age = 0;
+          char *ptr            = const_cast<char *>(val);
+          ptr += TS_HTTP_LEN_MAX_AGE;
+          while ((*ptr == ' ') || (*ptr == '\t')) {
+            ptr++;
+          }
+          if (*ptr == '=') {
+            ptr++;
+            max_age = atoi(ptr);
+          }
+          if (max_age > 0 && max_age < _max_age) {
+            _max_age = max_age;
+          }
+          // If we find even a single occurrence of private, the whole response must be private
+        } else if (strncasecmp(val, TS_HTTP_VALUE_PRIVATE, TS_HTTP_LEN_PRIVATE) == 0) {
+          found_private = true;
+          // Every requested document must have immutable for the final response to be immutable
+        } else if (strncasecmp(val, HTTP_IMMUTABLE, strlen(HTTP_IMMUTABLE)) == 0) {
+          found_immutable = true;
+        }
+      }
+    }
+    TSHandleMLocRelease(bufp, hdr_loc, field_loc);
+  }
+
+  if (!found_immutable) {
+    LOG_DEBUG("Did not see an immutable cache control. The response will be not be immutable");
+    _immutable = false;
+  }
+
+  if (found_private) {
+    LOG_DEBUG("Saw a private cache control. The response will be private");
+    _publicity = Publicity::PRIVATE;
+  }
+}
+
+string
+CacheControlHeader::generate() const
+{
+  unsigned int max_age;
+  char line_buf[256];
+  const char *publicity;
+  const char *immutable;
+
+  // Previously, all combo_cache documents were public. However, that's a bug. If any requested document is private the combo_cache
+  // document should private as well.
+  if (_publicity == Publicity::PUBLIC || _publicity == Publicity::DEFAULT) {
+    publicity = TS_HTTP_VALUE_PUBLIC;
+  } else {
+    publicity = TS_HTTP_VALUE_PRIVATE;
+  }
+
+  immutable = (_immutable ? ", " HTTP_IMMUTABLE : "");
+  max_age   = (_max_age == numeric_limits<unsigned int>::max() ? 315360000 : _max_age); // default is 10 years
+
+  sprintf(line_buf, "Cache-Control: max-age=%u, %s%s\r\n", max_age, publicity, immutable);
+  return string(line_buf);
+}
+
 // forward declarations
 static int handleReadRequestHeader(TSCont contp, TSEvent event, void *edata);
 static bool isComboHandlerRequest(TSMBuffer bufp, TSMLoc hdr_loc, TSMLoc url_loc);
@@ -200,13 +337,8 @@ static bool writeResponse(InterceptData &int_data);
 static bool writeErrorResponse(InterceptData &int_data, int &n_bytes_written);
 static bool writeStandardHeaderFields(InterceptData &int_data, int &n_bytes_written);
 static void prepareResponse(InterceptData &int_data, ByteBlockList &body_blocks, string &resp_header_fields);
-static bool getContentType(TSMBuffer bufp, TSMLoc hdr_loc, string &resp_header_fields);
-static int getMaxAge(TSMBuffer bufp, TSMLoc hdr_loc);
 static bool getDefaultBucket(TSHttpTxn txnp, TSMBuffer bufp, TSMLoc hdr_obj, ClientRequest &creq);
 
-// libesi TLS key.
-pthread_key_t threadKey = 0;
-
 void
 TSPluginInit(int argc, const char *argv[])
 {
@@ -216,12 +348,43 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = "dev@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[combo_handler][%s] plugin registration failed.", __FUNCTION__);
+    TSError("[combo_handler][%s] plugin registration failed", __FUNCTION__);
     return;
   }
 
-  if ((argc > 1) && (strcmp(argv[1], "-") != 0)) {
-    COMBO_HANDLER_PATH = argv[1];
+  if (argc > 1) {
+    int c;
+    static const struct option longopts[] = {
+      {"max-files", required_argument, nullptr, 'f'},
+      {nullptr, 0, nullptr, 0},
+    };
+
+    int longindex = 0;
+    optind        = 1; // Force restart to avoid problems with other plugins.
+    while ((c = getopt_long(argc, const_cast<char *const *>(argv), "f:", longopts, &longindex)) != -1) {
+      switch (c) {
+      case 'f': {
+        char *tmp = nullptr;
+        long n    = strtol(optarg, &tmp, 0);
+        if (tmp == optarg) {
+          TSError("[%s] %s requires a numeric argument", DEBUG_TAG, longopts[longindex].name);
+        } else if (n < 1) {
+          TSError("[%s] %s must be a positive number", DEBUG_TAG, longopts[longindex].name);
+        } else {
+          MaxFileCount = n;
+          TSDebug(DEBUG_TAG, "Max files set to %u", MaxFileCount);
+        }
+        break;
+      }
+      default:
+        TSError("[%s] Unrecognized option '%s'", DEBUG_TAG, argv[optind - 1]);
+        break;
+      }
+    }
+  }
+
+  if (argc >= optind && (argv[optind][0] != '-' || argv[optind][1])) {
+    COMBO_HANDLER_PATH = argv[optind];
     if (COMBO_HANDLER_PATH == "/") {
       COMBO_HANDLER_PATH.clear();
     } else {
@@ -232,28 +395,40 @@ TSPluginInit(int argc, const char *argv[])
         COMBO_HANDLER_PATH.erase(COMBO_HANDLER_PATH.size() - 1, 1);
       }
     }
-  } else {
-    COMBO_HANDLER_PATH = DEFAULT_COMBO_HANDLER_PATH;
   }
-  COMBO_HANDLER_PATH_SIZE = static_cast<int>(COMBO_HANDLER_PATH.size());
-  LOG_DEBUG("Combo handler path is [%s]", COMBO_HANDLER_PATH.c_str());
+  ++optind;
+  LOG_DEBUG("Combo handler path is [%.*s]", length(COMBO_HANDLER_PATH), COMBO_HANDLER_PATH.data());
 
-  SIG_KEY_NAME = ((argc > 2) && (strcmp(argv[2], "-") != 0)) ? argv[2] : "";
-  LOG_DEBUG("Signature key is [%s]", SIG_KEY_NAME.c_str());
+  SIG_KEY_NAME = (argc > optind && (argv[optind][0] != '-' || argv[optind][1])) ? argv[optind] : "";
+  ++optind;
+  LOG_DEBUG("Signature key is [%.*s]", length(SIG_KEY_NAME), SIG_KEY_NAME.data());
 
-  if ((argc > 3) && (strcmp(argv[3], "-") != 0)) {
-    stringstream strstream(argv[3]);
+  if (argc > optind && (argv[optind][0] != '-' || argv[optind][1])) {
+    stringstream strstream(argv[optind++]);
     string header;
     while (getline(strstream, header, ':')) {
-      HEADER_WHITELIST.push_back(header);
+      HEADER_ALLOWLIST.push_back(header);
     }
   }
+  ++optind;
 
-  for (unsigned int i = 0; i < HEADER_WHITELIST.size(); i++) {
-    LOG_DEBUG("WhiteList: %s", HEADER_WHITELIST[i].c_str());
+  for (unsigned int i = 0; i < HEADER_ALLOWLIST.size(); i++) {
+    LOG_DEBUG("AllowList: %s", HEADER_ALLOWLIST[i].c_str());
   }
 
-  TSReleaseAssert(pthread_key_create(&threadKey, nullptr) == 0);
+  std::string content_type_allowlist_filespec = (argc > optind && (argv[optind][0] != '-' || argv[optind][1])) ? argv[optind] : "";
+  if (content_type_allowlist_filespec.empty()) {
+    LOG_DEBUG("No Content-Type allowlist file specified (all content types allowed)");
+  } else {
+    // If we have a path and it's not an absolute path, make it relative to the
+    // configuration directory.
+    if (content_type_allowlist_filespec[0] != '/') {
+      content_type_allowlist_filespec = std::string(TSConfigDirGet()) + '/' + content_type_allowlist_filespec;
+    }
+    LOG_DEBUG("Content-Type allowlist file: %s", content_type_allowlist_filespec.c_str());
+    ContentTypeHandler::loadAllowList(content_type_allowlist_filespec);
+  }
+  ++optind;
 
   TSCont rrh_contp = TSContCreate(handleReadRequestHeader, nullptr);
   if (!rrh_contp) {
@@ -263,11 +438,11 @@ TSPluginInit(int argc, const char *argv[])
 
   TSHttpHookAdd(TS_HTTP_OS_DNS_HOOK, rrh_contp);
 
-  if (TSHttpArgIndexReserve(DEBUG_TAG, "will save plugin-enable flag here", &arg_idx) != TS_SUCCESS) {
+  if (TSUserArgIndexReserve(TS_USER_ARGS_TXN, DEBUG_TAG, "will save plugin-enable flag here", &arg_idx) != TS_SUCCESS) {
     LOG_ERROR("failed to reserve private data slot");
     return;
   } else {
-    LOG_DEBUG("arg_idx: %d", arg_idx);
+    LOG_DEBUG("txn_arg_idx: %d", arg_idx);
   }
 
   Utils::init(&TSDebug, &TSError);
@@ -294,14 +469,14 @@ handleReadRequestHeader(TSCont /* contp ATS_UNUSED */, TSEvent event, void *edat
     return 0;
   }
 
-  if (1 != reinterpret_cast<intptr_t>(TSHttpTxnArgGet(txnp, arg_idx))) {
+  if (1 != reinterpret_cast<intptr_t>(TSUserArgGet(txnp, arg_idx))) {
     LOG_DEBUG("combo is disabled for this channel");
     TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
     return 0;
   }
 
   LOG_DEBUG("combo is enabled for this channel");
-  LOG_DEBUG("handling TS_EVENT_HTTP_OS_DNS event...");
+  LOG_DEBUG("handling TS_EVENT_HTTP_OS_DNS event");
 
   TSEvent reenable_to_event = TS_EVENT_HTTP_CONTINUE;
   TSMBuffer bufp;
@@ -362,8 +537,8 @@ isComboHandlerRequest(TSMBuffer bufp, TSMLoc hdr_loc, TSMLoc url_loc)
         LOG_ERROR("Could not get path from request URL");
         retval = false;
       } else {
-        retval =
-          (path_len == COMBO_HANDLER_PATH_SIZE) && (strncasecmp(path, COMBO_HANDLER_PATH.c_str(), COMBO_HANDLER_PATH_SIZE) == 0);
+        retval = (path_len == length(COMBO_HANDLER_PATH)) &&
+                 (strncasecmp(path, COMBO_HANDLER_PATH.data(), COMBO_HANDLER_PATH.size()) == 0);
         LOG_DEBUG("Path [%.*s] is %s combo handler path", path_len, path, (retval ? "a" : "not a"));
       }
     }
@@ -382,7 +557,7 @@ getDefaultBucket(TSHttpTxn /* txnp ATS_UNUSED */, TSMBuffer bufp, TSMLoc hdr_obj
 
   field_loc = TSMimeHdrFieldFind(bufp, hdr_obj, TS_MIME_FIELD_HOST, -1);
   if (field_loc == TS_NULL_MLOC) {
-    LOG_ERROR("Host field not found.");
+    LOG_ERROR("Host field not found");
     return false;
   }
 
@@ -397,18 +572,6 @@ getDefaultBucket(TSHttpTxn /* txnp ATS_UNUSED */, TSMBuffer bufp, TSMLoc hdr_obj
   creq.defaultBucket = string(host, host_len);
   defaultBucketFound = true;
 
-  /*
-  for(int i = 0 ; i < host_len; i++)
-    {
-      if (host[i] == '.')
-        {
-          creq.defaultBucket = string(host, i);
-          defaultBucketFound = true;
-          break;
-        }
-    }
-  */
-
   TSHandleMLocRelease(bufp, hdr_obj, field_loc);
 
   LOG_DEBUG("defaultBucket: %s", creq.defaultBucket.data());
@@ -475,7 +638,7 @@ parseQueryParameters(const char *query, int query_len, ClientRequest &creq)
               LOG_DEBUG("Signature [%.*s] on query [%.*s] is invalid", param_len - 4, param + 4, param_start_pos, query);
             }
           } else {
-            LOG_DEBUG("Verification not configured; ignoring signature...");
+            LOG_DEBUG("Verification not configured, ignoring signature");
           }
           break; // nothing useful after the signature
         }
@@ -545,7 +708,7 @@ parseQueryParameters(const char *query, int query_len, ClientRequest &creq)
     creq.file_urls.clear();
   }
 
-  if (creq.file_urls.size() > MAX_FILE_COUNT) {
+  if (creq.file_urls.size() > MaxFileCount) {
     creq.status = TS_HTTP_STATUS_BAD_REQUEST;
     LOG_ERROR("too many files in url");
     creq.file_urls.clear();
@@ -649,7 +812,7 @@ handleServerEvent(TSCont contp, TSEvent event, void *edata)
   }
 
   if (int_data->read_complete && int_data->write_complete) {
-    LOG_DEBUG("Completed request processing. Shutting down...");
+    LOG_DEBUG("Completed request processing, shutting down");
     delete int_data;
     TSContDestroy(contp);
   }
@@ -787,40 +950,34 @@ writeResponse(InterceptData &int_data)
 static void
 prepareResponse(InterceptData &int_data, ByteBlockList &body_blocks, string &resp_header_fields)
 {
-  bool got_content_type = false;
-
   if (int_data.creq.status == TS_HTTP_STATUS_OK) {
     HttpDataFetcherImpl::ResponseData resp_data;
     TSMLoc field_loc;
-    int max_age      = 0;
-    bool got_max_age = false;
     time_t expires_time;
     bool got_expires_time = false;
-    int num_headers       = HEADER_WHITELIST.size();
+    int num_headers       = HEADER_ALLOWLIST.size();
     int flags_list[num_headers];
+    CacheControlHeader cch;
 
     for (int i = 0; i < num_headers; i++) {
       flags_list[i] = 0;
     }
 
+    ContentTypeHandler cth(resp_header_fields);
+
     for (StringList::iterator iter = int_data.creq.file_urls.begin(); iter != int_data.creq.file_urls.end(); ++iter) {
       if (int_data.fetcher->getData(*iter, resp_data) && resp_data.status == TS_HTTP_STATUS_OK) {
         body_blocks.push_back(ByteBlock(resp_data.content, resp_data.content_len));
-        if (find(HEADER_WHITELIST.begin(), HEADER_WHITELIST.end(), TS_MIME_FIELD_CONTENT_TYPE) == HEADER_WHITELIST.end()) {
-          if (!got_content_type) {
-            got_content_type = getContentType(resp_data.bufp, resp_data.hdr_loc, resp_header_fields);
+        if (find(HEADER_ALLOWLIST.begin(), HEADER_ALLOWLIST.end(), TS_MIME_FIELD_CONTENT_TYPE) == HEADER_ALLOWLIST.end()) {
+          if (!cth.nextObjectHeader(resp_data.bufp, resp_data.hdr_loc)) {
+            LOG_ERROR("Content type missing or forbidden for requested URL [%s]", iter->c_str());
+            int_data.creq.status = TS_HTTP_STATUS_FORBIDDEN;
+            break;
           }
         }
 
-        int curr_field_max_age = getMaxAge(resp_data.bufp, resp_data.hdr_loc);
-        if (curr_field_max_age > 0) {
-          if (!got_max_age) {
-            max_age     = curr_field_max_age;
-            got_max_age = true;
-          } else if (curr_field_max_age < max_age) {
-            max_age = curr_field_max_age;
-          }
-        }
+        // Load this document's Cache-Control header into our managing object
+        cch.update(resp_data.bufp, resp_data.hdr_loc);
 
         field_loc = TSMimeHdrFieldFind(resp_data.bufp, resp_data.hdr_loc, TS_MIME_FIELD_EXPIRES, TS_MIME_LEN_EXPIRES);
         if (field_loc != TS_NULL_MLOC) {
@@ -843,7 +1000,7 @@ prepareResponse(InterceptData &int_data, ByteBlockList &body_blocks, string &res
             continue;
           }
 
-          const string &header = HEADER_WHITELIST[i];
+          const string &header = HEADER_ALLOWLIST[i];
 
           field_loc = TSMimeHdrFieldFind(resp_data.bufp, resp_data.hdr_loc, header.c_str(), header.size());
           if (field_loc != TS_NULL_MLOC) {
@@ -878,22 +1035,18 @@ prepareResponse(InterceptData &int_data, ByteBlockList &body_blocks, string &res
       }
     }
     if (int_data.creq.status == TS_HTTP_STATUS_OK) {
-      if (find(HEADER_WHITELIST.begin(), HEADER_WHITELIST.end(), TS_MIME_FIELD_CACHE_CONTROL) == HEADER_WHITELIST.end()) {
-        if (got_max_age && max_age > 0) {
-          char line_buf[128];
-          int line_size = sprintf(line_buf, "Cache-Control: max-age=%d, public\r\n", max_age);
-          resp_header_fields.append(line_buf, line_size);
-        } else {
-          resp_header_fields.append("Cache-Control: max-age=315360000, public\r\n"); // set 10-years max-age
-        }
+      // Add in Cache-Control header
+      if (find(HEADER_ALLOWLIST.begin(), HEADER_ALLOWLIST.end(), TS_MIME_FIELD_CACHE_CONTROL) == HEADER_ALLOWLIST.end()) {
+        resp_header_fields.append(cch.generate());
       }
-      if (find(HEADER_WHITELIST.begin(), HEADER_WHITELIST.end(), TS_MIME_FIELD_EXPIRES) == HEADER_WHITELIST.end()) {
+      if (find(HEADER_ALLOWLIST.begin(), HEADER_ALLOWLIST.end(), TS_MIME_FIELD_EXPIRES) == HEADER_ALLOWLIST.end()) {
         if (got_expires_time) {
           if (expires_time <= 0) {
             resp_header_fields.append("Expires: 0\r\n");
           } else {
             char line_buf[128];
-            int line_size = strftime(line_buf, 128, "Expires: %a, %d %b %Y %T GMT\r\n", gmtime(&expires_time));
+            struct tm gm_expires_time;
+            int line_size = strftime(line_buf, 128, "Expires: %a, %d %b %Y %T GMT\r\n", gmtime_r(&expires_time, &gm_expires_time));
             resp_header_fields.append(line_buf, line_size);
           }
         }
@@ -914,10 +1067,9 @@ prepareResponse(InterceptData &int_data, ByteBlockList &body_blocks, string &res
   }
 }
 
-static bool
-getContentType(TSMBuffer bufp, TSMLoc hdr_loc, string &resp_header_fields)
+bool
+ContentTypeHandler::nextObjectHeader(TSMBuffer bufp, TSMLoc hdr_loc)
 {
-  bool retval      = false;
   TSMLoc field_loc = TSMimeHdrFieldFind(bufp, hdr_loc, TS_MIME_FIELD_CONTENT_TYPE, TS_MIME_LEN_CONTENT_TYPE);
   if (field_loc != TS_NULL_MLOC) {
     bool values_added = false;
@@ -926,54 +1078,86 @@ getContentType(TSMBuffer bufp, TSMLoc hdr_loc, string &resp_header_fields)
     int n_values = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
     for (int i = 0; i < n_values; ++i) {
       value = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, i, &value_len);
-      if (!values_added) {
-        resp_header_fields.append("Content-Type: ");
-        values_added = true;
-      } else {
-        resp_header_fields.append(", ");
+      ts::TextView tv{value, value_len};
+      tv = tv.prefix(';').rtrim(std::string_view(" \t"));
+      if (_content_type_allowlist.empty()) {
+        ;
+      } else if (std::find_if(_content_type_allowlist.begin(), _content_type_allowlist.end(), [tv](ts::TextView tv2) -> bool {
+                   return strcasecmp(tv, tv2) == 0;
+                 }) == _content_type_allowlist.end()) {
+        return false;
+      } else if (tv.empty()) {
+        // allowlist is bad, contains an empty string.
+        return false;
+      }
+      if (!_added_content_type) {
+        if (!values_added) {
+          _resp_header_fields.append("Content-Type: ");
+          values_added = true;
+        } else {
+          _resp_header_fields.append(", ");
+        }
+        _resp_header_fields.append(value, value_len);
       }
-      resp_header_fields.append(value, value_len);
     }
     TSHandleMLocRelease(bufp, hdr_loc, field_loc);
     if (values_added) {
-      resp_header_fields.append("\r\n");
-      retval = true;
+      _resp_header_fields.append("\r\n");
+
+      // Assume that the Content-type field from the first header covers all the responses being combined.
+      _added_content_type = true;
     }
+    return true;
   }
-  return retval;
+  // No content type header field so doesn't pass allowlist if there is one.
+  return _content_type_allowlist.empty();
 }
 
-static int
-getMaxAge(TSMBuffer bufp, TSMLoc hdr_loc)
+void
+ContentTypeHandler::loadAllowList(std::string const &file_spec)
 {
-  int max_age = 0;
-  const char *value, *ptr;
-  int value_len = 0;
+  std::fstream fs;
+  char line_buffer[256];
+  bool extra_junk_on_line{false};
+  int line_num = 0;
 
-  TSMLoc field_loc = TSMimeHdrFieldFind(bufp, hdr_loc, TS_MIME_FIELD_CACHE_CONTROL, TS_MIME_LEN_CACHE_CONTROL);
-  if (field_loc != TS_NULL_MLOC) {
-    int n_values = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
-    if ((n_values != TS_ERROR) && (n_values > 0)) {
-      for (int i = 0; i < n_values; i++) {
-        value = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, i, &value_len);
-        ptr   = value;
-        if (strncmp(value, TS_HTTP_VALUE_MAX_AGE, TS_HTTP_LEN_MAX_AGE) == 0) {
-          ptr += TS_HTTP_LEN_MAX_AGE;
-          while ((*ptr == ' ') || (*ptr == '\t')) {
-            ptr++;
-          }
-          if (*ptr == '=') {
-            ptr++;
-            max_age = atoi(ptr);
-          }
-          break;
-        }
+  fs.open(file_spec);
+  if (fs.good()) {
+    for (;;) {
+      ++line_num;
+      fs.getline(line_buffer, sizeof(line_buffer));
+      if (!fs.good()) {
+        break;
       }
+      constexpr std::string_view bs{" \t"sv};
+      ts::TextView line{line_buffer, std::size_t(fs.gcount() - 1)};
+      line.ltrim(bs);
+      if (line.empty() || line[0] == '#') {
+        // Empty/comment line.
+        continue;
+      }
+      ts::TextView content_type{line.take_prefix_at(bs)};
+      line.trim(bs);
+      if (line.size() && (line[0] != '#')) {
+        extra_junk_on_line = true;
+        break;
+      }
+      _content_type_allowlist.emplace_back(content_type);
     }
-    TSHandleMLocRelease(bufp, hdr_loc, field_loc);
   }
-
-  return max_age;
+  if (fs.fail() && !(fs.eof() && (fs.gcount() == 0))) {
+    LOG_ERROR("Error reading Content-Type allowlist config file %s, line %d", file_spec.c_str(), line_num);
+  } else if (extra_junk_on_line) {
+    LOG_ERROR("More than one type on line %d in Content-Type allowlist config file %s", line_num, file_spec.c_str());
+  } else if (_content_type_allowlist.empty()) {
+    LOG_ERROR("Content-type allowlist config file %s must have at least one entry", file_spec.c_str());
+  } else {
+    // End of file.
+    return;
+  }
+  _content_type_allowlist.clear();
+  // An empty string marks object as bad.
+  _content_type_allowlist.emplace_back("");
 }
 
 static const char INVARIANT_FIELD_LINES[]    = {"Vary: Accept-Encoding\r\n"};
@@ -982,7 +1166,7 @@ static const char INVARIANT_FIELD_LINES_SIZE = sizeof(INVARIANT_FIELD_LINES) - 1
 static bool
 writeStandardHeaderFields(InterceptData &int_data, int &n_bytes_written)
 {
-  if (find(HEADER_WHITELIST.begin(), HEADER_WHITELIST.end(), TS_MIME_FIELD_VARY) == HEADER_WHITELIST.end()) {
+  if (find(HEADER_ALLOWLIST.begin(), HEADER_ALLOWLIST.end(), TS_MIME_FIELD_VARY) == HEADER_ALLOWLIST.end()) {
     if (TSIOBufferWrite(int_data.output.buffer, INVARIANT_FIELD_LINES, INVARIANT_FIELD_LINES_SIZE) == TS_ERROR) {
       LOG_ERROR("Error while writing invariant fields");
       return false;
@@ -990,10 +1174,12 @@ writeStandardHeaderFields(InterceptData &int_data, int &n_bytes_written)
     n_bytes_written += INVARIANT_FIELD_LINES_SIZE;
   }
 
-  if (find(HEADER_WHITELIST.begin(), HEADER_WHITELIST.end(), TS_MIME_FIELD_LAST_MODIFIED) == HEADER_WHITELIST.end()) {
+  if (find(HEADER_ALLOWLIST.begin(), HEADER_ALLOWLIST.end(), TS_MIME_FIELD_LAST_MODIFIED) == HEADER_ALLOWLIST.end()) {
     time_t time_now = static_cast<time_t>(TShrtime() / 1000000000); // it returns nanoseconds!
     char last_modified_line[128];
-    int last_modified_line_size = strftime(last_modified_line, 128, "Last-Modified: %a, %d %b %Y %T GMT\r\n", gmtime(&time_now));
+    struct tm gmnow;
+    int last_modified_line_size =
+      strftime(last_modified_line, 128, "Last-Modified: %a, %d %b %Y %T GMT\r\n", gmtime_r(&time_now, &gmnow));
     if (TSIOBufferWrite(int_data.output.buffer, last_modified_line, last_modified_line_size) == TS_ERROR) {
       LOG_ERROR("Error while writing last-modified fields");
       return false;
@@ -1030,8 +1216,8 @@ writeErrorResponse(InterceptData &int_data, int &n_bytes_written)
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 {
-  TSHttpTxnArgSet(rh, arg_idx, (void *)1); /* Save for later hooks */
-  return TSREMAP_NO_REMAP;                 /* Continue with next remap plugin in chain */
+  TSUserArgSet(rh, arg_idx, (void *)1); /* Save for later hooks */
+  return TSREMAP_NO_REMAP;              /* Continue with next remap plugin in chain */
 }
 
 /*
