@@ -30,7 +30,7 @@
 class TestHttpDataFetcher : public HttpDataFetcher
 {
 public:
-  TestHttpDataFetcher() : _n_pending_requests(0), _return_data(true) {}
+  TestHttpDataFetcher() {}
   bool
   addFetchRequest(const std::string &url, FetchedDataProcessor *callback_obj = nullptr)
   {
@@ -84,7 +84,7 @@ public:
   };
 
 private:
-  int _n_pending_requests;
+  int _n_pending_requests = 0;
   std::string _data;
-  bool _return_data;
+  bool _return_data = true;
 };
