@@ -46,8 +46,8 @@ read_request(TSHttpTxn txnp, Config *const config)
     if (!header.hasKey(SLICER_MIME_FIELD_INFO, SLICER_MIME_LEN_INFO)) {
       // check if any previous plugin has monkeyed with the transaction status
       TSHttpStatus const txnstat = TSHttpTxnStatusGet(txnp);
-      if (0 != static_cast<int>(txnstat)) {
-        DEBUG_LOG("txn status change detected (%d), skipping plugin\n", (int)txnstat);
+      if (TS_HTTP_STATUS_NONE != txnstat) {
+        DEBUG_LOG("txn status change detected (%d), skipping plugin\n", static_cast<int>(txnstat));
         return false;
       }
 
