@@ -16,6 +16,8 @@
  * limitations under the License.
  */
 
+#pragma once
+
 #include <ts/ts.h>
 #include <ts/remap.h>
 #include <cstring>
@@ -62,12 +64,12 @@ enum ExpansionField {
 };
 
 struct SslHdrExpansion {
-  SslHdrExpansion() : name(), scope(SSL_HEADERS_SCOPE_NONE), field(SSL_HEADERS_FIELD_NONE) {}
+  SslHdrExpansion() : name() {}
   std::string name; // HTTP header name
-  ExpansionScope scope;
-  ExpansionField field;
+  ExpansionScope scope = SSL_HEADERS_SCOPE_NONE;
+  ExpansionField field = SSL_HEADERS_FIELD_NONE;
 
-  // noncopyable but moveable
+  // noncopyable but movable
   SslHdrExpansion(const SslHdrExpansion &) = delete;
   SslHdrExpansion &operator=(const SslHdrExpansion &) = delete;
   SslHdrExpansion(SslHdrExpansion &&)                 = default;
@@ -81,11 +83,9 @@ struct SslHdrInstance {
   ~SslHdrInstance();
 
   expansion_list expansions;
-  AttachOptions attach;
+  AttachOptions attach = SSL_HEADERS_ATTACH_SERVER;
   TSCont cont;
 
-  void register_hooks();
-
   // noncopyable
   SslHdrInstance(const SslHdrInstance &) = delete;
   SslHdrInstance &operator=(const SslHdrInstance &) = delete;
