@@ -20,6 +20,8 @@
 #include <errno.h>
 #include <pthread.h>
 #include <getopt.h>
+#include <inttypes.h>
+#include <pthread.h>
 
 #include "ts_lua_util.h"
 
@@ -39,6 +41,9 @@ static uint64_t ts_lua_g_http_next_id = 0;
 static ts_lua_main_ctx *ts_lua_main_ctx_array   = NULL;
 static ts_lua_main_ctx *ts_lua_g_main_ctx_array = NULL;
 
+static pthread_key_t lua_g_state_key;
+static pthread_key_t lua_state_key;
+
 // records.config entry injected by plugin
 static char const *const ts_lua_mgmt_state_str   = "proxy.config.plugin.lua.max_states";
 static char const *const ts_lua_mgmt_state_regex = "^[1-9][0-9]*$";
@@ -128,7 +133,7 @@ create_lua_vms()
       ts_lua_max_state_count = TS_LUA_MAX_STATE_COUNT;
     } else {
       ts_lua_max_state_count = (int)mgmt_state;
-      TSDebug(TS_LUA_DEBUG_TAG, "[%s] found %s: [%d]", __FUNCTION__, ts_lua_mgmt_state_str, (int)ts_lua_max_state_count);
+      TSDebug(TS_LUA_DEBUG_TAG, "[%s] found %s: [%d]", __FUNCTION__, ts_lua_mgmt_state_str, ts_lua_max_state_count);
     }
 
     if (ts_lua_max_state_count < 1) {
@@ -150,7 +155,7 @@ create_lua_vms()
     return NULL;
   }
 
-  // Initalize the GC numbers, no need to lock here
+  // Initialize the GC numbers, no need to lock here
   for (int index = 0; index < ts_lua_max_state_count; ++index) {
     ts_lua_main_ctx *const main_ctx = (ctx_array + index);
     lua_State *const lstate         = main_ctx->lua;
@@ -206,7 +211,7 @@ statsHandler(TSCont contp, TSEvent event, void *edata)
   collectStats(plugin_stats);
   publishStats(plugin_stats);
 
-  TSContSchedule(contp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
+  TSContScheduleOnPool(contp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
 
   return TS_EVENT_NONE;
 }
@@ -306,6 +311,8 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
   if (NULL == ts_lua_main_ctx_array) {
     ts_lua_main_ctx_array = create_lua_vms();
     if (NULL != ts_lua_main_ctx_array) {
+      pthread_key_create(&lua_state_key, NULL);
+
       TSCont const lcontp = TSContCreate(lifecycleHandler, TSMutexCreate());
       TSContDataSet(lcontp, ts_lua_main_ctx_array);
       TSLifecycleHookAdd(TS_LIFECYCLE_MSG_HOOK, lcontp);
@@ -317,7 +324,7 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
         TSDebug(TS_LUA_DEBUG_TAG, "Starting up stats management continuation");
         TSCont const scontp = TSContCreate(statsHandler, TSMutexCreate());
         TSContDataSet(scontp, plugin_stats);
-        TSContSchedule(scontp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
+        TSContScheduleOnPool(scontp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
       }
     } else {
       return TS_ERROR;
@@ -474,9 +481,13 @@ ts_lua_remap_plugin_init(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 
   int remap     = (rri == NULL ? 0 : 1);
   instance_conf = (ts_lua_instance_conf *)ih;
-  req_id        = __sync_fetch_and_add(&ts_lua_http_next_id, 1);
 
-  main_ctx = &ts_lua_main_ctx_array[req_id % instance_conf->states];
+  main_ctx = pthread_getspecific(lua_state_key);
+  if (main_ctx == NULL) {
+    req_id   = __sync_fetch_and_add(&ts_lua_http_next_id, 1);
+    main_ctx = &ts_lua_main_ctx_array[req_id % instance_conf->states];
+    pthread_setspecific(lua_state_key, main_ctx);
+  }
 
   TSMutexLock(main_ctx->mutexp);
 
@@ -510,7 +521,7 @@ ts_lua_remap_plugin_init(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 
   ts_lua_set_cont_info(L, NULL);
   if (lua_pcall(L, 0, 1, 0) != 0) {
-    TSError("[ts_lua] lua_pcall failed: %s", lua_tostring(L, -1));
+    TSError("[ts_lua][%s] lua_pcall failed: %s", __FUNCTION__, lua_tostring(L, -1));
     ret = TSREMAP_NO_REMAP;
 
   } else {
@@ -555,73 +566,6 @@ configHandler(TSCont contp, TSEvent event ATS_UNUSED, void *edata ATS_UNUSED)
   return 0;
 }
 
-static int
-vconnHookHandler(TSCont contp, TSEvent event, void *edata)
-{
-  TSVConn vconn = (TSVConn)edata;
-
-  int ret;
-  uint64_t req_id;
-
-  lua_State *l;
-
-  ts_lua_main_ctx *main_ctx;
-  ts_lua_vconn_ctx *vconn_ctx;
-
-  ts_lua_instance_conf *conf = (ts_lua_instance_conf *)TSContDataGet(contp);
-
-  req_id = __sync_fetch_and_add(&ts_lua_g_http_next_id, 1);
-
-  main_ctx = &ts_lua_g_main_ctx_array[req_id % conf->states];
-
-  TSDebug(TS_LUA_DEBUG_TAG, "[%s] req_id: %" PRId64, __FUNCTION__, req_id);
-
-  TSMutexLock(main_ctx->mutexp);
-
-  vconn_ctx        = ts_lua_create_vconn_ctx(main_ctx, conf);
-  vconn_ctx->vconn = vconn;
-  l                = vconn_ctx->lua;
-
-  switch (event) {
-  case TS_EVENT_VCONN_START:
-    lua_getglobal(l, TS_LUA_FUNCTION_G_VCONN_START);
-    break;
-
-  default:
-    ts_lua_destroy_vconn_ctx(vconn_ctx);
-    TSMutexUnlock(main_ctx->mutexp);
-    TSVConnReenable(vconn);
-    return 0;
-  }
-
-  if (lua_type(l, -1) != LUA_TFUNCTION) {
-    lua_pop(l, 1);
-    ts_lua_destroy_vconn_ctx(vconn_ctx);
-    TSMutexUnlock(main_ctx->mutexp);
-
-    TSVConnReenable(vconn);
-    return 0;
-  }
-
-  if (lua_pcall(l, 0, 1, 0) != 0) {
-    TSError("[ts_lua][%s] lua_pcall failed: %s", __FUNCTION__, lua_tostring(l, -1));
-  }
-
-  ret = lua_tointeger(l, -1);
-  lua_pop(l, 1);
-
-  ts_lua_destroy_vconn_ctx(vconn_ctx);
-
-  TSMutexUnlock(main_ctx->mutexp);
-  if (ret) {
-    TSError("[ts_lua][%s] error returned", __FUNCTION__);
-  } else {
-    TSDebug(TS_LUA_DEBUG_TAG, "[%s] no error returned", __FUNCTION__);
-  }
-  TSVConnReenable(vconn);
-  return 0;
-}
-
 static int
 globalHookHandler(TSCont contp, TSEvent event ATS_UNUSED, void *edata)
 {
@@ -643,11 +587,14 @@ globalHookHandler(TSCont contp, TSEvent event ATS_UNUSED, void *edata)
 
   ts_lua_instance_conf *conf = (ts_lua_instance_conf *)TSContDataGet(contp);
 
-  req_id = __sync_fetch_and_add(&ts_lua_g_http_next_id, 1);
+  main_ctx = pthread_getspecific(lua_g_state_key);
+  if (main_ctx == NULL) {
+    req_id = __sync_fetch_and_add(&ts_lua_g_http_next_id, 1);
+    TSDebug(TS_LUA_DEBUG_TAG, "[%s] req_id: %" PRId64, __FUNCTION__, req_id);
+    main_ctx = &ts_lua_g_main_ctx_array[req_id % conf->states];
+    pthread_setspecific(lua_g_state_key, main_ctx);
+  }
 
-  main_ctx = &ts_lua_g_main_ctx_array[req_id % conf->states];
-
-  TSDebug(TS_LUA_DEBUG_TAG, "[%s] req_id: %" PRId64, __FUNCTION__, req_id);
   TSMutexLock(main_ctx->mutexp);
 
   http_ctx           = ts_lua_create_http_ctx(main_ctx, conf);
@@ -702,6 +649,7 @@ globalHookHandler(TSCont contp, TSEvent event ATS_UNUSED, void *edata)
     // to allow API(s) to fetch the pointers again when it re-enters the hook
     if (http_ctx->client_response_hdrp != NULL) {
       TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
+      http_ctx->client_response_bufp = NULL;
       http_ctx->client_response_hdrp = NULL;
     }
     lua_getglobal(l, TS_LUA_FUNCTION_G_SEND_RESPONSE);
@@ -754,12 +702,21 @@ globalHookHandler(TSCont contp, TSEvent event ATS_UNUSED, void *edata)
   ts_lua_set_cont_info(l, NULL);
 
   if (lua_pcall(l, 0, 1, 0) != 0) {
-    TSError("[ts_lua] lua_pcall failed: %s", lua_tostring(l, -1));
+    TSError("[ts_lua][%s] lua_pcall failed: %s", __FUNCTION__, lua_tostring(l, -1));
   }
 
   ret = lua_tointeger(l, -1);
   lua_pop(l, 1);
 
+  // client response can be changed within a transaction
+  // (e.g. due to the follow redirect feature). So, clearing the pointers
+  // to allow API(s) to fetch the pointers again when it re-enters the hook
+  if (http_ctx->client_response_hdrp != NULL) {
+    TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
+    http_ctx->client_response_bufp = NULL;
+    http_ctx->client_response_hdrp = NULL;
+  }
+
   if (http_ctx->has_hook) {
     // add a hook to release resources for context
     TSDebug(TS_LUA_DEBUG_TAG, "[%s] has txn hook -> adding txn close hook handler to release resources", __FUNCTION__);
@@ -790,12 +747,14 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = "dev@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[ts_lua] Plugin registration failed");
+    TSError("[ts_lua][%s] Plugin registration failed", __FUNCTION__);
   }
 
   if (NULL == ts_lua_g_main_ctx_array) {
     ts_lua_g_main_ctx_array = create_lua_vms();
     if (NULL != ts_lua_g_main_ctx_array) {
+      pthread_key_create(&lua_g_state_key, NULL);
+
       TSCont const contp = TSContCreate(lifecycleHandler, TSMutexCreate());
       TSContDataSet(contp, ts_lua_g_main_ctx_array);
       TSLifecycleHookAdd(TS_LIFECYCLE_MSG_HOOK, contp);
@@ -805,7 +764,7 @@ TSPluginInit(int argc, const char *argv[])
       if (NULL != plugin_stats) {
         TSCont const scontp = TSContCreate(statsHandler, TSMutexCreate());
         TSContDataSet(scontp, plugin_stats);
-        TSContSchedule(scontp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
+        TSContScheduleOnPool(scontp, TS_LUA_STATS_TIMEOUT, TS_THREAD_POOL_TASK);
       }
     } else {
       return;
@@ -975,26 +934,6 @@ TSPluginInit(int argc, const char *argv[])
 
   ts_lua_destroy_http_ctx(http_ctx);
 
-  TSCont vconn_contp = TSContCreate(vconnHookHandler, NULL);
-  if (!vconn_contp) {
-    TSError("[ts_lua][%s] could not create vconn continuation", __FUNCTION__);
-    return;
-  }
-  TSContDataSet(vconn_contp, conf);
-
-  // adding hook based on whther the lua global vconn function exists
-  ts_lua_vconn_ctx *vconn_ctx = ts_lua_create_vconn_ctx(main_ctx, conf);
-  lua_State *vl               = vconn_ctx->lua;
-
-  lua_getglobal(vl, TS_LUA_FUNCTION_G_VCONN_START);
-  if (lua_type(vl, -1) == LUA_TFUNCTION) {
-    TSHttpHookAdd(TS_VCONN_START_HOOK, vconn_contp);
-    TSDebug(TS_LUA_DEBUG_TAG, "vconn_start_hook added");
-  }
-  lua_pop(vl, 1);
-
-  ts_lua_destroy_vconn_ctx(vconn_ctx);
-
   // support for reload as global plugin
   if (reload) {
     TSCont config_contp = TSContCreate(configHandler, NULL);
