@@ -16,13 +16,12 @@
   limitations under the License.
 */
 
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include "ts_lua_util.h"
 #include "ts_lua_http_intercept.h"
 #include "ts_lua_http_config.h"
 #include "ts_lua_http_cntl.h"
 #include "ts_lua_http_milestone.h"
+#include "ts_lua_http_txn_info.h"
 
 typedef enum {
   TS_LUA_CACHE_LOOKUP_MISS,
@@ -73,6 +72,7 @@ static int ts_lua_http_set_cache_lookup_status(lua_State *L);
 static int ts_lua_http_set_cache_url(lua_State *L);
 static int ts_lua_http_get_cache_lookup_url(lua_State *L);
 static int ts_lua_http_set_cache_lookup_url(lua_State *L);
+static int ts_lua_http_redo_cache_lookup(lua_State *L);
 static int ts_lua_http_get_parent_proxy(lua_State *L);
 static int ts_lua_http_set_parent_proxy(lua_State *L);
 static int ts_lua_http_get_parent_selection_url(lua_State *L);
@@ -85,6 +85,7 @@ static int ts_lua_http_resp_cache_transformed(lua_State *L);
 static int ts_lua_http_resp_cache_untransformed(lua_State *L);
 
 static int ts_lua_http_get_client_protocol_stack(lua_State *L);
+static int ts_lua_http_get_server_protocol_stack(lua_State *L);
 static int ts_lua_http_server_push(lua_State *L);
 static int ts_lua_http_is_websocket(lua_State *L);
 static int ts_lua_http_get_plugin_tag(lua_State *L);
@@ -94,6 +95,7 @@ static int ts_lua_http_is_internal_request(lua_State *L);
 static int ts_lua_http_is_aborted(lua_State *L);
 static int ts_lua_http_skip_remapping_set(lua_State *L);
 static int ts_lua_http_transaction_count(lua_State *L);
+static int ts_lua_http_server_transaction_count(lua_State *L);
 static int ts_lua_http_redirect_url_set(lua_State *L);
 static int ts_lua_http_get_server_state(lua_State *L);
 
@@ -103,13 +105,6 @@ static int ts_lua_http_get_remap_to_url(lua_State *L);
 static int ts_lua_http_get_server_fd(lua_State *L);
 static int ts_lua_http_get_client_fd(lua_State *L);
 
-static int ts_lua_http_get_client_received_error(lua_State *L);
-static int ts_lua_http_get_client_sent_error(lua_State *L);
-static int ts_lua_http_get_server_received_error(lua_State *L);
-static int ts_lua_http_get_server_sent_error(lua_State *L);
-
-static int ts_lua_http_get_ssn_remote_addr(lua_State *L);
-
 static void ts_lua_inject_server_state_variables(lua_State *L);
 
 static void ts_lua_inject_http_resp_transform_api(lua_State *L);
@@ -130,6 +125,7 @@ ts_lua_inject_http_api(lua_State *L)
   ts_lua_inject_http_config_api(L);
   ts_lua_inject_http_cntl_api(L);
   ts_lua_inject_http_milestone_api(L);
+  ts_lua_inject_txn_info_api(L);
   ts_lua_inject_http_misc_api(L);
 
   lua_setfield(L, -2, "http");
@@ -166,6 +162,9 @@ ts_lua_inject_http_cache_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_http_set_cache_lookup_url);
   lua_setfield(L, -2, "set_cache_lookup_url");
 
+  lua_pushcfunction(L, ts_lua_http_redo_cache_lookup);
+  lua_setfield(L, -2, "redo_cache_lookup");
+
   lua_pushcfunction(L, ts_lua_http_get_parent_proxy);
   lua_setfield(L, -2, "get_parent_proxy");
 
@@ -221,6 +220,9 @@ ts_lua_inject_http_misc_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_http_get_client_protocol_stack);
   lua_setfield(L, -2, "get_client_protocol_stack");
 
+  lua_pushcfunction(L, ts_lua_http_get_server_protocol_stack);
+  lua_setfield(L, -2, "get_server_protocol_stack");
+
   lua_pushcfunction(L, ts_lua_http_server_push);
   lua_setfield(L, -2, "server_push");
 
@@ -248,6 +250,9 @@ ts_lua_inject_http_misc_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_http_transaction_count);
   lua_setfield(L, -2, "transaction_count");
 
+  lua_pushcfunction(L, ts_lua_http_server_transaction_count);
+  lua_setfield(L, -2, "server_transaction_count");
+
   lua_pushcfunction(L, ts_lua_http_redirect_url_set);
   lua_setfield(L, -2, "redirect_url_set");
 
@@ -266,21 +271,6 @@ ts_lua_inject_http_misc_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_http_get_client_fd);
   lua_setfield(L, -2, "get_client_fd");
 
-  lua_pushcfunction(L, ts_lua_http_get_client_received_error);
-  lua_setfield(L, -2, "get_client_received_error");
-
-  lua_pushcfunction(L, ts_lua_http_get_client_sent_error);
-  lua_setfield(L, -2, "get_client_sent_error");
-
-  lua_pushcfunction(L, ts_lua_http_get_server_received_error);
-  lua_setfield(L, -2, "get_server_received_error");
-
-  lua_pushcfunction(L, ts_lua_http_get_server_sent_error);
-  lua_setfield(L, -2, "get_server_sent_error");
-
-  lua_pushcfunction(L, ts_lua_http_get_ssn_remote_addr);
-  lua_setfield(L, -2, "get_ssn_remote_addr");
-
   ts_lua_inject_server_state_variables(L);
 }
 
@@ -448,7 +438,28 @@ ts_lua_http_set_cache_lookup_url(lua_State *L)
         TSHttpTxnCacheLookupUrlSet(http_ctx->txnp, http_ctx->client_request_bufp, new_url_loc) == TS_SUCCESS) {
       TSDebug(TS_LUA_DEBUG_TAG, "Set cache lookup URL");
     } else {
-      TSError("[ts_lua] Failed to set cache lookup URL");
+      TSError("[ts_lua][%s] Failed to set cache lookup URL", __FUNCTION__);
+    }
+  }
+
+  return 0;
+}
+
+static int
+ts_lua_http_redo_cache_lookup(lua_State *L)
+{
+  const char *url;
+  size_t url_len;
+
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  url = luaL_checklstring(L, 1, &url_len);
+
+  if (url && url_len) {
+    if (TSHttpTxnRedoCacheLookup(http_ctx->txnp, url, url_len) != TS_SUCCESS) {
+      TSError("[ts_lua][%s] Failed to redo cache lookup", __FUNCTION__);
     }
   }
 
@@ -560,7 +571,7 @@ ts_lua_http_set_parent_selection_url(lua_State *L)
         TSHttpTxnParentSelectionUrlSet(http_ctx->txnp, http_ctx->client_request_bufp, new_url_loc) == TS_SUCCESS) {
       TSDebug(TS_LUA_DEBUG_TAG, "Set parent selection URL");
     } else {
-      TSError("[ts_lua] Failed to set parent selection URL");
+      TSError("[ts_lua][%s] Failed to set parent selection URL", __FUNCTION__);
     }
   }
 
@@ -581,7 +592,7 @@ ts_lua_http_set_cache_url(lua_State *L)
 
   if (url && url_len) {
     if (TSCacheUrlSet(http_ctx->txnp, url, url_len) != TS_SUCCESS) {
-      TSError("[ts_lua] Failed to set cache url");
+      TSError("[ts_lua][%s] Failed to set cache url", __FUNCTION__);
     }
   }
 
@@ -651,6 +662,23 @@ ts_lua_http_get_client_protocol_stack(lua_State *L)
   return count;
 }
 
+static int
+ts_lua_http_get_server_protocol_stack(lua_State *L)
+{
+  char const *results[10];
+  int count = 0;
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  TSHttpTxnServerProtocolStackGet(http_ctx->txnp, 10, results, &count);
+  for (int i = 0; i < count; i++) {
+    lua_pushstring(L, results[i]);
+  }
+
+  return count;
+}
+
 static int
 ts_lua_http_server_push(lua_State *L)
 {
@@ -782,6 +810,19 @@ ts_lua_http_transaction_count(lua_State *L)
   return 1;
 }
 
+static int
+ts_lua_http_server_transaction_count(lua_State *L)
+{
+  ts_lua_http_ctx *http_ctx;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  int n = TSHttpTxnServerSsnTransactionCount(http_ctx->txnp);
+  lua_pushnumber(L, n);
+
+  return 1;
+}
+
 static int
 ts_lua_http_redirect_url_set(lua_State *L)
 {
@@ -900,109 +941,6 @@ ts_lua_http_get_client_fd(lua_State *L)
   return 1;
 }
 
-static int
-ts_lua_http_get_client_received_error(lua_State *L)
-{
-  uint32_t class = 0;
-  uint64_t code  = 0;
-  ts_lua_http_ctx *http_ctx;
-
-  GET_HTTP_CONTEXT(http_ctx, L);
-
-  TSHttpTxnClientReceivedErrorGet(http_ctx->txnp, &class, &code);
-  lua_pushnumber(L, class);
-  lua_pushnumber(L, code);
-
-  return 2;
-}
-
-static int
-ts_lua_http_get_client_sent_error(lua_State *L)
-{
-  uint32_t class = 0;
-  uint64_t code  = 0;
-  ts_lua_http_ctx *http_ctx;
-
-  GET_HTTP_CONTEXT(http_ctx, L);
-
-  TSHttpTxnClientSentErrorGet(http_ctx->txnp, &class, &code);
-  lua_pushnumber(L, class);
-  lua_pushnumber(L, code);
-
-  return 2;
-}
-
-static int
-ts_lua_http_get_server_received_error(lua_State *L)
-{
-  uint32_t class = 0;
-  uint64_t code  = 0;
-  ts_lua_http_ctx *http_ctx;
-
-  GET_HTTP_CONTEXT(http_ctx, L);
-
-  TSHttpTxnServerReceivedErrorGet(http_ctx->txnp, &class, &code);
-  lua_pushnumber(L, class);
-  lua_pushnumber(L, code);
-
-  return 2;
-}
-
-static int
-ts_lua_http_get_server_sent_error(lua_State *L)
-{
-  uint32_t class = 0;
-  uint64_t code  = 0;
-  ts_lua_http_ctx *http_ctx;
-
-  GET_HTTP_CONTEXT(http_ctx, L);
-
-  TSHttpTxnServerSentErrorGet(http_ctx->txnp, &class, &code);
-  lua_pushnumber(L, class);
-  lua_pushnumber(L, code);
-
-  return 2;
-}
-
-static int
-ts_lua_http_get_ssn_remote_addr(lua_State *L)
-{
-  struct sockaddr const *client_ip;
-  ts_lua_http_ctx *http_ctx;
-  int port;
-  int family;
-  char cip[128];
-
-  GET_HTTP_CONTEXT(http_ctx, L);
-
-  TSHttpSsn ssn = TSHttpTxnSsnGet(http_ctx->txnp);
-  TSVConn vconn = TSHttpSsnClientVConnGet(ssn);
-  client_ip     = TSNetVConnRemoteAddrGet(vconn);
-
-  if (client_ip == NULL) {
-    lua_pushnil(L);
-    lua_pushnil(L);
-    lua_pushnil(L);
-
-  } else {
-    if (client_ip->sa_family == AF_INET) {
-      port = ntohs(((struct sockaddr_in *)client_ip)->sin_port);
-      inet_ntop(AF_INET, (const void *)&((struct sockaddr_in *)client_ip)->sin_addr, cip, sizeof(cip));
-      family = AF_INET;
-    } else {
-      port = ntohs(((struct sockaddr_in6 *)client_ip)->sin6_port);
-      inet_ntop(AF_INET6, (const void *)&((struct sockaddr_in6 *)client_ip)->sin6_addr, cip, sizeof(cip));
-      family = AF_INET6;
-    }
-
-    lua_pushstring(L, cip);
-    lua_pushnumber(L, port);
-    lua_pushnumber(L, family);
-  }
-
-  return 3;
-}
-
 static int
 ts_lua_http_resp_transform_get_upstream_bytes(lua_State *L)
 {
@@ -1010,8 +948,7 @@ ts_lua_http_resp_transform_get_upstream_bytes(lua_State *L)
 
   transform_ctx = ts_lua_get_http_transform_ctx(L);
   if (transform_ctx == NULL) {
-    TSError("[ts_lua] missing transform_ctx");
-    TSReleaseAssert(!"Unexpected fetch of transform_ctx");
+    TSError("[ts_lua][%s] missing transform_ctx", __FUNCTION__);
     return 0;
   }
 
@@ -1027,8 +964,7 @@ ts_lua_http_resp_transform_get_upstream_watermark_bytes(lua_State *L)
 
   transform_ctx = ts_lua_get_http_transform_ctx(L);
   if (transform_ctx == NULL) {
-    TSError("[ts_lua] missing transform_ctx");
-    TSReleaseAssert(!"Unexpected fetch of transform_ctx");
+    TSError("[ts_lua][%s] missing transform_ctx", __FUNCTION__);
     return 0;
   }
 
@@ -1045,8 +981,7 @@ ts_lua_http_resp_transform_set_upstream_watermark_bytes(lua_State *L)
 
   transform_ctx = ts_lua_get_http_transform_ctx(L);
   if (transform_ctx == NULL) {
-    TSError("[ts_lua] missing transform_ctx");
-    TSReleaseAssert(!"Unexpected fetch of transform_ctx");
+    TSError("[ts_lua][%s] missing transform_ctx", __FUNCTION__);
     return 0;
   }
 
@@ -1065,8 +1000,7 @@ ts_lua_http_resp_transform_set_downstream_bytes(lua_State *L)
 
   transform_ctx = ts_lua_get_http_transform_ctx(L);
   if (transform_ctx == NULL) {
-    TSError("[ts_lua] missing transform_ctx");
-    TSReleaseAssert(!"Unexpected fetch of transform_ctx");
+    TSError("[ts_lua][%s] missing transform_ctx", __FUNCTION__);
     return 0;
   }
 
