@@ -26,7 +26,6 @@
 #include "ts_lua_cached_response.h"
 #include "ts_lua_context.h"
 #include "ts_lua_hook.h"
-#include "ts_lua_vconn.h"
 #include "ts_lua_http.h"
 #include "ts_lua_misc.h"
 #include "ts_lua_log.h"
@@ -43,6 +42,20 @@ static void ts_lua_init_globals(lua_State *L);
 static void ts_lua_inject_ts_api(lua_State *L);
 static ts_lua_ctx_stats *ts_lua_create_ctx_stats();
 static void ts_lua_destroy_ctx_stats(ts_lua_ctx_stats *stats);
+static void ts_lua_update_server_response_hdrp(ts_lua_http_ctx *http_ctx);
+
+/**
+   Update http_ctx->server_response_hdrp if there.
+
+   This is required in the beginning of toughing response, because holding old pointer could be freed by core.
+ */
+void
+ts_lua_update_server_response_hdrp(ts_lua_http_ctx *http_ctx)
+{
+  if (http_ctx->server_response_hdrp) {
+    TSHttpTxnServerRespGet(http_ctx->txnp, &http_ctx->server_response_bufp, &http_ctx->server_response_hdrp);
+  }
+}
 
 int
 ts_lua_create_vm(ts_lua_main_ctx *arr, int n)
@@ -455,8 +468,6 @@ ts_lua_inject_ts_api(lua_State *L)
   ts_lua_inject_context_api(L);
   ts_lua_inject_hook_api(L);
 
-  ts_lua_inject_vconn_api(L);
-
   ts_lua_inject_http_api(L);
   ts_lua_inject_intercept_api(L);
   ts_lua_inject_misc_api(L);
@@ -579,95 +590,6 @@ ts_lua_destroy_async_ctx(ts_lua_http_ctx *http_ctx)
   TSfree(http_ctx);
 }
 
-void
-ts_lua_set_vconn_ctx(lua_State *L, ts_lua_vconn_ctx *ctx)
-{
-  lua_pushliteral(L, "__ts_vconn_ctx");
-  lua_pushlightuserdata(L, ctx);
-  lua_rawset(L, LUA_GLOBALSINDEX);
-}
-
-ts_lua_vconn_ctx *
-ts_lua_get_vconn_ctx(lua_State *L)
-{
-  ts_lua_vconn_ctx *ctx;
-
-  lua_pushliteral(L, "__ts_vconn_ctx");
-  lua_rawget(L, LUA_GLOBALSINDEX);
-  ctx = lua_touserdata(L, -1);
-
-  lua_pop(L, 1); // pop the ctx out
-
-  return ctx;
-}
-
-ts_lua_vconn_ctx *
-ts_lua_create_vconn_ctx(ts_lua_main_ctx *main_ctx, ts_lua_instance_conf *conf)
-{
-  ts_lua_vconn_ctx *vconn_ctx;
-
-  vconn_ctx = TSmalloc(sizeof(ts_lua_vconn_ctx));
-  memset(vconn_ctx, 0, sizeof(*vconn_ctx));
-
-  lua_State *L = main_ctx->lua;
-  lua_State *l = lua_newthread(L);
-
-  lua_pushlightuserdata(L, conf);
-  lua_rawget(L, LUA_REGISTRYINDEX);
-
-  /* new globals table for coroutine */
-  lua_newtable(l);
-  lua_pushvalue(l, -1);
-  lua_setfield(l, -2, "_G");
-  lua_newtable(l);
-  lua_xmove(L, l, 1);
-  lua_setfield(l, -2, "__index");
-  lua_setmetatable(l, -2);
-
-  lua_replace(l, LUA_GLOBALSINDEX);
-
-  // init coroutine
-  vconn_ctx->ref = luaL_ref(L, LUA_REGISTRYINDEX);
-
-  vconn_ctx->lua  = l;
-  vconn_ctx->mctx = main_ctx;
-
-  // update thread stats
-  ts_lua_ctx_stats *const stats = main_ctx->stats;
-
-  TSMutexLock(stats->mutexp);
-  ++stats->threads;
-  if (stats->threads_max < stats->threads) {
-    stats->threads_max = stats->threads;
-  }
-  TSMutexUnlock(stats->mutexp);
-
-  vconn_ctx->instance_conf = conf;
-
-  ts_lua_set_vconn_ctx(l, vconn_ctx);
-  ts_lua_create_context_table(l);
-
-  return vconn_ctx;
-}
-
-void
-ts_lua_destroy_vconn_ctx(ts_lua_vconn_ctx *vconn_ctx)
-{
-  // update thread stats
-  ts_lua_main_ctx *const main_ctx = vconn_ctx->mctx;
-  ts_lua_ctx_stats *const stats   = main_ctx->stats;
-
-  TSMutexLock(stats->mutexp);
-  --stats->threads;
-  TSMutexUnlock(stats->mutexp);
-
-  if (vconn_ctx->lua) {
-    luaL_unref(vconn_ctx->lua, LUA_REGISTRYINDEX, vconn_ctx->ref);
-  }
-
-  TSfree(vconn_ctx);
-}
-
 void
 ts_lua_set_http_ctx(lua_State *L, ts_lua_http_ctx *ctx)
 {
@@ -1005,6 +927,7 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
     break;
 
   case TS_EVENT_HTTP_READ_RESPONSE_HDR:
+    ts_lua_update_server_response_hdrp(http_ctx);
 
     lua_getglobal(L, TS_LUA_FUNCTION_READ_RESPONSE);
 
@@ -1021,6 +944,7 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
     // to allow API(s) to fetch the pointers again when it re-enters the hook
     if (http_ctx->client_response_hdrp != NULL) {
       TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
+      http_ctx->client_response_bufp = NULL;
       http_ctx->client_response_hdrp = NULL;
     }
 
@@ -1030,6 +954,12 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    if (http_ctx->client_response_hdrp != NULL) {
+      TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
+      http_ctx->client_response_bufp = NULL;
+      http_ctx->client_response_hdrp = NULL;
+    }
+
     break;
 
   case TS_EVENT_HTTP_READ_REQUEST_HDR:
@@ -1078,10 +1008,12 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
     break;
 
   case TS_EVENT_HTTP_TXN_CLOSE:
+    ts_lua_update_server_response_hdrp(http_ctx);
+
     lua_getglobal(L, TS_LUA_FUNCTION_TXN_CLOSE);
     if (lua_type(L, -1) == LUA_TFUNCTION) {
       if (lua_pcall(L, 0, 1, 0)) {
-        TSError("[ts_lua] lua_pcall failed: %s", lua_tostring(L, -1));
+        TSError("[ts_lua][%s] lua_pcall failed: %s", __FUNCTION__, lua_tostring(L, -1));
       }
     }
 
@@ -1110,7 +1042,7 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
     break;
 
   default: // coroutine failed
-    TSError("[ts_lua] lua_resume failed: %s", lua_tostring(L, -1));
+    TSError("[ts_lua][%s] lua_resume failed: %s", __FUNCTION__, lua_tostring(L, -1));
     rc = -1;
     lua_pop(L, 1);
     break;
