@@ -93,13 +93,9 @@ uriEncode(const String &in, bool isObjectName)
     } else if (isObjectName && i == '/') {
       /* Encode the forward slash character, '/', everywhere except in the object key name. */
       result << "/";
-    } else if (i == '+') {
-      /* Only written in the example code, but a plus sign is treated as a space regardless of the position and it must be encoded
-       * as "%20" instead of "%2B" */
-      result << "%20";
     } else {
       /* Letters in the hexadecimal value must be upper-case, for example "%1A". */
-      result << "%" << std::uppercase << std::setfill('0') << std::setw(2) << std::hex << (int)i;
+      result << "%" << std::uppercase << std::setfill('0') << std::setw(2) << std::hex << static_cast<int>(i);
     }
   }
 
@@ -264,9 +260,9 @@ getPayloadSha256(bool signPayload)
   }
 
   unsigned char payloadHash[SHA256_DIGEST_LENGTH];
-  SHA256((const unsigned char *)"", 0, payloadHash); /* empty content */
+  SHA256(reinterpret_cast<const unsigned char *>(""), 0, payloadHash); /* empty content */
 
-  return base16Encode((char *)payloadHash, SHA256_DIGEST_LENGTH);
+  return base16Encode(reinterpret_cast<char *>(payloadHash), SHA256_DIGEST_LENGTH);
 }
 
 /**
@@ -307,11 +303,6 @@ getCanonicalRequestSha256Hash(TsInterface &api, bool signPayload, const StringSe
   str = api.getPath(&length);
   String path("/");
   path.append(str, length);
-  str = api.getParams(&length);
-  if (length > 0) {
-    path.append(";", 1);
-    path.append(str, length);
-  }
   String canonicalUri = canonicalEncode(path, /* isObjectName */ true);
   sha256Update(&canonicalRequestSha256Ctx, canonicalUri);
   sha256Update(&canonicalRequestSha256Ctx, "\n");
@@ -429,7 +420,7 @@ getCanonicalRequestSha256Hash(TsInterface &api, bool signPayload, const StringSe
 #ifdef AWS_AUTH_V4_DETAILED_DEBUG_OUTPUT
   std::cout << "</CanonicalRequest>" << std::endl;
 #endif
-  return base16Encode((char *)canonicalRequestSha256Hash, SHA256_DIGEST_LENGTH);
+  return base16Encode(reinterpret_cast<char *>(canonicalRequestSha256Hash), SHA256_DIGEST_LENGTH);
 }
 
 /**
@@ -676,10 +667,10 @@ getSignature(const char *awsSecret, size_t awsSecretLen, const char *awsRegion,
       HMAC(EVP_sha256(), dateKey, dateKeyLen, (unsigned char *)awsRegion, awsRegionLen, dateRegionKey, &dateRegionKeyLen) &&
       HMAC(EVP_sha256(), dateRegionKey, dateRegionKeyLen, (unsigned char *)awsService, awsServiceLen, dateRegionServiceKey,
            &dateRegionServiceKeyLen) &&
-      HMAC(EVP_sha256(), dateRegionServiceKey, dateRegionServiceKeyLen, (unsigned char *)"aws4_request", 12, signingKey,
-           &signingKeyLen) &&
-      HMAC(EVP_sha256(), signingKey, signingKeyLen, (unsigned char *)stringToSign, stringToSignLen, (unsigned char *)signature,
-           &len)) {
+      HMAC(EVP_sha256(), dateRegionServiceKey, dateRegionServiceKeyLen, reinterpret_cast<const unsigned char *>("aws4_request"), 12,
+           signingKey, &signingKeyLen) &&
+      HMAC(EVP_sha256(), signingKey, signingKeyLen, (unsigned char *)stringToSign, stringToSignLen,
+           reinterpret_cast<unsigned char *>(signature), &len)) {
     return len;
   }
 
