@@ -37,12 +37,16 @@
 #include <openssl/sha.h>
 #include <openssl/hmac.h>
 
+#include <chrono>
+#include <atomic>
+#include <thread>
+#include <mutex>
+#include <shared_mutex>
+
 #include <ts/ts.h>
 #include <ts/remap.h>
 #include "tscore/ink_config.h"
 
-// Special snowflake here, only availbale when building inside the ATS source tree.
-#include "tscore/ink_atomic.h"
 #include "aws_auth_v4.h"
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -78,7 +82,6 @@ loadRegionMap(StringMap &m, const String &filename)
 
   std::ifstream ifstr;
   String line;
-  unsigned lineno = 0;
 
   ifstr.open(path.c_str());
   if (!ifstr) {
@@ -93,8 +96,6 @@ loadRegionMap(StringMap &m, const String &filename)
   while (std::getline(ifstr, line)) {
     String::size_type pos;
 
-    ++lineno;
-
     // Allow #-prefixed comments.
     pos = line.find_first_of('#');
     if (pos != String::npos) {
@@ -137,7 +138,7 @@ loadRegionMap(StringMap &m, const String &filename)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// Cache for the secrets file, to avoid reading / loding them repeatedly on
+// Cache for the secrets file, to avoid reading / loading them repeatedly on
 // a reload of remap.config. This gets cached for 60s (not configurable).
 //
 class S3Config;
@@ -148,7 +149,31 @@ public:
   S3Config *get(const char *fname);
 
 private:
-  std::unordered_map<std::string, std::pair<S3Config *, int>> _cache;
+  struct _ConfigData {
+    // This is incremented before and after cnf and load_time are set.
+    // Thus, an odd value indicates an update is in progress.
+    std::atomic<unsigned> update_status{0};
+
+    // A config from a file and the last time it was loaded.
+    // config should be written before load_time.  That way,
+    // if config is read after load_time, the load time will
+    // never indicate config is fresh when it isn't.
+    std::atomic<S3Config *> config;
+    std::atomic<time_t> load_time;
+
+    _ConfigData() {}
+
+    _ConfigData(S3Config *config_, time_t load_time_) : config(config_), load_time(load_time_) {}
+
+    _ConfigData(_ConfigData &&lhs)
+    {
+      update_status = lhs.update_status.load();
+      config        = lhs.config.load();
+      load_time     = lhs.load_time.load();
+    }
+  };
+
+  std::unordered_map<std::string, _ConfigData> _cache;
   static const int _ttl = 60;
 };
 
@@ -158,6 +183,7 @@ ConfigCache gConfCache;
 // One configuration setup
 //
 int event_handler(TSCont, TSEvent, void *); // Forward declaration
+int config_reloader(TSCont, TSEvent, void *);
 
 class S3Config
 {
@@ -167,14 +193,25 @@ public:
     if (get_cont) {
       _cont = TSContCreate(event_handler, nullptr);
       TSContDataSet(_cont, static_cast<void *>(this));
+
+      _conf_rld = TSContCreate(config_reloader, TSMutexCreate());
+      TSContDataSet(_conf_rld, static_cast<void *>(this));
     }
   }
 
   ~S3Config()
   {
-    _secret_len = _keyid_len = 0;
+    _secret_len = _keyid_len = _token_len = 0;
     TSfree(_secret);
     TSfree(_keyid);
+    TSfree(_token);
+    TSfree(_conf_fname);
+    if (_conf_rld_act) {
+      TSActionCancel(_conf_rld_act);
+    }
+    if (_conf_rld) {
+      TSContDestroy(_conf_rld);
+    }
     if (_cont) {
       TSContDestroy(_cont);
     }
@@ -192,13 +229,16 @@ public:
     /* Optional parameters, issue warning if v2 parameters are used with v4 and vice-versa (wrong parameters are ignored anyways) */
     if (2 == _version) {
       if (_v4includeHeaders_modified && !_v4includeHeaders.empty()) {
-        TSError("[%s] headers are not being signed with AWS auth v2, included headers parameter ignored", PLUGIN_NAME);
+        TSDebug("[%s] headers are not being signed with AWS auth v2, included headers parameter ignored", PLUGIN_NAME);
       }
       if (_v4excludeHeaders_modified && !_v4excludeHeaders.empty()) {
-        TSError("[%s] headers are not being signed with AWS auth v2, excluded headers parameter ignored", PLUGIN_NAME);
+        TSDebug("[%s] headers are not being signed with AWS auth v2, excluded headers parameter ignored", PLUGIN_NAME);
       }
       if (_region_map_modified && !_region_map.empty()) {
-        TSError("[%s] region map is not used with AWS auth v2, parameter ignored", PLUGIN_NAME);
+        TSDebug("[%s] region map is not used with AWS auth v2, parameter ignored", PLUGIN_NAME);
+      }
+      if (nullptr != _token || _token_len > 0) {
+        TSDebug("[%s] session token support with AWS auth v2 is not implemented, parameter ignored", PLUGIN_NAME);
       }
     } else {
       /* 4 == _version */
@@ -207,37 +247,29 @@ public:
     return true;
   }
 
-  void
-  acquire()
-  {
-    ink_atomic_increment(&_ref_count, 1);
-  }
-
-  void
-  release()
-  {
-    TSDebug(PLUGIN_NAME, "ref_count is %d", _ref_count);
-    if (1 >= ink_atomic_decrement(&_ref_count, 1)) {
-      TSDebug(PLUGIN_NAME, "configuration deleted, due to ref-counting");
-      delete this;
-    }
-  }
-
   // Used to copy relevant configurations that can be configured in a config file. Note: we intentionally
   // don't override/use the assignment operator, since we only copy things IF they have been modified.
   void
   copy_changes_from(const S3Config *src)
   {
     if (src->_secret) {
+      TSfree(_secret);
       _secret     = TSstrdup(src->_secret);
       _secret_len = src->_secret_len;
     }
 
     if (src->_keyid) {
+      TSfree(_keyid);
       _keyid     = TSstrdup(src->_keyid);
       _keyid_len = src->_keyid_len;
     }
 
+    if (src->_token) {
+      TSfree(_token);
+      _token     = TSstrdup(src->_token);
+      _token_len = src->_token_len;
+    }
+
     if (src->_version_modified) {
       _version          = src->_version;
       _version_modified = true;
@@ -262,6 +294,13 @@ public:
       _region_map          = src->_region_map;
       _region_map_modified = true;
     }
+
+    _expiration = src->_expiration;
+
+    if (src->_conf_fname) {
+      TSfree(_conf_fname);
+      _conf_fname = TSstrdup(src->_conf_fname);
+    }
   }
 
   // Getters
@@ -283,6 +322,12 @@ public:
     return _keyid;
   }
 
+  const char *
+  token() const
+  {
+    return _token;
+  }
+
   int
   secret_len() const
   {
@@ -295,6 +340,12 @@ public:
     return _keyid_len;
   }
 
+  int
+  token_len() const
+  {
+    return _token_len;
+  }
+
   int
   version() const
   {
@@ -319,6 +370,24 @@ public:
     return _region_map;
   }
 
+  long
+  expiration() const
+  {
+    return _expiration;
+  }
+
+  const char *
+  conf_fname() const
+  {
+    return _conf_fname;
+  }
+
+  int
+  incr_conf_reload_count()
+  {
+    return _conf_reload_count++;
+  }
+
   // Setters
   void
   set_secret(const char *s)
@@ -335,6 +404,13 @@ public:
     _keyid_len = strlen(s);
   }
   void
+  set_token(const char *s)
+  {
+    TSfree(_token);
+    _token     = TSstrdup(s);
+    _token_len = strlen(s);
+  }
+  void
   set_virt_host(bool f = true)
   {
     _virt_host          = f;
@@ -373,6 +449,25 @@ public:
     _region_map_modified = true;
   }
 
+  void
+  set_expiration(const char *s)
+  {
+    _expiration = strtol(s, nullptr, 10);
+  }
+
+  void
+  set_conf_fname(const char *s)
+  {
+    TSfree(_conf_fname);
+    _conf_fname = TSstrdup(s);
+  }
+
+  void
+  reset_conf_reload_count()
+  {
+    _conf_reload_count = 0;
+  }
+
   // Parse configs from an external file
   bool parse_config(const std::string &filename);
 
@@ -382,26 +477,43 @@ public:
   schedule(TSHttpTxn txnp) const
   {
     TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_REQUEST_HDR_HOOK, _cont);
-    TSHttpTxnHookAdd(txnp, TS_HTTP_TXN_CLOSE_HOOK, _cont); // To release the config lease
   }
 
+  void
+  schedule_conf_reload(long delay)
+  {
+    if (_conf_rld_act != nullptr && !TSActionDone(_conf_rld_act)) {
+      TSActionCancel(_conf_rld_act);
+    }
+    _conf_rld_act = TSContScheduleOnPool(_conf_rld, delay * 1000, TS_THREAD_POOL_NET);
+  }
+
+  std::shared_mutex reload_mutex;
+  std::atomic_bool reload_waiting = false;
+
 private:
   char *_secret            = nullptr;
   size_t _secret_len       = 0;
   char *_keyid             = nullptr;
   size_t _keyid_len        = 0;
+  char *_token             = nullptr;
+  size_t _token_len        = 0;
   bool _virt_host          = false;
   int _version             = 2;
   bool _version_modified   = false;
   bool _virt_host_modified = false;
   TSCont _cont             = nullptr;
-  int _ref_count           = 1;
+  TSCont _conf_rld         = nullptr;
+  TSAction _conf_rld_act   = nullptr;
   StringSet _v4includeHeaders;
   bool _v4includeHeaders_modified = false;
   StringSet _v4excludeHeaders;
   bool _v4excludeHeaders_modified = false;
   StringMap _region_map;
   bool _region_map_modified = false;
+  long _expiration          = 0;
+  char *_conf_fname         = nullptr;
+  int _conf_reload_count    = 0;
 };
 
 bool
@@ -431,7 +543,7 @@ S3Config::parse_config(const std::string &config_fname)
         continue;
       }
 
-      // Skip trailig white spaces
+      // Skip trailing white spaces
       pos2 = pos1;
       pos1 = pos2 + strlen(pos2) - 1;
       while ((pos1 > pos2) && isspace(*pos1)) {
@@ -446,6 +558,8 @@ S3Config::parse_config(const std::string &config_fname)
         set_secret(pos2 + 11);
       } else if (0 == strncasecmp(pos2, "access_key=", 11)) {
         set_keyid(pos2 + 11);
+      } else if (0 == strncasecmp(pos2, "session_token=", 14)) {
+        set_token(pos2 + 14);
       } else if (0 == strncasecmp(pos2, "version=", 8)) {
         set_version(pos2 + 8);
       } else if (0 == strncasecmp(pos2, "virtual_host", 12)) {
@@ -456,6 +570,8 @@ S3Config::parse_config(const std::string &config_fname)
         set_exclude_headers(pos2 + 19);
       } else if (0 == strncasecmp(pos2, "v4-region-map=", 14)) {
         set_region_map(pos2 + 14);
+      } else if (0 == strncasecmp(pos2, "expiration=", 11)) {
+        set_expiration(pos2 + 11);
       } else {
         // ToDo: warnings?
       }
@@ -477,6 +593,8 @@ S3Config::parse_config(const std::string &config_fname)
 S3Config *
 ConfigCache::get(const char *fname)
 {
+  S3Config *s3;
+
   struct timeval tv;
 
   gettimeofday(&tv, nullptr);
@@ -487,44 +605,56 @@ ConfigCache::get(const char *fname)
   auto it = _cache.find(config_fname);
 
   if (it != _cache.end()) {
-    if (tv.tv_sec > (it->second.second + _ttl)) {
-      // Update the cached configuration file.
-      S3Config *s3 = new S3Config(false); // false == this config does not get the continuation
+    unsigned update_status = it->second.update_status;
+    if (tv.tv_sec > (it->second.load_time + _ttl)) {
+      if (!(update_status & 1) && it->second.update_status.compare_exchange_strong(update_status, update_status + 1)) {
+        TSDebug(PLUGIN_NAME, "Configuration from %s is stale, reloading", config_fname.c_str());
+        s3 = new S3Config(false); // false == this config does not get the continuation
 
-      TSDebug(PLUGIN_NAME, "Configuration from %s is stale, reloading", config_fname.c_str());
-      it->second.second = tv.tv_sec;
-      if (nullptr != it->second.first) {
-        // The previous config update / reload attempt did not fail, safe to call release.
-        it->second.first->release();
-      }
-      if (s3->parse_config(config_fname)) {
-        it->second.first = s3;
+        if (s3->parse_config(config_fname)) {
+          s3->set_conf_fname(fname);
+        } else {
+          // Failed the configuration parse... Set the cache response to nullptr
+          delete s3;
+          s3 = nullptr;
+          TSAssert(!"Configuration parsing / caching failed");
+        }
+
+        delete it->second.config;
+        it->second.config    = s3;
+        it->second.load_time = tv.tv_sec;
+
+        // Update is complete.
+        ++it->second.update_status;
       } else {
-        // Failed the configuration parse... Set the cache response to nullptr
-        s3->release();
-        it->second.first = nullptr;
+        // This thread lost the race with another thread that is also reloading
+        // the config for this file. Wait for the other thread to finish reloading.
+        while (it->second.update_status & 1) {
+          // Hopefully yielding will sleep the thread at least until the next
+          // scheduler interrupt, preventing a busy wait.
+          std::this_thread::yield();
+        }
+        s3 = it->second.config;
       }
     } else {
       TSDebug(PLUGIN_NAME, "Configuration from %s is fresh, reusing", config_fname.c_str());
+      s3 = it->second.config;
     }
-    return it->second.first;
   } else {
     // Create a new cached file.
-    S3Config *s3 = new S3Config(false); // false == this config does not get the continuation
+    s3 = new S3Config(false); // false == this config does not get the continuation
 
+    TSDebug(PLUGIN_NAME, "Parsing and caching configuration from %s, version:%d", config_fname.c_str(), s3->version());
     if (s3->parse_config(config_fname)) {
-      _cache[config_fname] = std::make_pair(s3, tv.tv_sec);
-      TSDebug(PLUGIN_NAME, "Parsing and caching configuration from %s, version:%d", config_fname.c_str(), s3->version());
+      s3->set_conf_fname(fname);
+      _cache.emplace(config_fname, _ConfigData(s3, tv.tv_sec));
     } else {
-      s3->release();
-      return nullptr;
+      delete s3;
+      s3 = nullptr;
+      TSAssert(!"Configuration parsing / caching failed");
     }
-
-    return s3;
   }
-
-  TSAssert(!"Configuration parsing / caching failed");
-  return nullptr;
+  return s3;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -592,6 +722,7 @@ S3Request::set_header(const char *header, int header_len, const char *val, int v
     bool first = true;
 
     while (field_loc) {
+      tmp = TSMimeHdrFieldNextDup(_bufp, _hdr_loc, field_loc);
       if (first) {
         first = false;
         if (TS_SUCCESS == TSMimeHdrFieldValueStringSet(_bufp, _hdr_loc, field_loc, -1, val, val_len)) {
@@ -600,7 +731,6 @@ S3Request::set_header(const char *header, int header_len, const char *val, int v
       } else {
         TSMimeHdrFieldDestroy(_bufp, _hdr_loc, field_loc);
       }
-      tmp = TSMimeHdrFieldNextDup(_bufp, _hdr_loc, field_loc);
       TSHandleMLocRelease(_bufp, _hdr_loc, field_loc);
       field_loc = tmp;
     }
@@ -613,7 +743,7 @@ S3Request::set_header(const char *header, int header_len, const char *val, int v
   return ret;
 }
 
-// dst poinsts to starting offset of dst buffer
+// dst points to starting offset of dst buffer
 // dst_len remaining space in buffer
 static size_t
 str_concat(char *dst, size_t dst_len, const char *src, size_t src_len)
@@ -664,6 +794,12 @@ S3Request::authorizeV4(S3Config *s3)
     return TS_HTTP_STATUS_INTERNAL_SERVER_ERROR;
   }
 
+  /* set X-Amz-Security-Token if we have a token */
+  if (nullptr != s3->token() && '\0' != *(s3->token()) &&
+      !set_header(X_AMZ_SECURITY_TOKEN.data(), X_AMZ_SECURITY_TOKEN.size(), s3->token(), s3->token_len())) {
+    return TS_HTTP_STATUS_INTERNAL_SERVER_ERROR;
+  }
+
   String auth = util.getAuthorizationHeader();
   if (auth.empty()) {
     return TS_HTTP_STATUS_INTERNAL_SERVER_ERROR;
@@ -805,22 +941,22 @@ S3Request::authorizeV2(S3Config *s3)
 #endif
   HMAC_Init_ex(ctx, s3->secret(), s3->secret_len(), EVP_sha1(), nullptr);
   HMAC_Update(ctx, (unsigned char *)method, method_len);
-  HMAC_Update(ctx, (unsigned char *)"\n", 1);
+  HMAC_Update(ctx, reinterpret_cast<const unsigned char *>("\n"), 1);
   HMAC_Update(ctx, (unsigned char *)con_md5, con_md5_len);
-  HMAC_Update(ctx, (unsigned char *)"\n", 1);
+  HMAC_Update(ctx, reinterpret_cast<const unsigned char *>("\n"), 1);
   HMAC_Update(ctx, (unsigned char *)con_type, con_type_len);
-  HMAC_Update(ctx, (unsigned char *)"\n", 1);
-  HMAC_Update(ctx, (unsigned char *)date, date_len);
-  HMAC_Update(ctx, (unsigned char *)"\n/", 2);
+  HMAC_Update(ctx, reinterpret_cast<const unsigned char *>("\n"), 1);
+  HMAC_Update(ctx, reinterpret_cast<unsigned char *>(date), date_len);
+  HMAC_Update(ctx, reinterpret_cast<const unsigned char *>("\n/"), 2);
 
   if (host && host_endp) {
     HMAC_Update(ctx, (unsigned char *)host, host_endp - host);
-    HMAC_Update(ctx, (unsigned char *)"/", 1);
+    HMAC_Update(ctx, reinterpret_cast<const unsigned char *>("/"), 1);
   }
 
   HMAC_Update(ctx, (unsigned char *)path, path_len);
   if (param) {
-    HMAC_Update(ctx, (unsigned char *)";", 1); // TSUrlHttpParamsGet() does not include ';'
+    HMAC_Update(ctx, reinterpret_cast<const unsigned char *>(";"), 1); // TSUrlHttpParamsGet() does not include ';'
     HMAC_Update(ctx, (unsigned char *)param, param_len);
   }
 
@@ -832,7 +968,7 @@ S3Request::authorizeV2(S3Config *s3)
 #endif
 
   // Do the Base64 encoding and set the Authorization header.
-  if (TS_SUCCESS == TSBase64Encode((const char *)hmac, hmac_len, hmac_b64, sizeof(hmac_b64) - 1, &hmac_b64_len)) {
+  if (TS_SUCCESS == TSBase64Encode(reinterpret_cast<const char *>(hmac), hmac_len, hmac_b64, sizeof(hmac_b64) - 1, &hmac_b64_len)) {
     char auth[256]; // This is way bigger than any string we can think of.
     int auth_len = snprintf(auth, sizeof(auth), "AWS %s:%.*s", s3->keyid(), static_cast<int>(hmac_b64_len), hmac_b64);
 
@@ -855,40 +991,103 @@ S3Request::authorizeV2(S3Config *s3)
 int
 event_handler(TSCont cont, TSEvent event, void *edata)
 {
-  TSHttpTxn txnp = static_cast<TSHttpTxn>(edata);
-  S3Config *s3   = static_cast<S3Config *>(TSContDataGet(cont));
-
-  S3Request request(txnp);
-  TSHttpStatus status  = TS_HTTP_STATUS_INTERNAL_SERVER_ERROR;
+  TSHttpTxn txnp       = static_cast<TSHttpTxn>(edata);
+  S3Config *s3         = static_cast<S3Config *>(TSContDataGet(cont));
   TSEvent enable_event = TS_EVENT_HTTP_CONTINUE;
 
-  switch (event) {
-  case TS_EVENT_HTTP_SEND_REQUEST_HDR:
-    if (request.initialize()) {
-      status = request.authorize(s3);
-    }
+  {
+    S3Request request(txnp);
+    TSHttpStatus status = TS_HTTP_STATUS_INTERNAL_SERVER_ERROR;
 
-    if (TS_HTTP_STATUS_OK == status) {
-      TSDebug(PLUGIN_NAME, "Succesfully signed the AWS S3 URL");
-    } else {
-      TSDebug(PLUGIN_NAME, "Failed to sign the AWS S3 URL, status = %d", status);
-      TSHttpTxnStatusSet(txnp, status);
-      enable_event = TS_EVENT_HTTP_ERROR;
+    switch (event) {
+    case TS_EVENT_HTTP_SEND_REQUEST_HDR:
+      if (request.initialize()) {
+        while (s3->reload_waiting) {
+          std::this_thread::yield();
+        }
+
+        std::shared_lock lock(s3->reload_mutex);
+        status = request.authorize(s3);
+      }
+
+      if (TS_HTTP_STATUS_OK == status) {
+        TSDebug(PLUGIN_NAME, "Successfully signed the AWS S3 URL");
+      } else {
+        TSDebug(PLUGIN_NAME, "Failed to sign the AWS S3 URL, status = %d", status);
+        TSHttpTxnStatusSet(txnp, status);
+        enable_event = TS_EVENT_HTTP_ERROR;
+      }
+      break;
+    default:
+      TSError("[%s] Unknown event for this plugin", PLUGIN_NAME);
+      TSDebug(PLUGIN_NAME, "unknown event for this plugin");
+      break;
     }
-    break;
-  case TS_EVENT_HTTP_TXN_CLOSE:
-    s3->release(); // Release the configuration lease when txn closes
-    break;
-  default:
-    TSError("[%s] Unknown event for this plugin", PLUGIN_NAME);
-    TSDebug(PLUGIN_NAME, "unknown event for this plugin");
-    break;
+    // Most get S3Request out of scope in case the later plugins invalidate the TSAPI
+    // objects it references.  Some cases were causing asserts from the destructor
   }
 
   TSHttpTxnReenable(txnp, enable_event);
   return 0;
 }
 
+// If the token has more than one hour to expire, reload is scheduled one hour before expiration.
+// If the token has less than one hour to expire, reload is scheduled 15 minutes before expiration.
+// If the token has less than 15 minutes to expire, reload is scheduled at the expiration time.
+static long
+cal_reload_delay(long time_diff)
+{
+  if (time_diff > 3600) {
+    return time_diff - 3600;
+  } else if (time_diff > 900) {
+    return time_diff - 900;
+  } else {
+    return time_diff;
+  }
+}
+
+int
+config_reloader(TSCont cont, TSEvent event, void *edata)
+{
+  TSDebug(PLUGIN_NAME, "reloading configs");
+  S3Config *s3          = static_cast<S3Config *>(TSContDataGet(cont));
+  S3Config *file_config = gConfCache.get(s3->conf_fname());
+
+  if (!file_config || !file_config->valid()) {
+    TSError("[%s] requires both shared and AWS secret configuration", PLUGIN_NAME);
+    return TS_ERROR;
+  }
+
+  s3->reload_waiting = true;
+  {
+    std::unique_lock lock(s3->reload_mutex);
+    s3->copy_changes_from(file_config);
+  }
+  s3->reload_waiting = false;
+
+  if (s3->expiration() == 0) {
+    TSDebug(PLUGIN_NAME, "disabling auto config reload");
+  } else {
+    // auto reload is scheduled to be 5 minutes before the expiration time to get some headroom
+    long time_diff = s3->expiration() -
+                     std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now().time_since_epoch()).count();
+    if (time_diff > 0) {
+      long delay = cal_reload_delay(time_diff);
+      TSDebug(PLUGIN_NAME, "scheduling config reload with %ld seconds delay", delay);
+      s3->reset_conf_reload_count();
+      s3->schedule_conf_reload(delay);
+    } else {
+      TSDebug(PLUGIN_NAME, "config expiration time is in the past, re-checking in 1 minute");
+      if (s3->incr_conf_reload_count() == 10) {
+        TSError("[%s] tried to reload config automatically but failed, please try manual reloading the config", PLUGIN_NAME);
+      }
+      s3->schedule_conf_reload(60);
+    }
+  }
+
+  return TS_SUCCESS;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // Initialize the plugin.
 //
@@ -925,6 +1124,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     {const_cast<char *>("v4-include-headers"), required_argument, nullptr, 'i'},
     {const_cast<char *>("v4-exclude-headers"), required_argument, nullptr, 'e'},
     {const_cast<char *>("v4-region-map"), required_argument, nullptr, 'm'},
+    {const_cast<char *>("session_token"), required_argument, nullptr, 't'},
     {nullptr, no_argument, nullptr, '\0'},
   };
 
@@ -945,7 +1145,6 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
       if (!file_config) {
         TSError("[%s] invalid configuration file, %s", PLUGIN_NAME, optarg);
         *ih = nullptr;
-        s3->release();
         return TS_ERROR;
       }
       break;
@@ -955,6 +1154,9 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     case 's':
       s3->set_secret(optarg);
       break;
+    case 't':
+      s3->set_token(optarg);
+      break;
     case 'h':
       s3->set_virt_host();
       break;
@@ -985,15 +1187,30 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
   // Make sure we got both the shared secret and the AWS secret
   if (!s3->valid()) {
     TSError("[%s] requires both shared and AWS secret configuration", PLUGIN_NAME);
-    s3->release();
     *ih = nullptr;
     return TS_ERROR;
   }
 
-  // Note that we don't acquire() the s3 config, it's implicit that we hold at least one ref
+  if (s3->expiration() == 0) {
+    TSDebug(PLUGIN_NAME, "disabling auto config reload");
+  } else {
+    // auto reload is scheduled to be 5 minutes before the expiration time to get some headroom
+    long time_diff = s3->expiration() -
+                     std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now().time_since_epoch()).count();
+    if (time_diff > 0) {
+      long delay = cal_reload_delay(time_diff);
+      TSDebug(PLUGIN_NAME, "scheduling config reload with %ld seconds delay", delay);
+      s3->reset_conf_reload_count();
+      s3->schedule_conf_reload(delay);
+    } else {
+      TSDebug(PLUGIN_NAME, "config expiration time is in the past, re-checking in 1 minute");
+      s3->schedule_conf_reload(60);
+    }
+  }
+
   *ih = static_cast<void *>(s3);
-  TSDebug(PLUGIN_NAME, "New rule: secret_key=%s, access_key=%s, virtual_host=%s, version=%d", s3->secret(), s3->keyid(),
-          s3->virt_host() ? "yes" : "no", s3->version());
+  TSDebug(PLUGIN_NAME, "New rule: access_key=%s, virtual_host=%s, version=%d", s3->keyid(), s3->virt_host() ? "yes" : "no",
+          s3->version());
 
   return TS_SUCCESS;
 }
@@ -1002,8 +1219,7 @@ void
 TSRemapDeleteInstance(void *ih)
 {
   S3Config *s3 = static_cast<S3Config *>(ih);
-
-  s3->release();
+  delete s3;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1016,7 +1232,6 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
 
   if (s3) {
     TSAssert(s3->valid());
-    s3->acquire(); // Increasement ref-count
     // Now schedule the continuation to update the URL when going to origin.
     // Note that in most cases, this is a No-Op, assuming you have reasonable
     // cache hit ratio. However, the scheduling is next to free (very cheap).
