@@ -24,6 +24,8 @@
 #pragma once
 
 #include <sys/time.h>
+#include <string_view>
+#include <string>
 
 #include "tscore/ink_assert.h"
 #include "tscore/ink_apidefs.h"
@@ -32,6 +34,8 @@
 #include "HdrHeap.h"
 #include "HdrToken.h"
 
+#include "tscpp/util/TextView.h"
+
 /***********************************************************************
  *                                                                     *
  *                              Defines                                *
@@ -57,9 +61,6 @@ enum MimeParseState {
   MIME_PARSE_AFTER,    ///< After a field.
 };
 
-#define MIME_SCANNER_TYPE_LINE 0
-#define MIME_SCANNER_TYPE_FIELD 1
-
 /***********************************************************************
  *                                                                     *
  *                              Assertions                             *
@@ -116,7 +117,7 @@ struct MIMEField {
   bool
   is_cooked()
   {
-    return (m_flags & MIME_FIELD_SLOT_FLAGS_COOKED);
+    return (m_flags & MIME_FIELD_SLOT_FLAGS_COOKED) ? true : false;
   }
 
   bool
@@ -134,12 +135,14 @@ struct MIMEField {
   bool
   supports_commas() const
   {
-    if (m_wks_idx >= 0)
-      return (hdrtoken_index_to_flags(m_wks_idx) & MIME_FLAGS_COMMAS);
-    else
-      return true; // by default, assume supports commas
+    if (m_wks_idx >= 0) {
+      return (hdrtoken_index_to_flags(m_wks_idx) & HTIF_COMMAS);
+    }
+    return true; // by default, assume supports commas
   }
 
+  /// @return The name of @a this field.
+  std::string_view name_get() const;
   const char *name_get(int *length) const;
 
   /** Find the index of the value in the multi-value field.
@@ -156,7 +159,10 @@ struct MIMEField {
   */
   int value_get_index(const char *value, int length) const;
 
+  /// @return The value of @a this field.
+  std::string_view value_get() const;
   const char *value_get(int *length) const;
+
   int32_t value_get_int() const;
   uint32_t value_get_uint() const;
   int64_t value_get_int64() const;
@@ -277,13 +283,76 @@ struct MIMEHdrImpl : public HdrHeapObjImpl {
  *                                                                     *
  ***********************************************************************/
 
+/** A pre-parser used to extract MIME "lines" from raw input for further parsing.
+ *
+ * This maintains an internal line buffer which is used to keeping content between calls
+ * when the parse has not yet completed.
+ *
+ */
 struct MIMEScanner {
-  char *m_line;           // buffered line being built up
-  int m_line_length;      // size of real live data in buffer
-  int m_line_size;        // total allocated size of buffer
-  MimeParseState m_state; ///< Parsing machine state.
+  using self_type = MIMEScanner; ///< Self reference type.
+public:
+  /// Type of input scanning.
+  enum ScanType {
+    LINE  = 0, ///< Scan a single line.
+    FIELD = 1, ///< Scan with line folding enabled.
+  };
+
+  void init();  ///< Pseudo-constructor required by proxy allocation.
+  void clear(); ///< Pseudo-destructor required by proxy allocation.
+
+  /// @return The size of the internal line buffer.
+  size_t get_buffered_line_size() const;
+
+  /** Scan @a input for MIME data delimited by CR/LF end of line markers.
+   *
+   * @param input [in,out] Text to scan.
+   * @param output [out] Parsed text from @a input, if any.
+   * @param output_shares_input [out] Whether @a output is in @a input.
+   * @param eof_p [in] The source for @a input is done, no more data will ever be available.
+   * @param scan_type [in] Whether to check for line folding.
+   * @return The result of scanning.
+   *
+   * @a input is updated to remove text that was scanned. @a output is updated to be a view of the
+   * scanned @a input. This is separate because @a output may be a view of @a input or a view of the
+   * internal line buffer. Which of these cases obtains is returned in @a output_shares_input. This
+   * is @c true if @a output is a view of @a input, and @c false if @a output is a view of the
+   * internal buffer, but is only set if the result is not @c PARSE_RESULT_CONT (that is, it is not
+   * set until scanning has completed). If @a scan_type is @c FIELD then folded lines are
+   * accumulated in to a single line stored in the internal buffer. Otherwise the scanning
+   * terminates at the first CR/LF.
+   */
+  ParseResult get(ts::TextView &input, ts::TextView &output, bool &output_shares_input, bool eof_p, ScanType scan_type);
+
+protected:
+  /** Append @a text to the internal buffer.
+   *
+   * @param text Text to append.
+   * @return @a this
+   *
+   * A copy of @a text is appended to the internal line buffer.
+   */
+  self_type &append(ts::TextView text);
+
+  static constexpr MimeParseState INITIAL_PARSE_STATE = MIME_PARSE_BEFORE;
+  std::string m_line;                          ///< Internally buffered line data for field coalescence.
+  MimeParseState m_state{INITIAL_PARSE_STATE}; ///< Parsing machine state.
 };
 
+inline size_t
+MIMEScanner::get_buffered_line_size() const
+{
+  return m_line.size();
+}
+
+inline void
+MIMEScanner::clear()
+{
+  std::string empty;        // GAH! @c swap isn't defined to take r-value reference!
+  std::swap(m_line, empty); // make sure the memory is released.
+  m_state = INITIAL_PARSE_STATE;
+}
+
 struct MIMEParser {
   MIMEScanner m_scanner;
   int32_t m_field;
@@ -389,6 +458,7 @@ extern const char *MIME_FIELD_FORWARDED;
 extern const char *MIME_FIELD_SEC_WEBSOCKET_KEY;
 extern const char *MIME_FIELD_SEC_WEBSOCKET_VERSION;
 extern const char *MIME_FIELD_HTTP2_SETTINGS;
+extern const char *MIME_FIELD_EARLY_DATA;
 
 extern const char *MIME_VALUE_BYTES;
 extern const char *MIME_VALUE_CHUNKED;
@@ -490,7 +560,6 @@ extern int MIME_LEN_ATS_INTERNAL;
 extern int MIME_LEN_X_ID;
 extern int MIME_LEN_X_FORWARDED_FOR;
 extern int MIME_LEN_FORWARDED;
-
 extern int MIME_LEN_BYTES;
 extern int MIME_LEN_CHUNKED;
 extern int MIME_LEN_CLOSE;
@@ -513,11 +582,10 @@ extern int MIME_LEN_PROXY_REVALIDATE;
 extern int MIME_LEN_PUBLIC;
 extern int MIME_LEN_S_MAXAGE;
 extern int MIME_LEN_NEED_REVALIDATE_ONCE;
-
 extern int MIME_LEN_SEC_WEBSOCKET_KEY;
 extern int MIME_LEN_SEC_WEBSOCKET_VERSION;
-
 extern int MIME_LEN_HTTP2_SETTINGS;
+extern int MIME_LEN_EARLY_DATA;
 
 extern int MIME_WKSIDX_ACCEPT;
 extern int MIME_WKSIDX_ACCEPT_CHARSET;
@@ -595,6 +663,7 @@ extern int MIME_WKSIDX_X_ID;
 extern int MIME_WKSIDX_SEC_WEBSOCKET_KEY;
 extern int MIME_WKSIDX_SEC_WEBSOCKET_VERSION;
 extern int MIME_WKSIDX_HTTP2_SETTINGS;
+extern int MIME_WKSIDX_EARLY_DATA;
 
 /***********************************************************************
  *                                                                     *
@@ -657,11 +726,9 @@ inkcoreapi MIMEField *mime_hdr_prepare_for_value_set(HdrHeap *heap, MIMEHdrImpl
 
 void mime_field_destroy(MIMEHdrImpl *mh, MIMEField *field);
 
-const char *mime_field_name_get(const MIMEField *field, int *length);
 void mime_field_name_set(HdrHeap *heap, MIMEHdrImpl *mh, MIMEField *field, int16_t name_wks_idx_or_neg1, const char *name,
                          int length, bool must_copy_string);
 
-inkcoreapi const char *mime_field_value_get(const MIMEField *field, int *length);
 int32_t mime_field_value_get_int(const MIMEField *field);
 uint32_t mime_field_value_get_uint(const MIMEField *field);
 int64_t mime_field_value_get_int64(const MIMEField *field);
@@ -691,16 +758,10 @@ void mime_field_name_value_set(HdrHeap *heap, MIMEHdrImpl *mh, MIMEField *field,
 void mime_field_value_append(HdrHeap *heap, MIMEHdrImpl *mh, MIMEField *field, const char *value, int length, bool prepend_comma,
                              const char separator);
 
-void mime_scanner_init(MIMEScanner *scanner);
-void mime_scanner_clear(MIMEScanner *scanner);
-void mime_scanner_append(MIMEScanner *scanner, const char *data, int data_size);
-ParseResult mime_scanner_get(MIMEScanner *S, const char **raw_input_s, const char *raw_input_e, const char **output_s,
-                             const char **output_e, bool *output_shares_raw_input, bool raw_input_eof, int raw_input_scan_type);
-
 void mime_parser_init(MIMEParser *parser);
 void mime_parser_clear(MIMEParser *parser);
 ParseResult mime_parser_parse(MIMEParser *parser, HdrHeap *heap, MIMEHdrImpl *mh, const char **real_s, const char *real_e,
-                              bool must_copy_strings, bool eof);
+                              bool must_copy_strings, bool eof, bool remove_ws_from_field_name, size_t max_hdr_field_size = 131070);
 
 void mime_hdr_describe(HdrHeapObjImpl *raw, bool recurse);
 void mime_field_block_describe(HdrHeapObjImpl *raw, bool recurse);
@@ -709,6 +770,8 @@ int mime_hdr_print(HdrHeap *heap, MIMEHdrImpl *mh, char *buf_start, int buf_leng
                    int *buf_chars_to_skip_inout);
 int mime_mem_print(const char *src_d, int src_l, char *buf_start, int buf_length, int *buf_index_inout,
                    int *buf_chars_to_skip_inout);
+int mime_mem_print_lc(const char *src_d, int src_l, char *buf_start, int buf_length, int *buf_index_inout,
+                      int *buf_chars_to_skip_inout);
 int mime_field_print(MIMEField *field, char *buf_start, int buf_length, int *buf_index_inout, int *buf_chars_to_skip_inout);
 
 const char *mime_str_u16_set(HdrHeap *heap, const char *s_str, int s_len, const char **d_str, uint16_t *d_len, bool must_copy);
@@ -727,12 +790,12 @@ uint32_t mime_parse_uint(const char *buf, const char *end = nullptr);
 int64_t mime_parse_int64(const char *buf, const char *end = nullptr);
 int mime_parse_rfc822_date_fastcase(const char *buf, int length, struct tm *tp);
 time_t mime_parse_date(const char *buf, const char *end = nullptr);
-int mime_parse_day(const char *&buf, const char *end, int *day);
-int mime_parse_month(const char *&buf, const char *end, int *month);
-int mime_parse_mday(const char *&buf, const char *end, int *mday);
-int mime_parse_year(const char *&buf, const char *end, int *year);
-int mime_parse_time(const char *&buf, const char *end, int *hour, int *min, int *sec);
-int mime_parse_integer(const char *&buf, const char *end, int *integer);
+bool mime_parse_day(const char *&buf, const char *end, int *day);
+bool mime_parse_month(const char *&buf, const char *end, int *month);
+bool mime_parse_mday(const char *&buf, const char *end, int *mday);
+bool mime_parse_year(const char *&buf, const char *end, int *year);
+bool mime_parse_time(const char *&buf, const char *end, int *hour, int *min, int *sec);
+bool mime_parse_integer(const char *&buf, const char *end, int *integer);
 
 /***********************************************************************
  *                                                                     *
@@ -746,7 +809,9 @@ int mime_parse_integer(const char *&buf, const char *end, int *integer);
 inline const char *
 MIMEField::name_get(int *length) const
 {
-  return (mime_field_name_get(this, length));
+  auto name{this->name_get()};
+  *length = int(name.size());
+  return name.data();
 }
 
 /*-------------------------------------------------------------------------
@@ -755,11 +820,10 @@ MIMEField::name_get(int *length) const
 inline void
 MIMEField::name_set(HdrHeap *heap, MIMEHdrImpl *mh, const char *name, int length)
 {
-  int16_t name_wks_idx;
   const char *name_wks;
 
   if (hdrtoken_is_wks(name)) {
-    name_wks_idx = hdrtoken_wks_to_index(name);
+    int16_t name_wks_idx = hdrtoken_wks_to_index(name);
     mime_field_name_set(heap, mh, this, name_wks_idx, name, length, true);
   } else {
     int field_name_wks_idx = hdrtoken_tokenize(name, length, &name_wks);
@@ -790,37 +854,39 @@ MIMEField::name_is_valid() const
 inline const char *
 MIMEField::value_get(int *length) const
 {
-  return (mime_field_value_get(this, length));
+  auto value{this->value_get()};
+  *length = int(value.size());
+  return value.data();
 }
 
 inline int32_t
 MIMEField::value_get_int() const
 {
-  return (mime_field_value_get_int(this));
+  return mime_field_value_get_int(this);
 }
 
 inline uint32_t
 MIMEField::value_get_uint() const
 {
-  return (mime_field_value_get_uint(this));
+  return mime_field_value_get_uint(this);
 }
 
 inline int64_t
 MIMEField::value_get_int64() const
 {
-  return (mime_field_value_get_int64(this));
+  return mime_field_value_get_int64(this);
 }
 
 inline time_t
 MIMEField::value_get_date() const
 {
-  return (mime_field_value_get_date(this));
+  return mime_field_value_get_date(this);
 }
 
 inline int
 MIMEField::value_get_comma_list(StrList *list) const
 {
-  return (mime_field_value_get_comma_list(this, list));
+  return mime_field_value_get_comma_list(this, list);
 }
 
 /*-------------------------------------------------------------------------
@@ -904,9 +970,9 @@ MIMEField::has_dups() const
  ***********************************************************************/
 
 struct MIMEFieldIter {
-  MIMEFieldIter() : m_slot(0), m_block(nullptr) {}
-  uint32_t m_slot;
-  MIMEFieldBlockImpl *m_block;
+  MIMEFieldIter() {}
+  uint32_t m_slot             = 0;
+  MIMEFieldBlockImpl *m_block = nullptr;
 };
 
 /*-------------------------------------------------------------------------
@@ -951,10 +1017,12 @@ public:
 
   int print(char *buf, int bufsize, int *bufindex, int *chars_to_skip);
 
-  int parse(MIMEParser *parser, const char **start, const char *end, bool must_copy_strs, bool eof);
+  int parse(MIMEParser *parser, const char **start, const char *end, bool must_copy_strs, bool eof, bool remove_ws_from_field_name,
+            size_t max_hdr_field_size = UINT16_MAX);
 
   int value_get_index(const char *name, int name_length, const char *value, int value_length) const;
   const char *value_get(const char *name, int name_length, int *value_length) const;
+  std::string_view value_get(std::string_view const &name) const; // Convenience overload.
   int32_t value_get_int(const char *name, int name_length) const;
   uint32_t value_get_uint(const char *name, int name_length) const;
   int64_t value_get_int64(const char *name, int name_length) const;
@@ -971,7 +1039,7 @@ public:
   void value_append(const char *name, int name_length, const char *value, int value_length, bool prepend_comma = false,
                     const char separator = ',');
 
-  void field_value_set(MIMEField *field, const char *value, int value_length);
+  void field_value_set(MIMEField *field, const char *value, int value_length, bool reuse_heaps = false);
   void field_value_set_int(MIMEField *field, int32_t value);
   void field_value_set_uint(MIMEField *field, uint32_t value);
   void field_value_set_int64(MIMEField *field, int64_t value);
@@ -1027,6 +1095,12 @@ public:
   // No gratuitous copies & refcounts!
   MIMEHdr(const MIMEHdr &m) = delete;
   MIMEHdr &operator=(const MIMEHdr &m) = delete;
+
+private:
+  // Interface to replace (overwrite) field value without
+  // changing the heap as long as the new value is not longer
+  // than the current value
+  bool field_value_replace(MIMEField *field, const char *value, int value_length);
 };
 
 /*-------------------------------------------------------------------------
@@ -1107,7 +1181,7 @@ MIMEHdr::field_create(const char *name, int length)
     mime_field_name_set(m_heap, m_mime, field, field_name_wks_idx, name, length, true);
   }
 
-  return (field);
+  return field;
 }
 
 /*-------------------------------------------------------------------------
@@ -1225,7 +1299,8 @@ MIMEHdr::print(char *buf, int bufsize, int *bufindex, int *chars_to_skip)
   -------------------------------------------------------------------------*/
 
 inline int
-MIMEHdr::parse(MIMEParser *parser, const char **start, const char *end, bool must_copy_strs, bool eof)
+MIMEHdr::parse(MIMEParser *parser, const char **start, const char *end, bool must_copy_strs, bool eof,
+               bool remove_ws_from_field_name, size_t max_hdr_field_size)
 {
   if (!m_heap)
     m_heap = new_HdrHeap();
@@ -1233,7 +1308,7 @@ MIMEHdr::parse(MIMEParser *parser, const char **start, const char *end, bool mus
   if (!m_mime)
     m_mime = mime_hdr_create(m_heap);
 
-  return mime_parser_parse(parser, m_heap, m_mime, start, end, must_copy_strs, eof);
+  return mime_parser_parse(parser, m_heap, m_mime, start, end, must_copy_strs, eof, remove_ws_from_field_name, max_hdr_field_size);
 }
 
 /*-------------------------------------------------------------------------
@@ -1242,10 +1317,11 @@ inline int
 MIMEHdr::value_get_index(const char *name, int name_length, const char *value, int value_length) const
 {
   const MIMEField *field = field_find(name, name_length);
-  if (field)
+
+  if (field) {
     return field->value_get_index(value, value_length);
-  else
-    return -1;
+  }
+  return -1;
 }
 
 /*-------------------------------------------------------------------------
@@ -1254,13 +1330,23 @@ MIMEHdr::value_get_index(const char *name, int name_length, const char *value, i
 inline const char *
 MIMEHdr::value_get(const char *name, int name_length, int *value_length_return) const
 {
-  //    ink_assert(valid());
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (mime_field_value_get(field, value_length_return));
-  else
-    return (nullptr);
+  if (field) {
+    return field->value_get(value_length_return);
+  }
+  return nullptr;
+}
+
+inline std::string_view
+MIMEHdr::value_get(std::string_view const &name) const
+{
+  MIMEField const *field = field_find(name.data(), name.size());
+
+  if (field) {
+    return field->value_get();
+  }
+  return {};
 }
 
 inline int32_t
@@ -1268,10 +1354,10 @@ MIMEHdr::value_get_int(const char *name, int name_length) const
 {
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (mime_field_value_get_int(field));
-  else
-    return (0);
+  if (field) {
+    return mime_field_value_get_int(field);
+  }
+  return 0;
 }
 
 inline uint32_t
@@ -1279,10 +1365,10 @@ MIMEHdr::value_get_uint(const char *name, int name_length) const
 {
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (mime_field_value_get_uint(field));
-  else
-    return (0);
+  if (field) {
+    return mime_field_value_get_uint(field);
+  }
+  return 0;
 }
 
 inline int64_t
@@ -1290,10 +1376,10 @@ MIMEHdr::value_get_int64(const char *name, int name_length) const
 {
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (mime_field_value_get_int64(field));
-  else
-    return (0);
+  if (field) {
+    return mime_field_value_get_int64(field);
+  }
+  return 0;
 }
 
 inline time_t
@@ -1301,10 +1387,10 @@ MIMEHdr::value_get_date(const char *name, int name_length) const
 {
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (mime_field_value_get_date(field));
-  else
-    return (0);
+  if (field) {
+    return mime_field_value_get_date(field);
+  }
+  return 0;
 }
 
 inline int
@@ -1312,19 +1398,32 @@ MIMEHdr::value_get_comma_list(const char *name, int name_length, StrList *list)
 {
   const MIMEField *field = field_find(name, name_length);
 
-  if (field)
-    return (field->value_get_comma_list(list));
-  else
-    return (0);
+  if (field) {
+    return field->value_get_comma_list(list);
+  }
+  return 0;
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-inline void
-MIMEHdr::field_value_set(MIMEField *field, const char *value, int value_length)
+inline bool
+MIMEHdr::field_value_replace(MIMEField *field, const char *value, int value_length)
 {
-  field->value_set(m_heap, m_mime, value, value_length);
+  if (field->m_len_value >= value_length) {
+    memcpy((char *)field->m_ptr_value, value, value_length);
+    field->m_len_value = value_length;
+    return true;
+  }
+  return false;
+}
+
+inline void
+MIMEHdr::field_value_set(MIMEField *field, const char *value, int value_length, bool reuse_heaps)
+{
+  if (!reuse_heaps || !field_value_replace(field, value, value_length)) {
+    field->value_set(m_heap, m_mime, value, value_length);
+  }
 }
 
 inline void
@@ -1479,7 +1578,7 @@ MIMEHdr::get_age()
 inline int64_t
 MIMEHdr::get_content_length() const
 {
-  return (value_get_int64(MIME_FIELD_CONTENT_LENGTH, MIME_LEN_CONTENT_LENGTH));
+  return value_get_int64(MIME_FIELD_CONTENT_LENGTH, MIME_LEN_CONTENT_LENGTH);
 }
 
 /*-------------------------------------------------------------------------
@@ -1488,7 +1587,7 @@ MIMEHdr::get_content_length() const
 inline time_t
 MIMEHdr::get_date()
 {
-  return (value_get_date(MIME_FIELD_DATE, MIME_LEN_DATE));
+  return value_get_date(MIME_FIELD_DATE, MIME_LEN_DATE);
 }
 
 /*-------------------------------------------------------------------------
@@ -1497,7 +1596,7 @@ MIMEHdr::get_date()
 inline time_t
 MIMEHdr::get_expires()
 {
-  return (value_get_date(MIME_FIELD_EXPIRES, MIME_LEN_EXPIRES));
+  return value_get_date(MIME_FIELD_EXPIRES, MIME_LEN_EXPIRES);
 }
 
 /*-------------------------------------------------------------------------
@@ -1506,7 +1605,7 @@ MIMEHdr::get_expires()
 inline time_t
 MIMEHdr::get_if_modified_since()
 {
-  return (value_get_date(MIME_FIELD_IF_MODIFIED_SINCE, MIME_LEN_IF_MODIFIED_SINCE));
+  return value_get_date(MIME_FIELD_IF_MODIFIED_SINCE, MIME_LEN_IF_MODIFIED_SINCE);
 }
 
 /*-------------------------------------------------------------------------
@@ -1515,7 +1614,7 @@ MIMEHdr::get_if_modified_since()
 inline time_t
 MIMEHdr::get_if_unmodified_since()
 {
-  return (value_get_date(MIME_FIELD_IF_UNMODIFIED_SINCE, MIME_LEN_IF_UNMODIFIED_SINCE));
+  return value_get_date(MIME_FIELD_IF_UNMODIFIED_SINCE, MIME_LEN_IF_UNMODIFIED_SINCE);
 }
 
 /*-------------------------------------------------------------------------
@@ -1524,7 +1623,7 @@ MIMEHdr::get_if_unmodified_since()
 inline time_t
 MIMEHdr::get_last_modified()
 {
-  return (value_get_date(MIME_FIELD_LAST_MODIFIED, MIME_LEN_LAST_MODIFIED));
+  return value_get_date(MIME_FIELD_LAST_MODIFIED, MIME_LEN_LAST_MODIFIED);
 }
 
 /*-------------------------------------------------------------------------
@@ -1533,7 +1632,7 @@ MIMEHdr::get_last_modified()
 inline time_t
 MIMEHdr::get_if_range_date()
 {
-  return (value_get_date(MIME_FIELD_IF_RANGE, MIME_LEN_IF_RANGE));
+  return value_get_date(MIME_FIELD_IF_RANGE, MIME_LEN_IF_RANGE);
 }
 
 /*-------------------------------------------------------------------------
@@ -1542,7 +1641,7 @@ MIMEHdr::get_if_range_date()
 inline int32_t
 MIMEHdr::get_max_forwards()
 {
-  return (value_get_int(MIME_FIELD_MAX_FORWARDS, MIME_LEN_MAX_FORWARDS));
+  return value_get_int(MIME_FIELD_MAX_FORWARDS, MIME_LEN_MAX_FORWARDS);
 }
 
 /*-------------------------------------------------------------------------
@@ -1554,7 +1653,7 @@ MIMEHdr::get_warning(int idx)
   (void)idx;
   // FIXME: what do we do here?
   ink_release_assert(!"unimplemented");
-  return (0);
+  return 0;
 }
 
 /*-------------------------------------------------------------------------
@@ -1563,7 +1662,7 @@ MIMEHdr::get_warning(int idx)
 inline uint32_t
 MIMEHdr::get_cooked_cc_mask()
 {
-  return (m_mime->m_cooked_stuff.m_cache_control.m_mask);
+  return m_mime->m_cooked_stuff.m_cache_control.m_mask;
 }
 
 /*-------------------------------------------------------------------------
@@ -1572,7 +1671,7 @@ MIMEHdr::get_cooked_cc_mask()
 inline int32_t
 MIMEHdr::get_cooked_cc_max_age()
 {
-  return (m_mime->m_cooked_stuff.m_cache_control.m_secs_max_age);
+  return m_mime->m_cooked_stuff.m_cache_control.m_secs_max_age;
 }
 
 /*-------------------------------------------------------------------------
@@ -1581,7 +1680,7 @@ MIMEHdr::get_cooked_cc_max_age()
 inline int32_t
 MIMEHdr::get_cooked_cc_s_maxage()
 {
-  return (m_mime->m_cooked_stuff.m_cache_control.m_secs_s_maxage);
+  return m_mime->m_cooked_stuff.m_cache_control.m_secs_s_maxage;
 }
 
 /*-------------------------------------------------------------------------
@@ -1590,7 +1689,7 @@ MIMEHdr::get_cooked_cc_s_maxage()
 inline int32_t
 MIMEHdr::get_cooked_cc_max_stale()
 {
-  return (m_mime->m_cooked_stuff.m_cache_control.m_secs_max_stale);
+  return m_mime->m_cooked_stuff.m_cache_control.m_secs_max_stale;
 }
 
 /*-------------------------------------------------------------------------
@@ -1599,7 +1698,7 @@ MIMEHdr::get_cooked_cc_max_stale()
 inline int32_t
 MIMEHdr::get_cooked_cc_min_fresh()
 {
-  return (m_mime->m_cooked_stuff.m_cache_control.m_secs_min_fresh);
+  return m_mime->m_cooked_stuff.m_cache_control.m_secs_min_fresh;
 }
 
 /*-------------------------------------------------------------------------
@@ -1608,7 +1707,7 @@ MIMEHdr::get_cooked_cc_min_fresh()
 inline bool
 MIMEHdr::get_cooked_pragma_no_cache()
 {
-  return (m_mime->m_cooked_stuff.m_pragma.m_no_cache);
+  return m_mime->m_cooked_stuff.m_pragma.m_no_cache;
 }
 
 /*-------------------------------------------------------------------------
