@@ -36,20 +36,20 @@
 #include <cstdlib>
 #include <cstdio>
 #include <bitset>
-
-#ifdef HAVE_CTYPE_H
+#include <map>
 #include <cctype>
-#endif
+#include <string_view>
 
 #include "tscore/ink_platform.h"
 #include "tscore/ink_inet.h"
+#include "tscore/ink_resolver.h"
 #include "tscore/IpMap.h"
 #include "tscore/Regex.h"
-#include "string_view"
 #include "tscore/BufferWriter.h"
 #include "HttpProxyAPIEnums.h"
 #include "ProxyConfig.h"
 #include "records/P_RecProcess.h"
+#include "HttpConnectionCount.h"
 
 static const unsigned HTTP_STATUS_NUMBER = 600;
 using HttpStatusBitset                   = std::bitset<HTTP_STATUS_NUMBER>;
@@ -65,13 +65,14 @@ enum {
   http_current_client_transactions_stat,
   http_total_incoming_connections_stat,
   http_current_server_transactions_stat,
+  http_pooled_server_connections_stat,
 
   //  Http Abort information (from HttpNetConnection)
   http_ua_msecs_counts_errors_pre_accept_hangups_stat,
 
   // Http Total Connections Stats
   //
-  // it is assumed that this inequality will always be satisifed:
+  // it is assumed that this inequality will always be satisfied:
   //   http_total_client_connections_stat >=
   //     http_total_client_connections_ipv4_stat +
   //     http_total_client_connections_ipv6_stat
@@ -84,6 +85,7 @@ enum {
   http_total_parent_switches_stat,
   http_total_parent_retries_exhausted_stat,
   http_total_parent_marked_down_count,
+  http_background_fill_total_count_stat,
   http_current_parent_proxy_connections_stat,
   http_current_server_connections_stat,
   http_current_cache_connections_stat,
@@ -119,7 +121,6 @@ enum {
   http_cache_deletes_stat,
 
   http_tunnels_stat,
-  http_throttled_proxy_only_stat,
 
   // document size stats
   http_user_agent_request_header_total_size_stat,
@@ -176,7 +177,6 @@ enum {
   // cache result stats
   http_cache_hit_fresh_stat,
   http_cache_hit_mem_fresh_stat,
-  http_cache_hit_rww_stat,
   http_cache_hit_reval_stat,
   http_cache_hit_ims_stat,
   http_cache_hit_stale_served_stat,
@@ -327,9 +327,43 @@ enum {
 
   http_origin_connections_throttled_stat,
 
+  http_origin_connect_adjust_thread_stat,
+  http_cache_open_write_adjust_thread_stat,
+
+  http_origin_shutdown_pool_lock_contention,
+  http_origin_shutdown_migration_failure,
+  http_origin_shutdown_tunnel_server,
+  http_origin_shutdown_tunnel_server_no_keep_alive,
+  http_origin_shutdown_tunnel_server_eos,
+  http_origin_shutdown_tunnel_server_plugin_tunnel,
+  http_origin_shutdown_tunnel_server_detach,
+  http_origin_shutdown_tunnel_client,
+  http_origin_shutdown_tunnel_transform_read,
+  http_origin_shutdown_release_no_sharing,
+  http_origin_shutdown_release_no_server,
+  http_origin_shutdown_release_no_keep_alive,
+  http_origin_shutdown_release_invalid_response,
+  http_origin_shutdown_release_invalid_request,
+  http_origin_shutdown_release_modified,
+  http_origin_shutdown_release_misc,
+  http_origin_shutdown_cleanup_entry,
+  http_origin_shutdown_tunnel_abort,
+
+  http_dead_server_no_requests,
+
   http_stat_count
 };
 
+enum CacheOpenWriteFailAction_t {
+  CACHE_WL_FAIL_ACTION_DEFAULT                           = 0x00,
+  CACHE_WL_FAIL_ACTION_ERROR_ON_MISS                     = 0x01,
+  CACHE_WL_FAIL_ACTION_STALE_ON_REVALIDATE               = 0x02,
+  CACHE_WL_FAIL_ACTION_ERROR_ON_MISS_STALE_ON_REVALIDATE = 0x03,
+  CACHE_WL_FAIL_ACTION_ERROR_ON_MISS_OR_REVALIDATE       = 0x04,
+  CACHE_WL_FAIL_ACTION_READ_RETRY                        = 0x05,
+  TOTAL_CACHE_WL_FAIL_ACTION_TYPES
+};
+
 extern RecRawStatBlock *http_rsb;
 
 /* Stats should only be accessed using these macros */
@@ -357,11 +391,11 @@ extern RecRawStatBlock *http_rsb;
 //   (corresponds to a "*" in the config file)
 /////////////////////////////////////////////////////////////
 struct HttpConfigPortRange {
-  int low;
-  int high;
-  HttpConfigPortRange *next;
+  int low                   = 0;
+  int high                  = 0;
+  HttpConfigPortRange *next = nullptr;
 
-  HttpConfigPortRange() : low(0), high(0), next(nullptr) {}
+  HttpConfigPortRange() {}
   ~HttpConfigPortRange()
   {
     if (next)
@@ -397,383 +431,294 @@ OptionBitSet optStrToBitset(std::string_view optConfigStr, ts::FixedBufferWriter
 
 } // namespace HttpForwarded
 
+namespace RedirectEnabled
+{
+enum class AddressClass {
+  INVALID = -1,
+  DEFAULT,
+  PRIVATE,
+  LOOPBACK,
+  MULTICAST,
+  LINKLOCAL,
+  ROUTABLE,
+  SELF,
+};
+
+enum class Action {
+  INVALID = -1,
+  RETURN,
+  REJECT,
+  FOLLOW,
+};
+
+static std::map<std::string, AddressClass> address_class_map = {
+  {"default", AddressClass::DEFAULT},     {"private", AddressClass::PRIVATE},     {"loopback", AddressClass::LOOPBACK},
+  {"multicast", AddressClass::MULTICAST}, {"linklocal", AddressClass::LINKLOCAL}, {"routable", AddressClass::ROUTABLE},
+  {"self", AddressClass::SELF},
+};
+
+static std::map<std::string, Action> action_map = {
+  {"return", Action::RETURN},
+  {"reject", Action::REJECT},
+  {"follow", Action::FOLLOW},
+};
+} // namespace RedirectEnabled
+
 /////////////////////////////////////////////////////////////
 // This is a little helper class, used by the HttpConfigParams
 // and State (txn) structure. It allows for certain configs
 // to be overridable per transaction more easily.
 struct OverridableHttpConfigParams {
-  OverridableHttpConfigParams()
-    : maintain_pristine_host_hdr(1),
-      chunking_enabled(1),
-      negative_caching_enabled(0),
-      negative_revalidating_enabled(0),
-      cache_when_to_revalidate(0),
-      keep_alive_enabled_in(1),
-      keep_alive_enabled_out(1),
-      keep_alive_post_out(1),
-      server_session_sharing_match(TS_SERVER_SESSION_SHARING_MATCH_BOTH),
-      auth_server_session_private(1),
-      fwd_proxy_auth_to_parent(0),
-      uncacheable_requests_bypass_parent(1),
-      attach_server_session_to_client(0),
-      forward_connect_method(0),
-      insert_age_in_response(1),
-      anonymize_remove_from(0),
-      anonymize_remove_referer(0),
-      anonymize_remove_user_agent(0),
-      anonymize_remove_cookie(0),
-      anonymize_remove_client_ip(0),
-      anonymize_insert_client_ip(1),
-      proxy_response_server_enabled(1),
-      proxy_response_hsts_include_subdomains(0),
-      insert_squid_x_forwarded_for(1),
-      insert_forwarded(HttpForwarded::OptionBitSet()),
-      send_http11_requests(1),
-      cache_http(1),
-      cache_ignore_client_no_cache(1),
-      cache_ignore_client_cc_max_age(0),
-      cache_ims_on_client_no_cache(1),
-      cache_ignore_server_no_cache(0),
-      cache_responses_to_cookies(1),
-      cache_ignore_auth(0),
-      cache_urls_that_look_dynamic(1),
-      cache_required_headers(2),
-      cache_range_lookup(1),
-      cache_range_write(0),
-      allow_multi_range(0),
-      cache_enable_default_vary_headers(0),
-      ignore_accept_mismatch(0),
-      ignore_accept_language_mismatch(0),
-      ignore_accept_encoding_mismatch(0),
-      ignore_accept_charset_mismatch(0),
-      insert_request_via_string(1),
-      insert_response_via_string(0),
-      doc_in_cache_skip_dns(1),
-      flow_control_enabled(0),
-      normalize_ae(0),
-      srv_enabled(0),
-      parent_failures_update_hostdb(0),
-      cache_open_write_fail_action(0),
-      post_check_content_length_enabled(1),
-      request_buffer_enabled(0),
-      allow_half_open(1),
-      ssl_client_verify_server(0),
-      redirect_use_orig_cache_key(0),
-      number_of_redirections(0),
-      proxy_response_hsts_max_age(-1),
-      negative_caching_lifetime(1800),
-      negative_revalidating_lifetime(1800),
-      sock_recv_buffer_size_out(0),
-      sock_send_buffer_size_out(0),
-      sock_option_flag_out(0),
-      sock_packet_mark_out(0),
-      sock_packet_tos_out(0),
-      server_tcp_init_cwnd(0),
-      request_hdr_max_size(131072),
-      response_hdr_max_size(131072),
-      cache_heuristic_min_lifetime(3600),
-      cache_heuristic_max_lifetime(86400),
-      cache_guaranteed_min_lifetime(0),
-      cache_guaranteed_max_lifetime(31536000),
-      cache_max_stale_age(604800),
-      keep_alive_no_activity_timeout_in(120),
-      keep_alive_no_activity_timeout_out(120),
-      transaction_no_activity_timeout_in(30),
-      transaction_no_activity_timeout_out(30),
-      transaction_active_timeout_out(0),
-      transaction_active_timeout_in(900),
-      websocket_active_timeout(3600),
-      websocket_inactive_timeout(600),
-      origin_max_connections(0),
-      origin_max_connections_queue(0),
-      connect_attempts_max_retries(0),
-      connect_attempts_max_retries_dead_server(3),
-      connect_attempts_rr_retries(3),
-      connect_attempts_timeout(30),
-      post_connect_attempts_timeout(1800),
-      parent_connect_attempts(4),
-      parent_retry_time(300),
-      parent_fail_threshold(10),
-      per_parent_connect_attempts(2),
-      parent_connect_timeout(30),
-      down_server_timeout(300),
-      client_abort_threshold(10),
-      max_cache_open_read_retries(-1),
-      cache_open_read_retry_time(10),
-      cache_generation_number(-1),
-      max_cache_open_write_retries(1),
-      background_fill_active_timeout(60),
-      http_chunking_size(4096),
-      flow_high_water_mark(0),
-      flow_low_water_mark(0),
-      default_buffer_size_index(8),
-      default_buffer_water_mark(32768),
-      slow_log_threshold(0),
-      body_factory_template_base(nullptr),
-      body_factory_template_base_len(0),
-      proxy_response_server_string(nullptr),
-      proxy_response_server_string_len(0),
-      global_user_agent_header(nullptr),
-      global_user_agent_header_size(0),
-      cache_heuristic_lm_factor(0.10),
-      background_fill_threshold(0.5),
-      client_cert_filename(nullptr),
-      client_cert_filepath(nullptr),
-      cache_vary_default_text(nullptr),
-      cache_vary_default_images(nullptr),
-      cache_vary_default_other(nullptr)
-  {
-  }
+  OverridableHttpConfigParams() : insert_forwarded(HttpForwarded::OptionBitSet()) {}
 
   // A simple rules here:
   //   * Place all MgmtByte configs before all other configs
-  MgmtByte maintain_pristine_host_hdr;
-  MgmtByte chunking_enabled;
+  MgmtByte maintain_pristine_host_hdr = 1;
+  MgmtByte chunking_enabled           = 1;
 
   ////////////////////////////////
   //  Negative Response Caching //
   ////////////////////////////////
-  MgmtByte negative_caching_enabled;
-  MgmtByte negative_revalidating_enabled;
+  MgmtByte negative_caching_enabled      = 0;
+  MgmtByte negative_revalidating_enabled = 0;
 
-  MgmtByte cache_when_to_revalidate;
+  MgmtByte cache_when_to_revalidate = 0;
 
-  MgmtByte keep_alive_enabled_in;
-  MgmtByte keep_alive_enabled_out;
-  MgmtByte keep_alive_post_out; // share server sessions for post
+  MgmtByte keep_alive_enabled_in  = 1;
+  MgmtByte keep_alive_enabled_out = 1;
+  MgmtByte keep_alive_post_out    = 1; // share server sessions for post
 
-  MgmtByte server_session_sharing_match;
-  //  MgmtByte share_server_sessions;
-  MgmtByte auth_server_session_private;
-  MgmtByte fwd_proxy_auth_to_parent;
-  MgmtByte uncacheable_requests_bypass_parent;
-  MgmtByte attach_server_session_to_client;
+  MgmtInt server_min_keep_alive_conns         = 0;
+  MgmtByte server_session_sharing_match       = 0;
+  char *server_session_sharing_match_str      = nullptr;
+  MgmtByte auth_server_session_private        = 1;
+  MgmtByte fwd_proxy_auth_to_parent           = 0;
+  MgmtByte uncacheable_requests_bypass_parent = 1;
+  MgmtByte attach_server_session_to_client    = 0;
 
-  MgmtByte forward_connect_method;
+  MgmtByte forward_connect_method = 0;
 
-  MgmtByte insert_age_in_response;
+  MgmtByte insert_age_in_response = 1;
 
   ///////////////////////////////////////////////////////////////////
   // Privacy: fields which are removed from the user agent request //
   ///////////////////////////////////////////////////////////////////
-  MgmtByte anonymize_remove_from;
-  MgmtByte anonymize_remove_referer;
-  MgmtByte anonymize_remove_user_agent;
-  MgmtByte anonymize_remove_cookie;
-  MgmtByte anonymize_remove_client_ip;
-  MgmtByte anonymize_insert_client_ip;
+  MgmtByte anonymize_remove_from       = 0;
+  MgmtByte anonymize_remove_referer    = 0;
+  MgmtByte anonymize_remove_user_agent = 0;
+  MgmtByte anonymize_remove_cookie     = 0;
+  MgmtByte anonymize_remove_client_ip  = 0;
+  MgmtByte anonymize_insert_client_ip  = 1;
 
-  MgmtByte proxy_response_server_enabled;
-  MgmtByte proxy_response_hsts_include_subdomains;
+  MgmtByte proxy_response_server_enabled          = 1;
+  MgmtByte proxy_response_hsts_include_subdomains = 0;
 
   /////////////////////
   // X-Forwarded-For //
   /////////////////////
-  MgmtByte insert_squid_x_forwarded_for;
+  MgmtByte insert_squid_x_forwarded_for = 1;
 
   ///////////////
   // Forwarded //
   ///////////////
   HttpForwarded::OptionBitSet insert_forwarded;
+  MgmtInt proxy_protocol_out = -1;
 
   //////////////////////
   //  Version Hell    //
   //////////////////////
-  MgmtByte send_http11_requests;
+  MgmtByte send_http11_requests = 1;
 
   ///////////////////
   // cache control //
   ///////////////////
-  MgmtByte cache_http;
-  MgmtByte cache_ignore_client_no_cache;
-  MgmtByte cache_ignore_client_cc_max_age;
-  MgmtByte cache_ims_on_client_no_cache;
-  MgmtByte cache_ignore_server_no_cache;
-  MgmtByte cache_responses_to_cookies;
-  MgmtByte cache_ignore_auth;
-  MgmtByte cache_urls_that_look_dynamic;
-  MgmtByte cache_required_headers;
-  MgmtByte cache_range_lookup;
-  MgmtByte cache_range_write;
-  MgmtByte allow_multi_range;
+  MgmtByte cache_http                     = 1;
+  MgmtByte cache_ignore_client_no_cache   = 1;
+  MgmtByte cache_ignore_client_cc_max_age = 0;
+  MgmtByte cache_ims_on_client_no_cache   = 1;
+  MgmtByte cache_ignore_server_no_cache   = 0;
+  MgmtByte cache_responses_to_cookies     = 1;
+  MgmtByte cache_ignore_auth              = 0;
+  MgmtByte cache_urls_that_look_dynamic   = 1;
+  MgmtByte cache_required_headers         = 2;
+  MgmtByte cache_range_lookup             = 1;
+  MgmtByte cache_range_write              = 0;
+  MgmtByte allow_multi_range              = 0;
 
-  MgmtByte cache_enable_default_vary_headers;
+  MgmtByte ignore_accept_mismatch          = 0;
+  MgmtByte ignore_accept_language_mismatch = 0;
+  MgmtByte ignore_accept_encoding_mismatch = 0;
+  MgmtByte ignore_accept_charset_mismatch  = 0;
 
-  MgmtByte ignore_accept_mismatch;
-  MgmtByte ignore_accept_language_mismatch;
-  MgmtByte ignore_accept_encoding_mismatch;
-  MgmtByte ignore_accept_charset_mismatch;
-
-  MgmtByte insert_request_via_string;
-  MgmtByte insert_response_via_string;
+  MgmtByte insert_request_via_string  = 1;
+  MgmtByte insert_response_via_string = 0;
 
   //////////////////////
   //  DOC IN CACHE NO DNS//
   //////////////////////
-  MgmtByte doc_in_cache_skip_dns;
-  MgmtByte flow_control_enabled;
+  MgmtByte doc_in_cache_skip_dns = 1;
+  MgmtByte flow_control_enabled  = 0;
 
   ////////////////////////////////
   // Optimize gzip alternates   //
   ////////////////////////////////
-  MgmtByte normalize_ae;
+  MgmtByte normalize_ae = 0;
 
   //////////////////////////
   // hostdb/dns variables //
   //////////////////////////
-  MgmtByte srv_enabled;
-  MgmtByte parent_failures_update_hostdb;
+  MgmtByte srv_enabled                   = 0;
+  MgmtByte parent_failures_update_hostdb = 0;
 
-  MgmtByte cache_open_write_fail_action;
+  MgmtByte cache_open_write_fail_action = 0;
 
   ////////////////////////
   // Check Post request //
   ////////////////////////
-  MgmtByte post_check_content_length_enabled;
+  MgmtByte post_check_content_length_enabled = 1;
 
   ////////////////////////////////////////////////
   // Buffer post body before connecting servers //
   ////////////////////////////////////////////////
-  MgmtByte request_buffer_enabled;
+  MgmtByte request_buffer_enabled = 0;
 
   /////////////////////////////////////////////////
   // Keep connection open after client sends FIN //
   /////////////////////////////////////////////////
-  MgmtByte allow_half_open;
+  MgmtByte allow_half_open = 1;
 
-  /////////////////////////////
-  // server verification mode//
-  /////////////////////////////
-  MgmtByte ssl_client_verify_server;
+  //////////////////////////////
+  // server verification mode //
+  //////////////////////////////
+  MgmtByte ssl_client_verify_server         = 0;
+  char *ssl_client_verify_server_policy     = nullptr;
+  char *ssl_client_verify_server_properties = nullptr;
+  char *ssl_client_sni_policy               = nullptr;
 
   //////////////////
   // Redirection  //
   //////////////////
-  MgmtByte redirect_use_orig_cache_key;
-  MgmtInt number_of_redirections;
+  MgmtByte redirect_use_orig_cache_key = 0;
+  MgmtInt number_of_redirections       = 0;
 
-  MgmtInt proxy_response_hsts_max_age;
+  MgmtInt proxy_response_hsts_max_age = -1;
 
   ////////////////////////////////
   //  Negative cache lifetimes  //
   ////////////////////////////////
-  MgmtInt negative_caching_lifetime;
-  MgmtInt negative_revalidating_lifetime;
+  MgmtInt negative_caching_lifetime      = 1800;
+  MgmtInt negative_revalidating_lifetime = 1800;
 
   ///////////////////////////////////////
   // origin server connection settings //
   ///////////////////////////////////////
-  MgmtInt sock_recv_buffer_size_out;
-  MgmtInt sock_send_buffer_size_out;
-  MgmtInt sock_option_flag_out;
-  MgmtInt sock_packet_mark_out;
-  MgmtInt sock_packet_tos_out;
-
-  ///////////////////////////////
-  // Initial congestion window //
-  ///////////////////////////////
-  MgmtInt server_tcp_init_cwnd;
+  MgmtInt sock_recv_buffer_size_out = 0;
+  MgmtInt sock_send_buffer_size_out = 0;
+  MgmtInt sock_option_flag_out      = 0;
+  MgmtInt sock_packet_mark_out      = 0;
+  MgmtInt sock_packet_tos_out       = 0;
 
   ///////////////
   // Hdr Limit //
   ///////////////
-  MgmtInt request_hdr_max_size;
-  MgmtInt response_hdr_max_size;
+  MgmtInt request_hdr_max_size  = 131072;
+  MgmtInt response_hdr_max_size = 131072;
 
   /////////////////////
   // cache variables //
   /////////////////////
-  MgmtInt cache_heuristic_min_lifetime;
-  MgmtInt cache_heuristic_max_lifetime;
-  MgmtInt cache_guaranteed_min_lifetime;
-  MgmtInt cache_guaranteed_max_lifetime;
-  MgmtInt cache_max_stale_age;
+  MgmtInt cache_heuristic_min_lifetime  = 3600;
+  MgmtInt cache_heuristic_max_lifetime  = 86400;
+  MgmtInt cache_guaranteed_min_lifetime = 0;
+  MgmtInt cache_guaranteed_max_lifetime = 31536000;
+  MgmtInt cache_max_stale_age           = 604800;
 
   ///////////////////////////////////////////////////
   // connection variables. timeouts are in seconds //
   ///////////////////////////////////////////////////
-  MgmtInt keep_alive_no_activity_timeout_in;
-  MgmtInt keep_alive_no_activity_timeout_out;
-  MgmtInt transaction_no_activity_timeout_in;
-  MgmtInt transaction_no_activity_timeout_out;
-  MgmtInt transaction_active_timeout_out;
-  MgmtInt transaction_active_timeout_in;
-  MgmtInt websocket_active_timeout;
-  MgmtInt websocket_inactive_timeout;
-  MgmtInt origin_max_connections;
-  MgmtInt origin_max_connections_queue;
+  MgmtInt keep_alive_no_activity_timeout_in   = 120;
+  MgmtInt keep_alive_no_activity_timeout_out  = 120;
+  MgmtInt transaction_no_activity_timeout_in  = 30;
+  MgmtInt transaction_no_activity_timeout_out = 30;
+  MgmtInt transaction_active_timeout_out      = 0;
+  MgmtInt transaction_active_timeout_in       = 900;
+  MgmtInt websocket_active_timeout            = 3600;
+  MgmtInt websocket_inactive_timeout          = 600;
 
   ////////////////////////////////////
   // origin server connect attempts //
   ////////////////////////////////////
-  MgmtInt connect_attempts_max_retries;
-  MgmtInt connect_attempts_max_retries_dead_server;
-  MgmtInt connect_attempts_rr_retries;
-  MgmtInt connect_attempts_timeout;
-  MgmtInt post_connect_attempts_timeout;
+  MgmtInt connect_attempts_max_retries             = 0;
+  MgmtInt connect_attempts_max_retries_dead_server = 3;
+  MgmtInt connect_attempts_rr_retries              = 3;
+  MgmtInt connect_attempts_timeout                 = 30;
+  MgmtInt post_connect_attempts_timeout            = 1800;
+
+  MgmtInt connect_dead_policy = 2;
 
   ////////////////////////////////////
   // parent proxy connect attempts //
   ///////////////////////////////////
-  MgmtInt parent_connect_attempts;
-  MgmtInt parent_retry_time;
-  MgmtInt parent_fail_threshold;
-  MgmtInt per_parent_connect_attempts;
-  MgmtInt parent_connect_timeout;
+  MgmtInt parent_connect_attempts     = 4;
+  MgmtInt parent_retry_time           = 300;
+  MgmtInt parent_fail_threshold       = 10;
+  MgmtInt per_parent_connect_attempts = 2;
+  MgmtInt parent_connect_timeout      = 30;
 
-  MgmtInt down_server_timeout;
-  MgmtInt client_abort_threshold;
+  MgmtInt down_server_timeout    = 300;
+  MgmtInt client_abort_threshold = 1000;
 
   // open read failure retries.
-  MgmtInt max_cache_open_read_retries;
-  MgmtInt cache_open_read_retry_time; // time is in mseconds
-  MgmtInt cache_generation_number;
+  MgmtInt max_cache_open_read_retries = -1;
+  MgmtInt cache_open_read_retry_time  = 10; // time is in mseconds
+  MgmtInt cache_generation_number     = -1;
 
   // open write failure retries.
-  MgmtInt max_cache_open_write_retries;
+  MgmtInt max_cache_open_write_retries = 1;
 
-  MgmtInt background_fill_active_timeout;
+  MgmtInt background_fill_active_timeout = 60;
 
-  MgmtInt http_chunking_size;          // Maximum chunk size for chunked output.
-  MgmtByte http_drop_chunked_trailers; ///< Whether to drop chunked trailers.
-  MgmtInt flow_high_water_mark;        ///< Flow control high water mark.
-  MgmtInt flow_low_water_mark;         ///< Flow control low water mark.
+  MgmtInt http_chunking_size   = 4096; // Maximum chunk size for chunked output.
+  MgmtInt flow_high_water_mark = 0;    ///< Flow control high water mark.
+  MgmtInt flow_low_water_mark  = 0;    ///< Flow control low water mark.
 
-  MgmtInt default_buffer_size_index;
-  MgmtInt default_buffer_water_mark;
-  MgmtInt slow_log_threshold;
+  MgmtInt default_buffer_size_index = 8;
+  MgmtInt default_buffer_water_mark = 32768;
+  MgmtInt slow_log_threshold        = 0;
+
+  OutboundConnTrack::TxnConfig outbound_conntrack;
 
   ///////////////////////////////////////////////////////////////////
   // Server header                                                 //
   ///////////////////////////////////////////////////////////////////
-  char *body_factory_template_base;
-  size_t body_factory_template_base_len;
-  char *proxy_response_server_string;      // This does not get free'd by us!
-  size_t proxy_response_server_string_len; // Updated when server_string is set.
+  char *body_factory_template_base        = nullptr;
+  size_t body_factory_template_base_len   = 0;
+  char *proxy_response_server_string      = nullptr; // This does not get free'd by us!
+  size_t proxy_response_server_string_len = 0;       // Updated when server_string is set.
 
   ///////////////////////////////////////////////////////////////////
-  // Global User Agent header                                                 //
+  // Global User Agent header                                      //
   ///////////////////////////////////////////////////////////////////
-  char *global_user_agent_header;       // This does not get free'd by us!
-  size_t global_user_agent_header_size; // Updated when user_agent is set.
+  char *global_user_agent_header       = nullptr; // This does not get free'd by us!
+  size_t global_user_agent_header_size = 0;       // Updated when user_agent is set.
 
-  MgmtFloat cache_heuristic_lm_factor;
-  MgmtFloat background_fill_threshold;
+  MgmtFloat cache_heuristic_lm_factor = 0.10;
+  MgmtFloat background_fill_threshold = 0.5;
 
   // Various strings, good place for them here ...
-  char *client_cert_filename;
-  char *client_cert_filepath;
+  char *ssl_client_cert_filename        = nullptr;
+  char *ssl_client_private_key_filename = nullptr;
+  char *ssl_client_ca_cert_filename     = nullptr;
 
-  char *cache_vary_default_text;
-  char *cache_vary_default_images;
-  char *cache_vary_default_other;
+  // Host Resolution order
+  HostResData host_res_data;
 };
 
 /////////////////////////////////////////////////////////////
 //
 // struct HttpConfigParams
 //
-// configuration parameters as they apear in the global
+// configuration parameters as they appear in the global
 // configuration file.
 /////////////////////////////////////////////////////////////
 struct HttpConfigParams : public ConfigInfo {
@@ -800,9 +745,8 @@ public:
   IpAddr proxy_protocol_ip4, proxy_protocol_ip6;
   IpMap config_proxy_protocol_ipmap;
 
-  MgmtInt server_max_connections            = 0;
-  MgmtInt origin_min_keep_alive_connections = 0; // TODO: This one really ought to be overridable, but difficult right now.
-  MgmtInt max_websocket_connections         = -1;
+  MgmtInt server_max_connections    = 0;
+  MgmtInt max_websocket_connections = -1;
 
   char *proxy_request_via_string    = nullptr;
   char *proxy_response_via_string   = nullptr;
@@ -830,6 +774,13 @@ public:
   MgmtInt post_copy_size = 2048;
   MgmtInt max_post_size  = 0;
 
+  MgmtInt max_payload_iobuf_index = BUFFER_SIZE_INDEX_32K;
+  MgmtInt max_msg_iobuf_index     = BUFFER_SIZE_INDEX_32K;
+
+  char *redirect_actions_string                        = nullptr;
+  IpMap *redirect_actions_map                          = nullptr;
+  RedirectEnabled::Action redirect_actions_self_action = RedirectEnabled::Action::INVALID;
+
   ///////////////////////////////////////////////////////////////////
   // Put all MgmtByte members down here, avoids additional padding //
   ///////////////////////////////////////////////////////////////////
@@ -849,7 +800,7 @@ public:
   MgmtByte referer_filter_enabled  = 0;
   MgmtByte referer_format_redirect = 0;
 
-  MgmtByte strict_uri_parsing = 2;
+  MgmtByte strict_uri_parsing = 0;
 
   MgmtByte reverse_proxy_enabled = 0;
   MgmtByte url_remap_required    = 1;
@@ -861,11 +812,12 @@ public:
 
   MgmtByte send_100_continue_response = 0;
   MgmtByte disallow_post_100_continue = 0;
-  MgmtByte parser_allow_non_http      = 1;
   MgmtByte keepalive_internal_vc      = 0;
 
   MgmtByte server_session_sharing_pool = TS_SERVER_SESSION_SHARING_POOL_THREAD;
 
+  OutboundConnTrack::GlobalConfig outbound_conntrack;
+
   // bitset to hold the status codes that will BE cached with negative caching enabled
   HttpStatusBitset negative_caching_list;
 
@@ -875,7 +827,11 @@ public:
 
   MgmtInt body_factory_response_max_size = 8192;
 
-  MgmtByte scheme_proto_mismatch_policy = 2;
+  MgmtInt http_request_line_max_size = 65535;
+  MgmtInt http_hdr_field_max_size    = 131070;
+
+  MgmtByte http_host_sni_policy = 0;
+
   // noncopyable
   /////////////////////////////////////
   // operator = and copy constructor //
@@ -899,9 +855,14 @@ public:
   inkcoreapi static HttpConfigParams *acquire();
   inkcoreapi static void release(HttpConfigParams *params);
 
+  static bool load_server_session_sharing_match(const char *key, MgmtByte &mask);
+
   // parse ssl ports configuration string
   static HttpConfigPortRange *parse_ports_list(char *ports_str);
 
+  // parse redirect configuration string
+  static IpMap *parse_redirect_actions(char *redirect_actions_string, RedirectEnabled::Action &self_action);
+
 public:
   static int m_id;
   static HttpConfigParams m_master;
@@ -923,17 +884,43 @@ inline HttpConfigParams::~HttpConfigParams()
   ats_free(proxy_response_via_string);
   ats_free(anonymize_other_header_list);
   ats_free(oride.body_factory_template_base);
+  ats_free(oride.server_session_sharing_match_str);
   ats_free(oride.proxy_response_server_string);
   ats_free(oride.global_user_agent_header);
-  ats_free(oride.client_cert_filename);
-  ats_free(oride.client_cert_filepath);
-  ats_free(oride.cache_vary_default_text);
-  ats_free(oride.cache_vary_default_images);
-  ats_free(oride.cache_vary_default_other);
+  ats_free(oride.ssl_client_cert_filename);
+  ats_free(oride.ssl_client_private_key_filename);
+  ats_free(oride.ssl_client_ca_cert_filename);
   ats_free(connect_ports_string);
   ats_free(reverse_proxy_no_host_redirect);
+  ats_free(redirect_actions_string);
+  ats_free(oride.ssl_client_sni_policy);
+  ats_free(oride.host_res_data.conf_value);
 
-  if (connect_ports) {
-    delete connect_ports;
-  }
+  delete connect_ports;
+  delete redirect_actions_map;
 }
+
+/** Enable a dynamic configuration variable.
+ *
+ * @param name Configuration var name.
+ * @param cb Callback to do the actual update of the master record.
+ * @param cookie Extra data for @a cb
+ *
+ * The purpose of this is to unite the different ways and times a configuration variable needs
+ * to be loaded. These are
+ * - Process start.
+ * - Dynamic update.
+ * - Plugin API update.
+ *
+ * @a cb is expected to perform the update. It must return a @c bool which is
+ * - @c true if the value was changed.
+ * - @c false if the value was not changed.
+ *
+ * Based on that, a run time configuration update is triggered or not.
+ *
+ * In addition, this invokes @a cb and passes it the information in the configuration variable
+ * global table in order to perform the initial loading of the value. No update is triggered for
+ * that call as it is not needed.
+ *
+ */
+extern void Enable_Config_Var(std::string_view const &name, bool (*cb)(const char *, RecDataT, RecData, void *), void *cookie);
