@@ -19,6 +19,7 @@
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
+
  */
 
 #pragma once
@@ -27,12 +28,14 @@
 
 #include "tscore/ink_config.h"
 #include "AclFiltering.h"
-#include "Main.h"
 #include "URL.h"
 #include "RemapPluginInfo.h"
+#include "PluginFactory.h"
 #include "tscore/Regex.h"
 #include "tscore/List.h"
 
+class NextHopSelectionStrategy;
+
 /**
  * Used to store http referer strings (and/or regexp)
  **/
@@ -56,7 +59,7 @@ public:
 class redirect_tag_str
 {
 public:
-  redirect_tag_str() : next(nullptr), chunk_str(nullptr), type(0) {}
+  redirect_tag_str() {}
   ~redirect_tag_str()
   {
     type = 0;
@@ -66,9 +69,9 @@ public:
     }
   }
 
-  redirect_tag_str *next;
-  char *chunk_str;
-  char type; /* s - string, r - referer, t - url_to, f - url_from, o - origin url */
+  redirect_tag_str *next = nullptr;
+  char *chunk_str        = nullptr;
+  char type              = 0; /* s - string, r - referer, t - url_to, f - url_from, o - origin url */
   static redirect_tag_str *parse_format_redirect_url(char *url);
 };
 
@@ -80,22 +83,20 @@ class url_mapping
 public:
   ~url_mapping();
 
-  bool add_plugin(remap_plugin_info *i, void *ih);
-  remap_plugin_info *get_plugin(std::size_t) const;
-  void *get_instance(std::size_t) const;
+  bool add_plugin_instance(RemapPluginInst *i);
+  RemapPluginInst *get_plugin_instance(std::size_t) const;
 
   std::size_t
-  plugin_count() const
+  plugin_instance_count() const
   {
-    return _plugin_list.size();
+    return _plugin_inst_list.size();
   }
 
-  void delete_instance(unsigned int index);
-  void Print();
+  void Print() const;
 
   int from_path_len = 0;
   URL fromURL;
-  URL toUrl; // Default TO-URL (from remap.config)
+  URL toURL; // Default TO-URL (from remap.config)
   bool homePageRedirect              = false;
   bool unique                        = false; // INKqa11970 - unique mapping
   bool default_redirect_url          = false;
@@ -110,6 +111,7 @@ public:
   bool ip_allow_check_enabled_p      = false;
   acl_filter_rule *filter            = nullptr; // acl filtering (list of rules)
   LINK(url_mapping, link);                      // For use with the main Queue linked list holding all the mapping
+  std::shared_ptr<NextHopSelectionStrategy> strategy = nullptr;
 
   int
   getRank() const
@@ -123,8 +125,7 @@ public:
   };
 
 private:
-  std::vector<remap_plugin_info *> _plugin_list;
-  std::vector<void *> _instance_data;
+  std::vector<RemapPluginInst *> _plugin_inst_list;
   int _rank = 0;
 };
 
@@ -135,8 +136,8 @@ private:
 class UrlMappingContainer
 {
 public:
-  UrlMappingContainer() : _mapping(nullptr), _toURLPtr(nullptr), _heap(nullptr) {}
-  explicit UrlMappingContainer(HdrHeap *heap) : _mapping(nullptr), _toURLPtr(nullptr), _heap(heap) {}
+  UrlMappingContainer() {}
+  explicit UrlMappingContainer(HdrHeap *heap) : _heap(heap) {}
   ~UrlMappingContainer() { deleteToURL(); }
   URL *
   getToURL() const
@@ -160,7 +161,7 @@ public:
   {
     deleteToURL();
     _mapping  = m;
-    _toURLPtr = m ? &(m->toUrl) : nullptr;
+    _toURLPtr = m ? &(m->toURL) : nullptr;
   }
 
   void
@@ -201,8 +202,8 @@ public:
   UrlMappingContainer &operator=(const UrlMappingContainer &rhs) = delete;
 
 private:
-  url_mapping *_mapping;
-  URL *_toURLPtr;
+  url_mapping *_mapping = nullptr;
+  URL *_toURLPtr        = nullptr;
   URL _toURL;
-  HdrHeap *_heap;
+  HdrHeap *_heap = nullptr;
 };
