@@ -22,14 +22,18 @@
  */
 
 #include "HPACK.h"
-#include "HuffmanCodec.h"
 
+#include "tscpp/util/LocalBuffer.h"
+#include "tscpp/util/TextView.h"
+
+namespace
+{
 // [RFC 7541] 4.1. Calculating Table Size
 // The size of an entry is the sum of its name's length in octets (as defined in Section 5.2),
 // its value's length in octets, and 32.
 const static unsigned ADDITIONAL_OCTETS = 32;
 
-typedef enum {
+using TS_HPACK_STATIC_TABLE_ENTRY = enum {
   TS_HPACK_STATIC_TABLE_0 = 0,
   TS_HPACK_STATIC_TABLE_AUTHORITY,
   TS_HPACK_STATIC_TABLE_METHOD_GET,
@@ -93,78 +97,73 @@ typedef enum {
   TS_HPACK_STATIC_TABLE_VIA,
   TS_HPACK_STATIC_TABLE_WWW_AUTHENTICATE,
   TS_HPACK_STATIC_TABLE_ENTRY_NUM
-} TS_HPACK_STATIC_TABLE_ENTRY;
-
-struct StaticTable {
-  StaticTable(const char *n, const char *v) : name(n), value(v), name_size(strlen(name)), value_size(strlen(value)) {}
-  const char *name;
-  const char *value;
-  const int name_size;
-  const int value_size;
 };
 
-static const StaticTable STATIC_TABLE[] = {{"", ""},
-                                           {":authority", ""},
-                                           {":method", "GET"},
-                                           {":method", "POST"},
-                                           {":path", "/"},
-                                           {":path", "/index.html"},
-                                           {":scheme", "http"},
-                                           {":scheme", "https"},
-                                           {":status", "200"},
-                                           {":status", "204"},
-                                           {":status", "206"},
-                                           {":status", "304"},
-                                           {":status", "400"},
-                                           {":status", "404"},
-                                           {":status", "500"},
-                                           {"accept-charset", ""},
-                                           {"accept-encoding", "gzip, deflate"},
-                                           {"accept-language", ""},
-                                           {"accept-ranges", ""},
-                                           {"accept", ""},
-                                           {"access-control-allow-origin", ""},
-                                           {"age", ""},
-                                           {"allow", ""},
-                                           {"authorization", ""},
-                                           {"cache-control", ""},
-                                           {"content-disposition", ""},
-                                           {"content-encoding", ""},
-                                           {"content-language", ""},
-                                           {"content-length", ""},
-                                           {"content-location", ""},
-                                           {"content-range", ""},
-                                           {"content-type", ""},
-                                           {"cookie", ""},
-                                           {"date", ""},
-                                           {"etag", ""},
-                                           {"expect", ""},
-                                           {"expires", ""},
-                                           {"from", ""},
-                                           {"host", ""},
-                                           {"if-match", ""},
-                                           {"if-modified-since", ""},
-                                           {"if-none-match", ""},
-                                           {"if-range", ""},
-                                           {"if-unmodified-since", ""},
-                                           {"last-modified", ""},
-                                           {"link", ""},
-                                           {"location", ""},
-                                           {"max-forwards", ""},
-                                           {"proxy-authenticate", ""},
-                                           {"proxy-authorization", ""},
-                                           {"range", ""},
-                                           {"referer", ""},
-                                           {"refresh", ""},
-                                           {"retry-after", ""},
-                                           {"server", ""},
-                                           {"set-cookie", ""},
-                                           {"strict-transport-security", ""},
-                                           {"transfer-encoding", ""},
-                                           {"user-agent", ""},
-                                           {"vary", ""},
-                                           {"via", ""},
-                                           {"www-authenticate", ""}};
+constexpr HpackHeaderField STATIC_TABLE[] = {{"", ""},
+                                             {":authority", ""},
+                                             {":method", "GET"},
+                                             {":method", "POST"},
+                                             {":path", "/"},
+                                             {":path", "/index.html"},
+                                             {":scheme", "http"},
+                                             {":scheme", "https"},
+                                             {":status", "200"},
+                                             {":status", "204"},
+                                             {":status", "206"},
+                                             {":status", "304"},
+                                             {":status", "400"},
+                                             {":status", "404"},
+                                             {":status", "500"},
+                                             {"accept-charset", ""},
+                                             {"accept-encoding", "gzip, deflate"},
+                                             {"accept-language", ""},
+                                             {"accept-ranges", ""},
+                                             {"accept", ""},
+                                             {"access-control-allow-origin", ""},
+                                             {"age", ""},
+                                             {"allow", ""},
+                                             {"authorization", ""},
+                                             {"cache-control", ""},
+                                             {"content-disposition", ""},
+                                             {"content-encoding", ""},
+                                             {"content-language", ""},
+                                             {"content-length", ""},
+                                             {"content-location", ""},
+                                             {"content-range", ""},
+                                             {"content-type", ""},
+                                             {"cookie", ""},
+                                             {"date", ""},
+                                             {"etag", ""},
+                                             {"expect", ""},
+                                             {"expires", ""},
+                                             {"from", ""},
+                                             {"host", ""},
+                                             {"if-match", ""},
+                                             {"if-modified-since", ""},
+                                             {"if-none-match", ""},
+                                             {"if-range", ""},
+                                             {"if-unmodified-since", ""},
+                                             {"last-modified", ""},
+                                             {"link", ""},
+                                             {"location", ""},
+                                             {"max-forwards", ""},
+                                             {"proxy-authenticate", ""},
+                                             {"proxy-authorization", ""},
+                                             {"range", ""},
+                                             {"referer", ""},
+                                             {"refresh", ""},
+                                             {"retry-after", ""},
+                                             {"server", ""},
+                                             {"set-cookie", ""},
+                                             {"strict-transport-security", ""},
+                                             {"transfer-encoding", ""},
+                                             {"user-agent", ""},
+                                             {"vary", ""},
+                                             {"via", ""},
+                                             {"www-authenticate", ""}};
+
+constexpr std::string_view HPACK_HDR_FIELD_COOKIE        = STATIC_TABLE[TS_HPACK_STATIC_TABLE_COOKIE].name;
+constexpr std::string_view HPACK_HDR_FIELD_AUTHORIZATION = STATIC_TABLE[TS_HPACK_STATIC_TABLE_AUTHORIZATION].name;
 
 /**
   Threshold for total HdrHeap size which used by HPAK Dynamic Table.
@@ -181,10 +180,10 @@ static const StaticTable STATIC_TABLE[] = {{"", ""},
 */
 static constexpr uint32_t HPACK_HDR_HEAP_THRESHOLD = sizeof(MIMEHdrImpl) + sizeof(MIMEFieldBlockImpl) * (2 + 7 + 15);
 
-/******************
- * Local functions
- ******************/
-static inline bool
+//
+// Local functions
+//
+bool
 hpack_field_is_literal(HpackField ftype)
 {
   return ftype == HpackField::INDEXED_LITERAL || ftype == HpackField::NOINDEX_LITERAL || ftype == HpackField::NEVERINDEX_LITERAL;
@@ -218,60 +217,57 @@ hpack_parse_field_type(uint8_t ftype)
   return HpackField::NOINDEX_LITERAL;
 }
 
-/************************
- * HpackIndexingTable
- ************************/
-HpackLookupResult
-HpackIndexingTable::lookup(const MIMEFieldWrapper &field) const
+//
+// HpackStaticTable
+//
+namespace HpackStaticTable
 {
-  int target_name_len = 0, target_value_len = 0;
-  const char *target_name  = field.name_get(&target_name_len);
-  const char *target_value = field.value_get(&target_value_len);
-  return lookup(target_name, target_name_len, target_value, target_value_len);
-}
+  HpackLookupResult
+  lookup(const HpackHeaderField &header)
+  {
+    HpackLookupResult result;
 
-HpackLookupResult
-HpackIndexingTable::lookup(const char *name, int name_len, const char *value, int value_len) const
-{
-  HpackLookupResult result;
-  const unsigned int entry_num = TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table->length();
+    for (unsigned int index = 1; index < TS_HPACK_STATIC_TABLE_ENTRY_NUM; ++index) {
+      std::string_view name  = STATIC_TABLE[index].name;
+      std::string_view value = STATIC_TABLE[index].value;
 
-  for (unsigned int index = 1; index < entry_num; ++index) {
-    const char *table_name, *table_value;
-    int table_name_len = 0, table_value_len = 0;
-
-    if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
-      // static table
-      table_name      = STATIC_TABLE[index].name;
-      table_value     = STATIC_TABLE[index].value;
-      table_name_len  = STATIC_TABLE[index].name_size;
-      table_value_len = STATIC_TABLE[index].value_size;
-    } else {
-      // dynamic table
-      const MIMEField *m_field = _dynamic_table->get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
-
-      table_name  = m_field->name_get(&table_name_len);
-      table_value = m_field->value_get(&table_value_len);
-    }
-
-    // Check whether name (and value) are matched
-    if (ptr_len_casecmp(name, name_len, table_name, table_name_len) == 0) {
-      if ((value_len == table_value_len) && (memcmp(value, table_value, value_len) == 0)) {
-        result.index      = index;
-        result.match_type = HpackMatch::EXACT;
-        break;
-      } else if (!result.index) {
-        result.index      = index;
-        result.match_type = HpackMatch::NAME;
+      // Check whether name (and value) are matched
+      if (memcmp(header.name, name) == 0) {
+        if (memcmp(header.value, value) == 0) {
+          result.index      = index;
+          result.index_type = HpackIndex::STATIC;
+          result.match_type = HpackMatch::EXACT;
+          break;
+        } else if (!result.index) {
+          result.index      = index;
+          result.index_type = HpackIndex::STATIC;
+          result.match_type = HpackMatch::NAME;
+        }
       }
     }
+
+    return result;
   }
-  if (result.match_type != HpackMatch::NONE) {
-    if (result.index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
-      result.index_type = HpackIndex::STATIC;
-    } else {
-      result.index_type = HpackIndex::DYNAMIC;
-    }
+} // namespace HpackStaticTable
+} // namespace
+
+//
+// HpackIndexingTable
+//
+HpackLookupResult
+HpackIndexingTable::lookup(const HpackHeaderField &header) const
+{
+  // static table
+  HpackLookupResult result = HpackStaticTable::lookup(header);
+
+  // if result is not EXACT match, lookup dynamic table
+  if (result.match_type == HpackMatch::EXACT) {
+    return result;
+  }
+
+  // dynamic table
+  if (HpackLookupResult dt_result = this->_dynamic_table.lookup(header); dt_result.match_type == HpackMatch::EXACT) {
+    return dt_result;
   }
 
   return result;
@@ -287,11 +283,11 @@ HpackIndexingTable::get_header_field(uint32_t index, MIMEFieldWrapper &field) co
 
   if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
     // static table
-    field.name_set(STATIC_TABLE[index].name, STATIC_TABLE[index].name_size);
-    field.value_set(STATIC_TABLE[index].value, STATIC_TABLE[index].value_size);
-  } else if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table->length()) {
+    field.name_set(STATIC_TABLE[index].name.data(), STATIC_TABLE[index].name.size());
+    field.value_set(STATIC_TABLE[index].value.data(), STATIC_TABLE[index].value.size());
+  } else if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table.length()) {
     // dynamic table
-    const MIMEField *m_field = _dynamic_table->get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
+    const MIMEField *m_field = _dynamic_table.get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
 
     int name_len, value_len;
     const char *name  = m_field->name_get(&name_len);
@@ -310,32 +306,38 @@ HpackIndexingTable::get_header_field(uint32_t index, MIMEFieldWrapper &field) co
 }
 
 void
-HpackIndexingTable::add_header_field(const MIMEField *field)
+HpackIndexingTable::add_header_field(const HpackHeaderField &header)
 {
-  _dynamic_table->add_header_field(field);
+  _dynamic_table.add_header_field(header);
 }
 
 uint32_t
 HpackIndexingTable::maximum_size() const
 {
-  return _dynamic_table->maximum_size();
+  return _dynamic_table.maximum_size();
 }
 
 uint32_t
 HpackIndexingTable::size() const
 {
-  return _dynamic_table->size();
+  return _dynamic_table.size();
 }
 
 void
 HpackIndexingTable::update_maximum_size(uint32_t new_size)
 {
-  _dynamic_table->update_maximum_size(new_size);
+  _dynamic_table.update_maximum_size(new_size);
 }
 
 //
 // HpackDynamicTable
 //
+HpackDynamicTable::HpackDynamicTable(uint32_t size) : _maximum_size(size)
+{
+  _mhdr = new MIMEHdr();
+  _mhdr->create();
+}
+
 HpackDynamicTable::~HpackDynamicTable()
 {
   this->_headers.clear();
@@ -358,12 +360,9 @@ HpackDynamicTable::get_header_field(uint32_t index) const
 }
 
 void
-HpackDynamicTable::add_header_field(const MIMEField *field)
+HpackDynamicTable::add_header_field(const HpackHeaderField &header)
 {
-  int name_len, value_len;
-  const char *name     = field->name_get(&name_len);
-  const char *value    = field->value_get(&value_len);
-  uint32_t header_size = ADDITIONAL_OCTETS + name_len + value_len;
+  uint32_t header_size = ADDITIONAL_OCTETS + header.name.size() + header.value.size();
 
   if (header_size > _maximum_size) {
     // [RFC 7541] 4.4. Entry Eviction When Adding New Entries
@@ -385,13 +384,43 @@ HpackDynamicTable::add_header_field(const MIMEField *field)
     this->_current_size += header_size;
     this->_evict_overflowed_entries();
 
-    MIMEField *new_field = this->_mhdr->field_create(name, name_len);
-    new_field->value_set(this->_mhdr->m_heap, this->_mhdr->m_mime, value, value_len);
+    MIMEField *new_field = this->_mhdr->field_create(header.name.data(), header.name.size());
+    new_field->value_set(this->_mhdr->m_heap, this->_mhdr->m_mime, header.value.data(), header.value.size());
     this->_mhdr->field_attach(new_field);
     this->_headers.push_front(new_field);
   }
 }
 
+HpackLookupResult
+HpackDynamicTable::lookup(const HpackHeaderField &header) const
+{
+  HpackLookupResult result;
+  const unsigned int entry_num = TS_HPACK_STATIC_TABLE_ENTRY_NUM + this->length();
+
+  for (unsigned int index = TS_HPACK_STATIC_TABLE_ENTRY_NUM; index < entry_num; ++index) {
+    const MIMEField *m_field = this->_headers.at(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
+    std::string_view name    = m_field->name_get();
+    std::string_view value   = m_field->value_get();
+
+    // TODO: replace `strcasecmp` with `memcmp`
+    // Check whether name (and value) are matched
+    if (strcasecmp(header.name, name) == 0) {
+      if (memcmp(header.value, value) == 0) {
+        result.index      = index;
+        result.index_type = HpackIndex::DYNAMIC;
+        result.match_type = HpackMatch::EXACT;
+        break;
+      } else if (!result.index) {
+        result.index      = index;
+        result.index_type = HpackIndex::DYNAMIC;
+        result.match_type = HpackMatch::NAME;
+      }
+    }
+  }
+
+  return result;
+}
+
 uint32_t
 HpackDynamicTable::maximum_size() const
 {
@@ -432,17 +461,18 @@ HpackDynamicTable::_evict_overflowed_entries()
     return;
   }
 
-  for (auto h = this->_headers.rbegin(); h != this->_headers.rend(); ++h) {
+  while (!this->_headers.empty()) {
+    auto h = this->_headers.back();
     int name_len, value_len;
-    (*h)->name_get(&name_len);
-    (*h)->value_get(&value_len);
+    h->name_get(&name_len);
+    h->value_get(&value_len);
 
     this->_current_size -= ADDITIONAL_OCTETS + name_len + value_len;
 
     if (this->_mhdr_old && this->_mhdr_old->fields_count() != 0) {
-      this->_mhdr_old->field_delete(*h, false);
+      this->_mhdr_old->field_delete(h, false);
     } else {
-      this->_mhdr->field_delete(*h, false);
+      this->_mhdr->field_delete(h, false);
     }
 
     this->_headers.pop_back();
@@ -477,6 +507,9 @@ HpackDynamicTable::_mime_hdr_gc()
   }
 }
 
+//
+// Global functions
+//
 int64_t
 encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t index)
 {
@@ -505,7 +538,7 @@ encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t
 }
 
 int64_t
-encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
+encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t *buf_end, const HpackHeaderField &header,
                                               uint32_t index, HpackIndexingTable &indexing_table, HpackField type)
 {
   uint8_t *p = buf_start;
@@ -516,7 +549,7 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
 
   switch (type) {
   case HpackField::INDEXED_LITERAL:
-    indexing_table.add_header_field(header.field_get());
+    indexing_table.add_header_field(header);
     prefix = 6;
     flag   = 0x40;
     break;
@@ -547,20 +580,18 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   p += len;
 
   // Value String
-  int value_len;
-  const char *value = header.value_get(&value_len);
-  len               = xpack_encode_string(p, buf_end, value, value_len);
+  len = xpack_encode_string(p, buf_end, header.value.data(), header.value.size());
   if (len == -1) {
     return -1;
   }
   p += len;
 
-  Debug("hpack_encode", "Encoded field: %d: %.*s", index, value_len, value);
+  Debug("hpack_encode", "Encoded field: %d: %.*s", index, static_cast<int>(header.value.size()), header.value.data());
   return p - buf_start;
 }
 
 int64_t
-encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
+encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf_end, const HpackHeaderField &header,
                                           HpackIndexingTable &indexing_table, HpackField type)
 {
   uint8_t *p = buf_start;
@@ -571,7 +602,7 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
 
   switch (type) {
   case HpackField::INDEXED_LITERAL:
-    indexing_table.add_header_field(header.field_get());
+    indexing_table.add_header_field(header);
     flag = 0x40;
     break;
   case HpackField::NOINDEX_LITERAL:
@@ -588,34 +619,24 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   }
   *(p++) = flag;
 
-  // Convert field name to lower case to follow HTTP2 spec.
-  // This conversion is needed because WKSs in MIMEFields is old fashioned
-  Arena arena;
-  int name_len;
-  const char *name = header.name_get(&name_len);
-  char *lower_name = arena.str_store(name, name_len);
-  for (int i = 0; i < name_len; i++) {
-    lower_name[i] = ParseRules::ink_tolower(lower_name[i]);
-  }
-
   // Name String
-  len = xpack_encode_string(p, buf_end, lower_name, name_len);
+  len = xpack_encode_string(p, buf_end, header.name.data(), header.name.size());
   if (len == -1) {
     return -1;
   }
   p += len;
 
   // Value String
-  int value_len;
-  const char *value = header.value_get(&value_len);
-  len               = xpack_encode_string(p, buf_end, value, value_len);
+  len = xpack_encode_string(p, buf_end, header.value.data(), header.value.size());
   if (len == -1) {
     return -1;
   }
 
   p += len;
 
-  Debug("hpack_encode", "Encoded field: %.*s: %.*s", name_len, name, value_len, value);
+  Debug("hpack_encode", "Encoded field: %.*s: %.*s", static_cast<int>(header.name.size()), header.name.data(),
+        static_cast<int>(header.value.size()), header.value.data());
+
   return p - buf_start;
 }
 
@@ -738,7 +759,8 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 
   // Incremental Indexing adds header to header table as new entry
   if (isIncremental) {
-    indexing_table.add_header_field(header.field_get());
+    const MIMEField *field = header.field_get();
+    indexing_table.add_header_field({field->name_get(), field->value_get()});
   }
 
   // Print decoded header field
@@ -847,7 +869,11 @@ hpack_decode_header_block(HpackIndexingTable &indexing_table, HTTPHdr *hdr, cons
 
     field->name_get(&name_len);
     field->value_get(&value_len);
-    total_header_size += name_len + value_len;
+
+    // [RFC 7540] 6.5.2. SETTINGS_MAX_HEADER_LIST_SIZE:
+    // The value is based on the uncompressed size of header fields, including the length of the name and value in octets plus an
+    // overhead of 32 octets for each header field.
+    total_header_size += name_len + value_len + ADDITIONAL_OCTETS;
 
     if (total_header_size > max_header_size) {
       return HPACK_ERROR_SIZE_EXCEEDED_ERROR;
@@ -870,14 +896,13 @@ hpack_encode_header_block(HpackIndexingTable &indexing_table, uint8_t *out_buf,
 {
   uint8_t *cursor                  = out_buf;
   const uint8_t *const out_buf_end = out_buf + out_buf_len;
-  int64_t written                  = 0;
 
   ink_assert(http_hdr_type_get(hdr->m_http) != HTTP_TYPE_UNKNOWN);
 
   // Update dynamic table size
   if (maximum_table_size >= 0) {
     indexing_table.update_maximum_size(maximum_table_size);
-    written = encode_dynamic_table_size_update(cursor, out_buf_end, maximum_table_size);
+    int64_t written = encode_dynamic_table_size_update(cursor, out_buf_end, maximum_table_size);
     if (written == HPACK_ERROR_COMPRESSION_ERROR) {
       return HPACK_ERROR_COMPRESSION_ERROR;
     }
@@ -886,22 +911,33 @@ hpack_encode_header_block(HpackIndexingTable &indexing_table, uint8_t *out_buf,
 
   MIMEFieldIter field_iter;
   for (MIMEField *field = hdr->iter_get_first(&field_iter); field != nullptr; field = hdr->iter_get_next(&field_iter)) {
-    HpackField field_type;
-    MIMEFieldWrapper header(field, hdr->m_heap, hdr->m_http->m_fields_impl);
-    int name_len;
-    int value_len;
-    const char *name = header.name_get(&name_len);
-    header.value_get(&value_len);
+    // Convert field name to lower case to follow HTTP2 spec
+    // This conversion is needed because WKSs in MIMEFields is old fashioned
+    std::string_view original_name = field->name_get();
+    int name_len                   = original_name.size();
+    ts::LocalBuffer<char> local_buffer(name_len);
+    char *lower_name = local_buffer.data();
+    for (int i = 0; i < name_len; i++) {
+      lower_name[i] = ParseRules::ink_tolower(original_name[i]);
+    }
+
+    std::string_view name{lower_name, static_cast<size_t>(name_len)};
+    std::string_view value = field->value_get();
+
     // Choose field representation (See RFC7541 7.1.3)
     // - Authorization header obviously should not be indexed
     // - Short Cookie header should not be indexed because of low entropy
-    if ((ptr_len_casecmp(name, name_len, MIME_FIELD_COOKIE, MIME_LEN_COOKIE) == 0 && value_len < 20) ||
-        (ptr_len_casecmp(name, name_len, MIME_FIELD_AUTHORIZATION, MIME_LEN_AUTHORIZATION) == 0)) {
+    HpackField field_type;
+    if ((value.size() < 20 && memcmp(name, HPACK_HDR_FIELD_COOKIE) == 0) || memcmp(name, HPACK_HDR_FIELD_AUTHORIZATION) == 0) {
       field_type = HpackField::NEVERINDEX_LITERAL;
     } else {
       field_type = HpackField::INDEXED_LITERAL;
     }
+
+    HpackHeaderField header{name, value};
     const HpackLookupResult result = indexing_table.lookup(header);
+
+    int64_t written = 0;
     switch (result.match_type) {
     case HpackMatch::NONE:
       written = encode_literal_header_field_with_new_name(cursor, out_buf_end, header, indexing_table, field_type);
@@ -914,10 +950,9 @@ hpack_encode_header_block(HpackIndexingTable &indexing_table, uint8_t *out_buf,
       written = encode_indexed_header_field(cursor, out_buf_end, result.index);
       break;
     default:
-      // Does it happen?
-      written = 0;
       break;
     }
+
     if (written == HPACK_ERROR_COMPRESSION_ERROR) {
       return HPACK_ERROR_COMPRESSION_ERROR;
     }
