@@ -23,7 +23,10 @@
 
 #include "HTTP2.h"
 #include "HPACK.h"
+
 #include "tscore/ink_assert.h"
+#include "tscpp/util/LocalBuffer.h"
+
 #include "records/P_RecCore.h"
 #include "records/P_RecProcess.h"
 
@@ -45,10 +48,19 @@ const unsigned HTTP2_LEN_STATUS    = countof(":status") - 1;
 static size_t HTTP2_LEN_STATUS_VALUE_STR         = 3;
 static const uint32_t HTTP2_MAX_TABLE_SIZE_LIMIT = 64 * 1024;
 
+namespace
+{
+struct Http2HeaderName {
+  const char *name = nullptr;
+  int name_len     = 0;
+};
+
+Http2HeaderName http2_connection_specific_headers[5] = {};
+} // namespace
+
 // Statistics
 RecRawStatBlock *http2_rsb;
-static const char *const HTTP2_STAT_CURRENT_CLIENT_SESSION_NAME    = "proxy.process.http2.current_client_sessions"; // DEPRECATED
-static const char *const HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME = "proxy.process.http2.current_client_connections";
+static const char *const HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME        = "proxy.process.http2.current_client_connections";
 static const char *const HTTP2_STAT_CURRENT_ACTIVE_CLIENT_CONNECTION_NAME = "proxy.process.http2.current_active_client_connections";
 static const char *const HTTP2_STAT_CURRENT_CLIENT_STREAM_NAME            = "proxy.process.http2.current_client_streams";
 static const char *const HTTP2_STAT_TOTAL_CLIENT_STREAM_NAME              = "proxy.process.http2.total_client_streams";
@@ -71,10 +83,6 @@ static const char *const HTTP2_STAT_MAX_PING_FRAMES_PER_MINUTE_EXCEEDED_NAME =
   "proxy.process.http2.max_ping_frames_per_minute_exceeded";
 static const char *const HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED_NAME =
   "proxy.process.http2.max_priority_frames_per_minute_exceeded";
-static const char *const HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED_NAME =
-  "proxy.process.http2.max_rst_stream_frames_per_minute_exceeded";
-static const char *const HTTP2_STAT_MAX_CONTINUATION_FRAMES_PER_MINUTE_EXCEEDED_NAME =
-  "proxy.process.http2.max_continuation_frames_per_minute_exceeded";
 static const char *const HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE_NAME = "proxy.process.http2.insufficient_avg_window_update";
 
 union byte_pointer {
@@ -102,6 +110,7 @@ write_and_advance(byte_pointer &dst, uint32_t src)
 {
   byte_addressable_value<uint32_t> pval;
 
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   pval.value = htonl(src);
   memcpy(dst.u8, pval.bytes, sizeof(pval.bytes));
   dst.u8 += sizeof(pval.bytes);
@@ -112,6 +121,7 @@ write_and_advance(byte_pointer &dst, uint16_t src)
 {
   byte_addressable_value<uint16_t> pval;
 
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   pval.value = htons(src);
   memcpy(dst.u8, pval.bytes, sizeof(pval.bytes));
   dst.u8 += sizeof(pval.bytes);
@@ -232,6 +242,7 @@ http2_write_frame_header(const Http2FrameHeader &hdr, IOVec iov)
   }
 
   byte_addressable_value<uint32_t> length;
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   length.value = htonl(hdr.length);
   // MSB length.bytes[0] is unused.
   write_and_advance(ptr, length.bytes[1]);
@@ -408,105 +419,88 @@ http2_parse_window_update(IOVec iov, uint32_t &size)
 ParseResult
 http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
 {
-  MIMEField *field;
-
   ink_assert(http_hdr_type_get(headers->m_http) != HTTP_TYPE_UNKNOWN);
 
+  // HTTP Version
+  headers->version_set(HTTPVersion(1, 1));
+
   if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_REQUEST) {
-    const char *scheme, *authority, *path, *method;
-    int scheme_len, authority_len, path_len, method_len;
+    // :scheme
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME); field != nullptr && field->value_is_valid()) {
+      int scheme_len;
+      const char *scheme = field->value_get(&scheme_len);
 
-    // Get values of :scheme, :authority and :path to assemble requested URL
-    if ((field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
-      scheme = field->value_get(&scheme_len);
+      int scheme_wks_idx = hdrtoken_tokenize(scheme, scheme_len);
+      url_scheme_set(headers->m_heap, headers->m_http->u.req.m_url_impl, scheme, scheme_wks_idx, scheme_len, true);
 
-      const char *scheme_wks;
+      headers->field_delete(field);
+    } else {
+      return PARSE_RESULT_ERROR;
+    }
 
-      int scheme_wks_idx = hdrtoken_tokenize(scheme, scheme_len, &scheme_wks);
+    // :authority
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
+        field != nullptr && field->value_is_valid()) {
+      int authority_len;
+      const char *authority = field->value_get(&authority_len);
 
-      if (!(scheme_wks_idx > 0 && hdrtoken_wks_to_token_type(scheme_wks) == HDRTOKEN_TYPE_SCHEME)) {
-        // unkown scheme, validate the scheme
-        if (!validate_scheme({scheme, static_cast<size_t>(scheme_len)})) {
-          return PARSE_RESULT_ERROR;
-        }
+      url_host_set(headers->m_heap, headers->m_http->u.req.m_url_impl, authority, authority_len, true);
+
+      headers->field_delete(field);
+    } else {
+      return PARSE_RESULT_ERROR;
+    }
+
+    // :path
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH); field != nullptr && field->value_is_valid()) {
+      int path_len;
+      const char *path = field->value_get(&path_len);
+
+      // cut first '/' if there, because `url_print()` add '/' before printing path
+      if (path_len >= 1 && path[0] == '/') {
+        ++path;
+        --path_len;
       }
+
+      url_path_set(headers->m_heap, headers->m_http->u.req.m_url_impl, path, path_len, true);
+
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    if ((field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
-      authority = field->value_get(&authority_len);
-    } else {
-      return PARSE_RESULT_ERROR;
-    }
+    // :method
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD); field != nullptr && field->value_is_valid()) {
+      int method_len;
+      const char *method = field->value_get(&method_len);
 
-    if ((field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
-      path = field->value_get(&path_len);
-    } else {
-      return PARSE_RESULT_ERROR;
-    }
-
-    // Parse URL
-    Arena arena;
-    size_t url_length     = scheme_len + 3 + authority_len + path_len;
-    char *url             = arena.str_alloc(url_length);
-    const char *url_start = url;
-
-    memcpy(url, scheme, scheme_len);
-    memcpy(url + scheme_len, "://", 3);
-    memcpy(url + scheme_len + 3, authority, authority_len);
-    memcpy(url + scheme_len + 3 + authority_len, path, path_len);
-    url_parse(headers->m_heap, headers->m_http->u.req.m_url_impl, &url_start, url + url_length, true);
-    arena.str_free(url);
-
-    // Get value of :method
-    if ((field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
-      method = field->value_get(&method_len);
-
-      int method_wks_idx = hdrtoken_tokenize(method, method_len);
-      http_hdr_method_set(headers->m_heap, headers->m_http, method, method_wks_idx, method_len, false);
+      headers->method_set(method, method_len);
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
     // Combine Cookie headers ([RFC 7540] 8.1.2.5.)
-    field = headers->field_find(MIME_FIELD_COOKIE, MIME_LEN_COOKIE);
-    if (field) {
+    if (MIMEField *field = headers->field_find(MIME_FIELD_COOKIE, MIME_LEN_COOKIE); field != nullptr) {
       headers->field_combine_dups(field, true, ';');
     }
-
-    // Convert HTTP version to 1.1
-    int32_t version = HTTP_VERSION(1, 1);
-    http_hdr_version_set(headers->m_http, version);
-
-    // Remove HTTP/2 style headers
-    headers->field_delete(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
-    headers->field_delete(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
-    headers->field_delete(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
-    headers->field_delete(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
   } else {
-    int status_len;
-    const char *status;
+    // Set status from :status
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS); field != nullptr) {
+      int status_len;
+      const char *status = field->value_get(&status_len);
 
-    if ((field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS)) != nullptr) {
-      status = field->value_get(&status_len);
       headers->status_set(http_parse_status(status, status + status_len));
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
-
-    // Remove HTTP/2 style headers
-    headers->field_delete(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
   }
 
   // Check validity of all names and values
   MIMEFieldIter iter;
-  for (const MIMEField *field = headers->iter_get_first(&iter); field != nullptr; field = headers->iter_get_next(&iter)) {
-    if (!field->name_is_valid(is_control_BIT | is_ws_BIT) || !field->value_is_valid()) {
+  for (auto *mf = headers->iter_get_first(&iter); mf != nullptr; mf = headers->iter_get_next(&iter)) {
+    if (!mf->name_is_valid() || !mf->value_is_valid()) {
       return PARSE_RESULT_ERROR;
     }
   }
@@ -514,84 +508,147 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
   return PARSE_RESULT_DONE;
 }
 
+/**
+  Initialize HTTPHdr for HTTP/2
+
+  Reserve HTTP/2 Pseudo-Header Fields in front of HTTPHdr. Value of these header fields will be set by
+  `http2_convert_header_from_1_1_to_2()`. When a HTTPHdr for HTTP/2 headers is created, this should be called immediately.
+  Because all pseudo-header fields MUST appear in the header block before regular header fields.
+ */
 void
-http2_generate_h2_header_from_1_1(HTTPHdr *headers, HTTPHdr *h2_headers)
+http2_init_pseudo_headers(HTTPHdr &hdr)
 {
-  h2_headers->create(http_hdr_type_get(headers->m_http));
+  switch (http_hdr_type_get(hdr.m_http)) {
+  case HTTP_TYPE_REQUEST: {
+    MIMEField *method = hdr.field_create(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
+    hdr.field_attach(method);
 
-  if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_RESPONSE) {
-    // Add ':status' header field
-    char status_str[HTTP2_LEN_STATUS_VALUE_STR + 1];
-    snprintf(status_str, sizeof(status_str), "%d", headers->status_get());
-    MIMEField *status_field = h2_headers->field_create(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
-    status_field->value_set(h2_headers->m_heap, h2_headers->m_mime, status_str, HTTP2_LEN_STATUS_VALUE_STR);
-    h2_headers->field_attach(status_field);
+    MIMEField *scheme = hdr.field_create(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
+    hdr.field_attach(scheme);
 
-  } else if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_REQUEST) {
-    MIMEField *field;
-    const char *value;
-    int value_len;
+    MIMEField *authority = hdr.field_create(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
+    hdr.field_attach(authority);
 
-    // Add ':authority' header field
-    field = h2_headers->field_create(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
-    value = headers->host_get(&value_len);
-    if (headers->is_port_in_header()) {
-      int port            = headers->port_get();
-      char *host_and_port = (char *)ats_malloc(value_len + 8);
-      value_len           = snprintf(host_and_port, value_len + 8, "%.*s:%d", value_len, value, port);
-      field->value_set(h2_headers->m_heap, h2_headers->m_mime, host_and_port, value_len);
-      ats_free(host_and_port);
+    MIMEField *path = hdr.field_create(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
+    hdr.field_attach(path);
+
+    break;
+  }
+  case HTTP_TYPE_RESPONSE: {
+    MIMEField *status = hdr.field_create(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
+    hdr.field_attach(status);
+
+    break;
+  }
+  default:
+    ink_abort("HTTP_TYPE_UNKNOWN");
+  }
+}
+
+/**
+  Convert HTTP/1.1 HTTPHdr to HTTP/2
+
+  Assuming HTTP/2 Pseudo-Header Fields are reserved by `http2_init_pseudo_headers()`.
+ */
+ParseResult
+http2_convert_header_from_1_1_to_2(HTTPHdr *headers)
+{
+  switch (http_hdr_type_get(headers->m_http)) {
+  case HTTP_TYPE_REQUEST: {
+    // :method
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD); field != nullptr) {
+      int value_len;
+      const char *value = headers->method_get(&value_len);
+
+      field->value_set(headers->m_heap, headers->m_mime, value, value_len);
     } else {
-      field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
     }
-    h2_headers->field_attach(field);
 
-    // Add ':method' header field
-    field = h2_headers->field_create(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
-    value = headers->method_get(&value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(field);
+    // :scheme
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME); field != nullptr) {
+      int value_len;
+      const char *value = headers->scheme_get(&value_len);
 
-    // Add ':path' header field
-    field      = h2_headers->field_create(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
-    value      = headers->path_get(&value_len);
-    char *path = (char *)ats_malloc(value_len + 1);
-    path[0]    = '/';
-    memcpy(path + 1, value, value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, path, value_len + 1);
-    ats_free(path);
-    h2_headers->field_attach(field);
+      if (value != nullptr) {
+        field->value_set(headers->m_heap, headers->m_mime, value, value_len);
+      } else {
+        field->value_set(headers->m_heap, headers->m_mime, URL_SCHEME_HTTPS, URL_LEN_HTTPS);
+      }
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
 
-    // Add ':scheme' header field
-    field = h2_headers->field_create(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
-    value = headers->scheme_get(&value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(field);
+    // :authority
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY); field != nullptr) {
+      int value_len;
+      const char *value = headers->host_get(&value_len);
+
+      if (headers->is_port_in_header()) {
+        int port = headers->port_get();
+        ts::LocalBuffer<char> buf(value_len + 8);
+        char *host_and_port = buf.data();
+        value_len           = snprintf(host_and_port, value_len + 8, "%.*s:%d", value_len, value, port);
+
+        field->value_set(headers->m_heap, headers->m_mime, host_and_port, value_len);
+      } else {
+        field->value_set(headers->m_heap, headers->m_mime, value, value_len);
+      }
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+
+    // :path
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH); field != nullptr) {
+      int value_len;
+      const char *value = headers->path_get(&value_len);
+
+      ts::LocalBuffer<char> buf(value_len + 1);
+      char *path = buf.data();
+      path[0]    = '/';
+      memcpy(path + 1, value, value_len);
+
+      field->value_set(headers->m_heap, headers->m_mime, path, value_len + 1);
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+
+    // TODO: remove host/Host header
+    // [RFC 7540] 8.1.2.3. Clients that generate HTTP/2 requests directly SHOULD use the ":authority" pseudo-header field instead
+    // of the Host header field.
+
+    break;
+  }
+  case HTTP_TYPE_RESPONSE: {
+    // :status
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS); field != nullptr) {
+      // ink_small_itoa() requires 5+ buffer length
+      char status_str[HTTP2_LEN_STATUS_VALUE_STR + 3];
+      mime_format_int(status_str, headers->status_get(), sizeof(status_str));
+
+      field->value_set(headers->m_heap, headers->m_mime, status_str, HTTP2_LEN_STATUS_VALUE_STR);
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+    break;
+  }
+  default:
+    ink_abort("HTTP_TYPE_UNKNOWN");
   }
 
-  // Copy headers
   // Intermediaries SHOULD remove connection-specific header fields.
-  MIMEFieldIter field_iter;
-  for (MIMEField *field = headers->iter_get_first(&field_iter); field != nullptr; field = headers->iter_get_next(&field_iter)) {
-    const char *name;
-    int name_len;
-    const char *value;
-    int value_len;
-    name = field->name_get(&name_len);
-    if ((name_len == MIME_LEN_CONNECTION && strncasecmp(name, MIME_FIELD_CONNECTION, name_len) == 0) ||
-        (name_len == MIME_LEN_KEEP_ALIVE && strncasecmp(name, MIME_FIELD_KEEP_ALIVE, name_len) == 0) ||
-        (name_len == MIME_LEN_PROXY_CONNECTION && strncasecmp(name, MIME_FIELD_PROXY_CONNECTION, name_len) == 0) ||
-        (name_len == MIME_LEN_TRANSFER_ENCODING && strncasecmp(name, MIME_FIELD_TRANSFER_ENCODING, name_len) == 0) ||
-        (name_len == MIME_LEN_UPGRADE && strncasecmp(name, MIME_FIELD_UPGRADE, name_len) == 0)) {
-      continue;
+  for (auto &h : http2_connection_specific_headers) {
+    if (MIMEField *field = headers->field_find(h.name, h.name_len); field != nullptr) {
+      headers->field_delete(field);
     }
-    MIMEField *newfield;
-    name     = field->name_get(&name_len);
-    newfield = h2_headers->field_create(name, name_len);
-    value    = field->value_get(&value_len);
-    newfield->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(newfield);
   }
+
+  return PARSE_RESULT_DONE;
 }
 
 Http2ErrorCode
@@ -605,6 +662,7 @@ http2_encode_header_blocks(HTTPHdr *in, uint8_t *out, uint32_t out_len, uint32_t
   if (maximum_table_size == hpack_get_maximum_table_size(handle)) {
     maximum_table_size = -1;
   }
+
   // TODO: It would be better to split Cookie header value
   int64_t result = hpack_encode_header_block(handle, out, out_len, in, maximum_table_size);
   if (result < 0) {
@@ -664,13 +722,6 @@ http2_decode_header_blocks(HTTPHdr *hdr, const uint8_t *buf_start, const uint32_
         return Http2ErrorCode::HTTP2_ERROR_PROTOCOL_ERROR;
       }
     }
-    // Check whether header field name is lower case
-    // This check should be here but it will fail because WKSs in MIMEField is old fashioned.
-    // for (uint32_t i = 0; i < len; ++i) {
-    //   if (ParseRules::is_upalpha(value[i])) {
-    //     return Http2ErrorCode::HTTP2_ERROR_PROTOCOL_ERROR;
-    //   }
-    // }
   }
 
   // rfc7540,sec8.1.2.2: Any message containing connection-specific header
@@ -730,32 +781,33 @@ http2_decode_header_blocks(HTTPHdr *hdr, const uint8_t *buf_start, const uint32_
 }
 
 // Initialize this subsystem with librecords configs (for now)
-uint32_t Http2::max_concurrent_streams_in          = 100;
-uint32_t Http2::min_concurrent_streams_in          = 10;
-uint32_t Http2::max_active_streams_in              = 0;
-bool Http2::throttling                             = false;
-uint32_t Http2::stream_priority_enabled            = 0;
-uint32_t Http2::initial_window_size                = 65535;
-uint32_t Http2::max_frame_size                     = 16384;
-uint32_t Http2::header_table_size                  = 4096;
-uint32_t Http2::max_header_list_size               = 4294967295;
-uint32_t Http2::accept_no_activity_timeout         = 120;
-uint32_t Http2::no_activity_timeout_in             = 120;
-uint32_t Http2::active_timeout_in                  = 0;
-uint32_t Http2::push_diary_size                    = 256;
-uint32_t Http2::zombie_timeout_in                  = 0;
-float Http2::stream_error_rate_threshold           = 0.1;
-uint32_t Http2::max_settings_per_frame             = 7;
-uint32_t Http2::max_settings_per_minute            = 14;
-uint32_t Http2::max_settings_frames_per_minute     = 14;
-uint32_t Http2::max_ping_frames_per_minute         = 60;
-uint32_t Http2::max_priority_frames_per_minute     = 120;
-uint32_t Http2::max_rst_stream_frames_per_minute   = 200;
-uint32_t Http2::max_continuation_frames_per_minute = 120;
-float Http2::min_avg_window_update                 = 2560.0;
-uint32_t Http2::con_slow_log_threshold             = 0;
-uint32_t Http2::stream_slow_log_threshold          = 0;
-uint32_t Http2::header_table_size_limit            = 65536;
+uint32_t Http2::max_concurrent_streams_in      = 100;
+uint32_t Http2::min_concurrent_streams_in      = 10;
+uint32_t Http2::max_active_streams_in          = 0;
+bool Http2::throttling                         = false;
+uint32_t Http2::stream_priority_enabled        = 0;
+uint32_t Http2::initial_window_size            = 65535;
+uint32_t Http2::max_frame_size                 = 16384;
+uint32_t Http2::header_table_size              = 4096;
+uint32_t Http2::max_header_list_size           = 4294967295;
+uint32_t Http2::accept_no_activity_timeout     = 120;
+uint32_t Http2::no_activity_timeout_in         = 120;
+uint32_t Http2::active_timeout_in              = 0;
+uint32_t Http2::push_diary_size                = 256;
+uint32_t Http2::zombie_timeout_in              = 0;
+float Http2::stream_error_rate_threshold       = 0.1;
+uint32_t Http2::max_settings_per_frame         = 7;
+uint32_t Http2::max_settings_per_minute        = 14;
+uint32_t Http2::max_settings_frames_per_minute = 14;
+uint32_t Http2::max_ping_frames_per_minute     = 60;
+uint32_t Http2::max_priority_frames_per_minute = 120;
+float Http2::min_avg_window_update             = 2560.0;
+uint32_t Http2::con_slow_log_threshold         = 0;
+uint32_t Http2::stream_slow_log_threshold      = 0;
+uint32_t Http2::header_table_size_limit        = 65536;
+uint32_t Http2::write_buffer_block_size        = 262144;
+float Http2::write_size_threshold              = 0.5;
+uint32_t Http2::write_time_threshold           = 100;
 
 void
 Http2::init()
@@ -779,12 +831,13 @@ Http2::init()
   REC_EstablishStaticConfigInt32U(max_settings_frames_per_minute, "proxy.config.http2.max_settings_frames_per_minute");
   REC_EstablishStaticConfigInt32U(max_ping_frames_per_minute, "proxy.config.http2.max_ping_frames_per_minute");
   REC_EstablishStaticConfigInt32U(max_priority_frames_per_minute, "proxy.config.http2.max_priority_frames_per_minute");
-  REC_EstablishStaticConfigInt32U(max_rst_stream_frames_per_minute, "proxy.config.http2.max_rst_stream_frames_per_minute");
-  REC_EstablishStaticConfigInt32U(max_continuation_frames_per_minute, "proxy.config.http2.max_continuation_frames_per_minute");
   REC_EstablishStaticConfigFloat(min_avg_window_update, "proxy.config.http2.min_avg_window_update");
   REC_EstablishStaticConfigInt32U(con_slow_log_threshold, "proxy.config.http2.connection.slow.log.threshold");
   REC_EstablishStaticConfigInt32U(stream_slow_log_threshold, "proxy.config.http2.stream.slow.log.threshold");
   REC_EstablishStaticConfigInt32U(header_table_size_limit, "proxy.config.http2.header_table_size_limit");
+  REC_EstablishStaticConfigInt32U(write_buffer_block_size, "proxy.config.http2.write_buffer_block_size");
+  REC_EstablishStaticConfigFloat(write_size_threshold, "proxy.config.http2.write_size_threshold");
+  REC_EstablishStaticConfigInt32U(write_time_threshold, "proxy.config.http2.write_time_threshold");
 
   // If any settings is broken, ATS should not start
   ink_release_assert(http2_settings_parameter_is_valid({HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS, max_concurrent_streams_in}));
@@ -802,8 +855,6 @@ Http2::init()
 
   // Setup statistics
   http2_rsb = RecAllocateRawStatBlock(static_cast<int>(HTTP2_N_STATS));
-  RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_CURRENT_CLIENT_SESSION_NAME, RECD_INT, RECP_NON_PERSISTENT,
-                     static_cast<int>(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME, RECD_INT, RECP_NON_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT), RecRawStatSyncSum);
   HTTP2_CLEAR_DYN_STAT(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT);
@@ -847,150 +898,27 @@ Http2::init()
                      static_cast<int>(HTTP2_STAT_MAX_PING_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED_NAME, RECD_INT, RECP_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
-  RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED_NAME, RECD_INT, RECP_PERSISTENT,
-                     static_cast<int>(HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
-  RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_MAX_CONTINUATION_FRAMES_PER_MINUTE_EXCEEDED_NAME, RECD_INT,
-                     RECP_PERSISTENT, static_cast<int>(HTTP2_STAT_MAX_CONTINUATION_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE_NAME, RECD_INT, RECP_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE), RecRawStatSyncSum);
+
+  http2_init();
 }
 
-#if TS_HAS_TESTS
-
-void forceLinkRegressionHPACK();
+/**
+  mime_init() needs to be called
+ */
 void
-forceLinkRegressionHPACKCaller()
+http2_init()
 {
-  forceLinkRegressionHPACK();
+  ink_assert(MIME_FIELD_CONNECTION != nullptr);
+  ink_assert(MIME_FIELD_KEEP_ALIVE != nullptr);
+  ink_assert(MIME_FIELD_PROXY_CONNECTION != nullptr);
+  ink_assert(MIME_FIELD_TRANSFER_ENCODING != nullptr);
+  ink_assert(MIME_FIELD_UPGRADE != nullptr);
+
+  http2_connection_specific_headers[0] = {MIME_FIELD_CONNECTION, MIME_LEN_CONNECTION};
+  http2_connection_specific_headers[1] = {MIME_FIELD_KEEP_ALIVE, MIME_LEN_KEEP_ALIVE};
+  http2_connection_specific_headers[2] = {MIME_FIELD_PROXY_CONNECTION, MIME_LEN_PROXY_CONNECTION};
+  http2_connection_specific_headers[3] = {MIME_FIELD_TRANSFER_ENCODING, MIME_LEN_TRANSFER_ENCODING};
+  http2_connection_specific_headers[4] = {MIME_FIELD_UPGRADE, MIME_LEN_UPGRADE};
 }
-
-#include "tscore/TestBox.h"
-
-/***********************************************************************************
- *                                                                                 *
- *                       Regression test for HTTP/2                                *
- *                                                                                 *
- ***********************************************************************************/
-
-const static struct {
-  uint8_t ftype;
-  uint8_t fflags;
-  bool valid;
-} http2_frame_flags_test_case[] = {{HTTP2_FRAME_TYPE_DATA, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x20, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PING, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_PING, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x02, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x10, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x20, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x40, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x80, true}};
-
-static const uint8_t HTTP2_FRAME_FLAGS_MASKS[HTTP2_FRAME_TYPE_MAX] = {
-  HTTP2_FLAGS_DATA_MASK,          HTTP2_FLAGS_HEADERS_MASK,      HTTP2_FLAGS_PRIORITY_MASK, HTTP2_FLAGS_RST_STREAM_MASK,
-  HTTP2_FLAGS_SETTINGS_MASK,      HTTP2_FLAGS_PUSH_PROMISE_MASK, HTTP2_FLAGS_PING_MASK,     HTTP2_FLAGS_GOAWAY_MASK,
-  HTTP2_FLAGS_WINDOW_UPDATE_MASK, HTTP2_FLAGS_CONTINUATION_MASK,
-};
-
-REGRESSION_TEST(HTTP2_FRAME_FLAGS)(RegressionTest *t, int, int *pstatus)
-{
-  TestBox box(t, pstatus);
-  box = REGRESSION_TEST_PASSED;
-
-  for (auto i : http2_frame_flags_test_case) {
-    box.check((i.ftype >= HTTP2_FRAME_TYPE_MAX || (i.fflags & ~HTTP2_FRAME_FLAGS_MASKS[i.ftype]) == 0) == i.valid,
-              "Validation of frame flags (type: %d, flags: %d) are expected %d, but not", i.ftype, i.fflags, i.valid);
-  }
-}
-
-#endif /* TS_HAS_TESTS */
