@@ -118,7 +118,7 @@ public:
   {
   }
 
-  LogAccess(HttpSM *sm);
+  explicit LogAccess(HttpSM *sm);
 
   inkcoreapi ~LogAccess() {}
   inkcoreapi void init();
@@ -143,6 +143,7 @@ public:
   inkcoreapi int marshal_client_req_url_scheme(char *);         // STR
   inkcoreapi int marshal_client_req_http_version(char *);       // INT
   inkcoreapi int marshal_client_req_protocol_version(char *);   // STR
+  inkcoreapi int marshal_server_req_protocol_version(char *);   // STR
   inkcoreapi int marshal_client_req_squid_len(char *);          // INT
   inkcoreapi int marshal_client_req_header_len(char *);         // INT
   inkcoreapi int marshal_client_req_content_len(char *);        // INT
@@ -153,65 +154,72 @@ public:
   inkcoreapi int marshal_client_req_mptcp_state(char *);        // INT
   inkcoreapi int marshal_client_security_protocol(char *);      // STR
   inkcoreapi int marshal_client_security_cipher_suite(char *);  // STR
+  inkcoreapi int marshal_client_security_curve(char *);         // STR
+  inkcoreapi int marshal_client_security_alpn(char *);          // STR
   inkcoreapi int marshal_client_finish_status_code(char *);     // INT
   inkcoreapi int marshal_client_req_id(char *);                 // INT
   inkcoreapi int marshal_client_req_uuid(char *);               // STR
   inkcoreapi int marshal_client_rx_error_code(char *);          // STR
   inkcoreapi int marshal_client_tx_error_code(char *);          // STR
+  inkcoreapi int marshal_client_req_all_header_fields(char *);  // STR
 
   //
   // proxy -> client fields
   //
-  inkcoreapi int marshal_proxy_resp_content_type(char *);  // STR
-  inkcoreapi int marshal_proxy_resp_reason_phrase(char *); // STR
-  inkcoreapi int marshal_proxy_resp_squid_len(char *);     // INT
-  inkcoreapi int marshal_proxy_resp_content_len(char *);   // INT
-  inkcoreapi int marshal_proxy_resp_status_code(char *);   // INT
-  inkcoreapi int marshal_proxy_resp_header_len(char *);    // INT
-  inkcoreapi int marshal_proxy_finish_status_code(char *); // INT
-  inkcoreapi int marshal_cache_result_code(char *);        // INT
-  inkcoreapi int marshal_cache_result_subcode(char *);     // INT
-  inkcoreapi int marshal_proxy_host_port(char *);          // INT
-  inkcoreapi int marshal_cache_hit_miss(char *);           // INT
+  inkcoreapi int marshal_proxy_resp_content_type(char *);      // STR
+  inkcoreapi int marshal_proxy_resp_reason_phrase(char *);     // STR
+  inkcoreapi int marshal_proxy_resp_squid_len(char *);         // INT
+  inkcoreapi int marshal_proxy_resp_content_len(char *);       // INT
+  inkcoreapi int marshal_proxy_resp_status_code(char *);       // INT
+  inkcoreapi int marshal_proxy_resp_header_len(char *);        // INT
+  inkcoreapi int marshal_proxy_finish_status_code(char *);     // INT
+  inkcoreapi int marshal_cache_result_code(char *);            // INT
+  inkcoreapi int marshal_cache_result_subcode(char *);         // INT
+  inkcoreapi int marshal_proxy_host_port(char *);              // INT
+  inkcoreapi int marshal_cache_hit_miss(char *);               // INT
+  inkcoreapi int marshal_proxy_resp_all_header_fields(char *); // STR
 
   //
   // proxy -> server fields
   //
-  inkcoreapi int marshal_proxy_req_header_len(char *);  // INT
-  inkcoreapi int marshal_proxy_req_squid_len(char *);   // INT
-  inkcoreapi int marshal_proxy_req_content_len(char *); // INT
-  inkcoreapi int marshal_proxy_req_server_ip(char *);   // INT
-  inkcoreapi int marshal_proxy_req_server_port(char *); // INT
-  inkcoreapi int marshal_proxy_hierarchy_route(char *); // INT
-  inkcoreapi int marshal_next_hop_ip(char *);           // STR
-  inkcoreapi int marshal_next_hop_port(char *);         // INT
-  inkcoreapi int marshal_proxy_host_name(char *);       // STR
-  inkcoreapi int marshal_proxy_host_ip(char *);         // STR
-  inkcoreapi int marshal_proxy_req_is_ssl(char *);      // INT
+  inkcoreapi int marshal_proxy_req_header_len(char *);        // INT
+  inkcoreapi int marshal_proxy_req_squid_len(char *);         // INT
+  inkcoreapi int marshal_proxy_req_content_len(char *);       // INT
+  inkcoreapi int marshal_proxy_req_server_ip(char *);         // INT
+  inkcoreapi int marshal_proxy_req_server_port(char *);       // INT
+  inkcoreapi int marshal_proxy_hierarchy_route(char *);       // INT
+  inkcoreapi int marshal_next_hop_ip(char *);                 // STR
+  inkcoreapi int marshal_next_hop_port(char *);               // INT
+  inkcoreapi int marshal_proxy_host_name(char *);             // STR
+  inkcoreapi int marshal_proxy_host_ip(char *);               // STR
+  inkcoreapi int marshal_proxy_req_is_ssl(char *);            // INT
+  inkcoreapi int marshal_proxy_req_all_header_fields(char *); // STR
 
   //
   // server -> proxy fields
   //
-  inkcoreapi int marshal_server_host_ip(char *);           // INT
-  inkcoreapi int marshal_server_host_name(char *);         // STR
-  inkcoreapi int marshal_server_resp_status_code(char *);  // INT
-  inkcoreapi int marshal_server_resp_squid_len(char *);    // INT
-  inkcoreapi int marshal_server_resp_content_len(char *);  // INT
-  inkcoreapi int marshal_server_resp_header_len(char *);   // INT
-  inkcoreapi int marshal_server_resp_http_version(char *); // INT
-  inkcoreapi int marshal_server_resp_time_ms(char *);      // INT
-  inkcoreapi int marshal_server_resp_time_s(char *);       // INT
-  inkcoreapi int marshal_server_transact_count(char *);    // INT
-  inkcoreapi int marshal_server_connect_attempts(char *);  // INT
+  inkcoreapi int marshal_server_host_ip(char *);                // INT
+  inkcoreapi int marshal_server_host_name(char *);              // STR
+  inkcoreapi int marshal_server_resp_status_code(char *);       // INT
+  inkcoreapi int marshal_server_resp_squid_len(char *);         // INT
+  inkcoreapi int marshal_server_resp_content_len(char *);       // INT
+  inkcoreapi int marshal_server_resp_header_len(char *);        // INT
+  inkcoreapi int marshal_server_resp_http_version(char *);      // INT
+  inkcoreapi int marshal_server_resp_time_ms(char *);           // INT
+  inkcoreapi int marshal_server_resp_time_s(char *);            // INT
+  inkcoreapi int marshal_server_transact_count(char *);         // INT
+  inkcoreapi int marshal_server_connect_attempts(char *);       // INT
+  inkcoreapi int marshal_server_resp_all_header_fields(char *); // STR
 
   //
   // cache -> client fields
   //
-  inkcoreapi int marshal_cache_resp_status_code(char *);  // INT
-  inkcoreapi int marshal_cache_resp_squid_len(char *);    // INT
-  inkcoreapi int marshal_cache_resp_content_len(char *);  // INT
-  inkcoreapi int marshal_cache_resp_header_len(char *);   // INT
-  inkcoreapi int marshal_cache_resp_http_version(char *); // INT
+  inkcoreapi int marshal_cache_resp_status_code(char *);       // INT
+  inkcoreapi int marshal_cache_resp_squid_len(char *);         // INT
+  inkcoreapi int marshal_cache_resp_content_len(char *);       // INT
+  inkcoreapi int marshal_cache_resp_header_len(char *);        // INT
+  inkcoreapi int marshal_cache_resp_http_version(char *);      // INT
+  inkcoreapi int marshal_cache_resp_all_header_fields(char *); // STR
 
   inkcoreapi void set_client_req_url(char *, int);                // STR
   inkcoreapi void set_client_req_url_canon(char *, int);          // STR
@@ -244,10 +252,17 @@ public:
   inkcoreapi int marshal_client_http_transaction_priority_weight(char *);     // INT
   inkcoreapi int marshal_client_http_transaction_priority_dependence(char *); // INT
   inkcoreapi int marshal_cache_lookup_url_canon(char *);                      // STR
+  inkcoreapi int marshal_client_sni_server_name(char *);                      // STR
+  inkcoreapi int marshal_client_provided_cert(char *);                        // INT
+  inkcoreapi int marshal_proxy_provided_cert(char *);                         // INT
   inkcoreapi int marshal_version_build_number(char *);                        // STR
+  inkcoreapi int marshal_version_string(char *);                              // STR
   inkcoreapi int marshal_cache_read_retries(char *);                          // INT
   inkcoreapi int marshal_cache_write_retries(char *);                         // INT
   inkcoreapi int marshal_cache_collapsed_connection_success(char *);          // INT
+  inkcoreapi int marshal_proxy_protocol_version(char *);                      // STR
+  inkcoreapi int marshal_proxy_protocol_src_ip(char *);                       // STR
+  inkcoreapi int marshal_proxy_protocol_dst_ip(char *);                       // STR
 
   // named fields from within a http header
   //
@@ -280,6 +295,7 @@ public:
   inkcoreapi int marshal_milestone_fmt_time(TSMilestonesType ms, char *buf);
   inkcoreapi int marshal_milestone_fmt_ms(TSMilestonesType ms, char *buf);
   inkcoreapi int marshal_milestone_diff(TSMilestonesType ms1, TSMilestonesType ms2, char *buf);
+  inkcoreapi void set_http_header_field(LogField::Container container, char *field, char *buf, int len);
   //
   // unmarshalling routines
   //
@@ -302,14 +318,14 @@ public:
   static int unmarshal_ip(char **buf, IpEndpoint *dest);
   static int unmarshal_ip_to_str(char **buf, char *dest, int len);
   static int unmarshal_ip_to_hex(char **buf, char *dest, int len);
-  static int unmarshal_hierarchy(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
-  static int unmarshal_finish_status(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
-  static int unmarshal_cache_code(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
-  static int unmarshal_cache_hit_miss(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
-  static int unmarshal_cache_write_code(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
+  static int unmarshal_hierarchy(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
+  static int unmarshal_finish_status(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
+  static int unmarshal_cache_code(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
+  static int unmarshal_cache_hit_miss(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
+  static int unmarshal_cache_write_code(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
   static int unmarshal_client_protocol_stack(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
 
-  static int unmarshal_with_map(int64_t code, char *dest, int len, Ptr<LogFieldAliasMap> map, const char *msg = nullptr);
+  static int unmarshal_with_map(int64_t code, char *dest, int len, const Ptr<LogFieldAliasMap> &map, const char *msg = nullptr);
 
   static int unmarshal_record(char **buf, char *dest, int len);
 
@@ -361,10 +377,10 @@ private:
   char *m_cache_lookup_url_canon_str;
   int m_cache_lookup_url_canon_len;
 
-  void validate_unmapped_url(void);
-  void validate_unmapped_url_path(void);
+  void validate_unmapped_url();
+  void validate_unmapped_url_path();
 
-  void validate_lookup_url(void);
+  void validate_lookup_url();
 };
 
 inline int
