@@ -29,6 +29,7 @@
  ***************************************************************************/
 #include "tscore/ink_platform.h"
 
+#include "MIME.h"
 #include "LogUtils.h"
 #include "LogField.h"
 #include "LogBuffer.h"
@@ -170,7 +171,7 @@ struct cmp_str {
 };
 } // namespace
 
-typedef std::map<ts::ConstBuffer, TSMilestonesType, cmp_str> milestone_map;
+using milestone_map = std::map<ts::ConstBuffer, TSMilestonesType, cmp_str>;
 static milestone_map m_milestone_map;
 
 struct milestone {
@@ -326,7 +327,7 @@ LogField::LogField(const char *field, Container container, SetFunc _setfunc)
     m_milestone2(TS_MILESTONE_LAST_ENTRY),
     m_time_field(false),
     m_alias_map(nullptr),
-    m_set_func(_setfunc)
+    m_set_func(nullptr)
 {
   ink_assert(m_name != nullptr);
   ink_assert(m_symbol != nullptr);
@@ -347,7 +348,7 @@ LogField::LogField(const char *field, Container container, SetFunc _setfunc)
   case ESSH:
   case ECSSH:
   case SCFG:
-    m_unmarshal_func = (UnmarshalFunc) & (LogAccess::unmarshal_str);
+    m_unmarshal_func = reinterpret_cast<UnmarshalFunc>(&(LogAccess::unmarshal_str));
     break;
 
   case ICFG:
@@ -462,13 +463,39 @@ LogField::marshal_len(LogAccess *lad)
   }
 }
 
+bool
+LogField::isContainerUpdateFieldSupported(Container container)
+{
+  switch (container) {
+  case CQH:
+  case PSH:
+  case PQH:
+  case SSH:
+  case CSSH:
+  case ECQH:
+  case EPSH:
+  case EPQH:
+  case ESSH:
+  case ECSSH:
+  case SCFG:
+    return true;
+  default:
+    return false;
+  }
+}
+
 void
 LogField::updateField(LogAccess *lad, char *buf, int len)
 {
   if (m_container == NO_CONTAINER) {
     return (lad->*m_set_func)(buf, len);
+  } else {
+    if (isContainerUpdateFieldSupported(m_container)) {
+      return set_http_header_field(lad, m_container, this->m_name, buf, len);
+    } else {
+      // no set function defined for the container
+    }
   }
-  // else...// future enhancement
 }
 
 /*-------------------------------------------------------------------------
@@ -568,9 +595,9 @@ unsigned
 LogField::unmarshal(char **buf, char *dest, int len)
 {
   if (!m_alias_map) {
-    if (m_unmarshal_func == (UnmarshalFunc)LogAccess::unmarshal_str ||
-        m_unmarshal_func == (UnmarshalFunc)LogAccess::unmarshal_http_text) {
-      UnmarshalFuncWithSlice func = (UnmarshalFuncWithSlice)m_unmarshal_func;
+    if (m_unmarshal_func == reinterpret_cast<UnmarshalFunc>(LogAccess::unmarshal_str) ||
+        m_unmarshal_func == reinterpret_cast<UnmarshalFunc>(LogAccess::unmarshal_http_text)) {
+      UnmarshalFuncWithSlice func = reinterpret_cast<UnmarshalFuncWithSlice>(m_unmarshal_func);
       return (*func)(buf, dest, len, &m_slice);
     }
     return (*m_unmarshal_func)(buf, dest, len);
@@ -658,7 +685,7 @@ LogField::valid_container_name(char *name)
 {
   for (unsigned i = 1; i < countof(container_names); i++) {
     if (strcmp(name, container_names[i]) == 0) {
-      return (LogField::Container)i;
+      return static_cast<LogField::Container>(i);
     }
   }
 
@@ -670,7 +697,7 @@ LogField::valid_aggregate_name(char *name)
 {
   for (unsigned i = 1; i < countof(aggregate_names); i++) {
     if (strcmp(name, aggregate_names[i]) == 0) {
-      return (LogField::Aggregate)i;
+      return static_cast<LogField::Aggregate>(i);
     }
   }
 
@@ -694,6 +721,12 @@ LogField::fieldlist_contains_aggregates(const char *fieldlist)
   return false;
 }
 
+void
+LogField::set_http_header_field(LogAccess *lad, LogField::Container container, char *field, char *buf, int len)
+{
+  return lad->set_http_header_field(container, field, buf, len);
+}
+
 /*-------------------------------------------------------------------------
   LogFieldList
 
@@ -701,7 +734,7 @@ LogField::fieldlist_contains_aggregates(const char *fieldlist)
   heap with "new" and that each element is on at most ONE list.  To enforce
   this, items are copied by default, using the copy ctor.
   -------------------------------------------------------------------------*/
-LogFieldList::LogFieldList() : m_marshal_len(0) {}
+LogFieldList::LogFieldList() = default;
 
 LogFieldList::~LogFieldList()
 {
@@ -716,6 +749,7 @@ LogFieldList::clear()
     delete f; // safe given the semantics stated above
   }
   m_marshal_len = 0;
+  _badSymbols.clear();
 }
 
 void
@@ -750,11 +784,10 @@ LogFieldList::find_by_symbol(const char *symbol) const
 {
   LogField *field = nullptr;
 
-  if (Log::field_symbol_hash) {
-    if (ink_hash_table_lookup(Log::field_symbol_hash, (char *)symbol, (InkHashTableValue *)&field) && field) {
-      Debug("log-field-hash", "Field %s found", field->symbol());
-      return field;
-    }
+  if (auto it = Log::field_symbol_hash.find(symbol); it != Log::field_symbol_hash.end() && it->second) {
+    field = it->second;
+    Debug("log-field-hash", "Field %s found", field->symbol());
+    return field;
   }
   // trusty old method
 
@@ -823,3 +856,12 @@ LogFieldList::display(FILE *fd)
     f->display(fd);
   }
 }
+
+void
+LogFieldList::addBadSymbol(std::string_view badSymbol)
+{
+  if (_badSymbols.size() > 0) {
+    _badSymbols += ", ";
+  }
+  _badSymbols += badSymbol;
+}
