@@ -23,6 +23,9 @@
 
 #pragma once
 
+#include <string_view>
+#include <string>
+
 #include "tscore/ink_platform.h"
 #include "tscore/List.h"
 #include "tscore/TsBuffer.h"
@@ -47,7 +50,7 @@ struct LogSlice {
   // Initialize LogSlice by slice notation,
   // the str looks like: "xxx[0:30]".
   //
-  LogSlice(char *str);
+  explicit LogSlice(char *str);
 
   //
   // Convert slice notation to target string's offset,
@@ -76,7 +79,7 @@ public:
   typedef int (LogAccess::*MarshalFunc)(char *buf);
   typedef int (*UnmarshalFunc)(char **buf, char *dest, int len);
   typedef int (*UnmarshalFuncWithSlice)(char **buf, char *dest, int len, LogSlice *slice);
-  typedef int (*UnmarshalFuncWithMap)(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
+  typedef int (*UnmarshalFuncWithMap)(char **buf, char *dest, int len, const Ptr<LogFieldAliasMap> &map);
   typedef void (LogAccess::*SetFunc)(char *buf, int len);
 
   enum Type {
@@ -171,13 +174,15 @@ public:
     return m_time_field;
   }
 
+  inkcoreapi void set_http_header_field(LogAccess *lad, LogField::Container container, char *field, char *buf, int len);
   void set_aggregate_op(Aggregate agg_op);
   void update_aggregate(int64_t val);
 
-  static void init_milestone_container(void);
+  static void init_milestone_container();
   static Container valid_container_name(char *name);
   static Aggregate valid_aggregate_name(char *name);
   static bool fieldlist_contains_aggregates(const char *fieldlist);
+  static bool isContainerUpdateFieldSupported(Container container);
 
 private:
   char *m_name;
@@ -207,8 +212,6 @@ public:
   LogField &operator=(const LogField &rhs) = delete;
 
 private:
-  // luis, check where this is used and what it does
-  //    void init (char *name, char *symbol, Type type);
   LogField();
 };
 
@@ -245,14 +248,27 @@ public:
   unsigned count();
   void display(FILE *fd = stdout);
 
+  // Add a bad symbol seen in the log format to the list of bad symbols.
+  //
+  void addBadSymbol(std::string_view badSymbol);
+
+  // Return blank-separated list of added bad symbols.
+  //
+  std::string_view
+  badSymbols() const
+  {
+    return _badSymbols;
+  }
+
   // noncopyable
   // -- member functions that are not allowed --
   LogFieldList(const LogFieldList &rhs) = delete;
   LogFieldList &operator=(const LogFieldList &rhs) = delete;
 
 private:
-  unsigned m_marshal_len;
+  unsigned m_marshal_len = 0;
   Queue<LogField> m_field_list;
+  std::string _badSymbols;
 };
 
 /** Base IP address data.
