@@ -29,7 +29,6 @@
 #include "tscore/ink_platform.h"
 #include "LogBufferSink.h"
 
-class LogSock;
 class LogBuffer;
 struct LogBufferHeader;
 class LogObject;
@@ -44,7 +43,7 @@ class LogFile : public LogBufferSink, public RefCountObj
 {
 public:
   LogFile(const char *name, const char *header, LogFileFormat format, uint64_t signature, size_t ascii_buffer_size = 4 * 9216,
-          size_t max_line_size = 9216);
+          size_t max_line_size = 9216, int pipe_buffer_size = 0);
   LogFile(const LogFile &);
   ~LogFile() override;
 
@@ -59,7 +58,15 @@ public:
 
   int preproc_and_try_delete(LogBuffer *lb) override;
 
-  int roll(long interval_start, long interval_end);
+  bool trim_rolled(size_t rolling_max_count);
+  int roll(long interval_start, long interval_end, bool reopen_after_rolling = false);
+
+  /** Check whether the file at the log's filename exists and, if not, close
+   * the current file descriptor and reopen it.
+   *
+   * @return True if the file was re-opened, false otherwise.
+   */
+  bool reopen_if_moved();
 
   const char *
   get_name() const
@@ -120,6 +127,7 @@ public:
   uint64_t m_signature;       // signature of log object stored
   size_t m_ascii_buffer_size; // size of ascii buffer
   size_t m_max_line_size;     // size of longest log line (record)
+  int m_pipe_buffer_size;     // this is the size of the pipe buffer set by fcntl
   int m_fd;                   // this could back m_log or a pipe, depending on the situation
 
 public:
@@ -131,7 +139,3 @@ private:
   // -- member functions not allowed --
   LogFile();
 };
-
-/***************************************************************************
- LogFileList IS NOT USED
-****************************************************************************/
