@@ -34,7 +34,6 @@
 #include "LogFormat.h"
 #include "LogFile.h"
 #include "LogBuffer.h"
-#include "LogHost.h"
 #include "LogObject.h"
 #include "LogConfig.h"
 #include "Log.h"
@@ -142,7 +141,7 @@ LogFilter::parse(const char *name, Action action, const char *condition)
   LogFilter::Operator oper = LogFilter::N_OPERATORS;
   for (unsigned i = 0; i < LogFilter::N_OPERATORS; ++i) {
     if (strcasecmp(oper_str, LogFilter::OPERATOR_NAME[i]) == 0) {
-      oper = (LogFilter::Operator)i;
+      oper = static_cast<LogFilter::Operator>(i);
       break;
     }
   }
@@ -203,7 +202,7 @@ LogFilterString::_setValues(size_t n, char **value)
     for (size_t i = 0; i < n; ++i) {
       m_value[i]           = ats_strdup(value[i]);
       m_length[i]          = strlen(value[i]);
-      m_value_uppercase[i] = (char *)ats_malloc((unsigned int)m_length[i] + 1);
+      m_value_uppercase[i] = static_cast<char *>(ats_malloc(static_cast<unsigned int>(m_length[i]) + 1));
       size_t j;
       for (j = 0; j < m_length[i]; ++j) {
         m_value_uppercase[i][j] = ParseRules::ink_toupper(m_value[i][j]);
@@ -314,12 +313,15 @@ LogFilterString::wipe_this_entry(LogAccess *lad)
 
   static const unsigned BUFSIZE = 1024;
   char small_buf[BUFSIZE];
-  char *big_buf    = nullptr;
-  char *buf        = small_buf;
-  size_t marsh_len = m_field->marshal_len(lad); // includes null termination
+  char small_buf_upper[BUFSIZE];
+  char *big_buf       = nullptr;
+  char *big_buf_upper = nullptr;
+  char *buf           = small_buf;
+  char *buf_upper     = small_buf_upper;
+  size_t marsh_len    = m_field->marshal_len(lad); // includes null termination
 
   if (marsh_len > BUFSIZE) {
-    big_buf = (char *)ats_malloc(marsh_len);
+    big_buf = static_cast<char *>(ats_malloc(marsh_len));
     ink_assert(big_buf != nullptr);
     buf = big_buf;
   }
@@ -340,27 +342,36 @@ LogFilterString::wipe_this_entry(LogAccess *lad)
     // actual length, so we just use the fact that a MATCH is not possible
     // when marsh_len <= (length of the filter string)
     //
-    cond_satisfied = _checkConditionAndWipe(&strcmp, &buf, marsh_len, m_value, DATA_LENGTH_LARGER);
+    cond_satisfied = _checkConditionAndWipe(&strcmp, &buf, marsh_len, m_value, nullptr, DATA_LENGTH_LARGER);
     break;
   case CASE_INSENSITIVE_MATCH:
-    cond_satisfied = _checkConditionAndWipe(&strcasecmp, &buf, marsh_len, m_value, DATA_LENGTH_LARGER);
+    cond_satisfied = _checkConditionAndWipe(&strcasecmp, &buf, marsh_len, m_value, nullptr, DATA_LENGTH_LARGER);
     break;
   case CONTAIN:
-    cond_satisfied = _checkConditionAndWipe(&_isSubstring, &buf, marsh_len, m_value, DATA_LENGTH_LARGER);
+    cond_satisfied = _checkConditionAndWipe(&_isSubstring, &buf, marsh_len, m_value, nullptr, DATA_LENGTH_LARGER);
     break;
   case CASE_INSENSITIVE_CONTAIN:
-    for (size_t i = 0; i < marsh_len; i++) {
-      buf[i] = ParseRules::ink_toupper(buf[i]);
+    if (big_buf) {
+      big_buf_upper = static_cast<char *>(ats_malloc(static_cast<unsigned int>(marsh_len)));
+      buf_upper     = big_buf_upper;
+    } else {
+      buf = small_buf; // make clang happy
     }
-    cond_satisfied = _checkConditionAndWipe(&_isSubstring, &buf, marsh_len, m_value_uppercase, DATA_LENGTH_LARGER);
+    for (size_t i = 0; i < marsh_len; i++) {
+      buf_upper[i] = ParseRules::ink_toupper(buf[i]);
+    }
+    cond_satisfied = _checkConditionAndWipe(&_isSubstring, &buf, marsh_len, m_value_uppercase, buf_upper, DATA_LENGTH_LARGER);
     break;
   default:
     ink_assert(!"INVALID FILTER OPERATOR");
   }
 
-  m_field->updateField(lad, buf, strlen(buf));
+  if (cond_satisfied) {
+    m_field->updateField(lad, buf, strlen(buf));
+  }
 
   ats_free(big_buf);
+  ats_free(big_buf_upper);
   return cond_satisfied;
 }
 
@@ -393,7 +404,7 @@ LogFilterString::toss_this_entry(LogAccess *lad)
   size_t marsh_len    = m_field->marshal_len(lad); // includes null termination
 
   if (marsh_len > BUFSIZE) {
-    big_buf = (char *)ats_malloc((unsigned int)marsh_len);
+    big_buf = static_cast<char *>(ats_malloc(static_cast<unsigned int>(marsh_len)));
     ink_assert(big_buf != nullptr);
     buf = big_buf;
   }
@@ -421,7 +432,7 @@ LogFilterString::toss_this_entry(LogAccess *lad)
     break;
   case CASE_INSENSITIVE_CONTAIN: {
     if (big_buf) {
-      big_buf_upper = (char *)ats_malloc((unsigned int)marsh_len);
+      big_buf_upper = static_cast<char *>(ats_malloc(static_cast<unsigned int>(marsh_len)));
       buf_upper     = big_buf_upper;
     } else {
       buf = small_buf; // make clang happy
@@ -527,13 +538,16 @@ LogFilterInt::LogFilterInt(const char *name, LogField *field, LogFilter::Action
   size_t i           = 0;
   SimpleTokenizer tok(values, ',');
   size_t n = tok.getNumTokensRemaining();
+  auto field_map{field->map()}; // because clang-analyzer freaks out if this is inlined.
+                                // It doesn't realize the value is held in place by the smart
+                                // pointer in @c field.
 
   if (n) {
     val_array = new int64_t[n];
     char *t;
     while (t = tok.getNext(), t != nullptr) {
       int64_t ival;
-      if (!_convertStringToInt(t, &ival, field->map().get())) {
+      if (!_convertStringToInt(t, &ival, field_map.get())) {
         // conversion was successful, add entry to array
         //
         val_array[i++] = ival;
@@ -610,7 +624,7 @@ LogFilterInt::wipe_this_entry(LogAccess *lad)
   bool cond_satisfied = false;
   int64_t value;
 
-  m_field->marshal(lad, (char *)&value);
+  m_field->marshal(lad, reinterpret_cast<char *>(&value));
   // This used to do an ntohl() on value, but that breaks various filters.
   // Long term we should move IPs to their own log type.
 
@@ -648,7 +662,7 @@ LogFilterInt::toss_this_entry(LogAccess *lad)
   bool cond_satisfied = false;
   int64_t value;
 
-  m_field->marshal(lad, (char *)&value);
+  m_field->marshal(lad, reinterpret_cast<char *>(&value));
   // This used to do an ntohl() on value, but that breaks various filters.
   // Long term we should move IPs to their own log type.
 
@@ -767,14 +781,14 @@ void
 LogFilterIP::init()
 {
   m_type       = IP_FILTER;
-  m_num_values = m_map.getCount();
+  m_num_values = m_map.count();
 }
 
 /*-------------------------------------------------------------------------
   LogFilterIP::~LogFilterIP
   -------------------------------------------------------------------------*/
 
-LogFilterIP::~LogFilterIP() {}
+LogFilterIP::~LogFilterIP() = default;
 
 /*-------------------------------------------------------------------------
   LogFilterIP::operator==
@@ -839,15 +853,7 @@ LogFilterIP::toss_this_entry(LogAccess *lad)
 bool
 LogFilterIP::wipe_this_entry(LogAccess *)
 {
-#if 0
-  bool zret = WIPE_FIELD_VALUE == m_action && this->is_match(lad);
-  if (zret) {
-    // set to ADDR_ANY.
-  }
-  return zret;
-#else
   return false;
-#endif
 }
 
 /*-------------------------------------------------------------------------
@@ -886,7 +892,7 @@ LogFilterIP::display(FILE *fd)
 {
   ink_assert(fd != nullptr);
 
-  if (0 == m_map.getCount()) {
+  if (0 == m_map.count()) {
     fprintf(fd, "Filter \"%s\" is inactive, no values specified\n", m_name);
   } else {
     fprintf(fd, "Filter \"%s\" %sS records if %s %s ", m_name, ACTION_NAME[m_action], m_field->symbol(), OPERATOR_NAME[m_operator]);
@@ -927,7 +933,7 @@ filters_are_equal(LogFilter *filt1, LogFilter *filt2)
   add() function is overloaded for each sub-type of LogFilter.
   -------------------------------------------------------------------------*/
 
-LogFilterList::LogFilterList() : m_does_conjunction(true) {}
+LogFilterList::LogFilterList() = default;
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
