@@ -24,7 +24,7 @@
 #include "tscore/BaseLogFile.h"
 
 /*
- * This consturctor creates a BaseLogFile based on a given name.
+ * This constructor creates a BaseLogFile based on a given name.
  * This is the most common way BaseLogFiles are created.
  */
 BaseLogFile::BaseLogFile(const char *name) : m_name(ats_strdup(name))
@@ -33,7 +33,7 @@ BaseLogFile::BaseLogFile(const char *name) : m_name(ats_strdup(name))
 }
 
 /*
- * This consturctor creates a BaseLogFile based on a given name.
+ * This constructor creates a BaseLogFile based on a given name.
  * Similar to above constructor, but is overloaded with the object signature
  */
 BaseLogFile::BaseLogFile(const char *name, uint64_t sig) : m_name(ats_strdup(name)), m_signature(sig), m_has_signature(true)
@@ -46,15 +46,13 @@ BaseLogFile::BaseLogFile(const char *name, uint64_t sig) : m_name(ats_strdup(nam
  * This copy constructor creates a BaseLogFile based on a given copy.
  */
 BaseLogFile::BaseLogFile(const BaseLogFile &copy)
-  : m_fp(nullptr),
-    m_start_time(copy.m_start_time),
-    m_end_time(0L),
-    m_bytes_written(0),
+  : m_start_time(copy.m_start_time),
+
     m_name(ats_strdup(copy.m_name)),
     m_hostname(ats_strdup(copy.m_hostname)),
-    m_is_regfile(false),
+
     m_is_init(copy.m_is_init),
-    m_meta_info(nullptr),
+
     m_signature(copy.m_signature),
     m_has_signature(copy.m_has_signature)
 {
@@ -157,7 +155,7 @@ BaseLogFile::roll(long interval_start, long interval_end)
     // produce overlapping filenames (the problem is that we have
     // no easy way of keeping track of the timestamp of the first
     // transaction
-    log_log_trace("in BaseLogFile::roll(..), didn't use metadata starttime, used earlist available starttime\n");
+    log_log_trace("in BaseLogFile::roll(..), didn't use metadata starttime, used earliest available starttime\n");
     if (interval_start == 0) {
       start = m_start_time;
     } else {
@@ -169,8 +167,8 @@ BaseLogFile::roll(long interval_start, long interval_end)
 
   // Now that we have our timestamp values, convert them to the proper
   // timestamp formats and create the rolled file name.
-  timestamp_to_str((long)start, start_time_ext, sizeof(start_time_ext));
-  timestamp_to_str((long)end, end_time_ext, sizeof(start_time_ext));
+  timestamp_to_str(static_cast<long>(start), start_time_ext, sizeof(start_time_ext));
+  timestamp_to_str(static_cast<long>(end), end_time_ext, sizeof(start_time_ext));
   snprintf(roll_name, LOGFILE_ROLL_MAXPATHLEN, "%s%s%s.%s-%s%s", m_name.get(), (m_hostname.get() ? LOGFILE_SEPARATOR_STRING : ""),
            (m_hostname.get() ? m_hostname.get() : ""), start_time_ext, end_time_ext, LOGFILE_ROLLED_EXTENSION);
 
@@ -206,7 +204,7 @@ BaseLogFile::roll(long interval_start, long interval_end)
 }
 
 /*
- * The more convienent rolling function. Intended use is for less
+ * The more convenient rolling function. Intended use is for less
  * critical logs such as diags.log or traffic.out, since _exact_
  * timestamps may be less important
  *
@@ -238,8 +236,8 @@ BaseLogFile::roll()
 bool
 BaseLogFile::rolled_logfile(char *path)
 {
-  const int target_len = (int)strlen(LOGFILE_ROLLED_EXTENSION);
-  int len              = (int)strlen(path);
+  const int target_len = static_cast<int>(strlen(LOGFILE_ROLLED_EXTENSION));
+  int len              = static_cast<int>(strlen(path));
   if (len > target_len) {
     char *str = &path[len - target_len];
     if (!strcmp(str, LOGFILE_ROLLED_EXTENSION)) {
@@ -304,9 +302,9 @@ BaseLogFile::open_file(int perm)
     // The log file does not exist, so we create a new MetaInfo object
     //  which will save itself to disk right away (in the constructor)
     if (m_has_signature) {
-      m_meta_info = new BaseMetaInfo(m_name.get(), (long)time(nullptr), m_signature);
+      m_meta_info = new BaseMetaInfo(m_name.get(), static_cast<long>(time(nullptr)), m_signature);
     } else {
-      m_meta_info = new BaseMetaInfo(m_name.get(), (long)time(nullptr));
+      m_meta_info = new BaseMetaInfo(m_name.get(), static_cast<long>(time(nullptr)));
     }
   }
 
@@ -333,7 +331,7 @@ BaseLogFile::open_file(int perm)
     }
   }
 
-  // set m_bytes_written to force the rolling based on filesize.
+  // set m_bytes_written to force the rolling based on file size.
   m_bytes_written = fseek(m_fp, 0, SEEK_CUR);
 
   log_log_trace("BaseLogFile %s is now open (fd=%d)\n", m_name.get(), fileno(m_fp));
@@ -341,18 +339,32 @@ BaseLogFile::open_file(int perm)
   return LOG_FILE_NO_ERROR;
 }
 
-/*
- * Closes actual log file, not metainfo
+/**
+ * @brief Close the managed log file.
+ *
+ * @note This closes the actual log file, not its metainfo.
+ *
+ * @return The result of calling fclose on the file descriptor or 0 if the file
+ * was not open. If the result is non-zero, fclose failed and errno is set
+ * appropriately.
  */
-void
+int
 BaseLogFile::close_file()
 {
+  int ret = 0;
   if (is_open()) {
-    fclose(m_fp);
     log_log_trace("BaseLogFile %s is closed\n", m_name.get());
+
+    // Both log_log_trace and fclose may set errno. Thus, keep fclose after
+    // log_log_trace so that by the time this function completes if errno was
+    // set by fclose it will remain upon function return.
+    ret       = fclose(m_fp);
     m_fp      = nullptr;
     m_is_init = false;
+    delete m_meta_info;
+    m_meta_info = nullptr;
   }
+  return ret;
 }
 
 /*
@@ -452,7 +464,7 @@ BaseMetaInfo::_build_name(const char *filename)
 
   // 7 = 1 (dot at beginning) + 5 (".meta") + 1 (null terminating)
   //
-  _filename = (char *)ats_malloc(l + 7);
+  _filename = static_cast<char *>(ats_malloc(l + 7));
 
   if (i < 0) {
     ink_string_concatenate_strings(_filename, ".", filename, ".meta", nullptr);
@@ -483,7 +495,7 @@ BaseMetaInfo::_read_from_file()
         if (strcmp(t, "creation_time") == 0) {
           t = tok.getNext();
           if (t) {
-            _creation_time = (time_t)ink_atoi64(t);
+            _creation_time = static_cast<time_t>(ink_atoi64(t));
             _flags |= VALID_CREATION_TIME;
           }
         } else if (strcmp(t, "object_signature") == 0) {
@@ -523,7 +535,7 @@ BaseMetaInfo::_write_to_file()
   int n;
   if (_flags & VALID_CREATION_TIME) {
     log_log_trace("Writing creation time to %s\n", _filename);
-    n = snprintf(_buffer, BUF_SIZE, "creation_time = %lu\n", (unsigned long)_creation_time);
+    n = snprintf(_buffer, BUF_SIZE, "creation_time = %lu\n", static_cast<unsigned long>(_creation_time));
     // TODO modify this runtime check so that it is not an assertion
     ink_release_assert(n <= BUF_SIZE);
     if (write(fd, _buffer, n) == -1) {
@@ -537,7 +549,7 @@ BaseMetaInfo::_write_to_file()
     // TODO modify this runtime check so that it is not an assertion
     ink_release_assert(n <= BUF_SIZE);
     if (write(fd, _buffer, n) == -1) {
-      log_log_error("Could not write object_signaure\n");
+      log_log_error("Could not write object_signature\n");
     }
     log_log_trace("BaseMetaInfo::_write_to_file\n"
                   "\tfilename = %s\n"
