@@ -24,6 +24,7 @@
 #include "tscore/BufferWriter.h"
 #include "tscore/bwf_std_format.h"
 #include "tscore/ink_thread.h"
+#include "tscore/InkErrno.h"
 #include <unistd.h>
 #include <sys/param.h>
 #include <cctype>
@@ -32,6 +33,7 @@
 #include <cmath>
 #include <array>
 #include <chrono>
+#include <exception>
 
 using namespace std::literals;
 
@@ -98,12 +100,11 @@ BWFSpec::Property::Property()
 }
 
 /// Parse a format specification.
-BWFSpec::BWFSpec(TextView fmt)
+BWFSpec::BWFSpec(TextView fmt) : _name(fmt.take_prefix_at(':'))
 {
   TextView num; // temporary for number parsing.
   intmax_t n;
 
-  _name = fmt.take_prefix_at(':');
   // if it's parsable as a number, treat it as an index.
   n = tv_to_positive_decimal(_name, &num);
   if (num.size()) {
@@ -239,7 +240,7 @@ namespace bw_fmt
       size_t delta = min - extent;
       char *base   = w.auxBuffer();        // should be first byte of @a lw e.g. lw.data() - avoid const_cast.
       char *limit  = base + lw.capacity(); // first invalid byte.
-      char *dst;                           // used to track memory operation targest;
+      char *dst;                           // used to track memory operation target;
       char *last;                          // track limit of memory operation.
       size_t d2;
       switch (spec._align) {
@@ -452,7 +453,7 @@ namespace bw_fmt
     width -= static_cast<int>(n);
     std::string_view digits{buff + sizeof(buff) - n, n};
 
-    if (spec._align == BWFSpec::Align::SIGN) { // custom for signed case because prefix and digits are seperated.
+    if (spec._align == BWFSpec::Align::SIGN) { // custom for signed case because prefix and digits are separated.
       if (neg) {
         w.write(neg);
       }
@@ -467,24 +468,25 @@ namespace bw_fmt
       }
       w.write(digits);
     } else { // use generic Write_Aligned
-      Write_Aligned(w,
-                    [&]() {
-                      if (prefix1) {
-                        w.write(prefix1);
-                        if (prefix2) {
-                          w.write(prefix2);
-                        }
-                      }
-                      w.write(digits);
-                    },
-                    spec._align, width, spec._fill, neg);
+      Write_Aligned(
+        w,
+        [&]() {
+          if (prefix1) {
+            w.write(prefix1);
+            if (prefix2) {
+              w.write(prefix2);
+            }
+          }
+          w.write(digits);
+        },
+        spec._align, width, spec._fill, neg);
     }
     return w;
   }
 
-  /// Format for floating point values. Seperates floating point into a whole number and a
+  /// Format for floating point values. Separates floating point into a whole number and a
   /// fraction. The fraction is converted into an unsigned integer based on the specified
-  /// precision, spec._prec. ie. 3.1415 with precision two is seperated into two unsigned
+  /// precision, spec._prec. ie. 3.1415 with precision two is separated into two unsigned
   /// integers 3 and 14. The different pieces are assembled and placed into the BufferWriter.
   /// The default is two decimal places. ie. X.XX. The value is always written in base 10.
   ///
@@ -574,20 +576,21 @@ namespace bw_fmt
     std::string_view whole_digits{whole + sizeof(whole) - l, l};
     std::string_view frac_digits{fraction + sizeof(fraction) - r, r};
 
-    Write_Aligned(w,
-                  [&]() {
-                    w.write(whole_digits);
-                    w.write(dec);
-                    w.write(frac_digits);
-                  },
-                  spec._align, width, spec._fill, neg);
+    Write_Aligned(
+      w,
+      [&]() {
+        w.write(whole_digits);
+        w.write(dec);
+        w.write(frac_digits);
+      },
+      spec._align, width, spec._fill, neg);
 
     return w;
   }
 
   /// Write out the @a data as hexadecimal, using @a digits as the conversion.
   void
-  Hex_Dump(BufferWriter &w, std::string_view data, const char *digits)
+  Format_As_Hex(BufferWriter &w, std::string_view data, const char *digits)
   {
     const char *ptr = data.data();
     for (auto n = data.size(); n > 0; --n) {
@@ -608,32 +611,21 @@ bwformat(BufferWriter &w, BWFSpec const &spec, std::string_view sv)
   }
 
   if ('x' == spec._type || 'X' == spec._type) {
-    const char *digits = 'x' == spec._type ? bw_fmt::LOWER_DIGITS : bw_fmt::UPPER_DIGITS;
-    width -= sv.size() * 2;
-    if (spec._radix_lead_p) {
-      w.write('0');
-      w.write(spec._type);
-      width -= 2;
-    }
-    bw_fmt::Write_Aligned(w, [&w, &sv, digits]() { bw_fmt::Hex_Dump(w, sv, digits); }, spec._align, width, spec._fill, 0);
+    return bwformat(w, spec, bwf::detail::MemDump(sv.data(), sv.size()));
   } else {
     width -= sv.size();
-    bw_fmt::Write_Aligned(w, [&w, &sv]() { w.write(sv); }, spec._align, width, spec._fill, 0);
+    bw_fmt::Write_Aligned(
+      w, [&w, &sv]() { w.write(sv); }, spec._align, width, spec._fill, 0);
   }
   return w;
 }
 
 BufferWriter &
-bwformat(BufferWriter &w, BWFSpec const &spec, MemSpan const &span)
+bwformat(BufferWriter &w, BWFSpec const &spec, MemSpan<void> const &span)
 {
   static const BWFormat default_fmt{"{:#x}@{:p}"};
-  if (spec._ext.size() && 'd' == spec._ext.front()) {
-    const char *digits = 'X' == spec._type ? bw_fmt::UPPER_DIGITS : bw_fmt::LOWER_DIGITS;
-    if (spec._radix_lead_p) {
-      w.write('0');
-      w.write(digits[33]);
-    }
-    bw_fmt::Hex_Dump(w, span.view(), digits);
+  if ('x' == spec._type || 'X' == spec._type) {
+    bwformat(w, spec, bwf::detail::MemDump(span.data(), span.size()));
   } else {
     w.print(default_fmt, span.size(), span.data());
   }
@@ -905,8 +897,16 @@ bwformat(BufferWriter &w, BWFSpec const &spec, bwf::Errno const &e)
   if (spec.has_numeric_type()) {              // if numeric type, print just the numeric part.
     w.print(number_fmt, e._e);
   } else {
-    w.write(short_name(e._e));
-    w.write(strerror(e._e));
+    if (e._e < 0) {
+      w.write(InkStrerror(-e._e));
+    } else {
+      if (spec._ext.empty() || spec._ext.find('s')) {
+        w.write(short_name(e._e));
+      }
+      if (spec._ext.empty() || spec._ext.find('l')) {
+        w.write(strerror(e._e));
+      }
+    }
     if (spec._type != 's' && spec._type != 'S') {
       w.write(' ');
       w.print(number_fmt, e._e);
@@ -958,6 +958,28 @@ bwformat(BufferWriter &w, BWFSpec const &spec, bwf::OptionalAffix const &opts)
   return w.write(opts._prefix).write(opts._text).write(opts._suffix);
 }
 
+BufferWriter &
+bwformat(BufferWriter &w, BWFSpec const &spec, bwf::detail::MemDump const &hex)
+{
+  char fmt_type      = spec._type;
+  const char *digits = bw_fmt::UPPER_DIGITS;
+
+  if ('X' != fmt_type) {
+    fmt_type = 'x';
+    digits   = bw_fmt::LOWER_DIGITS;
+  }
+
+  int width = int(spec._min) - hex._view.size() * 2; // amount left to fill.
+  if (spec._radix_lead_p) {
+    w.write('0');
+    w.write(fmt_type);
+    width -= 2;
+  }
+  bw_fmt::Write_Aligned(
+    w, [&w, &hex, digits]() { bw_fmt::Format_As_Hex(w, hex._view, digits); }, spec._align, width, spec._fill, 0);
+  return w;
+}
+
 } // namespace ts
 
 namespace
@@ -965,12 +987,14 @@ namespace
 void
 BWF_Timestamp(ts::BufferWriter &w, ts::BWFSpec const &spec)
 {
-  // Unfortunately need to write to a temporary buffer or the sizing isn't correct if @a w is clipped
-  // because @c strftime returns 0 if the buffer isn't large enough.
-  char buff[32];
-  std::time_t t = std::time(nullptr);
-  auto n        = strftime(buff, sizeof(buff), "%Y %b %d %H:%M:%S", std::localtime(&t));
-  w.write(buff, n);
+  auto now   = std::chrono::system_clock::now();
+  auto epoch = std::chrono::system_clock::to_time_t(now);
+  ts::LocalBufferWriter<48> lw;
+
+  ctime_r(&epoch, lw.auxBuffer());
+  lw.fill(19); // clip trailing stuff, do not want.
+  lw.print(".{:03}", std::chrono::time_point_cast<std::chrono::milliseconds>(now).time_since_epoch().count() % 1000);
+  w.write(lw.view().substr(4));
 }
 
 void
