@@ -1,6 +1,3 @@
-#include "tscore/IpMap.h"
-#include "tscore/ink_inet.h"
-
 /** @file
     IP address map support.
 
@@ -46,6 +43,10 @@
     before we had IpAddr as a type.
 */
 
+#include "tscore/IpMap.h"
+#include "tscore/ink_inet.h"
+#include "tscore/BufferWriter.h"
+
 namespace ts
 {
 namespace detail
@@ -139,8 +140,9 @@ namespace detail
     using ArgType   = typename N::ArgType; ///< Import type.
     using Metric    = typename N::Metric;  ///< Import type.g482
 
-    IpMapBase() : _root(nullptr) {}
-    ~IpMapBase() { this->clear(); }
+    IpMapBase() = default;
+    IpMapBase(self_type &&that) : _root(that._root), _list(std::move(that._list)) { that._root = nullptr; }
+    ~IpMapBase();
     /** Mark a range.
         All addresses in the range [ @a min , @a max ] are marked with @a data.
         @return This object.
@@ -178,7 +180,7 @@ namespace detail
     */
     bool contains(ArgType target,      ///< Search target value.
                   void **ptr = nullptr ///< Client data return.
-                  ) const;
+    ) const;
 
     /** Remove all addresses in the map.
 
@@ -199,15 +201,15 @@ namespace detail
         Caller is responsible for ensuring that @a spot is in this container
         and the proper location for @a n.
     */
-    void insertAfter(N *spot, ///< Node in list.
-                     N *n     ///< Node to insert.
+    void insert_after(N *spot, ///< Node in list.
+                      N *n     ///< Node to insert.
     );
     /** Insert @a n before @a spot.
         Caller is responsible for ensuring that @a spot is in this container
         and the proper location for @a n.
     */
-    void insertBefore(N *spot, ///< Node in list.
-                      N *n     ///< Node to insert.
+    void insert_before(N *spot, ///< Node in list.
+                       N *n     ///< Node to insert.
     );
     /// Add node @a n as the first node.
     void prepend(N *n);
@@ -223,11 +225,15 @@ namespace detail
     void validate();
 
     /// @return The number of distinct ranges.
-    size_t getCount() const;
+    size_t count() const;
 
-    /// Print all spans.
-    /// @return This map.
-    self_type &print();
+    /** Generate formatted output.
+     *
+     * @param w Destination of the output.
+     * @param spec Format specification.
+     * @return @a w.
+     */
+    ts::BufferWriter &describe(ts::BufferWriter &w, ts::BWFSpec const &spec) const;
 
     // Helper methods.
     N *
@@ -256,21 +262,33 @@ namespace detail
       return static_cast<N *>(n->_right);
     }
     N *
-    getHead()
+    head()
     {
-      return static_cast<N *>(_list.getHead());
+      return static_cast<N *>(_list.head());
     }
     N *
-    getTail()
+    tail()
     {
-      return static_cast<N *>(_list.getTail());
+      return static_cast<N *>(_list.tail());
     }
 
-    N *_root; ///< Root node.
+    N *_root = nullptr; ///< Root node.
     /// In order list of nodes.
     /// For ugly compiler reasons, this is a list of base class pointers
     /// even though we really store @a N instances on it.
-    typedef IntrusiveDList<RBNode, &RBNode::_next, &RBNode::_prev> NodeList;
+    struct NodeLinkage {
+      static RBNode *&
+      next_ptr(RBNode *n)
+      {
+        return n->_next;
+      }
+      static RBNode *&
+      prev_ptr(RBNode *n)
+      {
+        return n->_prev;
+      }
+    };
+    using NodeList = ts::IntrusiveDList<NodeLinkage>;
     /// This keeps track of all allocated nodes in order.
     /// Iteration depends on this list being maintained.
     NodeList _list;
@@ -302,7 +320,7 @@ namespace detail
   IpMapBase<N>::clear()
   {
     // Delete everything.
-    N *n = static_cast<N *>(_list.getHead());
+    N *n = static_cast<N *>(_list.head());
     while (n) {
       N *x = n;
       n    = next(n);
@@ -321,22 +339,22 @@ namespace detail
     N *n = this->lowerBound(rmin);
     N *x = nullptr; // New node (if any).
     // Need copies because we will modify these.
-    Metric min = N::deref(rmin);
-    Metric max = N::deref(rmax);
+    Metric localmin = N::deref(rmin);
+    Metric localmax = N::deref(rmax);
 
     // Handle cases involving a node of interest to the left of the
     // range.
     if (n) {
-      if (n->_min < min) {
-        Metric min_1 = min;
+      if (n->_min < localmin) {
+        Metric min_1 = localmin;
         N::dec(min_1);         // dec is OK because min isn't zero.
         if (n->_max < min_1) { // no overlap or adj.
           n = next(n);
-        } else if (n->_max >= max) { // incoming range is covered, just discard.
+        } else if (n->_max >= localmax) { // incoming range is covered, just discard.
           return *this;
         } else if (n->_data != payload) { // different payload, clip range on left.
-          min = n->_max;
-          N::inc(min);
+          localmin = n->_max;
+          N::inc(localmin);
           n = next(n);
         } else { // skew overlap with same payload, use node and continue.
           x = n;
@@ -344,27 +362,30 @@ namespace detail
         }
       }
     } else {
-      n = this->getHead();
+      n = this->head();
     }
 
     // Work through the rest of the nodes of interest.
     // Invariant: n->_min >= min
 
-    // Careful here -- because max_plus1 might wrap we need to use it only
-    // if we can certain it didn't. This is done by ordering the range
-    // tests so that when max_plus1 is used when we know there exists a
-    // larger value than max.
-    Metric max_plus1 = max;
+    // Careful here -- because max_plus1 might wrap we need to use it only if we can be certain it
+    // didn't. This is done by ordering the range tests so that when max_plus1 is used when we know
+    // there exists a larger value than max.
+    Metric max_plus1 = localmax;
     N::inc(max_plus1);
+
     /* Notes:
-       - max (and thence max_plus1) never change during the loop.
-       - we must have either x != 0 or adjust min but not both.
+       - max (and thence also max_plus1) never change during the loop.
+       - we must have either x != 0 or adjust min but not both for each loop iteration.
     */
     while (n) {
       if (n->_data == payload) {
         if (x) {
-          if (n->_max <= max) {
-            // next range is covered, so we can remove and continue.
+          if (n->_max <= localmax) { // next range is covered, so we can remove and continue.
+#if defined(__clang_analyzer__)
+            x->_next = n->_next; // done in @c remove, but CA doesn't realize that.
+                                 // It's insufficient to assert(x->_next != n) after the remove.
+#endif
             this->remove(n);
             n = next(x);
           } else if (n->_min <= max_plus1) {
@@ -374,52 +395,52 @@ namespace detail
             return *this;
           } else {
             // have the space to finish off the range.
-            x->setMax(max);
+            x->setMax(localmax);
             return *this;
           }
-        } else {                // not carrying a span.
-          if (n->_max <= max) { // next range is covered - use it.
+        } else {                     // not carrying a span.
+          if (n->_max <= localmax) { // next range is covered - use it.
             x = n;
-            x->setMin(min);
+            x->setMin(localmin);
             n = next(n);
           } else if (n->_min <= max_plus1) {
-            n->setMin(min);
+            n->setMin(localmin);
             return *this;
           } else { // no overlap, space to complete range.
-            this->insertBefore(n, new N(min, max, payload));
+            this->insert_before(n, new N(localmin, localmax, payload));
             return *this;
           }
         }
       } else { // different payload
         if (x) {
-          if (max < n->_min) { // range ends before n starts, done.
-            x->setMax(max);
+          if (localmax < n->_min) { // range ends before n starts, done.
+            x->setMax(localmax);
             return *this;
-          } else if (max <= n->_max) { // range ends before n, done.
+          } else if (localmax <= n->_max) { // range ends before n, done.
             x->setMaxMinusOne(n->_min);
             return *this;
           } else { // n is contained in range, skip over it.
             x->setMaxMinusOne(n->_min);
-            x   = nullptr;
-            min = n->_max;
-            N::inc(min); // OK because n->_max maximal => next is null.
+            x        = nullptr;
+            localmin = n->_max;
+            N::inc(localmin); // OK because n->_max maximal => next is null.
             n = next(n);
           }
-        } else {               // no carry node.
-          if (max < n->_min) { // entirely before next span.
-            this->insertBefore(n, new N(min, max, payload));
+        } else {                    // no carry node.
+          if (localmax < n->_min) { // entirely before next span.
+            this->insert_before(n, new N(localmin, localmax, payload));
             return *this;
           } else {
-            if (min < n->_min) { // leading section, need node.
-              N *y = new N(min, n->_min, payload);
+            if (localmin < n->_min) { // leading section, need node.
+              N *y = new N(localmin, n->_min, payload);
               y->decrementMax();
-              this->insertBefore(n, y);
+              this->insert_before(n, y);
             }
-            if (max <= n->_max) { // nothing past node
+            if (localmax <= n->_max) { // nothing past node
               return *this;
             }
-            min = n->_max;
-            N::inc(min);
+            localmin = n->_max;
+            N::inc(localmin);
             n = next(n);
           }
         }
@@ -427,9 +448,9 @@ namespace detail
     }
     // Invariant: min is larger than any existing range maximum.
     if (x) {
-      x->setMax(max);
+      x->setMax(localmax);
     } else {
-      this->append(new N(min, max, payload));
+      this->append(new N(localmin, localmax, payload));
     }
     return *this;
   }
@@ -447,17 +468,14 @@ namespace detail
     Metric max_plus = N::deref(max);
     N::inc(max_plus);
 
-    /* Some subtlety - for IPv6 we overload the compare operators to do
-       the right thing, but we can't overload pointer
-       comparisons. Therefore we carefully never compare pointers in
-       this logic. Only @a min and @a max can be pointers, everything
-       else is an instance or a reference. Since there's no good reason
-       to compare @a min and @a max this isn't particularly tricky, but
-       it's good to keep in mind. If we were somewhat more clever, we
-       would provide static less than and equal operators in the
-       template class @a N and convert all the comparisons to use only
-       those two via static function call.
-    */
+    /* Some subtlety - for IPv6 we overload the compare operators to do the right thing, but we
+     * can't overload pointer comparisons. Therefore we carefully never compare pointers in this
+     * logic. Only @a min and @a max can be pointers, everything else is an instance or a reference.
+     * Since there's no good reason to compare @a min and @a max this isn't particularly tricky, but
+     * it's good to keep in mind. If we were somewhat more clever, we would provide static less than
+     * and equal operators in the template class @a N and convert all the comparisons to use only
+     * those two via static function call.
+     */
 
     /*  We have lots of special cases here primarily to minimize memory
         allocation by re-using an existing node as often as possible.
@@ -485,7 +503,7 @@ namespace detail
           // request span is covered by existing span.
           x = new N(min, max, payload); //
           n->setMin(max_plus);          // clip existing.
-          this->insertBefore(n, x);
+          this->insert_before(n, x);
           return *this;
         }
       } else if (n->_data == payload && n->_max >= min_1) {
@@ -517,20 +535,20 @@ namespace detail
         x = new N(min, max, payload);
         r = new N(max_plus, n->_max, n->_data);
         n->setMax(min_1);
-        this->insertAfter(n, x);
-        this->insertAfter(x, r);
+        this->insert_after(n, x);
+        this->insert_after(x, r);
         return *this; // done.
       }
       n = next(n); // lower bound span handled, move on.
       if (!x) {
         x = new N(min, max, payload);
         if (n) {
-          this->insertBefore(n, x);
+          this->insert_before(n, x);
         } else {
           this->append(x); // note that since n == 0 we'll just return.
         }
       }
-    } else if (nullptr != (n = this->getHead()) &&     // at least one node in tree.
+    } else if (nullptr != (n = this->head()) &&        // at least one node in tree.
                n->_data == payload &&                  // payload matches
                (n->_max <= max || n->_min <= max_plus) // overlap or adj.
     ) {
@@ -584,7 +602,7 @@ namespace detail
           x = new N(max, N::argue(n->_max), n->_data);
           x->incrementMin();
           n->setMaxMinusOne(N::deref(min));
-          this->insertAfter(n, x);
+          this->insert_after(n, x);
           return *this; // done.
         } else {
           n->setMaxMinusOne(N::deref(min)); // just clip overlap.
@@ -610,7 +628,7 @@ namespace detail
 
   template <typename N>
   void
-  IpMapBase<N>::insertAfter(N *spot, N *n)
+  IpMapBase<N>::insert_after(N *spot, N *n)
   {
     N *c = right(spot);
     if (!c) {
@@ -619,22 +637,26 @@ namespace detail
       spot->_next->setChild(n, N::LEFT);
     }
 
-    _list.insertAfter(spot, n);
+    _list.insert_after(spot, n);
     _root = static_cast<N *>(n->rebalanceAfterInsert());
   }
 
   template <typename N>
   void
-  IpMapBase<N>::insertBefore(N *spot, N *n)
+  IpMapBase<N>::insert_before(N *spot, N *n)
   {
-    N *c = left(spot);
-    if (!c) {
+    if (left(spot) == nullptr) {
       spot->setChild(n, N::LEFT);
     } else {
+// If there's a left child, there's a previous node, therefore spot->_prev is valid.
+// Clang analyzer doesn't realize this so it generates a false positive.
+#if defined(__clang_analyzer__)
+      ink_assert(spot->_prev != nullptr);
+#endif
       spot->_prev->setChild(n, N::RIGHT);
     }
 
-    _list.insertBefore(spot, n);
+    _list.insert_before(spot, n);
     _root = static_cast<N *>(n->rebalanceAfterInsert());
   }
 
@@ -645,7 +667,7 @@ namespace detail
     if (!_root) {
       _root = n;
     } else {
-      _root = static_cast<N *>(_list.getHead()->setChild(n, N::LEFT)->rebalanceAfterInsert());
+      _root = static_cast<N *>(_list.head()->setChild(n, N::LEFT)->rebalanceAfterInsert());
     }
     _list.prepend(n);
   }
@@ -657,7 +679,7 @@ namespace detail
     if (!_root) {
       _root = n;
     } else {
-      _root = static_cast<N *>(_list.getTail()->setChild(n, N::RIGHT)->rebalanceAfterInsert());
+      _root = static_cast<N *>(_list.tail()->setChild(n, N::RIGHT)->rebalanceAfterInsert());
     }
     _list.append(n);
   }
@@ -667,7 +689,7 @@ namespace detail
   IpMapBase<N>::remove(N *n)
   {
     _root = static_cast<N *>(n->remove());
-    _list.take(n);
+    _list.erase(n);
     delete n;
   }
 
@@ -695,9 +717,9 @@ namespace detail
 
   template <typename N>
   size_t
-  IpMapBase<N>::getCount() const
+  IpMapBase<N>::count() const
   {
-    return _list.getCount();
+    return _list.count();
   }
   //----------------------------------------------------------------------------
   template <typename N>
@@ -706,7 +728,7 @@ namespace detail
   {
 #if 0
   if (_root) _root->validate();
-  for ( Node* n = _list.getHead() ; n ; n = n->_next ) {
+  for ( Node* n = _list.head() ; n ; n = n->_next ) {
     Node* x;
     if (0 != (x = n->_next)) {
       if (x->_prev != n)
@@ -721,22 +743,27 @@ namespace detail
   }
 
   template <typename N>
-  IpMapBase<N> &
-  IpMapBase<N>::print()
+  ts::BufferWriter &
+  IpMapBase<N>::describe(ts::BufferWriter &w, ts::BWFSpec const &spec) const
   {
-#if 0
-  for ( Node* n = _list.getHead() ; n ; n = n->_next ) {
-    std::cout
-      << n << ": " << n->_min << '-' << n->_max << " [" << n->_data << "] "
-      << (n->_color == Node::BLACK ? "Black " : "Red   ") << "P=" << n->_parent << " L=" << n->_left << " R=" << n->_right
-      << std::endl;
-  }
-#endif
-    return *this;
+    auto pos = w.extent();
+    for (auto const &rb_node : _list) {
+      N const &n{static_cast<N const &>(rb_node)};
+      if (w.extent() > pos) {
+        w.write(',');
+      }
+      w.print("{::a}-{::a}={}", n.min(), n.max(), n._data);
+      if (std::string_view::npos != spec._ext.find('x')) {
+        w.print("[{};^{};<{};>{}]", n._color == N::BLACK ? "Black" : "Red", n._parent, n._left, n._right);
+      }
+    }
+    return w;
   }
 
+  template <typename N> IpMapBase<N>::~IpMapBase() { this->clear(); }
+
   //----------------------------------------------------------------------------
-  typedef Interval<in_addr_t, in_addr_t> Ip4Span;
+  using Ip4Span = Interval<in_addr_t, in_addr_t>;
 
   /** Node for IPv4 map.
       We store the address in host order in the @a _min and @a _max
@@ -822,7 +849,7 @@ namespace detail
     {
       return this->setMin(min + 1);
     }
-    /** Decremement the maximum value in place.
+    /** decrement the maximum value in place.
         @return This object.
     */
     self_type &
@@ -898,7 +925,13 @@ namespace detail
     /// is to use a pointer, not a reference.
     using ArgType = const ts::detail::Interval<sockaddr_in6, const sockaddr_in6 &>::Metric *;
 
-    /// Construct from pointers.
+    /** Construct from the argument type.
+     *
+     * @param min Minimum value in the range.
+     * @param max Maximum value in the range (inclusive).
+     * @param data Data to attach to the range.
+     */
+
     Ip6Node(ArgType min, ///< Minimum address (network order).
             ArgType max, ///< Maximum address (network order).
             void *data   ///< Client data.
@@ -906,30 +939,36 @@ namespace detail
       : Node(data), Ip6Span(*min, *max)
     {
     }
-    /// Construct with values.
-    Ip6Node(Metric const &min, ///< Minimum address (network order).
-            Metric const &max, ///< Maximum address (network order).
-            void *data         ///< Client data.
-            )
-      : Node(data), Ip6Span(min, max)
-    {
-    }
+
+    /** Construct from the underlying @c Metric type @a min to @a max
+     *
+     * @param min Minimum value in the range.
+     * @param max Maximum value in the range (inclusive).
+     * @param data Data to attach to the range.
+     */
+    Ip6Node(Metric const &min, Metric const &max, void *data) : Node(data), Ip6Span(min, max) {}
+
     /// @return The minimum value of the interval.
     sockaddr const *
     min() const override
     {
       return ats_ip_sa_cast(&_min);
     }
+
     /// @return The maximum value of the interval.
     sockaddr const *
     max() const override
     {
       return ats_ip_sa_cast(&_max);
     }
-    /// Set the client data.
+
+    /** Set the client @a data.
+     *
+     * @param data Client data.
+     * @return @a this
+     */
     self_type &
-    setData(void *data ///< Client data.
-            ) override
+    setData(void *data) override
     {
       _data = data;
       return *this;
@@ -996,7 +1035,7 @@ namespace detail
       inc(_min);
       return *this;
     }
-    /** Decremement the maximum value in place.
+    /** Decrement the maximum value in place.
         @return This object.
     */
     self_type &
@@ -1005,7 +1044,7 @@ namespace detail
       dec(_max);
       return *this;
     }
-    /** Increment the mininimum value in place.
+    /** Increment the minimum value in place.
         @return This object.
     */
     self_type &
@@ -1066,8 +1105,33 @@ namespace detail
     friend class ::IpMap;
   };
 } // namespace detail
+
+template <typename N>
+inline BufferWriter &
+bwformat(BufferWriter &w, BWFSpec const &spec, detail::IpMapBase<N> const &map)
+{
+  return map.describe(w, spec);
+}
+
 } // namespace ts
 //----------------------------------------------------------------------------
+IpMap::IpMap(IpMap::self_type &&that) noexcept : _m4(that._m4), _m6(that._m6)
+{
+  that._m4 = nullptr;
+  that._m6 = nullptr;
+}
+
+IpMap::self_type &
+IpMap::operator=(IpMap::self_type &&that)
+{
+  if (&that != this) {
+    this->clear();
+    std::swap(_m4, that._m4);
+    std::swap(_m6, that._m6);
+  }
+  return *this;
+}
+
 IpMap::~IpMap()
 {
   delete _m4;
@@ -1174,14 +1238,14 @@ IpMap::fill(in_addr_t min, in_addr_t max, void *data)
 }
 
 size_t
-IpMap::getCount() const
+IpMap::count() const
 {
   size_t zret = 0;
   if (_m4) {
-    zret += _m4->getCount();
+    zret += _m4->count();
   }
   if (_m6) {
-    zret += _m6->getCount();
+    zret += _m6->count();
   }
   return zret;
 }
@@ -1203,10 +1267,10 @@ IpMap::begin() const
 {
   Node *x = nullptr;
   if (_m4) {
-    x = _m4->getHead();
+    x = _m4->head();
   }
   if (!x && _m6) {
-    x = _m6->getHead();
+    x = _m6->head();
   }
   return iterator(this, x);
 }
@@ -1218,8 +1282,8 @@ IpMap::iterator::operator++()
     // If we go past the end of the list see if it was the v4 list
     // and if so, move to the v6 list (if it's there).
     Node *x = static_cast<Node *>(_node->_next);
-    if (!x && _tree->_m4 && _tree->_m6 && _node == _tree->_m4->getTail()) {
-      x = _tree->_m6->getHead();
+    if (!x && _tree->_m4 && _tree->_m6 && _node == _tree->_m4->tail()) {
+      x = _tree->_m6->head();
     }
     _node = x;
   }
@@ -1233,21 +1297,43 @@ IpMap::iterator::operator--()
     // At a node, try to back up. Handle the case where we back over the
     // start of the v6 addresses and switch to the v4, if there are any.
     Node *x = static_cast<Node *>(_node->_prev);
-    if (!x && _tree->_m4 && _tree->_m6 && _node == _tree->_m6->getHead()) {
-      x = _tree->_m4->getTail();
+    if (!x && _tree->_m4 && _tree->_m6 && _node == _tree->_m6->head()) {
+      x = _tree->_m4->tail();
     }
     _node = x;
   } else if (_tree) {
     // We were at the end. Back up to v6 if possible, v4 if not.
     if (_tree->_m6) {
-      _node = _tree->_m6->getTail();
+      _node = _tree->_m6->tail();
     }
     if (!_node && _tree->_m4) {
-      _node = _tree->_m4->getTail();
+      _node = _tree->_m4->tail();
     }
   }
   return *this;
 }
 
+ts::BufferWriter &
+IpMap::describe(ts::BufferWriter &w, ts::BWFSpec const &spec) const
+{
+  w.write("IPv4 ");
+  if (_m4) {
+    bwformat(w, spec, *_m4);
+  } else {
+    w.write("N/A");
+  }
+  w.write("\n");
+
+  w.write("IPv6 ");
+  if (_m6) {
+    bwformat(w, spec, *_m6);
+  } else {
+    w.write("N/A");
+  }
+  w.write("\n");
+
+  return w;
+}
+
 //----------------------------------------------------------------------------
 //----------------------------------------------------------------------------
