@@ -16,8 +16,6 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import os
-import subprocess
 Test.Summary = '''
 Test uri_signing plugin
 '''
@@ -26,70 +24,65 @@ Test.ContinueOnFail = False
 
 # Skip if plugins not present.
 Test.SkipUnless(Condition.PluginExists('uri_signing.so'))
-Test.SkipUnless(Condition.PluginExists('cachekey.so'))
 
 server = Test.MakeOriginServer("server")
 
 # Default origin test
-req_header = { "headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "",
-}
-res_header = { "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "",
-}
+req_header = {"headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "",
+              }
+res_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "",
+              }
 server.addResponse("sessionfile.log", req_header, res_header)
 
 # Test case for normal
-req_header = { "headers":
-	"GET /someasset.ts HTTP/1.1\r\nHost: somehost\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "",
-}
+req_header = {"headers":
+              "GET /someasset.ts HTTP/1.1\r\nHost: somehost\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "",
+              }
 
-res_header = { "headers":
-	"HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "somebody",
-}
+res_header = {"headers":
+              "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "somebody",
+              }
 
 server.addResponse("sessionfile.log", req_header, res_header)
 
 # Test case for crossdomain
-req_header = { "headers":
-	"GET /crossdomain.xml HTTP/1.1\r\nHost: somehost\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "",
-}
+req_header = {"headers":
+              "GET /crossdomain.xml HTTP/1.1\r\nHost: somehost\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "",
+              }
 
-res_header = { "headers":
-	"HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
-	"timestamp": "1469733493.993",
-	"body": "<crossdomain></crossdomain>",
-}
+res_header = {"headers":
+              "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n",
+              "timestamp": "1469733493.993",
+              "body": "<crossdomain></crossdomain>",
+              }
 
 server.addResponse("sessionfile.log", req_header, res_header)
 
 # http://user:password@host:port/path;params?query#fragment
 
 # Define default ATS
-ts = Test.MakeATSProcess("ts")
+ts = Test.MakeATSProcess("ts", enable_cache=False)
 #ts = Test.MakeATSProcess("ts", "traffic_server_valgrind.sh")
 
 ts.Disk.records_config.update({
-  'proxy.config.diags.debug.enabled': 1,
-  'proxy.config.diags.debug.tags': 'uri_signing|http',
-#  'proxy.config.diags.debug.tags': 'uri_signing',
-  'proxy.config.http.cache.http': 0,  # No cache
+    'proxy.config.diags.debug.enabled': 1,
+    'proxy.config.diags.debug.tags': 'uri_signing|http',
+    #  'proxy.config.diags.debug.tags': 'uri_signing',
 })
 
 # Use unchanged incoming URL.
-# This uses cachekey to handle the effective vs pristine url diff for the
-# first plugin issue that exists in 8.x. This is not necessary on 9x+
 ts.Disk.remap_config.AddLine(
     'map http://somehost/ http://127.0.0.1:{}/'.format(server.Variables.Port) +
-    ' @plugin=cachekey.so @pparam=--include-headers=foo' +
     ' @plugin=uri_signing.so @pparam={}/config.json'.format(Test.RunDirectory)
 )
 
@@ -202,7 +195,7 @@ ps.Streams.stderr = "gold/403.gold"
 tr.StillRunningAfter = server
 tr.StillRunningAfter = ts
 
-# 11 - multiple cookies
+# 9 - multiple cookies
 tr = Test.AddTestRun("multiple cookies, expired then good")
 ps = tr.Processes.Default
 ps.Command = curl_and_args + '"http://somehost/someasset.ts" -H "Cookie: URISigningPackage=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJpc3N1ZXIiLCJleHAiOjF9.GkdlOPHQc6BqS4Q6x79GeYuVFO2zuGbaPZZsJfD6ir8;URISigningPackage=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJpc3N1ZXIiLCJleHAiOjE5MjMwNTYwODR9.zw_wFQ-wvrWmfPLGj3hAUWn-GOHkiJZi2but4KV0paY"'
@@ -210,14 +203,3 @@ ps.ReturnCode = 0
 ps.Streams.stderr = "gold/200.gold"
 tr.StillRunningAfter = server
 tr.StillRunningAfter = ts
-
-
-# 12 - Check missing iss from the payload
-tr = Test.AddTestRun("Missing iss field in the payload")
-ps = tr.Processes.Default
-ps.Command = curl_and_args + '"http://somehost/someasset.ts?URISigningPackage=ewogICJ0eXAiOiAiSldUIiwKICAiYWxnIjogIkhTMjU2Igp9.ewogICJleHAiOiAxOTIzMDU2MDg0Cn0.zw_wFQ-wvrWmfPLGj3hAUWn-GOHkiJZi2but4KV0paY"'
-ps.ReturnCode = 0
-ps.Streams.stderr = "gold/403.gold"
-ts.Streams.stderr = Testers.ContainsExpression("Initial JWT Failure: iss is missing, must be present", "should fail the validation")
-tr.StillRunningAfter = server
-tr.StillRunningAfter = ts
